/* -*- Mode: C++ -*-
 * $Id: ActuatorInterface.cc,v 1.6 2004/08/18 12:05:17 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ActuatorInterface.cc
 * Unitname: BasicSystem
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: petan $ $Date: 2004/08/18 12:05:17 $
 *
 *
 * HISTORY
 *
 * $Log: ActuatorInterface.cc,v $
 * Revision 1.6  2004/08/18 12:05:17  petan
 * Added support for AgentStep
 *
 * Revision 1.5  2003/10/09 15:50:44  frehe
 * Clone the commands from a step before adding them to the actuator interface.
 *
 * Revision 1.4  2003/09/02 17:55:00  frehe
 * Added the semantics of the command pointers. RoboSoc will now take control of them.
 *
 * Revision 1.3  2000/10/23 12:05:50  frehe
 * Implemented the addToPlan(Step) method.
 *
 * Revision 1.2  2000/09/05 21:15:52  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/03 20:41:11  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ActuatorInterface.cc
 * Implementation file for the class ActuatorInterface.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.6 $
 * $Author: petan $
 * $Date: 2004/08/18 12:05:17 $
 */


#include "ActuatorInterface.h"
#include "Actuators.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE


ActuatorInterface::ActuatorInterface()
  : theActuators(NULL)
{
}


ActuatorInterface::~ActuatorInterface()
{
}


bool ActuatorInterface::addToPlan(Command* command)
{
  /*
  ServerCommand* scmd = dynamic_cast<ServerCommand*>(command);
  if ( scmd != 0 ) {
    std::cout << "addToPlan " << scmd->serverString() << std::endl;
  }
  */
  
  theActuators->executeCommand(command);
  return true;
  /* This is a possible new implementation
  Step step = currentPlan.getLastStep();
  currentPlan.removeLastStep();
  step.addCommand(command);
  currentPlan.addStep(step);
  return true;
  */
};


bool ActuatorInterface::addToPlan(Step step)
{
  bool result = true;
  while ( !step.empty() ) {
    result = result && addToPlan(step.getNextCommand()->clone());
    step.removeNextCommand();
  }
  //std::cout << "addToPlan step is empty" << std::endl;
  return result;
};

bool ActuatorInterface::addToPlan(AgentStep step)
{
  bool result = true;
  while ( !step.empty() ) {
    result = result && addToPlan(step.getNextCommand()->clone());
    step.removeNextCommand();
  }
  return result;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ActuatorInterface.icc"
#endif
