/* -*- Mode: C++ -*-
 * $Id: ControlInterface.h,v 1.8 2004/08/29 13:35:48 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ControlInterface.h
 * Unitname: BasicSystem
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2004/08/29 13:35:48 $
 *
 *
 * HISTORY
 *
 * $Log: ControlInterface.h,v $
 * Revision 1.8  2004/08/29 13:35:48  frehe
 * Added a onVisualCoachSensorData callback.
 *
 * Revision 1.7  2003/08/20 12:28:03  s02davro
 * New count counter, attentiontos etc.
 *
 * Revision 1.6  2003/07/21 08:53:06  s02davro
 * Added callback function when a substitution is noticed
 *
 * Revision 1.5  2002/09/04 13:25:08  s02davro
 * Removed deprecated methods in Controller and Decision
 *
 * Revision 1.4  2002/08/02 21:48:01  s02davro
 * Added some todos.
 *
 * Revision 1.3  2001/10/14 21:21:42  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.2  2001/08/13 16:43:11  frehe
 * Made updates to compile with new autotools and RCCParser.
 *
 * Revision 1.1  2000/09/03 20:41:08  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ControlInterface.h
 * Include file for the class ControlInterface.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2004/08/29 13:35:48 $
 */


#ifndef CONTROLINTERFACE_H
#define CONTROLINTERFACE_H


#include "ServerCommand.h"
#include "Decision.h"


RS_BEGIN_NAMESPACE


/**
 * @class ControlInterface ControlInterface.h
 * This class is the interface between the controller
 * and the decision maker.
 *
 * @todo Add documentation.
 * @todo Fix better syncronization (more information in Controll)
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2004/08/29 13:35:48 $
 */
class ControlInterface {
public:
  ControlInterface(Decision* const decision);
  virtual ~ControlInterface();
  
  virtual ServerCommand* initAgent();
  
  // Event handlers
  virtual void onActuatorSensorData();
  virtual void onAuralSensorData();
  virtual void onAuralCoachSensorData();
  virtual void onAuralRefereeSensorData();
  virtual void onAuralPlayerSensorData();
  virtual void onAuralSelfSensorData();
  virtual void onSubstitution();
  virtual void onInit();
  virtual void onPhysicalSensorData();
  virtual void onSensorError();
  virtual void onVisualCoachSensorData();
  virtual void onVisualSensorData();

  virtual void onBeforeSensorData();
  virtual void onAfterSensorData();
  virtual void onBeforeTick();
  virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

  virtual void onDelayedActions();
  virtual void onMissingActions();

protected:
  Decision* const theDecisionMaker;

  bool invariant() const;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ControlInterface.icc"
#endif


#endif // !CONTROLINTERFACE_H
