/* -*- Mode: C++ -*-
 * $Id: SensorInterface.cc,v 1.12 2002/08/31 15:05:44 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SensorInterface.cc
 * Unitname: BasicSystem
 * $Revision: 1.12 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 15:05:44 $
 *
 *
 * HISTORY
 *
 * $Log: SensorInterface.cc,v $
 * Revision 1.12  2002/08/31 15:05:44  s02davro
 * Updated the sensor interface to the new rccparser
 *
 * Revision 1.11  2002/08/02 21:49:49  s02davro
 * Added documentation, converted to abstract baseclass, old SensorInterface is now BasicSensorInterface.
 *
 * Revision 1.10  2002/01/20 17:17:13  frehe
 * Added a debug message to hearReferee.
 *
 * Revision 1.9  2002/01/16 09:53:52  frehe
 * Fixed the coach parsing of referee messages.
 *
 * Revision 1.8  2001/10/14 21:21:42  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.7  2001/09/17 21:53:38  frehe
 * Added the relative speed to the seeBall method.
 *
 * Revision 1.6  2001/09/01 23:52:23  frehe
 * Added some debug info.
 *
 * Revision 1.5  2001/08/19 23:32:23  frehe
 * Updated 2001-08-19
 *
 * Revision 1.4  2001/08/13 16:43:16  frehe
 * Made updates to compile with new autotools and RCCParser.
 *
 * Revision 1.3  2000/09/13 00:01:51  frehe
 * Made minor changes to the forEach methods.
 *
 * Revision 1.2  2000/09/05 21:15:53  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/03 20:41:13  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file SensorInterface.cc
 * Implementation file for the class SensorInterface.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.12 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:05:44 $
 */


#include "SensorInterface.h"
#include "rs_error.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE


SensorInterface::SensorInterface() 
  : forcedNewCycle(false),
    forcedNewCycleTime(Time(0,0)),
    currentTime(Time::currentTime),
    clockStopped(Time::clockStopped),
    lastCycleTime(Time(0,0)) {
}

SensorInterface::~SensorInterface() { 
} 

void SensorInterface::newCycle() {
  if ((lastCycleTime < currentTime) && 
      (!forcedNewCycle ||(forcedNewCycle && 
			  (forcedNewCycleTime < currentTime)))) {
    updateBeforeNewCycle();

    if (((currentTime.getT() - lastCycleTime.getT()) > 1) ||
	(currentTime.isClockStopped() && 
	 ((currentTime.getS() - lastCycleTime.getS()) > 1)))
      RS_WARNING(currentTime << ": [SensorInterface] Missed cycle(s)!"
		 << " lastCycleTime = " << lastCycleTime);
    
    lastCycleTime = currentTime;    
    forcedNewCycle = false;
  }
  else {
    RS_WARNING(currentTime 
	       << ": SKIPPED NEW CYCLE SINCE FORCED NEW CYCLE ALREADY!");
  }
}

void SensorInterface::forceNewCycle() {
  if ((lastCycleTime < currentTime) && 
      (!forcedNewCycle || (forcedNewCycle &&
			   (forcedNewCycleTime < currentTime)))) {
    forcedNewCycle = true;
    forcedNewCycleTime = currentTime;
    lastCycleTime = currentTime;
    RS_WARNING(currentTime << ": FORCED NEW CYCLE!");
    updateBeforeNewCycle();
  }
}


void SensorInterface::updateBeforeNewCycle() {
}



RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "SensorInterface.icc"
#endif
