/* -*- Mode: C++ -*-
 * $Id: Sensors.cc,v 1.47 2004/09/05 19:38:15 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Sensors.cc
 * Unitname: BasicSystem
 * $Revision: 1.47 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2004/09/05 19:38:15 $
 *
 *
 * HISTORY
 *
 * $Log: Sensors.cc,v $
 * Revision 1.47  2004/09/05 19:38:15  frehe
 * Improved support for focus.
 *
 * Revision 1.46  2004/09/04 10:38:12  frehe
 * Fixed bug where parsedSide was not reset after parsing.
 *
 * Revision 1.45  2004/08/29 14:42:16  frehe
 * Fixed support for getting the teamnames if only one is known.
 *
 * Revision 1.44  2004/08/27 14:57:53  frehe
 * Added support for (ok team_names...)
 *
 * Revision 1.43  2004/08/27 12:21:44  frehe
 * Fixed init message for trainers.
 *
 * Revision 1.42  2004/08/23 20:47:20  frehe
 * Added support for move ok.
 *
 * Revision 1.41  2004/08/23 13:23:23  frehe
 * Added support for (ok ear on).
 *
 * Revision 1.40  2003/10/14 18:07:52  s02davro
 * Added call for command to sensorinterface
 *
 * Revision 1.39  2003/10/02 15:41:43  s02davro
 * Fixed bug related to the offensive corner flags
 *
 * Revision 1.38  2003/09/02 17:55:01  frehe
 * Added the semantics of the command pointers. RoboSoc will now take control of them.
 *
 * Revision 1.37  2003/08/20 12:29:56  s02davro
 * ControlCommand fix, count counter
 *
 * Revision 1.36  2003/07/22 14:45:22  s02davro
 * Logging, arm, tackle, focus + bugfix
 *
 * Revision 1.35  2003/07/21 08:55:58  s02davro
 * Added support for new rccparser and server version 9.
 *
 * Revision 1.34  2002/12/07 17:29:04  s02davro
 * Hack in Sensor to fix trainer added CP_trainer to WorldFacts
 *
 * Revision 1.33  2002/09/26 15:23:41  s02davro
 * Bugfixes, close, team, coach
 *
 * Revision 1.32  2002/09/23 10:28:41  s02davro
 * Find bug in (close), this information needs to be handled - todo
 *
 * Revision 1.31  2002/09/21 15:38:57  s02davro
 * Update to new (1.0.3) RCCParser
 *
 * Revision 1.30  2002/08/31 15:06:16  s02davro
 * Redesign (again) now handles the coach fullstate information messages
 *
 * Revision 1.29  2002/08/13 10:23:17  s02davro
 * Update to handle ServerParam, PlayerParam and PlayerType messages.
 *
 * Revision 1.28  2002/08/02 21:51:49  s02davro
 * Complete rewrite, now uses rccparser-1.0 and clangparser from the soccerserver.
 *
 * Revision 1.26  2002/01/20 17:16:49  frehe
 * Fixed the problem with the coach not hearing the referee messages.
 *
 * Revision 1.25  2002/01/16 09:53:52  frehe
 * Fixed the coach parsing of referee messages.
 *
 * Revision 1.24  2002/01/15 17:19:41  frehe
 * Improved the coach parser.
 *
 * Revision 1.23  2002/01/11 15:27:18  frehe
 * Added support for coach freeform messages.
 *
 * Revision 1.22  2001/11/15 16:48:48  frehe
 * Added support for a coach.
 *
 * Revision 1.21  2001/10/14 21:21:43  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.20  2001/10/14 14:48:36  frehe
 * Removed old debug info.
 *
 * Revision 1.19  2001/09/22 17:18:28  frehe
 * Added a workaround to get teammates from rccparser.
 *
 * Revision 1.18  2001/09/17 21:53:59  frehe
 * Readded support for a debug coach.
 *
 * Revision 1.17  2001/09/17 21:30:09  andbo928
 * Implemented an ugly workaround to get ballspeeds working.
 *
 * Revision 1.16  2001/09/14 13:48:04  frehe
 * Fixed the speed bug.
 *
 * Revision 1.15  2001/09/03 19:39:36  frehe
 * Changed code to use the RS_BEGIN_TRY, RS_END_TRY and RS_CATCH macros.
 *
 * Revision 1.14  2001/09/01 23:52:23  frehe
 * Added some debug info.
 *
 * Revision 1.13  2001/08/30 21:41:47  frehe
 * Removed a call to distDirChng in RCCParser that crashed the agent.
 *
 * Revision 1.12  2001/08/29 16:37:32  frehe
 * Added a test on the validity of vec in getVectorUnknown(RCC_Vector2d).
 *
 * Revision 1.11  2001/08/29 12:35:26  frehe
 * Fixed a bug in sense_body handler.
 *
 * Revision 1.10  2001/08/22 21:06:59  frehe
 * Updated 2001-08-22
 *
 * Revision 1.9  2001/08/19 23:32:24  frehe
 * Updated 2001-08-19
 *
 * Revision 1.8  2001/08/17 07:15:16  frehe
 * Made changes to compile RoboSoc with RCCParser-0.10.
 *
 * Revision 1.7  2001/08/13 16:43:16  frehe
 * Made updates to compile with new autotools and RCCParser.
 *
 * Revision 1.6  2001/08/13 07:33:16  frehe
 * Updated from laptop 2001-08-13.
 *
 * Revision 1.5  2001/08/02 17:04:22  frehe
 * Updated 2001-08-02
 *
 * Revision 1.4  2000/10/24 15:15:17  frehe
 * Fixed the generation of the onNewCycle event.
 *
 * Revision 1.3  2000/09/05 21:15:53  frehe
 * Made minor changes.
 *
 * Revision 1.2  2000/09/04 08:46:45  frehe
 * Fixed a naming error in the sensors.
 *
 * Revision 1.1  2000/09/03 20:41:14  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Sensors.cc
 * Implementation file for the class Sensors.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.47 $
 * $Author: frehe $
 * $Date: 2004/09/05 19:38:15 $
 */


#include <sstream>
#include <cstdio>
#include <cstring>
#include <cctype>

#include "Sensors.h"
#include "Controller.h"
#include "SensorInterface.h"

#include "SetCyclesSinceIOBeforeShutdown.h"
#include "SetMinTicksBeforeSendingCommand.h"
#include "SetMaxTicksBeforeSendingCommand.h"
#include "SetTicksBeforeCommandWarning.h"
#include "SetTicksBetweenCycles.h"
#include "SetTicksBetweenSenses.h"
#include "SetTicksBetweenVisuals.h"

#include "string_utils.h"
#include "rs_error.h"
#include "rs_log.h"
#include "rs_try_catch.h"

RS_BEGIN_NAMESPACE


Sensors::Sensors(SensorInterface* sensor_interface, 
		 rcss::Parser& coachMessageParser)
  : rcc::Parser(coachMessageParser),
	 facts(WorldFacts::nonConstInstance()),
	 theController(NULL),
	 theSensorInterface(sensor_interface),
	 lastSensorType(SENSOR_UNKNOWN),
	 currentTime(0),
	 commandQueue(),
	 parsedSide(S_UNKNOWN),
	 parsedPlayMode(PM_UNKNOWN),
	 parsedViewQuality(VQ_UNKNOWN),
	 parsedViewWidth(VW_UNKNOWN),
	 parsedLocations(),
	 parsedFlagOffset(UIntUnknown()),
	 parsedUniformNumber(UIntUnknown()),
	 parsedGoalie(false),
	 parsedTeam(T_UNKNOWN),
	 parsedArmState(),
	 parsedFocusState(),
	 parsedTackleState() {}

void Sensors::init(Controller* controller) {
  theController = controller;
}

void Sensors::analyze(ServerCommand* command) {
  lastSensorType = SENSOR_ACTUATORS;
  commandQueue.push(command->getType());
  theSensorInterface->setLastCommand(command->clone());
  theSensorInterface->setLastSensorType(lastSensorType);
  delete command;
}

void Sensors::analyze(char* sensor_data) {
  lastSensorType = SENSOR_UNKNOWN;  
  std::istringstream strm(sensor_data);
  if (!parse(strm)) {
    RS_LOG( LA_WARNING, "Parse error on sensor data: '" << sensor_data << "'");
  }
  else if ( lastSensorType == SENSOR_UNKNOWN ) 
    RS_LOG( LA_WARNING, "Unknown sensor data: '" << sensor_data << "'");
  else 
    theSensorInterface->setLastSensorType(lastSensorType);
}

void Sensors::updateTime(int time) {
  setCurrentTime(time);
  if ( theController->updateTime(getCurrentTime()) ) {
    theSensorInterface->newCycle();
    RS_LOG( LA_CONTROLLER, "Time: " << toString(getCurrentTime()) 
	    << " --- NEW CYCLE ---");
  }
}

/*
 * Misc parsing
 */

void Sensors::doMsgParsed() {
}

void Sensors::doBuildScore( int time, int our, int opp ) {
  RS_LOG ( LA_SENSORS, "BuildScore: " << time << ", " << our << ", " << opp );
  facts->setScore(facts->getOurSide(), our);
  facts->setScore(facts->getTheirSide(), opp);
  updateTime(time);
  lastSensorType = SENSOR_AURAL_SCORE;

}

void Sensors::doBuildTeamName( const std::string& name ) {
  RS_LOG ( LA_SENSORS, "BuildTeamName: " << name );
  if ( parsedSide != S_UNKNOWN ) {
    RS_LOG ( LA_SENSORS, "BuildTeamName: got team name" );
    if ( facts->getOurSide() == parsedSide ) {
      facts->CP_team_name = name;
    } else {
      facts->CP_their_team_name = name;
    }
  } else {
    RS_LOG( LA_SENSORS, "BuildTeamName: " << facts->CP_team_name
	    << ", " << facts->CP_their_team_name );
    if ( name == facts->CP_team_name ) {
      RS_LOG ( LA_SENSORS, "BuildTeamName: our team" );
      parsedTeam = T_OUR_TEAM;
    } else if ( name == facts->CP_their_team_name ) {
      RS_LOG ( LA_SENSORS, "BuildTeamName: their team" );
      parsedTeam = T_THEIR_TEAM;
    } else {
      // We have encountered an unknown teamname.
      // If we have one of the team names already then
      // update the other
      if ( facts->isOurTeamNameKnown() ) {
	RS_LOG ( LA_SENSORS, "BuildTeamName: assume their team is " << name );
	facts->CP_their_team_name = name;
	parsedTeam = T_THEIR_TEAM;
      } else if ( facts->isTheirTeamNameKnown() ) {
	RS_LOG ( LA_SENSORS, "BuildTeamName: assume our team is " << name );
	facts->CP_team_name = name;
	parsedTeam = T_OUR_TEAM;
      } else {
	// We don't know either team so do nothing!
	RS_LOG ( LA_SENSORS, "BuildTeamName: do not know team" );
	parsedTeam = T_UNKNOWN;
      }
    }
  }
}

void Sensors::doBuildUNum( int unum ) {
  RS_LOG ( LA_SENSORS, "BuildUNum: " << unum);
  parsedUniformNumber = unum;
}

void Sensors::doBuildGoalie() {
  RS_LOG ( LA_SENSORS, "BuildGoalie" );
  parsedGoalie = true;
}

/*
 * CLang parsning.
 */

void Sensors::doParsedCLang( const std::string& msg ) {
  RS_LOG ( LA_SENSORS, "ParsedCLang: " << msg);
  lastSensorType = SENSOR_AURAL_COACH;
}

/*
 * Fullstate information parsing. Not implemented.
 */

void Sensors::doBuildFullState( int time ) {
  RS_LOG ( LA_SENSORS, "BuildFullState: " << time);
  RS_WARNING("Fullstate messages isn't implemented");
  lastSensorType = SENSOR_UNKNOWN;
}

void Sensors::doBuildCounts(int kick, int dash, int turn, int katch,
			    int move, int turn_neck, int change_view, int say) {
  RS_LOG ( LA_SENSORS, "BuildCounts: " << kick << ", " << dash << ", " 
  	   << turn << ", " << ", " << katch << ", " << move << ", " 
  	   << turn_neck << ", " << change_view << ", " << say);
  RS_WARNING("This BuildCounts isn't implemented");
  lastSensorType = SENSOR_UNKNOWN;
}

void Sensors::doBuildScore( int our, int opp ) {
  RS_LOG ( LA_SENSORS, "BuildScore: " << our << ", " << opp );
  facts->setScore(facts->getOurSide(), our);
  facts->setScore(facts->getTheirSide(), opp);
  lastSensorType = SENSOR_AURAL_SCORE;
}

void Sensors::doBuildBall( double x, double y, 
			   double delta_x, double delta_y ) {
  RS_LOG ( LA_SENSORS, "BuildBall: " << x << ", " << y << ", "
  	   << delta_x << ", " << delta_y);
  RS_WARNING("This doBuildBall isn't implemented!");
  lastSensorType = SENSOR_UNKNOWN;
}

void Sensors::doBuildPlayer( int unum, int type, double x, double y, 
			     double dx, double dy,
			     double body_dir, double head_dir,
			     int stamina, double effort, double recovery ) {
  // Not pointing, not goalie
  RS_LOG ( LA_SENSORS, "BuildPlayer: " << unum << ", " << type << ", " 
  	   << x  << ", " << y  << ", " << dx  << ", " << dy  << ", " << body_dir
  	   << ", " << head_dir << ", " << stamina  << ", " << effort
  	   << ", " << recovery);
  RS_WARNING("This doBuildPlayer isn't implemented!");
  lastSensorType = SENSOR_UNKNOWN;
}

void Sensors::doBuildPlayer( int unum, int type, double x, double y, 
			     double dx, double dy,
			     double body_dir, double head_dir,
			     double arm_mag, double arm_dir,
			     int stamina, double effort, double recovery ) {
  // Pointing, not goalie
  RS_LOG ( LA_SENSORS, "BuildPlayer: " << unum << ", " << type << ", " 
  	   << x  << ", " << y  << ", " << dx  << ", " << dy  << ", " << body_dir
  	   << ", " << head_dir << ", " << arm_mag << ", " << arm_dir << ", " 
  	   << stamina  << ", " << effort << ", " << recovery);
  RS_WARNING("This doBuildPlayer isn't implemented!");
  lastSensorType = SENSOR_UNKNOWN;
}

void Sensors::doBuildGoalie( int unum, double x, double y, 
			     double dx, double dy,
			     double body_dir, double head_dir,
			     int stamina, double effort, double recovery ) {
  // Not pointing, not goalie
  RS_LOG ( LA_SENSORS, "BuildGoalie: " << unum << ", " << ", " 
  	   << x  << ", " << y  << ", " << dx  << ", " << dy  << ", " << body_dir
  	   << ", " << head_dir << ", " << stamina  << ", " << effort
  	   << ", " << recovery);
  RS_WARNING("This doBuildGoalie isn't implemented!");
  lastSensorType = SENSOR_UNKNOWN;
}

void Sensors::doBuildGoalie( int unum, double x, double y, 
			     double dx, double dy,
			     double body_dir, double head_dir,
			     double arm_mag, double arm_dir,
			     int stamina, double effort, double recovery ) {
  // Pointing, not goalie
  RS_LOG ( LA_SENSORS, "BuildGoalie: " << unum << ", " << ", " 
  	   << x  << ", " << y  << ", " << dx  << ", " << dy  << ", " << body_dir
  	   << ", " << head_dir << ", " << arm_mag << ", " << arm_dir << ", "
  	   << stamina  << ", " << effort << ", " << recovery);
  RS_WARNING("This doBuildGoalie isn't implemented!");
  lastSensorType = SENSOR_UNKNOWN;
}

/*
 * PlayMode parsing.
 */

void Sensors::doBuildCatchBallLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildCatchBallLeftPlayMode");
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_GOALIE_GOT_BALL;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_GOALIE_GOT_BALL;
}

void Sensors::doBuildCatchBallRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildCatchBallRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_GOALIE_GOT_BALL;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_GOALIE_GOT_BALL;
}

void Sensors::doBuildBeforeKickOffPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildBeforeKickOffPlayMode" );
  parsedPlayMode = PM_BEFORE_KICK_OFF;
}

void Sensors::doBuildTimeOverPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildTimeOverPlayMode" );
  parsedPlayMode = PM_TIME_OVER;
}

void Sensors::doBuildPlayOnPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPlayOnPlayMode" );
  parsedPlayMode = PM_PLAY_ON;
}

void Sensors::doBuildKickOffLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildKickOffLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_KICK_OFF;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_KICK_OFF;
}

void Sensors::doBuildKickOffRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildKickOffRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_KICK_OFF;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_KICK_OFF;
}
 
void Sensors::doBuildKickInLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildKickInLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_KICK_IN;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_KICK_IN;
}
 
void Sensors::doBuildKickInRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildKickInRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_KICK_IN;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_KICK_IN;
}

void Sensors::doBuildFreeKickLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFreeKickLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FREE_KICK;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FREE_KICK;  
}

void Sensors::doBuildFreeKickRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFreeKickRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_FREE_KICK;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_FREE_KICK;  
}

void Sensors::doBuildCornerKickLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildCornerKickLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_CORNER_KICK;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_CORNER_KICK;  
}

void Sensors::doBuildCornerKickRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildCornerKickRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_CORNER_KICK;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_CORNER_KICK;  
}

void Sensors::doBuildGoalKickLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildGoalKickLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_GOAL_KICK;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_GOAL_KICK;  
}

void Sensors::doBuildGoalKickRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildGoalKickRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_GOAL_KICK;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_GOAL_KICK;  
}

void Sensors::doBuildAfterGoalLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildAfterGoalLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT) 
    parsedPlayMode = PM_OUR_GOAL;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_GOAL;
}
 
void Sensors::doBuildAfterGoalRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildAfterGoalRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT) 
    parsedPlayMode = PM_OUR_GOAL;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_GOAL;
}

void Sensors::doBuildDropBallPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildDropBallPlayMode" );
  parsedPlayMode = PM_DROP_BALL;
}

void Sensors::doBuildOffSideLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildOffSideLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_OFFSIDE_KICK;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_OFFSIDE_KICK;
}

void Sensors::doBuildOffsideRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildOffsideRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_OFFSIDE_KICK;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_OFFSIDE_KICK;
}
 
void Sensors::doBuildPenaltyKickLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyKickLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_KICK;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_KICK;
}

void Sensors::doBuildPenaltyKickRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyKickRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_KICK;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_KICK;
}
 
void Sensors::doBuildFirstHalfOverPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFirstHalfOverPlayMode" );
  parsedPlayMode = PM_HALF_TIME;
}

void Sensors::doBuildPausePlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPausePlayMode" );
  parsedPlayMode = PM_PAUSE;
}

void Sensors::doBuildHumanPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildHumanPlayMode" );
  parsedPlayMode = PM_HUMAN;
}

void Sensors::doBuildFoulLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulLeftPlayMode: ");
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL;  
}

void Sensors::doBuildFoulRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulRightPlayMode: ");
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL;
}
 
void Sensors::doBuildFoulChargeLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulChargeLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL_CHARGE;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL_CHARGE;
}

void Sensors::doBuildFoulChargeRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulChargeRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_FOUL_CHARGE;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_FOUL_CHARGE;
}

void Sensors::doBuildFoulPushLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulPushLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL_PUSH;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL_PUSH;
}

void Sensors::doBuildFoulPushRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulPushRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_FOUL_PUSH;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_FOUL_PUSH;
}

void Sensors::doBuildFoulMultipleAttackerLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulMultipleAttackerLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL_MULTIPLEATTACKER;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL_MULTIPLEATTACKER;
}

void Sensors::doBuildFoulMultipleAttackerRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulMultipleAttackerRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_FOUL_MULTIPLEATTACKER;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_FOUL_MULTIPLEATTACKER;
}

void Sensors::doBuildFoulBallOutLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulBallOutLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL_BALLOUT;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL_BALLOUT;
}

void Sensors::doBuildFoulBallOutRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFoulBallOutRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_FOUL_BALLOUT;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_FOUL_BALLOUT;
}

void Sensors::doBuildBackPassLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildBackPassLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL_BACKPASS;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL_BACKPASS;
}

void Sensors::doBuildBackPassRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildBackPassRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_FOUL_BACKPASS;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_FOUL_BACKPASS;
}

void Sensors::doBuildFreeKickFaultLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFreeKickFaultLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL_FREEKICKFAULT;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL_FREEKICKFAULT;
}

void Sensors::doBuildFreeKickFaultRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildFreeKickFaultRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_FOUL_FREEKICKFAULT;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_FOUL_FREEKICKFAULT;
}

void Sensors::doBuildCatchFaultLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildCatchFaultLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_FOUL_CATCHFAULT;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL_CATCHFAULT;
}

void Sensors::doBuildCatchFaultRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildCatchFaultRigthPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_FOUL_CATCHFAULT;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_FOUL_CATCHFAULT;
}

void Sensors::doBuildIndirectFreeKickLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildIndirectFreeKickLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_INDIRECT_FREE_KICK;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_INDIRECT_FREE_KICK;
}

void Sensors::doBuildIndirectFreeKickRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildIndirectFreeKickRigthPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_INDIRECT_FREE_KICK;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_INDIRECT_FREE_KICK;
}

void Sensors::doBuildPenaltySetupLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltySetupLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_SETUP;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_SETUP;
}

void Sensors::doBuildPenaltySetupRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltySetupRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_SETUP;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_SETUP;
}

void Sensors::doBuildPenaltyReadyLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyReadyLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_READY;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_READY;
}

void Sensors::doBuildPenaltyReadyRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyReadyRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_READY;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_READY;
}

void Sensors::doBuildPenaltyTakenLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyTakenLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_TAKEN;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_TAKEN;
}

void Sensors::doBuildPenaltyTakenRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyTakenRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_TAKEN;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_TAKEN;
}

void Sensors::doBuildPenaltyMissLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyMissLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_MISS;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_MISS;
}

void Sensors::doBuildPenaltyMissRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyMissRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_MISS;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_MISS;
}

void Sensors::doBuildPenaltyScoreLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyScoreLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_SCORE;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_SCORE;
}

void Sensors::doBuildPenaltyScoreRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyScoreRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_SCORE;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_SCORE;
}

void Sensors::doBuildPenaltyOnFieldLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyOnFieldLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_ONFIELD;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_ONFIELD;
}

void Sensors::doBuildPenaltyOnFieldRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyOnFieldRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_ONFIELD;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_ONFIELD;
}

void Sensors::doBuildPenaltyFoulLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyFoulLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_FOUL;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_FOUL;
}

void Sensors::doBuildPenaltyFoulRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyFoulRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_FOUL;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_FOUL;
}

void Sensors::doBuildPenaltyWinnerLeftPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyWinnerLeftPlayMode" );
  if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_OUR_PENALTY_WINNER;
  else if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_THEIR_PENALTY_WINNER;
}

void Sensors::doBuildPenaltyWinnerRightPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyWinnerRightPlayMode" );
  if (facts->getOurSide() == S_RIGHT)
    parsedPlayMode = PM_OUR_PENALTY_WINNER;
  else if (facts->getOurSide() == S_LEFT)
    parsedPlayMode = PM_THEIR_PENALTY_WINNER;
}

void Sensors::doBuildPenaltyDrawPlayMode() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyDraw" );
  parsedPlayMode = PM_PENALTY_DRAW;
}

/*
 * Side parsing.
 */

void Sensors::doBuildLeftSide() {
  RS_LOG ( LA_SENSORS, "BuildLeftSide" );
  parsedSide = S_LEFT;
}

void Sensors::doBuildRightSide() {
  RS_LOG ( LA_SENSORS, "BuildRightSide" );
  parsedSide = S_RIGHT;
}

void Sensors::doBuildOppSide() {
  RS_LOG ( LA_SENSORS, "BuildOppSide" );
  if (facts->getOurSide() == S_LEFT)
    parsedSide = S_RIGHT;
  else if (facts->getOurSide() == S_RIGHT)
    parsedSide = S_LEFT;
}
 
void Sensors::doBuildOurSide() {
  RS_LOG ( LA_SENSORS, "BuildOurSide" );
  parsedSide = facts->getOurSide();
}
 
void Sensors::doBuildNeutralSide() {
  RS_LOG ( LA_SENSORS, "BuildNeutralSide" );
  parsedSide = S_NEUTRAL;  
}

void Sensors::doBuildSenseBody( int time, double stamina, double effort,
				double speed_mag, double speed_head,
				double head_angle, int kick_count,
				int dash_count,	int turn_count,
				int say_count, int turn_neck_count,
				int catch_count, int move_count,
				int chg_view_count ) {
  RS_LOG ( LA_SENSORS, "BuildSenseBody: " << time  << ", " << stamina
  	   << ", " << effort << ", " << speed_mag  << ", " << speed_head
  	   << ", " << head_angle << ", " << kick_count << ", " << dash_count
  	   << ", " << turn_count << ", " << say_count << ", "
  	   << turn_neck_count  << ", " << catch_count  << ", " << move_count
  	   << ", " << chg_view_count);
  lastSensorType = SENSOR_SENSE;
  
  // @todo Move some things away to better place.

  // Immediate commands.
  static UInt change_views = 0;
  static UInt says         = 0;
  static UInt turn_necks   = 0;
  static UInt attentionTos = 0;
  static UInt pointTos     = 0;

  // Periodic (main) commands.
  static UInt catches      = 0;
  static UInt dashes       = 0;
  static UInt kicks        = 0;
  static UInt moves        = 0;
  static UInt turns        = 0;
  static UInt tackles      = 0;

  Int lastcycle_change_views = chg_view_count - change_views;
  Int lastcycle_says = say_count - says;
  Int lastcycle_turn_necks = turn_neck_count - turn_necks;
  Int lastcycle_attentionTos = parsedFocusState.focusCount - attentionTos;
  Int lastcycle_pointTos = parsedArmState.count - pointTos;

  Int lastcycle_catches = catch_count - catches;
  Int lastcycle_dashes = dash_count - dashes;
  Int lastcycle_kicks = kick_count - kicks;
  Int lastcycle_moves = move_count - moves;
  Int lastcycle_turns = turn_count - turns;
  Int lastcycle_tackles = parsedTackleState.tackleCount - tackles;
  
  change_views = chg_view_count;
  says = say_count;
  turn_necks = turn_neck_count;
  attentionTos = parsedFocusState.focusCount;
  pointTos = parsedArmState.count;

  catches = catch_count;
  dashes = dash_count;
  kicks = kick_count;
  moves = move_count;
  turns = turn_count;
  tackles = parsedTackleState.tackleCount;

  while (!commandQueue.empty()) {
    switch (commandQueue.front()) {
    case CMD_ChangeView:
      --lastcycle_change_views;
      break;
    case CMD_TurnNeck:
      --lastcycle_turn_necks;
      break;
    case CMD_Say:
      --lastcycle_says;
      break;
    case CMD_AttentionTo:
      --lastcycle_attentionTos;
      break;
    case CMD_PointTo:
      --lastcycle_pointTos;
      break;      
    case CMD_Catch:
      --lastcycle_catches;
      break;
    case CMD_Dash:
      --lastcycle_dashes;
      break;
    case CMD_Kick:
      --lastcycle_kicks;
      break;
    case CMD_Move:
      --lastcycle_moves;
      break;
    case CMD_Turn:
      --lastcycle_turns;
      break;
    case CMD_Tackle:
      --lastcycle_tackles;
      break;
    default:
      //CMD_Unknown, CMD_Bye, CMD_Init, CMD_SenseBody
      break;
    }
    commandQueue.pop();
  }

  if ( lastcycle_catches < 0 ) 
    RS_WARNING("The server missed the last catch command. " + toString(currentTime));
  if ( lastcycle_dashes < 0 ) 
    RS_WARNING("The server missed the last dash command. " + toString(currentTime));
  if ( lastcycle_kicks < 0 ) 
    RS_WARNING("The server missed the last kick command. " + toString(currentTime));
  if ( lastcycle_moves < 0 ) 
    RS_WARNING("The server missed the last move command. " + toString(currentTime));
  if ( lastcycle_turns < 0 ) 
    RS_WARNING("The server missed the last turn command. " + toString(currentTime));
  if ( lastcycle_tackles < 0 ) 
    RS_WARNING("The server missed the last tackle command. " + toString(currentTime));

  if ( lastcycle_change_views < 0 ) 
    RS_WARNING("The server missed the last change view command. " + toString(currentTime));
  if ( lastcycle_turn_necks < 0 ) 
    RS_WARNING("The server missed the last turn neck command. " + toString(currentTime));
  if ( lastcycle_says < 0 ) 
    RS_WARNING("The server missed the last say command. " + toString(currentTime));
  if ( lastcycle_attentionTos < 0 ) 
    RS_WARNING("The server missed the last attentionTo command. " + toString(currentTime));
  if ( lastcycle_pointTos < 0 ) 
    RS_WARNING("The server missed the last pointTo command. " + toString(currentTime));

  if ( lastcycle_catches < 0 || 
       lastcycle_dashes < 0 ||
       lastcycle_kicks < 0 ||
       lastcycle_moves < 0 ||
       lastcycle_turns < 0 ||
       lastcycle_tackles < 0 ||
       lastcycle_change_views < 0 ||
       lastcycle_turn_necks < 0 ||
       lastcycle_says < 0 ||
       lastcycle_attentionTos < 0 ||
       lastcycle_pointTos < 0)
    theController->delayedActions();
  
  if ( lastcycle_catches > 0 || 
       lastcycle_dashes > 0 ||
       lastcycle_kicks > 0 ||
       lastcycle_moves > 0 ||
       lastcycle_turns > 0 ||
       lastcycle_tackles > 0 ||
       lastcycle_change_views > 0 ||
       lastcycle_turn_necks > 0 ||
       lastcycle_says > 0 ||
       lastcycle_attentionTos > 0 ||
       lastcycle_pointTos > 0)
    theController->missingActions();
  
  updateTime(time);
  parsedAgent = ServerAgentObject 
    (parsedViewQuality, 
     parsedViewWidth,
     stamina,
     effort,
     FloatUE(ERROR_QUANTIZE, speed_mag, 0.01),
     AngleDegUE(ERROR_QUANTIZE, speed_head, facts->SP_angle_qstep),
     AngleDegUE(ERROR_QUANTIZE, head_angle, facts->SP_angle_qstep),
     catch_count, 
     chg_view_count, 
     dash_count,
     kick_count, 
     move_count, 
     say_count,
     turn_count, 
     turn_neck_count,
     parsedArmState,
     parsedFocusState,
     parsedTackleState);  
  theSensorInterface->sense(parsedAgent);
  parsedViewQuality = VQ_UNKNOWN;
  parsedViewWidth = VW_UNKNOWN;  
  parsedArmState = ServerArmState();
  parsedFocusState = ServerFocusState();
  parsedTackleState = ServerTackleState();
}

void Sensors::doBuildArmState(int moveable_in, int expires_in, 
			      double target_x, double target_y,
			      int count) {
  RS_LOG ( LA_SENSORS, "BuildArmState " << moveable_in << ", " 
  	   << expires_in << ", " << target_x << ", " << target_y 
  	   << ", " << count);
  parsedArmState = ServerArmState(moveable_in, expires_in,
				  Point(target_x, target_y),
				  count);
}

void Sensors::doBuildFocusState(int count) {
  RS_LOG ( LA_SENSORS, "BuildFocusState " << count);
  parsedFocusState = ServerFocusState(count);
}

void Sensors::doBuildFocusState(int unum, int count) {
  RS_LOG ( LA_SENSORS, "BuildFocusState " << unum << ", " << count);
  parsedFocusState = ServerFocusState(count, parsedSide, unum);
  parsedSide = S_UNKNOWN;
}

void Sensors::doBuildTackleState(int expires_in, int count) {
  RS_LOG ( LA_SENSORS, "BuildTackleState " << expires_in << ", " << count);
  parsedTackleState = ServerTackleState(expires_in, count);
}

/*
 * View Mode parsing.
 */

void Sensors::doBuildViewMode() {
  RS_LOG ( LA_SENSORS, "BuildViewMode" );
}

void Sensors::doBuildViewQualityHigh() {
  RS_LOG ( LA_SENSORS, "BuildViewQualityHigh" );
  parsedViewQuality = VQ_HIGH;
}

void Sensors::doBuildViewQualityLow() {
  RS_LOG ( LA_SENSORS, "BuildViewQualityLow" );
  parsedViewQuality = VQ_LOW;
}

void Sensors::doBuildViewWidthNarrow() {
  RS_LOG ( LA_SENSORS, "BuildViewWidthNarrow" );
  parsedViewWidth = VW_NARROW;
}

void Sensors::doBuildViewWidthNormal() {
  RS_LOG ( LA_SENSORS, "BuildViewWidthNormal" );
  parsedViewWidth = VW_NORMAL;
}

void Sensors::doBuildViewWidthWide() {
  RS_LOG ( LA_SENSORS, "BuildViewWidthWide" );
  parsedViewWidth = VW_WIDE;
}

/*
 * Parameter parsing.
 */

void Sensors::doBuildParam(const std::string &name, int value) {
  RS_LOG ( LA_SENSORS, "BuildParam " << name << ", " << value);
  facts->setOption(name, value);
}

void Sensors::doBuildParam(const std::string &name, double value) {
  RS_LOG ( LA_SENSORS, "BuildParam " << name << ", " << value);
  facts->setOption(name, value);
}

void Sensors::doBuildParam(const std::string &name, const std::string &value) {
  RS_LOG ( LA_SENSORS, "BuildParam " << name << ", " << value);
  facts->setOption(name, value);
}

void Sensors::doBuildServerParam() {
  RS_LOG ( LA_SENSORS, "BuildServerParam");
  facts->serverParamsInitialized();
  lastSensorType = SENSOR_PARAM;
}
 
void Sensors::doBuildPlayerParam() {
  RS_LOG ( LA_SENSORS, "BuildPlayerParam");
  facts->playerParamsInitialized();
  lastSensorType = SENSOR_PARAM;
}

void Sensors::doBuildPlayerType() {
  RS_LOG ( LA_SENSORS, "BuildPlayerType");
  facts->playerTypeComplete();
  lastSensorType = SENSOR_PARAM;
}

/*
 * Player visual parsning.
 */

void Sensors::doBuildVisual( int time ) {
  RS_LOG ( LA_SENSORS, "BuildVisual: " << time);  
  lastSensorType = SENSOR_VISUAL;
  updateTime(time);
  theSensorInterface->see(parsedLines,
			  parsedMarkers,
			  parsedPlayers,
			  parsedBall);
  parsedLines.clear();
  parsedMarkers.clear();
  parsedPlayers.clear();
  parsedBall = ServerBallObject();
  parsedAgent = ServerAgentObject();
}

void Sensors::doBuildLine( double dist, double dir ) {
  RS_LOG ( LA_SENSORS, "BuildLine: " << dist << ", " << dir);  
  parsedLines.push_back 
    (ServerLineObject (locationToLine(),
		       AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
		       FloatUE(ERROR_QUANTIZE, dist, facts->SP_land_qstep)));
  parsedLocations.clear();
}

void Sensors::doBuildLine( double dir ) {
  RS_LOG ( LA_SENSORS, "BuildLine: " << dir);
  parsedLines.push_back 
    (ServerLineObject (locationToLine(),
		       AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep)));
  parsedLocations.clear();    
}

// TODO! Handle "close" in a sane way.

void Sensors::doBuildFlag( double dist, double dir,
			   double dist_chg, double dir_chg ) {
  RS_LOG ( LA_SENSORS, "BuildFlag: " << dist
  	   << ", " << dir << ", " << dist_chg  << ", " << dir_chg);
  parsedMarkers.push_back
    (ServerMarkerObject
     (locationToMarker(),
      false, 
      AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
      FloatUE(ERROR_QUANTIZE, dist, facts->SP_land_qstep),
      FloatUE(ERROR_DISTCHANGE, dist_chg, facts->SP_dist_qstep,
	      dist, facts->CP_dist_change_qstep),
      AngleDegUE(ERROR_QUANTIZE, dir_chg, facts->CP_dir_change_qstep)));
  parsedLocations.clear();
  parsedFlagOffset = UIntUnknown();
}

void Sensors::doBuildFlag( bool close, double dist, double dir ) {
    RS_LOG ( LA_SENSORS, "BuildFlag: " << close << ", " << dist
  	   << ", " << dir);
  if (!close) {
    parsedMarkers.push_back
      (ServerMarkerObject
       (locationToMarker(),
	close,
	AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
	FloatUE(ERROR_QUANTIZE, dist, facts->SP_land_qstep)));
    parsedLocations.clear();
    parsedFlagOffset = UIntUnknown();
  }
}
 
void Sensors::doBuildFlag( bool close, double dir ) {
  RS_LOG ( LA_SENSORS, "BuildFlag: " << close  << ", " << dir);
  if (!close) {
    parsedMarkers.push_back
      (ServerMarkerObject
       (locationToMarker(), 
	close,
	AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep)));
    parsedLocations.clear();
    parsedFlagOffset = UIntUnknown();
  }
}

void Sensors::doBuildGoal( double dist, double dir,
			   double dist_chg, double dir_chg ) {
  RS_LOG ( LA_SENSORS, "BuildGoal:" << dist
  	   << ", " << dir  << ", " << dist_chg  << ", " << dir_chg);
  doBuildFlag( dist, dir, dist_chg, dir_chg);
}

void Sensors::doBuildGoal( bool close, double dist, double dir ) {
  RS_LOG ( LA_SENSORS, "BuildGoal: " << close  << ", " << dist
  	   << ", " << dir);
  if (!close)
    doBuildFlag( close, dist, dir);
}

void Sensors::doBuildGoal( bool close, double dir ) {
  RS_LOG ( LA_SENSORS, "BuildGoal: " << close  << ", " << dir );
  if (!close)
    doBuildFlag( close, dir);
}

void Sensors::doBuildPlayerVisBall( double dist, double dir,
				    double dist_chg, double dir_chg ) {
  RS_LOG ( LA_SENSORS, "BuildBall: " << dist
  	   << ", " << dir << ", " << dist_chg << ", " << dir_chg);
  parsedBall = 
    ServerBallObject
    (false,
     AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
     FloatUE(ERROR_QUANTIZE, dist, facts->SP_dist_qstep),
     FloatUE(ERROR_DISTCHANGE, dist_chg, facts->SP_dist_qstep,
	     dist, facts->CP_dist_change_qstep),
     AngleDegUE(ERROR_QUANTIZE, dir_chg, facts->CP_dir_change_qstep));
}

void Sensors::doBuildBall( bool close, double dist, double dir ) {
  RS_LOG ( LA_SENSORS, "BuildBall: " << close  << ", " << dist
  	   << ", " << dir);
  parsedBall =
    ServerBallObject(close,
		     AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
		     FloatUE(ERROR_QUANTIZE, dist, facts->SP_dist_qstep));    
}

void Sensors::doBuildBall( bool close, double dir ) {
  RS_LOG ( LA_SENSORS, "BuildBall:"  << ", " << close << ", " << dir);
  parsedBall =
    ServerBallObject(close,
		     AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep));
}

// Pointing players

// 1.
void Sensors::doBuildPlayer( double dist, double dir,
			     double dist_chg, double dir_chg,
			     double orientation, double head_orientation,
			     double point_dir, bool tackling) {
  RS_LOG ( LA_SENSORS, "BuildPlayer: " << dist
  	   << ", " << dir << ", " << dist_chg << ", " << dir_chg
  	   << ", " << orientation << ", " << head_orientation
  	   << ", " << point_dir << ", " << tackling);
  parsedPlayers.push_back
    (ServerPlayerObject
     (false,
      AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
      parsedGoalie,
      parsedTeam,
      parsedUniformNumber,
      FloatUE(ERROR_QUANTIZE, dist, facts->SP_dist_qstep),
      FloatUE(ERROR_DISTCHANGE, dist_chg, facts->SP_dist_qstep,
	      dist, facts->CP_dist_change_qstep),
      AngleDegUE(ERROR_QUANTIZE, dir_chg, facts->CP_dir_change_qstep),
      AngleDegUE(ERROR_QUANTIZE, orientation, facts->SP_angle_qstep),
      AngleDegUE(ERROR_QUANTIZE, head_orientation, facts->SP_angle_qstep),
      tackling,
      AngleDegUE(point_dir))); // Fix new (normal)error here.
  parsedTeam = T_UNKNOWN;
  parsedUniformNumber = UIntUnknown();
  parsedGoalie = false;  
}

// If we see a point direction, then the players isn't close.
// 3.
void Sensors::doBuildPlayer( double dist, double dir,
			     double point_dir, bool tackling) {
  RS_LOG ( LA_SENSORS, "BuildPlayer: " << ", " << dist
  	   << ", " << dir << ", " << point_dir << ", " << tackling);
  parsedPlayers.push_back
    (ServerPlayerObject
     (false,
      AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
      parsedGoalie,
      parsedTeam,
      parsedUniformNumber,
      FloatUE(ERROR_QUANTIZE, dist, facts->SP_dist_qstep),
      FloatUE(),
      AngleDegUE(),
      AngleDegUE(),
      AngleDegUE(),
      tackling,
      AngleDegUE(point_dir))); // Fix new (normal) error here.
  parsedTeam = T_UNKNOWN;
  parsedUniformNumber = UIntUnknown();
  parsedGoalie = false;
}


// Not pointing players

// 2.
void Sensors::doBuildPlayer( double dist, double dir,
			     double dist_chg, double dir_chg,
			     double orientation, double head_orientation,
			     bool tackling) {
  RS_LOG ( LA_SENSORS, "BuildPlayer: " << dist
  	   << ", " << dir << ", " << dist_chg << ", " << dir_chg
  	   << ", " << orientation << ", " << head_orientation
  	   << ", " << tackling);
  parsedPlayers.push_back
    (ServerPlayerObject
     (false,
      AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
      parsedGoalie,
      parsedTeam,
      parsedUniformNumber,
      FloatUE(ERROR_QUANTIZE, dist, facts->SP_dist_qstep),
      FloatUE(ERROR_DISTCHANGE, dist_chg, facts->SP_dist_qstep,
	      dist, facts->CP_dist_change_qstep),
      AngleDegUE(ERROR_QUANTIZE, dir_chg, facts->CP_dir_change_qstep),
      AngleDegUE(ERROR_QUANTIZE, orientation, facts->SP_angle_qstep),
      AngleDegUE(ERROR_QUANTIZE, head_orientation, facts->SP_angle_qstep),
      tackling));
  parsedTeam = T_UNKNOWN;
  parsedUniformNumber = UIntUnknown();
  parsedGoalie = false;  
}

// 4.
void Sensors::doBuildPlayer( bool close,
			     double dist, double dir,
			     bool tackling) {
  RS_LOG ( LA_SENSORS, "BuildPlayer: " << close << ", " << dist
  	   << ", " << dir << ", " << tackling);
  parsedPlayers.push_back
    (ServerPlayerObject
     (close,
      AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
      parsedGoalie,
      parsedTeam,
      parsedUniformNumber,
      FloatUE(ERROR_QUANTIZE, dist, facts->SP_dist_qstep),
      FloatUE(),
      AngleDegUE(),
      AngleDegUE(),
      AngleDegUE(),
      tackling));
  
  parsedTeam = T_UNKNOWN;
  parsedUniformNumber = UIntUnknown();
  parsedGoalie = false;
}

// 5. 
void Sensors::doBuildPlayer( bool close, double dir ) {
  RS_LOG ( LA_SENSORS, "BuildPlayer:" << close  << ", " << dir);
  parsedPlayers.push_back
    (ServerPlayerObject
     (close,
      AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep),
      parsedGoalie,
      parsedTeam,
      parsedUniformNumber));
  parsedTeam = T_UNKNOWN;
  parsedUniformNumber = UIntUnknown();
  parsedGoalie = false;
}


/*
 * Coach visual parsning.
 */

void Sensors::doBuildGlobalVisual(int time) {
  RS_LOG ( LA_SENSORS, "BuildGlobalVisual: " << time);
  lastSensorType = SENSOR_COACH_VISUAL;
  updateTime(time);
  theSensorInterface->seeGlobal(parsedCoachPlayers,
				parsedCoachBall);
  parsedCoachPlayers.clear();
  parsedCoachBall = CoachBallObject();  
}

void Sensors::doBuildGlobalGoal(double x, double y) {
  RS_LOG ( LA_SENSORS, "BuildGlobalGoal: " << x << ", " << y);
}

void Sensors::doBuildGlobalBall(double x, double y, double dx, double dy) {
  RS_LOG ( LA_SENSORS, "BuildGlobalBall: " << x << ", " << y 
  	   << ", " << dx << ", " << dy);
  parsedCoachBall = CoachBallObject(x, y, dx, dy);
}

void Sensors::doBuildGlobalPlayer(double x, double y, double dx, double dy, 
				  double bodydir, double neckdir, bool tackle) {
    RS_LOG ( LA_SENSORS, "BuildGlobalPlayer: " << x << ", " << y 
  	   << ", " << dx << ", " << dy << ", " << bodydir << ", " 
  	   << neckdir << ", " << tackle);
  parsedCoachPlayers.push_back
    (CoachPlayerObject
     (x, y, dx, dy, 
      parsedTeam,
      parsedUniformNumber.getValue(),
      parsedGoalie,
      bodydir, 
      neckdir,
      AngleDegUnknown(),
      tackle));
  parsedTeam = T_UNKNOWN;
  parsedUniformNumber = UIntUnknown();
  parsedGoalie = false;
}

void Sensors::doBuildGlobalPlayer(double x, double y, double dx, double dy, 
				  double bodydir, double neckdir, 
				  double point_dir, bool tackle) {
  RS_LOG ( LA_SENSORS, "BuildGlobalPlayer: " << x << ", " << y 
  	   << ", " << dx << ", " << dy << ", " << bodydir << ", " 
  	   << neckdir << ", " << point_dir << ", " << tackle);
  parsedCoachPlayers.push_back
    (CoachPlayerObject
     (x, y, dx, dy, 
      parsedTeam,
      parsedUniformNumber.getValue(),
      parsedGoalie,
      bodydir, 
      neckdir,
      point_dir,
      tackle));
  parsedTeam = T_UNKNOWN;
  parsedUniformNumber = UIntUnknown();
  parsedGoalie = false;
}

/*
 * Field Location parsning.
 */

void Sensors::doBuildFlagOffset( int offset ) {
  RS_LOG ( LA_SENSORS, "BuildFlagOffset" );
  parsedFlagOffset = offset;
} 

void Sensors::doBuildTopLocation() {
  RS_LOG ( LA_SENSORS, "BuildTopLocation" );
  parsedLocations.push_back(FL_TOP);
}
  
void Sensors::doBuildBottomLocation() {
  RS_LOG ( LA_SENSORS, "BuildBottomLocation" );
  parsedLocations.push_back(FL_BOTTOM);
}
 
void Sensors::doBuildLeftLocation() {
  RS_LOG ( LA_SENSORS, "BuildLeftLocation" );
  parsedLocations.push_back(FL_LEFT);
}
 
void Sensors::doBuildRightLocation() {
  RS_LOG ( LA_SENSORS, "BuildRightLocation" );
  parsedLocations.push_back(FL_RIGHT);
}
 
void Sensors::doBuildCenterLocation() {
  RS_LOG ( LA_SENSORS, "BuildCenterLocation" );
  parsedLocations.push_back(FL_CENTER);
}
 
void Sensors::doBuildPenaltyLocation() {
  RS_LOG ( LA_SENSORS, "BuildPenaltyLocation" );
  parsedLocations.push_back(FL_PENALTY);
}
 
void Sensors::doBuildGoalLocation() {
  RS_LOG ( LA_SENSORS, "BuildGoalLocation" );
  parsedLocations.push_back(FL_GOAL);
}

/*
 * Initialisation parsning.
 */
 
void Sensors::doBuildInit( int unum ) {
  RS_LOG ( LA_SENSORS, "BuildInit: " << unum);
  theSensorInterface->hearReferee(parsedPlayMode);
  std::string agentSide;
  if (parsedSide == S_LEFT) {
    facts->setSides(S_LEFT, S_RIGHT);
    agentSide = "left";
  }
  else {
    facts->setSides(S_RIGHT, S_LEFT);
    agentSide = "right";
  }
  facts->setUniformNumber(unum);
  std::cout << "-----------------------------------------------------------"
	    << "-------------" << std::endl
	    << "Connected player " << unum << " on team '"
	    << facts->CP_team_name << "'" << std::endl
	    << "It's playing on the " << agentSide << " side of the field."
	    << std::endl << "----------------------------------------------"
	    << "--------------------------" << std::endl;

  parsedSide = S_UNKNOWN;
  parsedPlayMode = PM_UNKNOWN;  
  lastSensorType = SENSOR_INIT;
}

void Sensors::doBuildInit() {
  RS_LOG ( LA_SENSORS, "BuildInit" );
  RS_WARNING("This doBuildInit isn't implemented!");
  lastSensorType = SENSOR_UNKNOWN;
}

void Sensors::doBuildCoachInit() {
  RS_LOG ( LA_SENSORS, "BuildCoachInit");
  if ( parsedSide == S_UNKNOWN ) {
    // We use left as the default side, it shouldn't really matter
    facts->setSides(S_LEFT, S_RIGHT);
    
    std::cout << "-----------------------------------------------------------"
	      << "-------------\n"
	      << "Connected trainer.\n"
	      << "----------------------------------------------"
	      << "--------------------------" << std::endl;
  } else {
    std::string coachSide;
    if (parsedSide == S_LEFT) {    
      facts->setSides(S_LEFT, S_RIGHT);
      coachSide = "left";
    } else {
      facts->setSides(S_RIGHT, S_LEFT);
      coachSide = "right";
    }
    std::cout << "-----------------------------------------------------------"
	      << "-------------\n"
	      << "Connected coach on team '" << facts->CP_team_name
	      << "'\n It's coaching the team on the " << coachSide
	      << " side of the field.\n"
	      << "----------------------------------------------"
	      << "--------------------------" << std::endl;
  }
  parsedSide = S_UNKNOWN;
  parsedPlayMode = PM_UNKNOWN;  
  lastSensorType = SENSOR_INIT;
}

/*
 * Audio parsning.
 */

void Sensors::doBuildCoachAudio( int time, const std::string& msg ) {
  RS_LOG ( LA_SENSORS, "BuildCoachAudio: " << time << ", " << msg);
  lastSensorType = SENSOR_AURAL_COACH;
  updateTime(time);
  theSensorInterface->hearCoach(msg);    
  parsedSide = S_UNKNOWN;
}

void Sensors::doBuildRefAudio( int time ) {
  RS_LOG ( LA_SENSORS, "BuildRefAudio: " << time );
  lastSensorType = SENSOR_AURAL_REFEREE;
  updateTime(time);
  theSensorInterface->hearReferee(parsedPlayMode);  
  parsedPlayMode = PM_UNKNOWN;
}


void Sensors::doBuildGoalRefAudio( int time, int score ) {
  RS_LOG ( LA_SENSORS, "BuildGoalRefAudio: " << time << ", " << score);
  lastSensorType = SENSOR_AURAL_REFEREE;
  updateTime(time);
  facts->setScore(parsedSide, score);
  if (parsedSide == facts->getOurSide())
    theSensorInterface->hearReferee(PM_OUR_GOAL);
  else
    theSensorInterface->hearReferee(PM_THEIR_GOAL);
  parsedSide = S_UNKNOWN;
}

void Sensors::doBuildUnknownRefAudio( int time, const std::string& message ) {
  RS_LOG ( LA_SENSORS, "BuildUnknownRefAudio: " << time << ", " << message);
  //RS_WARNING ("BuildUnknownRefAudio");
  //This is a normal message sent by the offline coach!
  lastSensorType = SENSOR_AURAL_REFEREE;
  updateTime(time);
  parsedSide = S_UNKNOWN;
}

// The client is online coach, hearing a player.
void Sensors::doBuildPlayerAudio( int time, const std::string& msg ) {
  RS_LOG ( LA_SENSORS, "BuildPlayerAudio: " << time << ", " << msg);
  lastSensorType = SENSOR_COACH_AURAL;
  updateTime(time);
  theSensorInterface->hearPlayerAsCoach(parsedTeam, parsedUniformNumber, msg);
  parsedSide = S_UNKNOWN;
}

// The client is a field player, hearing an opponent player.
void Sensors::doBuildPlayerAudio( int time, double dir, 
				  const std::string& msg ) {
  RS_LOG ( LA_SENSORS, "BuildPlayerAudio: " << time << ", " 
  	   << dir << ", " << msg);
  lastSensorType = SENSOR_AURAL_PLAYER;
  updateTime(time);
  theSensorInterface->hearOpponent
    (AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep), msg);
  parsedSide = S_UNKNOWN;
}

// The client is a field player, hearing a teammate.
void Sensors::doBuildPlayerAudio( int time, double dir, 
				  int unum, const std::string& msg ) {
  RS_LOG ( LA_SENSORS, "BuildPlayerAudio: " << time << ", " 
  	   << dir << ", " << unum << ", " << msg);
  lastSensorType = SENSOR_AURAL_PLAYER;
  updateTime(time);
  theSensorInterface->hearTeammate
    (unum, AngleDegUE(ERROR_QUANTIZE, dir, facts->SP_angle_qstep), msg);
  parsedSide = S_UNKNOWN;
}

// The client is a field player, hearing another teammate.
void Sensors::doBuildPlayerAudio( int time, int unum ) {
  RS_LOG ( LA_SENSORS, "BuildPlayerAudio: " << time << ", " << unum);
  lastSensorType = SENSOR_AURAL_PLAYER;
  updateTime(time);
  theSensorInterface->hearAnotherTeammate(unum);
  parsedSide = S_UNKNOWN;
}

// The client is a field player, hearing another opponent.
void Sensors::doBuildPlayerAudio( int time ) {
  RS_LOG ( LA_SENSORS, "BuildPlayerAudio: " << time);
  lastSensorType = SENSOR_AURAL_PLAYER;
  updateTime(time);
  theSensorInterface->hearAnotherOpponent();
  parsedSide = S_UNKNOWN;
}

void Sensors::doBuildSelfAudio( int time, const std::string& msg ) {
  RS_LOG ( LA_SENSORS, "BuildSelfAudio: " << time << ", " << msg);
  lastSensorType = SENSOR_AURAL_SELF;
  updateTime(time);
  theSensorInterface->hearSelf(msg);  
  parsedSide = S_UNKNOWN;
}

/*
 * Substitution parsning.
 */

void Sensors::doBuildSubstitution( int unum, int type ) {
  RS_LOG ( LA_SENSORS, "BuildSubstitution: " << unum << ", " << type);
  facts->setTeammateType(unum, type);
  lastSensorType = SENSOR_SUBSTITUTION;
}

void Sensors::doBuildSubstitution( int unum ) {
  RS_LOG ( LA_SENSORS, "BuildSubstitution: " << unum);
  lastSensorType = SENSOR_SUBSTITUTION;
}

/*
 * Clang version parsning.
 * This need to be handled if field players and online coach dosn't
 * handle the same versions of the coach languge. Not high priority.
 */

void Sensors::doBuildClangPlayerVersionMsg() {
  RS_LOG ( LA_SENSORS, "BuildClangPlayerVersionMsg");
  // Todo!
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildClangPlayerVersion(int min, int max) {
  RS_LOG ( LA_SENSORS, "BuildClangPlayerVersionMsg: " << min << ", " << max);
  // Todo!
  lastSensorType = SENSOR_OK;
}

/*
 * Error parsning.
 */

void Sensors::doBuildCantReconnect() {
  RS_LOG ( LA_SENSORS, "BuildCantReconnect" );
  RS_WARNING ( "BuildCantReconnect" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildInitError() {
  RS_LOG ( LA_SENSORS, "BuildInitError" );
  RS_WARNING(  "BuildInitError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildNoMoreTeamOrPlayerOrGoalieError() {
  RS_LOG ( LA_SENSORS, "BuildNoMoreTeamOrPlayerOrGoalieError" );
  RS_WARNING ( "BuildNoMoreTeamOrPlayerOrGoalieError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildNoSuchTeamOrAlreadyHaveCoachError() {
  RS_LOG ( LA_SENSORS, "BuildNoSuchTeamOrAlreadyHaveCoachError" );
  RS_WARNING ( "BuildNoSuchTeamOrAlreadyHaveCoachError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildTooManyMovesError() {
  RS_LOG ( LA_SENSORS, "BuildTooManyMovesError" );
  RS_WARNING ( "BuildTooManyMovesError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildUnknownCommandError() {
  RS_LOG ( LA_SENSORS, "BuildUnknownCommandError" );
  RS_WARNING ( "BuildUnknownCommandError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildIllegalCommandError() {
  RS_LOG ( LA_SENSORS, "BuildIllegalCommandError" );
  RS_WARNING ( "BuildIllegalCommandError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildSayMessageTooLongError() {
  RS_LOG ( LA_SENSORS, "BuildSayMessageTooLongError" );
  RS_WARNING ( "BuildSayMessageTooLongError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildIllegalModeError() {
  RS_LOG ( LA_SENSORS, "BuildIllegalModeError" );
  RS_WARNING ( "BuildIllegalModeError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildIllegalObjectFormError() {
  RS_LOG ( LA_SENSORS, "BuildIllegalObjectFormError" );
  RS_WARNING ( "BuildIllegalObjectFormError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildSaidTooManyFreeformMessagesError() {
  RS_LOG ( LA_SENSORS, "BuildSaidTooManyFreeformMessagesError" );
  RS_WARNING ( "BuildSaidTooManyFreeformMessagesError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildCannotSayFreeformWhilePlayonError() {
  RS_LOG ( LA_SENSORS, "BuildCannotSayFreeformWhilePlayonError" );
  RS_WARNING ( "BuildCannotSayFreeformWhilePlayonError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildSaidTooManyMetaMessagesError() {
  RS_LOG ( LA_SENSORS, "BuildSaidTooManyMetaMessagesError" );
  RS_WARNING ( "BuildSaidTooManyMetaMessagesError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildSaidTooManyAdviceMessagesError() {
  RS_LOG ( LA_SENSORS, "BuildSaidTooManyAdviceMessagesError" );
  RS_WARNING ( "BuildSaidTooManyAdviceMessagesError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildSaidTooManyInfoMessagesError() {
  RS_LOG ( LA_SENSORS, "BuildSaidTooManyInfoMessagesError" );
  RS_WARNING ( "BuildSaidTooManyInfoMessagesError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildSaidTooManyDefineMessagesError() {
  RS_LOG ( LA_SENSORS, "BuildSaidTooManyDefineMessagesError" );
  RS_WARNING ( "BuildSaidTooManyDefineMessagesError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildCouldNotParseSayError() {
  RS_LOG ( LA_SENSORS, "BuildCouldNotParseSayError" );
  RS_WARNING ( "BuildCouldNotParseSayError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildSaidTooManyMessagesError() {
  RS_LOG ( LA_SENSORS, "BuildSaidTooManyMessagesError" );
  RS_WARNING ( "BuildSaidTooManyMessagesError" );
  lastSensorType = SENSOR_ERROR;
}

void Sensors::doBuildUnknownError( const std::string& error ) {
  RS_LOG ( LA_SENSORS, "BuildUnknownError: " << error);
  RS_WARNING ( "BuildUnknownError: " << error);
  lastSensorType = SENSOR_ERROR;
}

/*
 * Warning parsning.
 */

void Sensors::doBuildNoTeamFoundWarning() {
  RS_LOG ( LA_SENSORS, "BuildNoTeamFoundWarning" );
  RS_WARNING ( "BuildNoTeamFoundWarning" );
  lastSensorType = SENSOR_WARNING;
}

void Sensors::doBuildNoSuchPlayerWarning() {
  RS_LOG ( LA_SENSORS, "BuildNoSuchPlayerWarning" );
  RS_WARNING ( "BuildNoSuchPlayerWarning" );
  lastSensorType = SENSOR_WARNING;
}

void Sensors::doBuildCannotSubWhilePlayOnWarning() {
  RS_LOG ( LA_SENSORS, "BuildCannotSubWhilePlayOnWarning" );
  RS_WARNING ( "BuildCannotSubWhilePlayOnWarning" );
  lastSensorType = SENSOR_WARNING;
}

void Sensors::doBuildNoSubsLeftWarning() {
  RS_LOG ( LA_SENSORS, "BuildNoSubsLeftWarning" );
  RS_WARNING ( "BuildNoSubsLeftWarning" );
  lastSensorType = SENSOR_WARNING;
}

void Sensors::doBuildMaxOfThatPlayerTypeOnFieldWarning() {
  RS_LOG ( LA_SENSORS, "BuildMaxOfThatPlayerTypeOnFieldWarning" );
  RS_WARNING ( "BuildMaxOfThatPlayerTypeOnFieldWarning" );
  lastSensorType = SENSOR_WARNING;
}

void Sensors::doBuildCannotChangeGoalieWarning() {
  RS_LOG ( LA_SENSORS, "BuildCannotChangeGoalieWarning" );
  RS_WARNING ( "BuildCannotChangeGoalieWarning" );
  lastSensorType = SENSOR_WARNING;
}

void Sensors::doBuildCompressionWarning() {
  RS_LOG ( LA_SENSORS, "BuildCompressionWarning" );
  RS_WARNING ( "BuildCompressionWarning" );
  lastSensorType = SENSOR_WARNING;
}

void Sensors::doBuildUnknownWarning(const std::string& warning) {
  RS_LOG ( LA_SENSORS, "BuildUnknownWarning: " << warning);
  RS_WARNING ( "BuildUnknownWarning: " << warning);
  lastSensorType = SENSOR_WARNING;
}

/*
 * OK parsning.
 */

void Sensors::doBuildClangVerOK( int min, int max ) {
  RS_LOG ( LA_SENSORS, "BuildClangVerOK: " << min << ", " << max );
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildEyeOK(bool on) {
  RS_LOG (LA_SENSORS, "BuildEyeOK: " << on);
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildEarOK(bool on) {
  RS_LOG (LA_SENSORS, "BuildEarOK: " << on);
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildMoveOK() {
  RS_LOG (LA_SENSORS, "BuildMoveOK");
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildSayOK() {
  RS_LOG (LA_SENSORS, "BuildSayOK");
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildTeamNamesOK() {
  RS_LOG (LA_SENSORS, "BuildTeamNamesOK");
  parsedSide = S_UNKNOWN;
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildChangePlayerTypeOK(int unum, int type) {
  RS_LOG ( LA_SENSORS, "BuildChangePlayerTypeOK: " << unum << ", " << type);
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildCompressionOK(int level) {
  RS_LOG ( LA_SENSORS, "BuildCompressionOK: " << level);
  lastSensorType = SENSOR_OK;
}

void Sensors::doBuildUnknownOK(const std::string& msg) {
  RS_LOG ( LA_SENSORS, "BuildUnknownOK: " << msg);
  lastSensorType = SENSOR_OK;
}

/* 
 * Private help functions.
 */

SideLineName Sensors::locationToLine() const {
  if (!parsedLocations.empty())
    switch (parsedLocations[0]) {
    case FL_TOP: {
      return SL_TOP;
    }
    case FL_LEFT: {
      return SL_LEFT;
    }
    case FL_RIGHT: {
      return SL_RIGHT;
    }
    case FL_BOTTOM: {
      return SL_BOTTOM;
    }
    default: {
      return SL_UNKNOWN;
    }
    }
  return SL_UNKNOWN;
}
  
MarkerName Sensors::locationToMarker() const {
  switch (parsedLocations.size()) {
  case 0: {
    return UNKNOWN_MARKER;
  }
  case 1: {
    if (parsedFlagOffset.isKnown()) {
      if (parsedFlagOffset.getValue() == 0) {
	switch (parsedLocations[0]) {
	case FL_LEFT:
	  return FLAG_L0;
	case FL_RIGHT:
	  return FLAG_R0;
	case FL_TOP:
	  return FLAG_T0;
	case FL_BOTTOM:
	  return FLAG_B0;
	default:
	  return UNKNOWN_MARKER;
	}
      }
      else { 
	return UNKNOWN_MARKER;
      }
    }
    else {
      switch (parsedLocations[0]) {
      case FL_CENTER:
	return FLAG_C;      
      case FL_RIGHT:
	return GOAL_R;
      case FL_LEFT:
	return GOAL_L;
      default:
	return UNKNOWN_MARKER;
      } 
    }
  }
  case 2: {
    switch (parsedLocations[0]) {
    case FL_CENTER: {
      switch (parsedLocations[1]) {
      case FL_TOP:
	return FLAG_CT;
      case FL_BOTTOM: 
	return FLAG_CB;
      default:
	return UNKNOWN_MARKER;
      }
    }
    case FL_LEFT: {
      switch (parsedLocations[1]) {      
      case FL_TOP: {
	if (parsedFlagOffset.isKnown()) {
	  switch (parsedFlagOffset.getValue()) {
	  case 10:
	    return FLAG_LT10;
	  case 20: 
	    return FLAG_LT20;
	  case 30:
	    return FLAG_LT30;
	  default:
	    return UNKNOWN_MARKER;
	  }
	}
	return FLAG_LT;
      }
      case FL_BOTTOM: {
	if (parsedFlagOffset.isKnown()) {
	  switch (parsedFlagOffset.getValue()) {
	  case 10: 
	    return FLAG_LB10;
	  case 20:
	    return FLAG_LB20;
	  case 30:
	    return FLAG_LB30;
	  default:
	    return UNKNOWN_MARKER;
	  }
	}
	return FLAG_LB;
      }      
      default:
	return UNKNOWN_MARKER;
      }
    }
    case FL_RIGHT: {
      switch (parsedLocations[1]) {      
      case FL_TOP: {
	if (parsedFlagOffset.isKnown()) {
	  switch (parsedFlagOffset.getValue()) {
	  case 10:
	    return FLAG_RT10;
	  case 20:
	    return FLAG_RT20;
	  case 30:
	    return FLAG_RT30;
	  default:
	    return UNKNOWN_MARKER;
	  }
	}
	return FLAG_RT;
      }
      case FL_BOTTOM: {
	if (parsedFlagOffset.isKnown()) {
	  switch (parsedFlagOffset.getValue()) {
	  case 10:
	    return FLAG_RB10;
	  case 20:
	    return FLAG_RB20;
	  case 30:
	    return FLAG_RB30;
	  default:
	    return UNKNOWN_MARKER;
	  }
	}
	return FLAG_RB;
      }
      default: 
	return UNKNOWN_MARKER;
      }
    }
    case FL_TOP: {
      if (parsedFlagOffset.isKnown()) {
	switch (parsedLocations[1]) {
	case FL_LEFT: {
	  switch (parsedFlagOffset.getValue()) {
	  case 10: 
	    return FLAG_TL10;
	  case 20:
	    return FLAG_TL20;
	  case 30:
	    return FLAG_TL30;
	  case 40:
	    return FLAG_TL40;
	  case 50:
	    return FLAG_TL50;
	  default:
	    return UNKNOWN_MARKER;
	  }
	}
	case FL_RIGHT: {
	  switch (parsedFlagOffset.getValue()) {
	  case 10:
	    return FLAG_TR10;
	  case 20:
	    return FLAG_TR20;
	  case 30:
	    return FLAG_TR30;
	  case 40:
	    return FLAG_TR40;
	  case 50:
	    return FLAG_TR50;
	  default:
	    return UNKNOWN_MARKER;
	  }
	}
	default:
	  return UNKNOWN_MARKER;
	}
      }
      return UNKNOWN_MARKER;
    }
    case FL_BOTTOM: {
      if (parsedFlagOffset.isKnown()) {
	switch (parsedLocations[1]) {
	case FL_LEFT: {
	  switch (parsedFlagOffset.getValue()) {
	  case 10:
	    return FLAG_BL10;
	  case 20:
	    return FLAG_BL20;
	  case 30:
	    return FLAG_BL30;
	  case 40:
	    return FLAG_BL40;
	  case 50:
	    return FLAG_BL50;
	  default:
	    return UNKNOWN_MARKER;
	  }
	}
	case FL_RIGHT: {
	  switch (parsedFlagOffset.getValue()) {
	  case 10:
	    return FLAG_BR10;
	  case 20:
	    return FLAG_BR20;
	  case 30:
	    return FLAG_BR30;
	  case 40:
	    return FLAG_BR40;
	  case 50:
	    return FLAG_BR50;
	  default:
	    return UNKNOWN_MARKER;
	  }
	}       
	default:
	  return UNKNOWN_MARKER;
	}
      }
      return UNKNOWN_MARKER;
    }
    default:
      return UNKNOWN_MARKER;
    }
  }
  case 3: {
    switch (parsedLocations[0]) {
    case FL_PENALTY: {
      switch (parsedLocations[1]) {
      case FL_LEFT: {
	switch (parsedLocations[2]) {
	case FL_TOP:
	  return FLAG_PLT;
	case FL_CENTER:
	  return FLAG_PLC;
	case FL_BOTTOM:
	  return FLAG_PLB;
	default:
	  return UNKNOWN_MARKER;
	}
      }
      case FL_RIGHT: {
	switch (parsedLocations[2]) {
	case FL_TOP:
	  return FLAG_PRT;
	case FL_CENTER:
	  return FLAG_PRC;
	case FL_BOTTOM:
	  return FLAG_PRB;
	default:
	  return UNKNOWN_MARKER;
	}
      }
      default:
	return UNKNOWN_MARKER;
      }
    }
    case FL_GOAL: {
      switch (parsedLocations[1]) {
      case FL_LEFT: {
	switch (parsedLocations[2]) {
	case FL_TOP:
	  return FLAG_GLT;
	case FL_BOTTOM:
	  return FLAG_GLB;
	default:
	  return UNKNOWN_MARKER;
	}
      }
      case FL_RIGHT: {
	switch (parsedLocations[2]) {
	case FL_TOP:
	  return FLAG_GRT;
	case FL_BOTTOM:
	  return FLAG_GRB;
	default:
	  return UNKNOWN_MARKER;
	}
      }
      default:
	return UNKNOWN_MARKER;
      }
    }
    default:
      return UNKNOWN_MARKER;
    }    
  default:
    return UNKNOWN_MARKER;
  }
  }
}

void Sensors::executeControlCommand(ControlCommand* cmd) throw() {
  if (SetCyclesSinceIOBeforeShutdown* ccmd = 
      dynamic_cast<SetCyclesSinceIOBeforeShutdown *>(cmd) ) {
    facts->CP_cycles_since_io_before_shutdown = ccmd->getCycles();
  } else if (SetMinTicksBeforeSendingCommand* ccmd = 
	     dynamic_cast<SetMinTicksBeforeSendingCommand *>(cmd) ) {
    facts->CP_min_ticks_before_sending_command = ccmd->getTicks();
  } else if (SetMaxTicksBeforeSendingCommand* ccmd = 
	     dynamic_cast<SetMaxTicksBeforeSendingCommand *>(cmd) ) {
    facts->CP_max_ticks_before_sending_command = ccmd->getTicks();
  } else if (SetTicksBeforeCommandWarning* ccmd = 
	     dynamic_cast<SetTicksBeforeCommandWarning *>(cmd) ) {
    facts->CP_ticks_before_command_warning = ccmd->getTicks();
  }
  delete cmd;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "Sensors.icc"
#endif


