/* -*- Mode: C++ -*-
 * $Id: WorldFacts.cc,v 1.15 2004/09/17 13:00:55 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WorldFacts.cc
 * Unitname: BasicSystem
 * $Revision: 1.15 $
 * Created by: David Rosn 2002-06-18
 * Last modified by $Author: frehe $ $Date: 2004/09/17 13:00:55 $
 *
 *
 * HISTORY
 *
 * $Log: WorldFacts.cc,v $
 * Revision 1.15  2004/09/17 13:00:55  frehe
 * Changed the cycles before shutdown to 20.
 *
 * Revision 1.14  2004/09/05 20:35:29  frehe
 * Added CP_no_logging.
 *
 * Revision 1.13  2004/08/31 17:12:59  frehe
 * Added CP_random_seed.
 *
 * Revision 1.12  2004/08/29 14:41:55  frehe
 * Added methods to test whether teamnames has been set or not.
 *
 * Revision 1.11  2004/08/29 13:35:17  frehe
 * Removed RoboSoc as the default teamname.
 *
 * Revision 1.10  2004/08/23 13:22:48  frehe
 * Added CP_module_dir
 *
 * Revision 1.9  2003/12/02 16:16:23  frehe
 * Added a new parameter CP_cycles_before_ball_unknown to represent the number of cycles the ball hasn't been seen before it's unknown.
 *
 * Revision 1.8  2003/10/13 23:22:35  frehe
 * Added extra debug output.
 *
 * Revision 1.7  2003/08/20 12:30:36  s02davro
 * Changed initial values of command_warning and min_ticks
 *
 * Revision 1.6  2003/07/22 14:45:59  s02davro
 * Changed type of pen_dist_x
 *
 * Revision 1.5  2003/07/21 08:56:36  s02davro
 * Added support for new server parameters (v.9), player parameters and player types
 *
 * Revision 1.4  2002/12/07 17:29:05  s02davro
 * Hack in Sensor to fix trainer added CP_trainer to WorldFacts
 *
 * Revision 1.3  2002/10/01 16:57:50  s02davro
 * Added version 8 parameters in WorldFacts
 *
 * Revision 1.2  2002/09/21 19:08:59  s02davro
 * Now the left goal is always ours
 *
 * Revision 1.1  2002/08/31 16:09:05  s02davro
 * Moved WorldFacts from Framework to BasicSystem
 *
 * Revision 1.1  2002/08/02 21:58:29  s02davro
 * Created to replace parameterView, better associated with the agents WorldModel.
 *
 */

/**
 * @file WorldFacts.cc
 * Implementation file for the class WorldFacts.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.15 $
 * $Author: frehe $
 * $Date: 2004/09/17 13:00:55 $
 */


#include "WorldFacts.h"
#include "rs_error.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE

WorldFacts* WorldFacts::onlyInstance(NULL);

const WorldFacts* WorldFacts::instance() {
  if (onlyInstance == NULL)
    onlyInstance = new WorldFacts();
  return onlyInstance;
}

WorldFacts* WorldFacts::nonConstInstance() {
  if (onlyInstance == NULL)
    onlyInstance = new WorldFacts();
  return onlyInstance;
}

WorldFacts::WorldFacts() 
  : serverOptionsInitialized(false),
    playerOptionsInitialized(false),
    playerTypes(10) {
  parameterInit();
  initOptions();
  initMarkers();
}

WorldFacts::~WorldFacts() {
}

void WorldFacts::init(int argc, char **argv, bool verbose) {
  if (onlyInstance == NULL)
    onlyInstance = new WorldFacts();
  onlyInstance->readCommandLine(argc, argv, verbose);
}


bool
WorldFacts::isOurTeamNameKnown() const
{
  return ("our_team" != CP_team_name);
}

  
bool
WorldFacts::isTheirTeamNameKnown() const
{
  return ("their_team" != CP_their_team_name);
}


void WorldFacts::parameterInit() {
  // Special options
  CP_no_logging                 = false;
  CP_team_name                  = "our_team";
  CP_their_team_name            = "their_team";
  CP_goalie                     = false;
  CP_trainer                    = false;
  CP_cycles_before_unknown_ball = 10;
  CP_cycles_since_io_before_shutdown  = 20;
  CP_min_ticks_before_sending_command = 1;
  CP_ticks_before_command_warning     = 7;
  CP_max_ticks_before_sending_command = 9;
  CP_clang_version_min          = 7;
  CP_clang_version_max          = 7;
  CP_dist_change_qstep          = 0.02;
  CP_dir_change_qstep           = 0.01;
  CP_speed_qstep                = 0.01;
  CP_random_seed                = 1;
 
  SP_pitch_length    		= 105.0;
  SP_pitch_width 	 	= 68.0;
  SP_pitch_margin	 	= 5.0;
  SP_penalty_area_length 	= 16.5;
  SP_penalty_area_width		= 40.32;
  SP_goal_area_length		= 5.5;
  SP_goal_area_width		= 18.32;
  SP_penalty_spot_dist		= 11.0;
  SP_corner_arc_r		= 1.0;
  SP_num_lines     		= 4;
  SP_num_markers   		= 55;
  SP_goal_width    		= 14.02;       
  SP_team_size     		= 11;
  SP_unum_far_length		= 20.0;
  SP_unum_too_far_length	= 40.0;
  SP_team_far_length		= 40.0;
  SP_team_too_far_length	= 60.0;
  SP_version	 		= 9.0;
  SP_host                       = "localhost";
  SP_buffer_size                = 4096;
  SP_port	    		= 6000;
  SP_coach_port    		= 6001;
  SP_connect_wait               = 300;
  SP_olcoach_port               = 6002;
  SP_verbose	 		= false;
  SP_drop_ball_time             = 200;
  SP_connect_wait               = 300;
  SP_game_over_wait             = 100;
  SP_kick_off_wait              = 100;
  SP_auto_mode                  = false;
  SP_team_l_start               = "";
  SP_team_r_start               = "";
  SP_slow_down_factor           = 1;
  SP_half_time     		= 300;
  SP_free_kick_margin		= 9.15;
  SP_after_goal_wait  		= 50;
  SP_back_passes                = true;
  SP_free_kick_faults           = true;
  SP_player_size   		= 0.3;
  SP_player_decay  		= 0.4;
  SP_player_rand 		= 0.1;
  SP_player_weight		= 60.0;
  SP_player_speed_max		= 1.0;
  SP_player_accel_max           = 1.0;
  SP_player_rand_factor_l       = 1.0;
  SP_player_rand_factor_r       = 1.0;
  SP_inertia_moment		= 5.0;
  SP_feel_distance 		= 3.0;  
  SP_dash_power_rate		= 0.006;
  SP_visible_angle 		= 90.0;
  SP_visible_distance           = 3;
  SP_hear_max                   = 2;
  SP_hear_inc                   = 1;
  SP_hear_decay    		= 2;
  SP_say_msg_size  		= 10;
  SP_audio_cut_dist		= 50.0;	
  SP_tackle_back_dist           = 0.5;
  SP_tackle_dist                = 2.5;
  SP_tackle_exponent            = 6;
  SP_tackle_power_rate          = 0.027;
  SP_tackle_width               = 1.25;
  SP_tackle_cycles              = 10;
  SP_point_to_ban               = 5;
  SP_point_to_duration          = 20;
  SP_stamina_min                = 0.0;
  SP_stamina_max                = 4000.0;
  SP_stamina_inc_max		= 45.0;
  SP_recover_dec_thr		= 0.3;
  SP_recover_dec   		= 0.002;
  SP_recover_min   		= 0.5;
  SP_recover_max   		= 1.0;
  SP_recover_init               = 1.0;
  SP_effort_dec_thr		= 0.3;
  SP_effort_dec    		= 0.005;	
  SP_effort_inc_thr		= 0.6;	
  SP_effort_inc    		= 0.01;
  SP_effort_min    		= 0.6;
  SP_effort_max    		= 1.0;
  SP_effort_init                = 0;
  SP_catchable_area_l  		= 2.0;	
  SP_catchable_area_w  		= 1.0;	
  SP_ckick_margin               = 1.0;
  SP_catch_probability		= 1.0;
  SP_catch_ban_cycle		= 5;        
  SP_goalie_max_moves           = 2;
  SP_proper_goal_kicks          = false;
  SP_max_goal_kicks             = 3;
  SP_ball_size     		= 0.085;		
  SP_ball_decay    		= 0.94;
  SP_ball_rand     		= 0.05;		
  SP_ball_weight   		= 0.2;
  SP_ball_speed_max		= 2.7;
  SP_ball_accel_max             = 2.7;
  SP_stopped_ball_vel           = 0.01;
  SP_wind_force    		= 0.0;
  SP_wind_dir		        = 0.0;	
  SP_wind_rand     		= 0.0;	
  SP_wind_ang                   = 0.0;
  SP_wind_none                  = true;
  SP_wind_random                = false;
  SP_kickable_margin		= 0.7;
  SP_kickable_area              = SP_kickable_margin + SP_ball_size + SP_player_size;
  SP_kick_power_rate		= 0.027;
  SP_kick_rand                  = 0.0;
  SP_kick_rand_factor_l         = 1.0;
  SP_kick_rand_factor_r         = 1.0;
  SP_control_radius             = 2.0;
  SP_control_radius_width       = 0.0; // Don't know default value.
  SP_dist_qstep    		= 0.1;
  SP_land_qstep            	= 0.01;
  SP_angle_qstep                = 1.0;
  SP_dist_qstep_l               = 0.1;
  SP_dist_qstep_r               = 0.1;  
  SP_land_qstep_l               = 0.01;
  SP_land_qstep_r               = 0.01;
  SP_angle_qstep_l              = 1.0;
  SP_angle_qstep_r              = 1.0;
  SP_maxpower     		= 100;
  SP_minpower     		= -100;
  SP_maxmoment    		= 180;
  SP_minmoment    		= -180;
  SP_maxneckang                 = 90;
  SP_minneckang                 = -90;
  SP_maxneckmoment              = 180;
  SP_minneckmoment              = -180;
  SP_say_coach_cnt_max          = 128;
  SP_say_coach_msg_size         = 128;
  SP_clang_win_size             = 300;
  SP_clang_define_win           = 1;
  SP_clang_meta_win             = 1;
  SP_clang_advice_win           = 1;
  SP_clang_info_win             = 1;
  SP_clang_rule_win             = 1;
  SP_clang_del_win              = 1;
  SP_clang_mess_delay           = 50;
  SP_clang_mess_per_cycle       = 1;
  SP_freeform_send_period       = 20;
  SP_freeform_wait_period       = 600;
  SP_send_vi_step               = 100;
  SP_coach                      = false;
  SP_coach_w_referee            = false;
  SP_old_coach_hear             = false;
  SP_use_offside   		= true;
  SP_offside_active_area_size   = 9.15;
  SP_offside_kick_margin        = 9.15;
  SP_forbid_kick_off_offside    = true;
  SP_start_goal_l               = 0;
  SP_start_goal_r               = 0;
  SP_fullstate_l                = false;
  SP_fullstate_r                = false;
  SP_team_actuator_noise        = false; 
  SP_slowness_on_top_left       = 1;
  SP_slowness_on_top_right      = 1; 
  SP_penalty_shoot_outs         = true;
  SP_pen_dist_x                 = 42.5;
  SP_pen_max_goalie_dist_x      = 4;
  SP_pen_allow_mult_kicks       = false;
  SP_pen_random_winner          = false;
  SP_pen_before_setup_wait      = 30;
  SP_pen_max_extra_kicks        = 10;
  SP_pen_nr_kicks               = 5;
  SP_pen_ready_wait             = 50;
  SP_pen_setup_wait             = 100;
  SP_pen_taken_wait             = 100;
  SP_keepaway                   = false;
  SP_keepaway_length            = 20;
  SP_keepaway_width             = 20;
  SP_keepaway_start             = -1;
  SP_keepaway_logging           = true;
  SP_keepaway_log_dir           = "./";
  SP_keepaway_log_fixed_name    = "rcssserver";
  SP_keepaway_log_fixed         = false;
  SP_keepaway_log_dated         = true;
  SP_log_date_format            = "%Y%m%d%H%M-";
  SP_game_logging               = true;
  SP_game_log_version           = 3;
  SP_game_log_dated             = true;
  SP_game_log_fixed             = false;
  SP_game_log_compression       = 0;
  SP_game_log_dir               = "./";
  SP_game_log_fixed_name        = "rcssserver";
  SP_text_logging               = true;
  SP_text_log_dated             = true;
  SP_text_log_fixed             = false;
  SP_text_log_compression       = 0;
  SP_text_log_dir               = "./";
  SP_text_log_fixed_name        = "rcssserver";
  SP_log_times                  = 0;
  SP_profile                    = false;
  SP_record_messages            = false;
  SP_sense_body_step            = 100;
  SP_simulator_step		= 100;
  SP_send_step     		= 150;
  SP_recv_step     		= 10;
  SP_lcm_step                   = 150; // Don't know default value.
  SP_landmark_file              = "~/.rcssserver-landmark.xml";
  SP_module_dir                 = "/home/TDDA14/pkg/rcsoccersim-cvs/lib/rcssserver/modules";
  SP_send_comms                 = false;
  SP_synch_mode                 = false;;
  SP_synch_offset               = 60;
  SP_synch_micro_sleep          = 1;
  playerTypes[0] = PlayerType();
  teammateTypes[1] = 0;
  teammateTypes[2] = 0;
  teammateTypes[3] = 0;
  teammateTypes[4] = 0;
  teammateTypes[5] = 0;
  teammateTypes[6] = 0;
  teammateTypes[7] = 0;
  teammateTypes[8] = 0;
  teammateTypes[9] = 0;
  teammateTypes[10] = 0;
  teammateTypes[11] = 0;
}

void WorldFacts::initOptions() {
  OptionType opt[] = {
    {"no_logging", (void *)&CP_no_logging, IT_BOOL},
    {"team_name", (void *)&CP_team_name, IT_STRING},
    {"their_team_name", (void *)&CP_their_team_name, IT_STRING},
    {"goalie", (void *)&CP_goalie,  IT_BOOL},
    {"trainer", (void *)&CP_trainer,  IT_BOOL},
    {"cycles_before_unknown_ball", (void *)&CP_cycles_before_unknown_ball,  IT_INT},
    {"cycles_since_io_before_shutdown", (void *)&CP_cycles_since_io_before_shutdown, IT_INT},
    {"min_ticks_before_sending_command",  (void *)&CP_min_ticks_before_sending_command, IT_INT},
    {"ticks_before_command_warning", (void *)&CP_ticks_before_command_warning, IT_INT},
    {"max_ticks_before_sending_command",  (void *)&CP_max_ticks_before_sending_command, IT_INT},  
    {"clang_version_min", (void *)&CP_clang_version_min, IT_INT},
    {"clang_version_max", (void *)&CP_clang_version_max, IT_INT},
    {"dist_change_qstep", (void *)&CP_dist_change_qstep, IT_FLOAT},
    {"dir_change_qstep", (void *)&CP_dir_change_qstep, IT_FLOAT},
    {"speed_qstep", (void *)&CP_speed_qstep, IT_FLOAT},
    {"random_seed", (void *)&CP_random_seed,  IT_INT},
    {"pitch_length", (void *)&SP_pitch_length, IT_FLOAT},
    {"pitch_width", (void *)&SP_pitch_width, IT_FLOAT},
    {"pitch_margin", (void *)&SP_pitch_margin, IT_FLOAT},
    {"penalty_area_length", (void *)&SP_penalty_area_length, IT_FLOAT},
    {"penalty_area_width", (void *)&SP_penalty_area_width, IT_FLOAT},
    {"goal_area_length", (void *)&SP_goal_area_length, IT_FLOAT},
    {"goal_area_width", (void *)&SP_goal_area_width, IT_FLOAT},
    {"penalty_spot_dist", (void *)&SP_penalty_spot_dist, IT_FLOAT},
    {"corner_arc_r", (void *)&SP_corner_arc_r, IT_FLOAT},
    {"num_lines", (void *)&SP_num_lines, IT_INT},
    {"num_markers", (void *)&SP_num_markers, IT_INT},
    {"goal_width", (void *)&SP_goal_width, IT_FLOAT},
    {"team_size", (void *)&SP_team_size, IT_INT},
    {"unum_far_length", (void *)&SP_unum_far_length, IT_FLOAT},
    {"unum_too_far_length", (void *)&SP_unum_too_far_length, IT_FLOAT},
    {"team_far_length", (void *)&SP_team_far_length, IT_FLOAT},
    {"team_too_far_length", (void *)&SP_team_too_far_length, IT_FLOAT},
    {"version", (void *)&SP_version, IT_FLOAT},
    {"host", (void *)&SP_host, IT_STRING},
    {"buffer_size", (void *)&SP_buffer_size, IT_INT},
    {"port", (void *)&SP_port, IT_INT},
    {"coach_port", (void *)&SP_coach_port, IT_INT},
    {"connect_wait", (void *)&SP_connect_wait, IT_INT},
    {"olcoach_port", (void *)&SP_olcoach_port, IT_INT},
    {"verbose", (void *)&SP_verbose,  IT_BOOL},
    {"drop_ball_time", (void *)&SP_drop_ball_time, IT_INT},
    {"connect_wait", (void *)&SP_connect_wait, IT_INT},
    {"game_over_wait", (void *)&SP_game_over_wait, IT_INT},
    {"kick_off_wait", (void *)&SP_kick_off_wait, IT_INT},
    {"auto_mode", (void *)&SP_auto_mode, IT_BOOL},
    {"team_l_start", (void *)&SP_team_l_start, IT_STRING},
    {"team_r_start", (void *)&SP_team_r_start, IT_STRING},
    {"slow_down_factor", (void *)&SP_slow_down_factor, IT_INT},
    {"half_time", (void *)&SP_half_time, IT_INT},
    {"free_kick_margin", (void *)&SP_free_kick_margin, IT_FLOAT},
    {"after_goal_wait", (void *)&SP_after_goal_wait, IT_INT},
    {"back_passes", (void *)&SP_back_passes,  IT_BOOL},
    {"free_kick_faults", (void *)&SP_free_kick_faults,  IT_BOOL},
    {"nr_extra_halfs", (void *)&SP_nr_extra_halfs, IT_INT},
    {"nr_normal_halfs", (void *)&SP_nr_normal_halfs, IT_INT},
    {"player_size", (void *)&SP_player_size, IT_FLOAT},
    {"player_decay", (void *)&SP_player_decay, IT_FLOAT},
    {"player_rand", (void *)&SP_player_rand, IT_FLOAT},
    {"player_weight", (void *)&SP_player_weight, IT_FLOAT},
    {"player_speed_max", (void *)&SP_player_speed_max, IT_FLOAT},
    {"player_accel_max", (void *)&SP_player_accel_max, IT_FLOAT},
    {"player_rand_factor_l", (void *)&SP_player_rand_factor_l, IT_FLOAT},
    {"player_rand_factor_r", (void *)&SP_player_rand_factor_r, IT_FLOAT},
    {"prand_factor_l", (void *)&SP_player_rand_factor_l, IT_FLOAT},
    {"prand_factor_r", (void *)&SP_player_rand_factor_r, IT_FLOAT},
    {"inertia_moment", (void *)&SP_inertia_moment, IT_FLOAT},
    {"feel_distance", (void *)&SP_feel_distance, IT_FLOAT},
    {"dash_power_rate", (void *)&SP_dash_power_rate, IT_FLOAT},
    {"visible_angle", (void *)&SP_visible_angle, IT_FLOAT},
    {"visible_distance", (void *)&SP_visible_distance, IT_FLOAT},
    {"hear_max", (void *)&SP_hear_max, IT_INT},
    {"hear_inc", (void *)&SP_hear_inc, IT_INT},
    {"hear_decay", (void *)&SP_hear_decay, IT_INT},
    {"say_msg_size", (void *)&SP_say_msg_size, IT_INT},
    {"audio_cut_dist", (void *)&SP_audio_cut_dist, IT_FLOAT},
    {"tackle_back_dist", (void *)&SP_tackle_back_dist, IT_FLOAT},
    {"tackle_dist", (void *)&SP_tackle_dist, IT_FLOAT},
    {"tackle_exponent", (void *)&SP_tackle_exponent, IT_INT},
    {"tackle_power_rate", (void *)&SP_tackle_power_rate, IT_FLOAT},
    {"tackle_width", (void *)&SP_tackle_width, IT_FLOAT},
    {"tackle_cycles", (void *)&SP_tackle_cycles, IT_INT},
    {"point_to_ban", (void *)&SP_point_to_ban, IT_INT},
    {"point_to_duration", (void *)&SP_point_to_duration, IT_INT},
    {"stamina_min", (void *)&SP_stamina_min, IT_FLOAT},
    {"stamina_max", (void *)&SP_stamina_max, IT_FLOAT},
    {"stamina_inc_max", (void *)&SP_stamina_inc_max, IT_FLOAT},
    {"recover_dec_thr", (void *)&SP_recover_dec_thr, IT_FLOAT},
    {"recover_dec", (void *)&SP_recover_dec, IT_FLOAT},
    {"recover_min", (void *)&SP_recover_min, IT_FLOAT},
    {"recover_max", (void *)&SP_recover_max, IT_FLOAT},
    {"recover_init", (void *)&SP_recover_init, IT_FLOAT},
    {"effort_dec_thr", (void *)&SP_effort_dec_thr, IT_FLOAT},
    {"effort_dec", (void *)&SP_effort_dec, IT_FLOAT},
    {"effort_inc_thr", (void *)&SP_effort_inc_thr, IT_FLOAT},
    {"effort_inc", (void *)&SP_effort_inc, IT_FLOAT},
    {"effort_min", (void *)&SP_effort_min, IT_FLOAT},
    {"effort_max", (void *)&SP_effort_max, IT_FLOAT},
    {"effort_init", (void *)&SP_effort_init, IT_INT},
    {"catchable_area_l", (void *)&SP_catchable_area_l, IT_FLOAT},
    {"catchable_area_w", (void *)&SP_catchable_area_w, IT_FLOAT},
    {"ckick_margin", (void *)&SP_ckick_margin, IT_FLOAT},
    {"catch_probability", (void *)&SP_catch_probability, IT_FLOAT},
    {"catch_ban_cycle", (void *)&SP_catch_ban_cycle, IT_INT},
    {"goalie_max_moves", (void *)&SP_goalie_max_moves, IT_INT},
    {"proper_goal_kicks", (void *)&SP_proper_goal_kicks,  IT_BOOL},
    {"max_goal_kicks", (void *)&SP_max_goal_kicks, IT_INT},
    {"ball_size", (void *)&SP_ball_size, IT_FLOAT},
    {"ball_decay", (void *)&SP_ball_decay, IT_FLOAT},
    {"ball_rand", (void *)&SP_ball_rand, IT_FLOAT},
    {"ball_weight", (void *)&SP_ball_weight, IT_FLOAT},
    {"ball_speed_max", (void *)&SP_ball_speed_max, IT_FLOAT},
    {"ball_accel_max", (void *)&SP_ball_accel_max, IT_FLOAT},
    {"stopped_ball_vel", (void *)&SP_stopped_ball_vel, IT_FLOAT},
    {"wind_force", (void *)&SP_wind_force, IT_FLOAT},
    {"wind_dir", (void *)&SP_wind_dir, IT_FLOAT},
    {"wind_rand", (void *)&SP_wind_rand, IT_FLOAT},
    {"wind_ang", (void *)&SP_wind_ang, IT_FLOAT},
    {"wind_none", (void *)&SP_wind_none,  IT_BOOL},
    {"wind_random", (void *)&SP_wind_random,  IT_BOOL},
    {"kickable_margin", (void *)&SP_kickable_margin, IT_FLOAT},
    {"kickable_area", (void *)&SP_kickable_area, IT_FLOAT},
    {"kick_power_rate", (void *)&SP_kick_power_rate, IT_FLOAT},
    {"kick_rand", (void *)&SP_kick_rand, IT_FLOAT},
    {"kick_rand_factor_l", (void *)&SP_kick_rand_factor_l, IT_FLOAT},
    {"kick_rand_factor_r", (void *)&SP_kick_rand_factor_r, IT_FLOAT},
    {"control_radius", (void *)&SP_control_radius, IT_FLOAT},
    {"control_radius_width", (void *)&SP_control_radius_width, IT_FLOAT},
    {"dist_qstep", (void *)&SP_dist_qstep, IT_FLOAT},
    {"land_qstep", (void *)&SP_land_qstep, IT_FLOAT},
    {"quantize_step", (void *)&SP_dist_qstep, IT_FLOAT},
    {"quantize_step_l", (void *)&SP_land_qstep, IT_FLOAT},
    {"angle_qstep", (void *)&SP_angle_qstep, IT_FLOAT},
    {"dist_qstep_l", (void *)&SP_dist_qstep_l, IT_FLOAT},
    {"dist_qstep_r", (void *)&SP_dist_qstep_r, IT_FLOAT},
    {"land_qstep_l", (void *)&SP_land_qstep_l, IT_FLOAT},
    {"land_qstep_r", (void *)&SP_land_qstep_r, IT_FLOAT},
    {"angle_qstep_l", (void *)&SP_angle_qstep_l, IT_FLOAT},
    {"angle_qstep_r", (void *)&SP_angle_qstep_r, IT_FLOAT},
    {"maxpower", (void *)&SP_maxpower, IT_FLOAT},
    {"minpower", (void *)&SP_minpower, IT_FLOAT},
    {"maxmoment", (void *)&SP_maxmoment, IT_FLOAT},
    {"minmoment", (void *)&SP_minmoment, IT_FLOAT},
    {"maxneckang", (void *)&SP_maxneckang, IT_FLOAT},
    {"minneckang", (void *)&SP_minneckang, IT_FLOAT},
    {"maxneckmoment", (void *)&SP_maxneckmoment, IT_FLOAT},
    {"minneckmoment", (void *)&SP_minneckmoment, IT_FLOAT},
    {"say_coach_cnt_max", (void *)&SP_say_coach_cnt_max, IT_INT},
    {"say_coach_msg_size", (void *)&SP_say_coach_msg_size, IT_INT},
    {"clang_win_size", (void *)&SP_clang_win_size, IT_INT},
    {"clang_define_win", (void *)&SP_clang_define_win, IT_INT},
    {"clang_meta_win", (void *)&SP_clang_meta_win, IT_INT},
    {"clang_advice_win", (void *)&SP_clang_advice_win, IT_INT},
    {"clang_info_win", (void *)&SP_clang_info_win, IT_INT},
    {"clang_rule_win", (void *)&SP_clang_rule_win, IT_INT},
    {"clang_del_win", (void *)&SP_clang_del_win, IT_INT},
    {"clang_mess_delay", (void *)&SP_clang_mess_delay, IT_INT},
    {"clang_mess_per_cycle", (void *)&SP_clang_mess_per_cycle, IT_INT},
    {"freeform_send_period", (void *)&SP_freeform_send_period, IT_INT},
    {"freeform_wait_period", (void *)&SP_freeform_wait_period, IT_INT},
    {"send_vi_step", (void *)&SP_send_vi_step, IT_INT},
    {"coach", (void *)&SP_coach,  IT_BOOL},
    {"coach_w_referee", (void *)&SP_coach_w_referee,  IT_BOOL},
    {"old_coach_hear", (void *)&SP_old_coach_hear,  IT_BOOL},
    {"use_offside", (void *)&SP_use_offside,  IT_BOOL},
    {"offside_active_area_size", (void *)&SP_offside_active_area_size, IT_FLOAT},
    {"offside_kick_margin", (void *)&SP_offside_kick_margin, IT_FLOAT},
    {"forbid_kick_off_offside", (void *)&SP_forbid_kick_off_offside,  IT_BOOL},
    {"start_goal_l", (void *)&SP_start_goal_l, IT_INT},
    {"start_goal_r", (void *)&SP_start_goal_r, IT_INT},
    {"fullstate_l", (void *)&SP_fullstate_l,  IT_BOOL},
    {"fullstate_r", (void *)&SP_fullstate_r,  IT_BOOL},
    {"team_actuator_noise", (void *)&SP_team_actuator_noise,  IT_BOOL},    
    {"slowness_on_top_left", (void *)&SP_slowness_on_top_left, IT_INT},
    {"slowness_on_top_right", (void *)&SP_slowness_on_top_right, IT_INT},
    {"slowness_on_top_for_left_team", (void *)&SP_slowness_on_top_left, IT_INT},
    {"slowness_on_top_for_right_team", (void *)&SP_slowness_on_top_right, IT_INT},
    {"penalty_shoot_outs", (void *)&SP_penalty_shoot_outs, IT_BOOL},
    {"pen_dist_x", (void *)&SP_pen_dist_x, IT_FLOAT},
    {"pen_max_goalie_dist_x", (void *)&SP_pen_max_goalie_dist_x, IT_INT},
    {"pen_allow_mult_kicks", (void *)&SP_pen_allow_mult_kicks, IT_BOOL},
    {"pen_random_winner", (void *)&SP_pen_random_winner, IT_BOOL},
    {"pen_before_setup_wait", (void *)&SP_pen_before_setup_wait, IT_INT},
    {"pen_max_extra_kicks", (void *)&SP_pen_max_extra_kicks, IT_INT},
    {"pen_nr_kicks", (void *)&SP_pen_nr_kicks, IT_INT},
    {"pen_ready_wait", (void *)&SP_pen_ready_wait, IT_INT},
    {"pen_setup_wait", (void *)&SP_pen_setup_wait, IT_INT},
    {"pen_taken_wait", (void *)&SP_pen_taken_wait, IT_INT},
    {"keepaway", (void *)&SP_keepaway, IT_BOOL},
    {"keepaway_length", (void *)&SP_keepaway_length, IT_INT},
    {"keepaway_width", (void *)&SP_keepaway_width, IT_INT},
    {"keepaway_start", (void *)&SP_keepaway_start, IT_INT},
    {"keepaway_logging", (void *)&SP_keepaway_logging, IT_BOOL},
    {"keepaway_log_dir", (void *)&SP_keepaway_log_dir, IT_STRING},
    {"keepaway_log_fixed_name", (void *)&SP_keepaway_log_fixed_name, IT_STRING},
    {"keepaway_log_fixed", (void *)&SP_keepaway_log_fixed, IT_BOOL},
    {"keepaway_log_dated", (void *)&SP_keepaway_log_dated, IT_BOOL},
    {"log_date_format", (void *)&SP_log_date_format, IT_STRING},
    {"game_logging", (void *)&SP_game_logging, IT_BOOL},
    {"game_log_version", (void *)&SP_game_log_version, IT_INT},
    {"game_log_dated", (void *)&SP_game_log_dated, IT_BOOL},
    {"game_log_fixed", (void *)&SP_game_log_fixed, IT_BOOL},
    {"game_log_compression", (void *)&SP_game_log_compression, IT_INT},
    {"game_log_dir", (void *)&SP_game_log_dir, IT_STRING},
    {"game_log_fixed_name", (void *)&SP_game_log_fixed_name, IT_STRING},
    {"text_logging", (void *)&SP_text_logging, IT_BOOL},
    {"text_log_dated", (void *)&SP_text_log_dated, IT_BOOL},
    {"text_log_fixed", (void *)&SP_text_log_fixed, IT_BOOL},
    {"text_log_compression", (void *)&SP_text_log_compression, IT_INT},
    {"text_log_dir", (void *)&SP_text_log_dir, IT_STRING},
    {"text_log_fixed_name", (void *)&SP_text_log_fixed_name, IT_STRING},
    {"log_times", (void *)&SP_log_times, IT_INT},
    {"profile", (void *)&SP_profile,  IT_BOOL},
    {"record_messages", (void *)&SP_record_messages,  IT_BOOL},
    {"sense_body_step", (void *)&SP_sense_body_step, IT_INT},
    {"simulator_step", (void *)&SP_simulator_step, IT_INT},
    {"send_step", (void *)&SP_send_step, IT_INT},
    {"recv_step", (void *)&SP_recv_step, IT_INT},
    {"lcm_step", (void *)&SP_lcm_step, IT_INT},
    {"landmark_file", (void *)&SP_landmark_file, IT_STRING},
    {"module_dir", (void *)&SP_module_dir, IT_STRING},
    {"send_comms", (void *)&SP_send_comms,  IT_BOOL},
    {"synch_mode", (void *)&SP_synch_mode,  IT_BOOL},
    {"synch_offset", (void *)&SP_synch_offset, IT_INT},
    {"synch_micro_sleep", (void *)&SP_synch_micro_sleep, IT_INT},
    { "\000", 0, IT_NONE }
  };
  
  int i = 0;
  while (opt[i].vptr != 0 && i < optsize) {
    strcpy(options[i].name, opt[i].name);
    options[i].vptr = opt[i].vptr;
    options[i].vsize = opt[i].vsize;
    ++i;
  }

  OptionType opt2[] = {
    {"player_types", (void *)&PP_player_types, IT_INT},
    {"subs_max", (void *)&PP_subs_max, IT_INT},
    {"player_speed_max_delta_min", (void *)&PP_player_speed_max_delta_min, IT_FLOAT},
    {"player_speed_max_delta_max", (void *)&PP_player_speed_max_delta_max, IT_FLOAT},
    {"stamina_inc_max_delta_factor", (void *)&PP_stamina_inc_max_delta_factor, IT_FLOAT},
    {"player_decay_delta_min", (void *)&PP_player_decay_delta_min, IT_FLOAT},
    {"player_decay_delta_max", (void *)&PP_player_decay_delta_max, IT_FLOAT},
    {"inertia_moment_delta_factor", (void *)&PP_inertia_moment_delta_factor, IT_FLOAT},
    {"dash_power_rate_delta_min", (void *)&PP_dash_power_rate_delta_min, IT_FLOAT},
    {"dash_power_rate_delta_max", (void *)&PP_dash_power_rate_delta_max, IT_FLOAT},
    {"player_size_delta_factor", (void *)&PP_player_size_delta_factor, IT_FLOAT},
    {"new_dash_power_rate_delta_min", (void *)&PP_new_dash_power_rate_delta_min, IT_FLOAT},
    {"new_dash_power_rate_delta_max", (void *)&PP_new_dash_power_rate_delta_max, IT_FLOAT},
    {"new_stamina_inc_max_delta_factor", (void *)&PP_new_stamina_inc_max_delta_factor, IT_FLOAT},
    {"kickable_margin_delta_min", (void *)&PP_kickable_margin_delta_min, IT_FLOAT},
    {"kickable_margin_delta_max", (void *)&PP_kickable_margin_delta_max, IT_FLOAT},
    {"kick_rand_delta_factor", (void *)&PP_kick_rand_delta_factor, IT_FLOAT},
    {"extra_stamina_delta_min", (void *)&PP_extra_stamina_delta_min, IT_FLOAT},
    {"extra_stamina_delta_max", (void *)&PP_extra_stamina_delta_max, IT_FLOAT},
    {"effort_max_delta_factor", (void *)&PP_effort_max_delta_factor, IT_FLOAT},
    {"effort_min_delta_factor", (void *)&PP_effort_min_delta_factor, IT_FLOAT},
    { "\000", 0, IT_NONE }
  };

  i = 0;
  while (opt2[i].vptr != 0 && i < poptsize) {
    strcpy(playeroptions[i].name, opt2[i].name);
    playeroptions[i].vptr = opt2[i].vptr;
    playeroptions[i].vsize = opt2[i].vsize;
    ++i;
  }

  OptionType opt3[] = {
    {"id", (void *)&(playertype.id), IT_INT},
    {"player_speed_max", (void *)&(playertype.player_speed_max), IT_FLOAT},
    {"stamina_inc_max", (void *)&(playertype.stamina_inc_max), IT_FLOAT},
    {"player_decay", (void *)&(playertype.player_decay), IT_FLOAT},
    {"inertia_moment", (void *)&(playertype.inertia_moment), IT_FLOAT},
    {"dash_power_rate", (void *)&(playertype.dash_power_rate), IT_FLOAT},
    {"player_size", (void *)&(playertype.player_size), IT_FLOAT},
    {"kickable_margin", (void *)&(playertype.kickable_margin), IT_FLOAT},
    {"kick_rand", (void *)&(playertype.kick_rand), IT_FLOAT},
    {"extra_stamina", (void *)&(playertype.extra_stamina), IT_FLOAT},
    {"effort_max", (void *)&(playertype.effort_max), IT_FLOAT},
    {"effort_min", (void *)&(playertype.effort_min), IT_FLOAT},
    { "\000", 0, IT_NONE }
  };

  i = 0;
  while (opt3[i].vptr != 0 && i < ptoptsize) {
    strcpy(playertypeoptions[i].name, opt3[i].name);
    playertypeoptions[i].vptr = opt3[i].vptr;
    playertypeoptions[i].vsize = opt3[i].vsize;
    ++i;
  }

}

void WorldFacts::readCommandLine(int argc, char** argv, int verbose) {
  // skip command name
  argv++ ; argc-- ;

  // first, search for option '-f' that's read a configuration file
  for(int i = 0 ; i < argc ; ++i) {
    if (!strcmp(*(argv + i),"-f")) {
      readOptions(*(argv+i+1), verbose);
    }
  }

  // next, analyze command line option
  while (argc) {
    if (!strcmp(*argv, "-f")) {
      argv += 2 ;
      argc -= 2 ;
      continue ;
    }

    if ( readOption(*argv + 1, *(argv + 1)) ) {
      argv += 2 ;
      argc -= 2 ;
    } else {
      argv++ ;
      argc-- ;
    }
  }
}


void WorldFacts::readOptions(const std::string& filename, int verbose) {
  FILE *file;

  file = fopen(filename.c_str(), "r");

  if (file == NULL) {
    if (verbose > 0) {
      fprintf(stderr, "can't open (%s)\n",
	      filename.c_str());
	}
    else {
      RS_WARNING("Can't open file " + filename 
		 + " in WorldFacts::readOptions!");
    }	  
    return;
  }

  static char linebuf[256];
  char *t;
  int i = 0;

  while (fgets(linebuf, 256, file) != 0) {
    ++i;
    if (verbose == 1 && i % 10 == 0)
      fprintf(stderr, ".");
                
    if (verbose > 2) {
      fprintf(stderr,linebuf);
      fflush(stderr);
    }

    if (linebuf[0] == '#'
	|| linebuf[0] == '\n'
	|| linebuf[0] == '\0')
      continue;

    t = linebuf;
    // replace the ':' by space
    while (*t != '\0') {
      if (*t == ':') *t = ' ';
      t++;
    }
    readOption(linebuf);
    
  }
}

bool WorldFacts::readOption(const char* linebuf, 
			    int verbose, const OptionType* opts) {
  int n;
  static char com[64];
  static char val[64];
  
  n = sscanf(linebuf,"%s %s", com, val) ;

  if (n < 1) {
    if (verbose > 0) {
      fprintf(stderr, "Read illegal line:  (%s)\n", linebuf);
    }
    else {
      RS_WARNING("Read illegal line: " + std::string(linebuf) 
		 + " in WorldFacts::readOption!");
      return false;
    }
  }
  return readOption(com, val, opts);
}

void WorldFacts::setOption(const std::string &name, std::string value) {
  if (!serverOptionsInitialized) {
    for (int p = 0 ; options[p].vptr != NULL ; p++) {
      if (name != options[p].name)
	continue;
      if (options[p].vsize == IT_STRING)
	*((std::string *)options[p].vptr) = value;
      else 
	RS_WARNING("Typematch problem for parameter '" << name << "' (= '" << value << "')");
      return;
    }
  }
  RS_WARNING("Tried to set unknown option '" << name << "'");
}

void WorldFacts::setOption(const std::string &name, int value) {
  if (!serverOptionsInitialized) {
    for (int p = 0 ; options[p].vptr != NULL ; p++) {
      if (name != options[p].name)
	continue;
      if (options[p].vsize == IT_BOOL) {
	if (value == 1) *((bool *)options[p].vptr) = true;
	else if (value == 0) *((bool *)options[p].vptr) = false;
	else 
	  RS_WARNING("Strange bool value: '" << name << "' = '" << value << "'");
      } 
      else if (options[p].vsize == IT_INT)
	*((int *)options[p].vptr) = value;
      else if (options[p].vsize == IT_FLOAT)
	*((double *)options[p].vptr) = (double)value;
      else 
	RS_WARNING("Typematch problem for parameter '" << name << "' (= '" << value << "')");
      
    }
    return;
  } else if (!playerOptionsInitialized) {
    for (int p = 0 ; playeroptions[p].vptr != NULL ; p++) {
      if (name != playeroptions[p].name)
	continue;
      if (playeroptions[p].vsize == IT_INT)
	*((int *)playeroptions[p].vptr) = value;
      else if (playeroptions[p].vsize == IT_FLOAT)
	*((double *)playeroptions[p].vptr) = (double)value;
      else 
	RS_WARNING("Typematch problem for parameter '" << name << "' (= '" << value << "')");
    }
    return;
  } else {
    for (int p = 0; playertypeoptions[p].vptr != NULL ; p++) {
      if (name != playertypeoptions[p].name)
	continue;
      if (playertypeoptions[p].vsize == IT_INT)
	*((int *)playertypeoptions[p].vptr) = value;
      else if (playertypeoptions[p].vsize == IT_FLOAT)
	*((double *)playertypeoptions[p].vptr) = (double)value;
      else 
	RS_WARNING("Typematch problem for parameter '" << name << "' (= '" << value << "')");
    }
    return;
  }
  RS_WARNING("Tried to set unknown option '" << name << "'");
}

void WorldFacts::setOption(const std::string &name, double value) {
  if (!serverOptionsInitialized) {
    for (int p = 0 ; options[p].vptr != NULL ; p++) {
      if (name != options[p].name)
	continue;
      if (options[p].vsize == IT_FLOAT)
	*((double *)options[p].vptr) = value;
      else 
	RS_WARNING("Typematch problem for parameter '" << name << "' (= '" << value << "')");    
      return;
    }
  } else if (!playerOptionsInitialized) {
    for (int p = 0 ; playeroptions[p].vptr != NULL ; p++) {
      if (name != playeroptions[p].name)
	continue;
      if (playeroptions[p].vsize == IT_FLOAT)
	*((double *)playeroptions[p].vptr) = value;
      else 
	RS_WARNING("Typematch problem for parameter '" << name << "' (= '" << value << "')");    
      return;
    }
  } else {
    for (int p = 0; playertypeoptions[p].vptr != NULL ; p++) {
      if (name != playertypeoptions[p].name)
	continue;
      else if (playertypeoptions[p].vsize == IT_FLOAT)
	*((double *)playertypeoptions[p].vptr) = value;
      else 
	RS_WARNING("Typematch problem for parameter '" << name << "' (= '" << value << "')");
    }
    return;
  }
  RS_WARNING("Tried to set unknown option '" << name << "'");
}

bool WorldFacts::readOption(const char* option, const char* value, 
			    const OptionType* opts) {
  int p;
  for (p = 0 ; opts[p].vptr != NULL ; p++) {
    if (strcmp(option, opts[p].name))
      continue ;

    std::cout << "Found match for " << option << " = " << value
	      << " namely " << opts[p].name << std::endl;
    
    switch(opts[p].vsize) {
    case IT_INT:
      *((Int *)opts[p].vptr) = atoi(value) ;
      break ;

    case IT_STRING:
      *((std::string *)opts[p].vptr) = value ;
      break ;

    case IT_FLOAT:
      *((Float *)opts[p].vptr) = atof(value) ;
      break ;

    case IT_BOOL:
      *((bool *)opts[p].vptr) = (!strcmp(value, "false")) ? false : true ;
      break ;

    case IT_NONE:
      break ;
    }
    break ;
  }

  if (opts[p].vptr == NULL) {
    RS_ERROR("Unrecognized option : " + std::string(option)
	     + " in WorldFacts::readOption");
    return false;
  } else {
    RS_WARNING("Setting option '" << option << "' to '" << value << "'");
  }

  return true;
}

const MarkerObject& WorldFacts::getMarker(MarkerName marker) const {
  return markers[marker];
}

void WorldFacts::pM(MarkerName mn, Point p) {
  markers.push_back(MarkerObject(mn, p));
}

void WorldFacts::initMarkers() {
  pM(GOAL_L,    Point(-SP_pitch_length/2.0,     0.0));
  pM(GOAL_R,    Point( SP_pitch_length/2.0,     0.0));
  pM(FLAG_C,    Point( 0.0,                     0.0));
  pM(FLAG_CT,   Point( 0.0,                    -SP_pitch_width/2.0));
  pM(FLAG_CB,   Point( 0.0,                     SP_pitch_width/2.0));
  pM(FLAG_LT,   Point(-SP_pitch_length/2.0,    -SP_pitch_width/2.0));
  pM(FLAG_LB,   Point(-SP_pitch_length/2.0,     SP_pitch_width/2.0));
  pM(FLAG_RT,   Point( SP_pitch_length/2.0,    -SP_pitch_width/2.0));
  pM(FLAG_RB,   Point( SP_pitch_length/2.0,     SP_pitch_width/2.0));
  pM(FLAG_PLT,  Point(-SP_pitch_length/2.0 + 
		       SP_penalty_area_length, -SP_penalty_area_width/2.0));
  pM(FLAG_PLC,  Point(-SP_pitch_length/2.0 + 
		       SP_penalty_area_length,  0));
  pM(FLAG_PLB,  Point(-SP_pitch_length/2.0 +
		       SP_penalty_area_length,  SP_penalty_area_width/2.0));
  pM(FLAG_PRT,  Point( SP_pitch_length/2.0 - 
		       SP_penalty_area_length, -SP_penalty_area_width/2.0));
  pM(FLAG_PRC,  Point( SP_pitch_length/2.0 -
		       SP_penalty_area_length,  0));
  pM(FLAG_PRB,  Point( SP_pitch_length/2.0 -
		       SP_penalty_area_length,  SP_penalty_area_width/2.0));
  pM(FLAG_GLT,  Point(-SP_pitch_length/2.0,    -SP_goal_width/2.0));
  pM(FLAG_GLB,  Point(-SP_pitch_length/2.0,     SP_goal_width/2.0));
  pM(FLAG_GRT,  Point( SP_pitch_length/2.0,    -SP_goal_width/2.0));
  pM(FLAG_GRB,  Point( SP_pitch_length/2.0,     SP_goal_width/2.0));
  pM(FLAG_TL50, Point(-50.0,                   -SP_pitch_width/2.0 -
		                                SP_pitch_margin));
  pM(FLAG_TL40, Point(-40.0,                   -SP_pitch_width/2.0 -
		                                SP_pitch_margin));
  pM(FLAG_TL30, Point(-30.0,                   -SP_pitch_width/2.0 -
		                                SP_pitch_margin));
  pM(FLAG_TL20, Point(-20.0,                   -SP_pitch_width/2.0 -
		                                SP_pitch_margin));
  pM(FLAG_TL10, Point(-10.0,                   -SP_pitch_width/2.0 -
		                                SP_pitch_margin));
  pM(FLAG_T0,   Point( 0.0,                    -SP_pitch_width/2.0 -
		                                SP_pitch_margin));
  pM(FLAG_TR10, Point( 10.0,		       -SP_pitch_width/2.0 - 
		                                SP_pitch_margin));
  pM(FLAG_TR20, Point( 20.0,                   -SP_pitch_width/2.0 -
		                                SP_pitch_margin));
  pM(FLAG_TR30, Point( 30.0,                   -SP_pitch_width/2.0  
		                               -SP_pitch_margin));
  pM(FLAG_TR40, Point( 40.0,                   -SP_pitch_width/2.0 -
		                                SP_pitch_margin));
  pM(FLAG_TR50, Point( 50.0,                   -SP_pitch_width/2.0 -
		                                SP_pitch_margin));    
  pM(FLAG_BL50, Point(-50.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BL40, Point(-40.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BL30, Point(-30.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BL20, Point(-20.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BL10, Point(-10.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_B0,   Point( 0.0,                     SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BR10, Point( 10.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BR20, Point( 20.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BR30, Point( 30.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BR40, Point( 40.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_BR50, Point( 50.0,                    SP_pitch_width/2.0 +
		                                SP_pitch_margin));
  pM(FLAG_LT30, Point(-SP_pitch_length/2.0 -
		       SP_pitch_margin,        -30));
  pM(FLAG_LT20, Point(-SP_pitch_length/2.0 -
		       SP_pitch_margin,        -20));
  pM(FLAG_LT10, Point(-SP_pitch_length/2.0 -
		       SP_pitch_margin,        -10));
  pM(FLAG_L0,   Point(-SP_pitch_length/2.0 -
		       SP_pitch_margin,         0));
  pM(FLAG_LB10, Point(-SP_pitch_length/2.0 - 
		       SP_pitch_margin,         10));
  pM(FLAG_LB20, Point(-SP_pitch_length/2.0 -
		       SP_pitch_margin,         20));
  pM(FLAG_LB30, Point(-SP_pitch_length/2.0 -
		       SP_pitch_margin,         30));    
  pM(FLAG_RT30, Point( SP_pitch_length/2.0 + 
		       SP_pitch_margin,        -30));
  pM(FLAG_RT20, Point( SP_pitch_length/2.0 +
		       SP_pitch_margin,        -20));
  pM(FLAG_RT10, Point( SP_pitch_length/2.0 +
		       SP_pitch_margin,        -10));
  pM(FLAG_R0,   Point( SP_pitch_length/2.0 + 
		       SP_pitch_margin,         0));
  pM(FLAG_RB10, Point( SP_pitch_length/2.0 +
		       SP_pitch_margin,         10));
  pM(FLAG_RB20, Point( SP_pitch_length/2.0 +
		       SP_pitch_margin,         20));
  pM(FLAG_RB30, Point( SP_pitch_length/2.0 +
		       SP_pitch_margin,         30));
}

void WorldFacts::setSides(Side our, Side their) {
  ourSide = our;
  theirSide = their;
}

void WorldFacts::setUniformNumber(UInt16 my_number) {
  uniformNumber = my_number;
}

Side WorldFacts::getOurSide() const {
  return ourSide;
}

Side WorldFacts::getTheirSide() const {
  return theirSide;
}

UInt16 WorldFacts::getUniformNumber() const {
  return uniformNumber;
}

UInt WorldFacts::getOurScore() const {
  return ourScore;
}

UInt WorldFacts::getTheirScore() const {
  return theirScore;
}

const MarkerObject& WorldFacts::getTheirGoal() const {
  return getMarker(GOAL_R);
}

const MarkerObject& WorldFacts::getOurGoal() const {
  return getMarker(GOAL_L);
}

void WorldFacts::setScore(Side side, UInt score) {
  if (side == getOurSide())
    ourScore = score;
  else
    theirScore = score;
}

void WorldFacts::serverParamsInitialized() {
  serverOptionsInitialized = true;
  PlayerType pt;
  pt.player_speed_max = SP_player_speed_max;
  pt.stamina_inc_max = SP_stamina_inc_max;
  pt.player_decay = SP_player_decay;
  pt.inertia_moment = SP_inertia_moment;
  pt.dash_power_rate = SP_dash_power_rate;
  pt.player_size = SP_player_size;
  pt.kickable_margin = SP_kickable_margin;
  pt.kick_rand = SP_kick_rand;
  pt.extra_stamina = 0;
  pt.effort_max = SP_effort_max;
  pt.effort_min = SP_effort_min;
  playerTypes[0] = pt;
  RS_LOG(LA_WORLDMODEL, "Server parameters initialized");
}

void WorldFacts::playerParamsInitialized() {
  playerOptionsInitialized = true;
  RS_LOG(LA_WORLDMODEL, "Player parameters initialized");
}

void WorldFacts::playerTypeComplete() {
  if (playertype.id == 0)
    playerTypes[0] = playertype;
  else 
    playerTypes.push_back(playertype);
  RS_LOG(LA_WORLDMODEL, "Added a player type");
}

const std::vector<PlayerType>& WorldFacts::getPlayerTypes() const {
  return playerTypes;
}

PlayerType WorldFacts::getPlayerType(UInt id) const {
  for (std::vector<PlayerType>::const_iterator i = playerTypes.begin();
       i != playerTypes.end(); ++i)
    if (i->id == id) return *i;
  RS_WARNING("Tried to get a unknown player type");
  return playerTypes[0];
}

PlayerType WorldFacts::getTeammateType(UInt number) const {
  if (teammateTypes.find(number) != teammateTypes.end())
    return getPlayerType(teammateTypes.find(number)->second);
  else
    return playerTypes[0];
}

PlayerType WorldFacts::getMyType() const {
  return getTeammateType(getUniformNumber());
}

void WorldFacts::setTeammateType(UInt number, UInt id) {
  if (id >= playerTypes.size()) {
    RS_WARNING("Tried to set players type to a strange one.");
  } else {
    if (number == getUniformNumber())
      RS_LOG(LA_WORLDMODEL, "You feel like a new field player");
    teammateTypes[number] = id;
  }
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "WorldFacts.icc"
#endif
