/* -*- Mode: C++ -*-
 * $Id: basic_system_types.h,v 1.10 2004/09/05 19:38:15 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: basic_system_types.h
 * Unitname: BasicSystem
 * $Revision: 1.10 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2004/09/05 19:38:15 $
 *
 *
 * HISTORY
 *
 * $Log: basic_system_types.h,v $
 * Revision 1.10  2004/09/05 19:38:15  frehe
 * Improved support for focus.
 *
 * Revision 1.9  2003/08/20 12:31:07  s02davro
 * Added Arm, Tackle and Focus states
 *
 * Revision 1.8  2003/07/22 14:46:23  s02davro
 * Added Arm, Foucs and Tackleobjects
 *
 * Revision 1.7  2003/07/21 08:57:35  s02davro
 * Moved away some stuff to Sensors, added support for arm focus and tackling
 *
 * Revision 1.6  2002/09/26 15:23:42  s02davro
 * Bugfixes, close, team, coach
 *
 * Revision 1.5  2002/08/31 15:07:06  s02davro
 * Added copyconstructor, assignment to all server game objects
 *
 * Revision 1.4  2002/08/02 21:45:52  s02davro
 * Changed the ServerGameObject classes. A lot.
 *
 * Revision 1.3  2001/10/14 21:21:43  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.2  2001/08/17 07:15:14  frehe
 * Made changes to compile RoboSoc with RCCParser-0.10.
 *
 * Revision 1.1  2000/09/03 20:41:11  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file basic_system_types.h
 * Include file with the types used by the RoboSoc basic system.
 *
 * @todo Move the server objects to their own files.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.10 $
 * $Author: frehe $
 * $Date: 2004/09/05 19:38:15 $
 */


#ifndef BASIC_SYSTEM_TYPES_H
#define BASIC_SYSTEM_TYPES_H

#include "library_types.h"
#include "library_types_with_unknown.h"
#include "library_types_with_ue.h"
#include "soccer_types.h"

RS_BEGIN_NAMESPACE

/** Used to represent the objects sent by the server
  to the client as see messages. */
class ServerGameObject {
public:
  bool close;
  AngleDegUE direction;
  FloatUE distance;
  ServerGameObject ( bool def_close = false, 
		     AngleDegUE def_direction = AngleDegUE(),
		     FloatUE def_distance = FloatUE()) 
    : close(def_close),
      direction(def_direction),
      distance(def_distance) {};
  ServerGameObject& operator= (const ServerGameObject& obj) {
    if (this != &obj) {
      close = obj.close;
      direction = obj.direction;
      distance = obj.distance;
    }
    return *this;
  };
};


/** Used to represent the marker objects sent by the
  server to the client as see messages. */
class ServerMarkerObject : public ServerGameObject {
public:
  MarkerName markerId;
  // Not used.
  FloatUE distanceChange;
  AngleDegUE directionChange;
  ServerMarkerObject ( MarkerName def_markerId = UNKNOWN_MARKER, 
		       bool def_close = false,
		       AngleDegUE def_direction = AngleDegUE(),
		       FloatUE def_distance = FloatUE(), 
		       FloatUE def_distanceChange = FloatUE(),
		       AngleDegUE def_directionChange = FloatUE())
    : ServerGameObject(def_close, def_direction, def_distance),
      markerId(def_markerId),
      distanceChange(def_distanceChange),
      directionChange(def_directionChange) {};
  ServerMarkerObject& operator= (const ServerMarkerObject& obj) {
    if (this != &obj) {
      ServerGameObject::operator=(obj);
      markerId = obj.markerId;
      distanceChange = obj.distanceChange;
      directionChange = obj.directionChange;
    }
    return *this;
  };
};


/** Used to represent the line objects sent by the
  server to the client as see messages. */
class ServerLineObject : public ServerGameObject {
public:
  SideLineName lineId;
  ServerLineObject ( SideLineName def_lineId = SL_UNKNOWN, 
		     AngleDegUE def_direction = AngleDegUE(),
		     FloatUE def_distance = FloatUE()) 
    : ServerGameObject(false, def_direction, def_distance),
      lineId(def_lineId) {};
  ServerLineObject& operator= (const ServerLineObject& obj) {
    if (this != &obj) {
      ServerGameObject::operator=(obj);
      lineId = obj.lineId;
    }
    return *this;
  };
};

/** Used to represent the mobile objects sent by the
    server to the client as see messages. */
class ServerMobileObject : public ServerGameObject {
public:
  FloatUE distanceChange;
  AngleDegUE directionChange;
  ServerMobileObject ( bool def_close = false, 
		       AngleDegUE def_direction = AngleDegUE(), 
		       FloatUE def_distance = FloatUE(),
		       FloatUE def_distanceChange = FloatUE(),
		       AngleDegUE def_directionChange = AngleDegUE()) 
    : ServerGameObject(def_close, def_direction, def_distance),
      distanceChange(def_distanceChange),
      directionChange(def_directionChange) {};
  ServerMobileObject& operator= (const ServerMobileObject& obj) {
    if (this != &obj) {
      ServerGameObject::operator=(obj);
      distanceChange = obj.distanceChange;
      directionChange = obj.directionChange;
    }
    return *this;      
  };
};

/** Used to represent the player objects sent by the
  server to the client as see messages. */
class ServerPlayerObject : public ServerMobileObject {
public:
  bool goalie;
  Team team;
  UIntUnknown uniformNumber;
  AngleDegUE relBodyDirection;
  AngleDegUE relNeckDirection;
  bool tackling;
  AngleDegUE pointDirection;
  ServerPlayerObject( bool def_close = false, 
		      AngleDegUE def_direction = AngleDegUE(),
		      bool def_goalie = false,
		      Team def_team = T_UNKNOWN,
		      UIntUnknown def_uniformNumber = UIntUnknown(),
		      FloatUE def_distance = FloatUE(),
		      FloatUE def_distanceChange = FloatUE(),
		      AngleDegUE def_directionChange = AngleDegUE(),
		      AngleDegUE def_relBodyDirection = AngleDegUE(),
		      AngleDegUE def_relNeckDirection = AngleDegUE(),
		      bool def_tackling = false,
		      AngleDegUE def_pointDirection = AngleDegUE()
		      )
    : ServerMobileObject(def_close, def_direction, def_distance, 
			 def_distanceChange, def_directionChange),
    goalie(def_goalie),
    team(def_team),
    uniformNumber(def_uniformNumber),
    relBodyDirection(def_relBodyDirection),
    relNeckDirection(def_relNeckDirection),
    tackling(def_tackling),
    pointDirection(def_pointDirection) {};
  ServerPlayerObject& operator= (const ServerPlayerObject& obj) {
    if (this != &obj) {
      ServerMobileObject::operator=(obj);
      goalie = obj.goalie;
      team = obj.team;
      uniformNumber = obj.uniformNumber;
      relBodyDirection = obj.relBodyDirection;
      relNeckDirection = obj.relNeckDirection;
      tackling = obj.tackling;
      pointDirection = obj.pointDirection;
    }
    return *this;
  };
};

class ServerBallObject : public ServerMobileObject {
public:
  ServerBallObject( bool def_close = false,
		    AngleDegUE def_direction = AngleDegUE(),
		    FloatUE def_distance = FloatUE(),
		    FloatUE def_distanceChange = FloatUE(),
		    AngleDegUE def_directionChange = AngleDegUE()) 
    : ServerMobileObject(def_close, def_direction, def_distance,
			 def_distanceChange, def_directionChange) {};
  ServerBallObject& operator= (const ServerBallObject& obj) {
    if (this != &obj) {
      ServerMobileObject::operator=(obj);
    }
    return *this;
  };
};

class ServerArmState {
public:
  ServerArmState( UInt def_moveableIn = 0,
		  UInt def_expiresIn = 0,
		  PointUnknown def_target = PointUnknown(),
		  UInt def_count = 0)
    : moveableIn(def_moveableIn),
      expiresIn(def_expiresIn),
      target(def_target),
      count(def_count) {}
  UInt moveableIn;
  UInt expiresIn;
  PointUnknown target;
  UInt count;
};

class ServerFocusState {
public:
  ServerFocusState (UInt def_focusCount = 0,
		    Side def_focusSide = S_UNKNOWN,
		    UIntUnknown def_focusUniformNr = UIntUnknown())
    : focusCount(def_focusCount),
      focusSide(def_focusSide),
      focusUniformNr(def_focusUniformNr) {}
  UInt focusCount;
  Side focusSide;
  UIntUnknown focusUniformNr;
};

class ServerTackleState {
public:
  ServerTackleState (UInt def_tackleExpiresIn = 0,
		     UInt def_tackleCount = 0) 
    : tackleExpiresIn(def_tackleExpiresIn),
      tackleCount(def_tackleCount) {}
  UInt tackleExpiresIn;
  UInt tackleCount;
};
    
class ServerAgentObject { 
public:
  ViewQuality viewQuality;
  ViewWidth viewWidth;
  Float stamina;
  Float effort;
  FloatUE speedAmount;
  AngleDegUE speedDirection;
  AngleDegUE relNeckDirection;
  UInt catches;
  UInt changeViews;
  UInt dashes;
  UInt kicks;
  UInt moves;
  UInt says;
  UInt turns;
  UInt turnNecks;
  ServerArmState armState;
  ServerFocusState focusState;
  ServerTackleState tackleState;
  ServerAgentObject ( ViewQuality def_vQ = VQ_HIGH,
		      ViewWidth def_vW = VW_NORMAL,
		      Float def_stamina = 4000,
		      Float def_effort = 1.0,
		      FloatUE def_speedAmount = FloatUE(),
		      AngleDegUE def_speedDirection = AngleDegUE(),
		      AngleDegUE def_relNeckDirection = AngleDegUE(),
		      UInt def_catches = 0,
		      UInt def_changeViews = 0,
		      UInt def_dashes = 0,
		      UInt def_kicks = 0,
		      UInt def_moves = 0,
		      UInt def_says = 0,
		      UInt def_turns = 0,
		      UInt def_turnNecks = 0,
		      ServerArmState def_armState = ServerArmState(),
		      ServerFocusState def_focusState = ServerFocusState(),
		      ServerTackleState def_tackleState = ServerTackleState()) 
    : viewQuality(def_vQ),
      viewWidth(def_vW),
      stamina(def_stamina),
      effort(def_effort),
      speedAmount(def_speedAmount),
      speedDirection(def_speedDirection),
      relNeckDirection(def_relNeckDirection),
      catches(def_catches),
      changeViews(def_changeViews),
      dashes(def_dashes),
      kicks(def_kicks),
      moves(def_moves),
      says(def_says),
      turns(def_turns),
      turnNecks(def_turnNecks),
      armState(def_armState),
      focusState(def_focusState),
      tackleState(def_tackleState) {};
  ServerAgentObject& operator= (const ServerAgentObject& obj) {
    if (this != &obj) {
      viewQuality = obj.viewQuality;
      viewWidth = obj.viewWidth;
      stamina = obj.stamina;
      effort = obj.effort;
      speedAmount = obj.speedAmount;
      speedDirection = obj.speedDirection;
      relNeckDirection = obj.relNeckDirection;
      catches = obj.catches;
      changeViews = obj.changeViews;
      dashes = obj.dashes;
      kicks = obj.kicks;
      moves = obj.moves;
      says = obj.says;
      turns = obj.turns;
      turnNecks = obj.turnNecks;
      armState = obj.armState;
      focusState = obj.focusState;
      tackleState = obj.tackleState;
    }
    return *this;
  };
};

/** Used to represent the player objects sent by the
  server to coach clients as see messages. */

class CoachMobileObject {
public:
  Float x;
  Float y;
  Float dx;
  Float dy;
  CoachMobileObject ( Float def_x = 0, 
		      Float def_y = 0,
		      Float def_dx = 0, 
		      Float def_dy = 0) 
    : x(def_x),
      y(def_y),
      dx(def_dx),
      dy(def_dy) {};
  CoachMobileObject& operator= (const CoachMobileObject& obj) {
    if (this != &obj) {
      x = obj.x;
      y = obj.y;
      dx = obj.dx;
      dy = obj.dy;
    }
    return *this;
  };
};


class CoachPlayerObject : public CoachMobileObject {
public:
  Team team;
  UInt unum;
  bool goalie;
  AngleDeg bodyDirection;
  AngleDeg neckDirection;
  AngleDegUnknown pointDirection;
  bool tackling;
  CoachPlayerObject ( Float def_x = 0,
		      Float def_y = 0,
		      Float def_dx = 0,
		      Float def_dy = 0,
		      Team def_team = T_UNKNOWN,		      
		      UInt def_unum = 0,
		      bool def_goalie = false,
		      AngleDeg def_bodyDirection = AngleDeg(),
		      AngleDeg def_neckDirection = AngleDeg(),
		      AngleDegUnknown def_pointdir = AngleDegUnknown(),
		      bool def_tackling = false)
    : CoachMobileObject(def_x, def_y, def_dx, def_dy),
      team(def_team),
      unum(def_unum),
      goalie(def_goalie),
      bodyDirection(def_bodyDirection),
      neckDirection(def_neckDirection),
      pointDirection(def_pointdir),
      tackling(def_tackling) {};
  CoachPlayerObject& operator= (const CoachPlayerObject& obj) {
    if (this != &obj) {
      CoachMobileObject::operator=(obj);
      team = obj.team;
      unum = obj.unum;
      goalie = obj.goalie;
      bodyDirection = obj.bodyDirection;
      neckDirection = obj.neckDirection;
      pointDirection = obj.pointDirection;
      tackling = obj.tackling;
    }
    return *this;
  };
};

class CoachBallObject : public CoachMobileObject {
public:
  CoachBallObject ( Float def_x = 0,
		    Float def_y = 0,
		    Float def_dx = 0,
		    Float def_dy = 0) 
    : CoachMobileObject(def_x, def_y, def_dx, def_dy) {};
  CoachBallObject& operator= (const CoachBallObject& obj) {
    if (this != &obj) {
      CoachMobileObject::operator=(obj);
    }
    return *this;
  };
};

class CoachHearingObject {
public:
  CoachHearingObject ( Team def_team = T_UNKNOWN,
		       UIntUnknown def_nr = UIntUnknown(),
		       std::string def_msg = "") 
    : team(def_team),
      nr(def_nr),
      msg(def_msg) {}
  Team team;
  UIntUnknown nr;
  std::string msg;
};


RS_END_NAMESPACE


#endif // !BASIC_SYSTEM_TYPES_H
