/* -*- Mode: C++ -*-
 * $Id: AttackerDecision.h,v 1.12 2005/09/19 17:41:56 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttackerDecision.h
 * Unitname: Framework/example/attacker
 * $Revision: 1.12 $
 * Created by: Fredrik Heintz 2004-08-31
 * Last modified by $Author: frehe $ $Date: 2005/09/19 17:41:56 $
 *
 *
 * HISTORY
 *
 * $Log: AttackerDecision.h,v $
 * Revision 1.12  2005/09/19 17:41:56  frehe
 * Created a default mode for the attacker to be used in three on three scenarios.
 *
 * Revision 1.11  2005/09/19 16:58:12  frehe
 * Fixed start of scenario.
 *
 * Revision 1.10  2005/09/15 09:12:42  frehe
 * Removed mAct and warnings.
 *
 * Revision 1.9  2005/09/08 14:18:46  frehe
 * Handle the new bookkeeping messages from the trainer.
 *
 * Revision 1.8  2005/09/08 13:42:57  frehe
 * Refactored out the different skills.
 *
 * Revision 1.7  2005/09/08 12:42:25  frehe
 * Added a test so that decide isn't called more than once a cycle.
 *
 * Revision 1.6  2004/09/13 19:40:59  frehe
 * Tried to use KickBallView and then changed back to AccelerateBallToVelocity.
 *
 * Revision 1.5  2004/09/13 14:38:12  frehe
 * Added support for the new coach messages for the scenarios.
 *
 * Revision 1.4  2004/09/12 23:28:45  frehe
 * Added a check to make sure we have new information about the ball.
 *
 * Revision 1.3  2004/09/08 18:33:43  frehe
 * Added support for different scenario modes.
 *
 * Revision 1.2  2004/09/02 21:47:31  frehe
 * Updated logging.
 *
 * Revision 1.1  2004/08/31 23:04:03  frehe
 * Added an example attacker.
 *
 */

/**
 * @file AttackerDecision.h
 * Include file for the class AttackerDecision.
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.12 $
 * $Author: frehe $
 * $Date: 2005/09/19 17:41:56 $
 */


#ifndef ATTACKER_DECISION_H
#define ATTACKER_DECISION_H


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "Score.h"
#include "Dribble.h"
#include "Pass.h"
#include "ReceivePass.h"
#include "AlignNeckToBody.h"


RS_BEGIN_NAMESPACE


class WorldFacts;
class GameView;
class AgentView;
class PlayerView;
class BallView;
class KickBallView;


/// A type representing different scenario types
enum Scenario {
  NONE,
  DEFAULT,
  SCORE,
  DRIBBLE,
  PASS
};


/**
 * @class AttackerDecision AttackerDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.12 $
 * $Author: frehe $
 * $Date: 2005/09/19 17:41:56 $
 */
class AttackerDecision : public Decision {
public:
  AttackerDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~AttackerDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  virtual bool decide();

  // Event handlers
  //virtual void onActuatorSensorData();
  virtual void onAuralCoachSensorData();
  virtual void onAuralRefereeSensorData();
  virtual void onAuralPlayerSensorData();
  virtual void onAuralSelfSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  //virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  //virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  //virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();


protected:
  const WorldFacts* facts;
  GameView* mGameView;
  AgentView* mAgentView;
  BallView* mBallView;
  KickBallView* mKickBallView;
  PlayerView* mPlayerView;
  bool mRunning;
  Scenario mScenario;
  bool mNewScenario;
  Time mNewScenarioStart;
  int mMinDribbleDist;
  int mMinPassDist;
  int mMaxPassDist;

  Score mScore;
  Dribble mDribble;
  Pass mPass;
  ReceivePass mReceivePass;
  AlignNeckToBody mAlignNeckToBody;
};


RS_END_NAMESPACE


#endif // !ATTACKER_DECISION_H
