/* -*- Mode: C++ -*-
 * $Id: Dribble.h,v 1.1 2005/09/08 13:42:57 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Dribble.h
 * Unitname: Framework/example/attacker
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2005-09-08
 * Last modified by $Author: frehe $ $Date: 2005/09/08 13:42:57 $
 *
 *
 * HISTORY
 *
 * $Log: Dribble.h,v $
 * Revision 1.1  2005/09/08 13:42:57  frehe
 * Refactored out the different skills.
 *
 */

/**
 * @file Dribble.h
 * Include file for the skill class Dribble.
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/09/08 13:42:57 $
 */


#ifndef DRIBBLE_H
#define DRIBBLE_H


#include "Skill.h"
#include "WorldModelInterface.h"


RS_BEGIN_NAMESPACE


class BallView;
class AgentView;
class PlayerView;


/**
 * @class Dribble Dribble.h
 * This is an example dribble skill.
 *
 * @pre The ball is kickable or the relative vector to the ball is known
 * @post Either kick the ball in a suitable direction or move towards the ball
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/09/08 13:42:57 $
 */
class Dribble : public Skill {
public:
  /** Construct a dribble skill. */
  Dribble();

  /** Destructor should clean up after the skill. */
  virtual ~Dribble();


  /** Reset any internal state in the skill. */
  virtual void reset();
  
  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent. You can assume
      that the preconditions of the skill are true (this can be tested
      by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal of the skill. */
  virtual AgentStep generateStep();


  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
      the description of the skill. */
  virtual FuzzyBool applicable();


  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();


private:
  /** Hide copy-constructor. */
  Dribble(const Dribble&);
  /** Hide assignment operator. */
  Dribble& operator= (const Dribble&);

  const WorldModelInterface* const worldModelInterface;
  const WorldFacts* facts;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


#endif // !DRIBBLE_H
