/* -*- Mode: C++ -*-
 * $Id: Pass.cc,v 1.2 2005/09/08 17:40:47 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Pass.cc
 * Unitname: Framework/example/attacker
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2005-09-08
 * Last modified by $Author: frehe $ $Date: 2005/09/08 17:40:47 $
 *
 *
 * HISTORY
 *
 * $Log: Pass.cc,v $
 * Revision 1.2  2005/09/08 17:40:47  frehe
 * Simplified the code.
 *
 * Revision 1.1  2005/09/08 13:42:57  frehe
 * Refactored out the different skills.
 *
 */

/**
 * @file Pass.cc
 * Implementation file for the skill class Pass.
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/09/08 17:40:47 $
 */


#include "Pass.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "view_constants.h"
#include "sector_helpers.h"
#include "rs_error.h"

#include "AccelerateBallToVelocity.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

Pass::Pass()
  : Skill(),
    worldModelInterface(WorldModelInterface::instance()),
    facts(worldModelInterface->getWorldFacts()),
    ballView(NULL),
    agentView(NULL),
    playerView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}


// --- DESTRUCTOR ---

Pass::~Pass()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}


// --- METHODS ---

void
Pass::reset()
{
}


Plan
Pass::generatePlan()
{
  // Return an empty plan.
  return Plan(generateStep());
}


AgentStep
Pass::generateStep()
{
  // Create an empty step
  AgentStep step;

  PlayerObject receiver = playerView->getTeammateClosestToAgent();
  if ( receiver.getTeam() != T_OUR_TEAM ) {
    RS_WARNING( "didn't find any teammates in pass");
  } else {
    if ( agentView->getPosition().isKnown()
	 && receiver.getPosition().isKnown() ) {
      Vector pass_dir(agentView->getPosition().getPoint().getVectorTo(receiver.getPosition().getPoint()));
      AccelerateBallToVelocity kick(pass_dir);
      if ( kick.applicable() ) {
	return kick.generateStep();
      }
    }
  }

  /*
  // Find the closest teammate and pass him
  if ( agentView->getPosition().isKnown() ) {
    Point my_pos = agentView->getPosition().getPoint();
    Vector closest_player(0, 99999);
    bool found = false;
    const std::vector<PlayerObject>& teammates = playerView->getTeammates();
    for ( std::vector<PlayerObject>::const_iterator i = teammates.begin();
	  i != teammates.end(); ++i ) {
      if ( i->getPosition().isKnown() ) {
	Vector tmp(my_pos, i->getPosition().getPoint());
	if ( tmp.getLength() < closest_player.getLength() ) {
	  closest_player = tmp;
	  found = true;
	}
      }
    }

    if ( found && ballView->isBallKickable() ) {
      AngleDeg kick_dir = closest_player.getAngle();
      if ( agentView->getPosition().isKnown() ) {
	Point agent_pos = agentView->getPosition().getPoint();

	// Find the best angle to kick in
	typedef std::vector<PlayerObject> PlayerObjects;
	const PlayerObjects& opps = playerView->getOpponents();
	for ( PlayerObjects::const_iterator p = opps.begin();
	      p != opps.end(); ++p ) {
	  if ( p->getPosition().isKnown() ) {
	    AngleDeg player_dir = agent_pos.getDirectionTo(p->getPosition().getPoint());
	    if ( abs(kick_dir - player_dir) < 30 ) {
	      if ( (kick_dir - player_dir) > 0 ) {
		kick_dir = (kick_dir - player_dir) - 30;
	      } else {
		kick_dir = (kick_dir - player_dir) + 30;
	      }
	    }
	  }
	}
      }

      closest_player.setAngle(kick_dir);
      closest_player.setLength(closest_player.getLength()/5);
      AccelerateBallToVelocity kick(closest_player);
      if ( kick.applicable() ) {
	return kick.generateStep();
      } else {
	std::cout << "AccelerateBallToVelocity kick(" << closest_player
		  << " not applicable" << std::endl;
      }
    }
  }
  */

  // Return the step
  return step;
}


FuzzyBool
Pass::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool
Pass::applicable()
{
  bool res = ( ballView->isBallKickable()
	       && playerView->getTeammates().size() > 0 );

  return res;
}


FuzzyBool
Pass::succeed()
{
  // The skill has not succeeded yet.
  return FuzzyBool(0.0);
}


FuzzyBool
Pass::failed()
{
  // The skill has not failed yet.
  return FuzzyBool(0.0);
}


RS_END_NAMESPACE
