/* -*- Mode: C++ -*-
 * $Id: Pass.h,v 1.1 2005/09/08 13:42:57 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Pass.h
 * Unitname: Framework/example/attacker
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2005-09-08
 * Last modified by $Author: frehe $ $Date: 2005/09/08 13:42:57 $
 *
 *
 * HISTORY
 *
 * $Log: Pass.h,v $
 * Revision 1.1  2005/09/08 13:42:57  frehe
 * Refactored out the different skills.
 *
 */

/**
 * @file Pass.h
 * Include file for the skill class Pass.
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/09/08 13:42:57 $
 */


#ifndef PASS_H
#define PASS_H


#include "Skill.h"
#include "WorldModelInterface.h"


RS_BEGIN_NAMESPACE


class BallView;
class AgentView;
class PlayerView;


/**
 * @class Pass Pass.h
 * This is an example pass skill.
 *
 * @pre At least one teammate is seen
 * @pre The ball is kickable
 * @post The ball is kicked towards the closest teammate
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/09/08 13:42:57 $
 */
class Pass : public Skill {
public:
  /** Construct a pass skill. */
  Pass();

  /** Destructor should clean up after the skill. */
  virtual ~Pass();


  /** Reset any internal state in the skill. */
  virtual void reset();
  
  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent. You can assume
      that the preconditions of the skill are true (this can be tested
      by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal of the skill. */
  virtual AgentStep generateStep();


  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
      the description of the skill. */
  virtual FuzzyBool applicable();


  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();


private:
  /** Hide copy-constructor. */
  Pass(const Pass&);
  /** Hide assignment operator. */
  Pass& operator= (const Pass&);

  const WorldModelInterface* const worldModelInterface;
  const WorldFacts* facts;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


#endif // !PASS_H
