/* -*- Mode: C++ -*-
 * $Id: ReceivePass.cc,v 1.3 2005/09/19 16:57:59 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ReceivePass.cc
 * Unitname: Framework/example/attacker
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2005-09-08
 * Last modified by $Author: frehe $ $Date: 2005/09/19 16:57:59 $
 *
 *
 * HISTORY
 *
 * $Log: ReceivePass.cc,v $
 * Revision 1.3  2005/09/19 16:57:59  frehe
 * Commented out debug print outs.
 *
 * Revision 1.2  2005/09/08 17:40:47  frehe
 * Simplified the code.
 *
 * Revision 1.1  2005/09/08 13:42:57  frehe
 * Refactored out the different skills.
 *
 */

/**
 * @file ReceivePass.cc
 * Implementation file for the skill class ReceivePass.
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2005/09/19 16:57:59 $
 */


#include "ReceivePass.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "view_constants.h"
#include "sector_helpers.h"
#include "rs_error.h"

#include "TurnBodyToPoint.h"
#include "DashToPoint.h"
#include "SearchBall.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ReceivePass::ReceivePass()
  : Skill(),
    worldModelInterface(WorldModelInterface::instance()),
    facts(worldModelInterface->getWorldFacts()),
    ballView(NULL),
    agentView(NULL),
    playerView(NULL),
    minPassDist(0),
    maxPassDist(0)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}


// --- DESTRUCTOR ---

ReceivePass::~ReceivePass()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}


// --- METHODS ---

void
ReceivePass::setMinPassDist(int min_pass_dist)
{
  minPassDist = min_pass_dist;
}


void
ReceivePass::setMaxPassDist(int max_pass_dist)
{
  maxPassDist = max_pass_dist;
}


void
ReceivePass::reset()
{
}


Plan
ReceivePass::generatePlan()
{
  // Return an empty plan.
  return Plan(generateStep());
}


AgentStep
ReceivePass::generateStep()
{
  // Create an empty step
  AgentStep step;

  // If we know the position of the ball, run for it!
  Point ball_pos;
  if ( ballView->getPosition(NEXT).isKnown() ) {
    ball_pos = ballView->getPosition(NEXT).getPoint();
  } else if ( ballView->getPosition().isKnown() ) {
    ball_pos = ballView->getPosition().getPoint();
  } else {
    SearchBall search_ball;
    if ( search_ball.applicable() ) {
      //std::cout << "search ball" << std::endl;
      return search_ball.generateStep();
    } else {
      // Something is not right here!!!
      std::cout << "search ball not applicable!" << std::endl;
    }
  }

  Point agent_pos;
  if ( agentView->getPosition().isKnown() ) {
    agent_pos = agentView->getPosition().getPoint();
  } else {
    // No agent position!
    std::cout << "no agent position" << std::endl;
    return step;
  }

  // If we are not turned towards the ball turn towards it
  // otherwise dash towards the ball
  AngleDeg dir_to_ball = agent_pos.getDirectionTo(ball_pos);
  if ( abs(dir_to_ball) > 15 ) {
    TurnBodyToPoint turn(ball_pos);
    if ( turn.applicable() ) {
      //std::cout << "turn to ball dir = " << dir_to_ball << std::endl;
      return turn.generateStep();
    }
  } else {
    DashToPoint dash(ball_pos);
    if ( dash.applicable() ) {
      //std::cout << "dash to ball" << std::endl;
      return dash.generateStep();
    }
  }

  /*
  if ( ballView->getPosition().isKnown() ) {
    // If the ball has passed the minimum pass distance the run for it
    Point ball_pos = ballView->getPosition().getPoint();
	  
    if ( ballView->getSpeedVector().isKnown() ) {
      // If we know the speed, simulate where the ball will end up
      Vector ball_vel = ballView->getSpeedVector().getVector();
      while ( ball_vel.getLength() > 0.05
	      && ball_pos.getX() < (-40+minPassDist+2) ) {
	// Get the next position
	ball_pos += ball_vel;
	// Reduce the ball speed
	ball_vel *= facts->SP_ball_decay;
      }
    } else {
      // Take the next position at least if available
      if ( ballView->getPosition(NEXT).isKnown() ) {
	ball_pos = ballView->getPosition(NEXT).getPoint();
      }
    }

    if ( agentView->getPosition().isKnown() ) {
      Point agent_pos = agentView->getPosition().getPoint();
      Vector ball_vec = agent_pos.getVectorTo(ball_pos);
	  
      // If the ball is close go for it
      if ( ball_pos.getX() > (-40+minPassDist)
	   || ( ball_vec.getLength() > 4
		&& ball_pos.getX() > (-40+minPassDist-5) ) ) {
	// If angle is too large, turn towards the ball
	if ( abs(ball_vec.getAngle()) > 15 ) {
	  step.setBodyCommand(new TurnCommand(agentView->getAngleForTurn(ball_vec.getAngle())));
	} else {
	  step.setBodyCommand(new DashCommand(agentView->getPowerForDash(ball_pos)));
	}
      }
    }
  } else {
    // else turn towards the ball position
    if ( ballView->getRelativeVector().isKnown() ) {
      Vector ball_vec = ballView->getRelativeVector().getVector();
	
      // Turn towards the ball
      if ( abs(ball_vec.getAngle()) > 15 ) {
	step.setBodyCommand(new TurnCommand(agentView->getAngleForTurn(ball_vec.getAngle())));
      }
    }
  }
  */

  // Return the step
  return step;
}


FuzzyBool
ReceivePass::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool
ReceivePass::applicable()
{
  /*
  bool res = ( ballView->getPosition().isKnown()
	       || ballView->getRelativeVector().isKnown() );
	       */
  bool res = ( agentView->getPosition().isKnown() );

  return res;
}


FuzzyBool
ReceivePass::succeed()
{
  // The skill has not succeeded yet.
  return FuzzyBool(0.0);
}


FuzzyBool
ReceivePass::failed()
{
  // The skill has not failed yet.
  return FuzzyBool(0.0);
}


RS_END_NAMESPACE
