/* -*- Mode: C++ -*-
 * $Id: ReceivePass.h,v 1.1 2005/09/08 13:42:57 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ReceivePass.h
 * Unitname: Framework/example/attacker
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2005-09-08
 * Last modified by $Author: frehe $ $Date: 2005/09/08 13:42:57 $
 *
 *
 * HISTORY
 *
 * $Log: ReceivePass.h,v $
 * Revision 1.1  2005/09/08 13:42:57  frehe
 * Refactored out the different skills.
 *
 */

/**
 * @file ReceivePass.h
 * Include file for the skill class ReceivePass.
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/09/08 13:42:57 $
 */


#ifndef RECEIVE_PASS_H
#define RECEIVE_PASS_H


#include "Skill.h"
#include "WorldModelInterface.h"


RS_BEGIN_NAMESPACE


class BallView;
class AgentView;
class PlayerView;


/**
 * @class ReceivePass ReceivePass.h
 * This is an example receive pass skill.
 *
 * @pre The position or relative vector of the ball is known
 * @post Intercept the ball
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/09/08 13:42:57 $
 */
class ReceivePass : public Skill {
public:
  /** Construct a receive pass skill. */
  ReceivePass();

  /** Destructor should clean up after the skill. */
  virtual ~ReceivePass();


  void setMinPassDist(int min_pass_dist);
  void setMaxPassDist(int max_pass_dist);


  /** Reset any internal state in the skill. */
  virtual void reset();

  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent. You can assume
      that the preconditions of the skill are true (this can be tested
      by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal of the skill. */
  virtual AgentStep generateStep();


  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
      the description of the skill. */
  virtual FuzzyBool applicable();


  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();


private:
  /** Hide copy-constructor. */
  ReceivePass(const ReceivePass&);
  /** Hide assignment operator. */
  ReceivePass& operator= (const ReceivePass&);

  const WorldModelInterface* const worldModelInterface;
  const WorldFacts* facts;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;

  int minPassDist;
  int maxPassDist;
};


RS_END_NAMESPACE


#endif // !RECEIVE_PASS_H
