/* -*- Mode: C++ -*-
 * $Id: Score.h,v 1.4 2005/09/08 13:42:58 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Score.h
 * Unitname: Framework/example/attacker
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2004-08-31
 * Last modified by $Author: frehe $ $Date: 2005/09/08 13:42:58 $
 *
 *
 * HISTORY
 *
 * $Log: Score.h,v $
 * Revision 1.4  2005/09/08 13:42:58  frehe
 * Refactored out the different skills.
 *
 * Revision 1.3  2005/09/08 08:58:49  frehe
 * Implemented the reset method.
 *
 * Revision 1.2  2004/09/20 10:42:40  frehe
 * Improved score.
 *
 * Revision 1.1  2004/08/31 23:04:03  frehe
 * Added an example attacker.
 *
 */

/**
 * @file Score.h
 * Include file for the skill class Score.
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2005/09/08 13:42:58 $
 */


#ifndef SCORE_H
#define SCORE_H


#include "Skill.h"
#include "WorldModelInterface.h"


RS_BEGIN_NAMESPACE


class BallView;
class AgentView;
class PlayerView;


/**
 * @class Score Score.h
 * This is an example score skill.
 *
 * @pre The ball is kickable
 * @pre The relative vector to the ball is known
 * @pre The agent position and body direction are known
 * @post The ball is moving fast towards the largest free sector to the goal
 *
 * @par Unit:
 *      Framework/example/attacker
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2005/09/08 13:42:58 $
 */
class Score : public Skill {
public:
  /** Construct a score skill. */
  Score();

  /** Destructor should clean up after the skill. */
  virtual ~Score();


  /** Reset any internal state in the skill. */
  virtual void reset();
  
  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent. You can assume
      that the preconditions of the skill are true (this can be tested
      by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal of the skill. */
  virtual AgentStep generateStep();


  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
      the description of the skill. */
  virtual FuzzyBool applicable();


  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();


private:
  /** Hide copy-constructor. */
  Score(const Score&);
  /** Hide assignment operator. */
  Score& operator= (const Score&);

  const WorldModelInterface* const worldModelInterface;
  const WorldFacts* facts;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


#endif // !SCORE_H
