/* -*- Mode: C++ -*-
 * $Id: DefenderDecision.h,v 1.4 2005/09/19 16:58:21 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DefenderDecision.h
 * Unitname: Framework/test/Team
 * $Revision: 1.4 $
 * Created by: Peter Andersson 2003-08-26
 * Last modified by $Author: frehe $ $Date: 2005/09/19 16:58:21 $
 *
 *
 * HISTORY
 *
 * $Log: DefenderDecision.h,v $
 * Revision 1.4  2005/09/19 16:58:21  frehe
 * Fixed start of scenario.
 *
 * Revision 1.3  2004/09/13 14:56:47  frehe
 * Wait until seen the ball before acting.
 *
 * Revision 1.2  2004/09/02 09:33:44  petan
 * Fixed defender mode 3-6
 *
 * Revision 1.1  2004/09/01 08:18:02  petan
 * Added defender
 *
 */

/**
 * @file DefenderDecision.h
 * Include file for the class DefenderDecision.
 *
 * @par Unit:
 *      Framework/test/Team
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2005/09/19 16:58:21 $
 */


#ifndef DEFENDERDECISION_H
#define DEFENDERDECISION_H


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "WorldModelInterface.h"
#include "SearchBall.h"
#include "TurnBodyToPoint.h"
#include "TurnBackToPoint.h"
#include "TurnNeckToBall.h"
#include "AlignNeckToBody.h"
#include "DashToPoint.h"

RS_BEGIN_NAMESPACE

class ViewManager;
class GameView;
class BallView;
class AgentView;
class PlayerView;

#define NO_DEFENDER_MODE 0
#define DEFENDER_MODE_1 1
#define DEFENDER_MODE_2 2
#define DEFENDER_MODE_3 3
#define DEFENDER_MODE_4 4
#define DEFENDER_MODE_5 5
#define DEFENDER_MODE_6 6

#define STATE_NORMAL 0
#define KEEP_TRACK_ON_BALL 1
#define KEEP_TRACK_ON_ATTACKER 2

/**
 * @class DefenderDecision DefenderDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2005/09/19 16:58:21 $
 */
class DefenderDecision : public Decision {
public:
  DefenderDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~DefenderDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  virtual bool decide();

  // Event handlers
  //virtual void onActuatorSensorData();
  virtual void onAuralCoachSensorData();
  virtual void onAuralRefereeSensorData();
  virtual void onAuralPlayerSensorData();
  virtual void onAuralSelfSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  //virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  //virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  //virtual void onNewCycle(const bool forced);
  //virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  AgentStep generateStep();
  Point preferredPosition();
  bool inZone(Point point);
  Point getAgentZonePosition();
  void setAgentZonePosition(Point point);
  void setZoneLeash(Float length);
  void setZoneRectangle(Point upperLeft, Float width, Float height);

  ViewManager* const viewManager;
  const WorldModelInterface* const worldModelInterface;
  const WorldFacts *facts;
  GameView* mGameView;
  BallView* mBallView;
  AgentView* mAgentView;
  PlayerView* mPlayerView;

  bool mRunning;
  bool mNewScenario;
  Time mNewScenarioStart;

  int mMode;
  int mState;
  bool mZoneLeash;
  Float mZoneLeashLength;
  Float mZoneRectangleX;
  Float mZoneRectangleY;
  Float mZoneRectangleX2;
  Float mZoneRectangleY2;
  Point mAgentZonePosition;

  SearchBall mSearchBall;  
  AlignNeckToBody mAlignNeckToBody;
  TurnBodyToPoint mTurnBodyToPoint;
  TurnBackToPoint mTurnBackToPoint;
  TurnNeckToBall mTurnNeckToBall;
  DashToPoint mDashToPoint;
};


RS_END_NAMESPACE


#endif // !TEAMDECISION_H
