/* -*- Mode: C++ -*-
 * $Id: GoalieDecision.h,v 1.5 2005/09/19 16:58:25 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GoalieDecision.h
 * Unitname: Framework/test/Team
 * $Revision: 1.5 $
 * Created by: Peter Andersson 2003-08-26
 * Last modified by $Author: frehe $ $Date: 2005/09/19 16:58:25 $
 *
 *
 * HISTORY
 *
 * $Log: GoalieDecision.h,v $
 * Revision 1.5  2005/09/19 16:58:25  frehe
 * Fixed start of scenario.
 *
 * Revision 1.4  2004/09/13 14:56:40  frehe
 * Wait until seen the ball before acting.
 *
 * Revision 1.3  2004/09/01 13:13:26  petan
 * Fixed teleportation and goalie modes
 *
 * Revision 1.2  2004/09/01 08:14:03  petan
 * Added first 2 modes of training
 *
 * Revision 1.1  2004/08/18 12:05:36  petan
 * Added example goalie
 *
 *
 */

/**
 * @file GoalieDecision.h
 * Include file for the class GoalieDecision.
 *
 * @par Unit:
 *      Framework/test/Team
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/19 16:58:25 $
 */


#ifndef GOALIEDECISION_H
#define GOALIEDECISION_H


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "WorldModelInterface.h"
#include "SearchBall.h"
#include "CatchBall.h"
#include "TurnBodyToPoint.h"
#include "TurnBackToPoint.h"
#include "TurnNeckToBall.h"
#include "AlignNeckToBody.h"
#include "DashToPoint.h"

RS_BEGIN_NAMESPACE

class ViewManager;
class GameView;
class BallView;
class AgentView;
class PlayerView;

#define NO_GOALIE_MODE 0
#define GOALIE_MODE_1 1
#define GOALIE_MODE_2 2
#define GOALIE_MODE_3 3
#define GOALIE_MODE_4 4
#define GOALIE_MODE_5 5

#define STATE_NORMAL 0
#define STATE_TELEPORTED 1

/**
 * @class GoalieDecision GoalieDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/19 16:58:25 $
 */
class GoalieDecision : public Decision {
public:
  GoalieDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~GoalieDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  virtual bool decide();

  // Event handlers
  //virtual void onActuatorSensorData();
  virtual void onAuralCoachSensorData();
  virtual void onAuralRefereeSensorData();
  virtual void onAuralPlayerSensorData();
  virtual void onAuralSelfSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  //virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  //virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  //virtual void onNewCycle(const bool forced);
  //virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  AgentStep generateStep();
  PointUE predictBall();
  Point preferredPosition();

  ViewManager* const viewManager;
  const WorldModelInterface* const worldModelInterface;
  const WorldFacts *facts;
  GameView* mGameView;
  BallView* mBallView;
  AgentView* mAgentView;
  PlayerView* mPlayerView;

  bool mRunning;
  bool mNewScenario;
  Time mNewScenarioStart;

  int mState;
  int mMode;
  CatchBall mCatchBall;
  SearchBall mSearchBall;  
  AlignNeckToBody mAlignNeckToBody;
  TurnBodyToPoint mTurnBodyToPoint;
  TurnBackToPoint mTurnBackToPoint;
  TurnNeckToBall mTurnNeckToBall;
  DashToPoint mDashToPoint;
};


RS_END_NAMESPACE


#endif // !TEAMDECISION_H
