/* -*- Mode: C++ -*-
 * $Id: AmIClosestToBall.cc,v 1.1 2003/09/22 18:00:31 aip3 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AmIClosestToBall.cc
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: aip3 2000-09-22
 * Last modified by $Author: aip3 $ $Date: 2003/09/22 18:00:31 $
 *
 *
 * HISTORY
 *
 * $Log: AmIClosestToBall.cc,v $
 * Revision 1.1  2003/09/22 18:00:31  aip3
 * Adding predicate AmIClosestToBall.
 * It tests if the agent is closest to the ball.
 * The test can be performed against all other visible players or just opponents.
 *
 */

/**
 * @file AmIClosestToBall.cc
 * Implementation file for the predicate class AmIClosestToBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: aip3 $
 * $Date: 2003/09/22 18:00:31 $
 */


#include "AmIClosestToBall.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


AmIClosestToBall::AmIClosestToBall(bool includeTeammates)
  : Predicate(), agentView(NULL), ballView(NULL), playerView(NULL), includeTeammates(includeTeammates)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}


AmIClosestToBall::AmIClosestToBall(const AmIClosestToBall& obj)
  : Predicate(obj), agentView(NULL), ballView(NULL), playerView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  includeTeammates = obj.includeTeammates;
}


AmIClosestToBall::~AmIClosestToBall()
{
  delete ballView;
  delete agentView;
  delete playerView;
}


AmIClosestToBall& AmIClosestToBall::operator= (const AmIClosestToBall& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }
  ballView = obj.ballView;
  agentView = obj.agentView;
  playerView = obj.playerView;
  includeTeammates = obj.includeTeammates;

  return *this;
}


FuzzyBool AmIClosestToBall::evaluate()
{
  PointUE myPos, otherPos, ballPos;
  myPos = agentView->getPosition();
  ballPos = ballView->getPosition();
  if (myPos.isUnknown()){
    return FuzzyBool(0.5);
  }
  if (ballPos.isUnknown()){
    return FuzzyBool(0.5);
  }
  //  std::cerr << "Ballpos: " << ballPos << std::endl;
  PlayerObject other = playerView->getOpponentClosestToBall();
  if (includeTeammates){
    PlayerObject tm = playerView->getTeammateClosestToBall();
    PointUE tmPos = tm.getPosition();
    if (tmPos.isKnown())
      if (VectorUE(tmPos, ballPos) < VectorUE(myPos, ballPos))
        return FuzzyBool(0.0);        // Teammate nrmare
  }
  otherPos = other.getPosition();
  if (otherPos.isUnknown())
    return FuzzyBool(1.0);         //Vet inget om motstndare
  if (VectorUE(myPos, ballPos) < 
      VectorUE(otherPos, ballPos)) 
    return FuzzyBool(1.0);
  return FuzzyBool(0.0);
}


void AmIClosestToBall::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "AmIClosestToBall.icc"
#endif
