/* -*- Mode: C++ -*-
 * $Id: AmIClosestToBall.h,v 1.3 2003/10/10 10:39:08 aip3 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AmIClosestToBall.h
 * Unitname: Framework/predicate
 * $Revision: 1.3 $
 * Created by: aip3 2003-09-22
 * Last modified by $Author: aip3 $ $Date: 2003/10/10 10:39:08 $
 *
 *
 * HISTORY
 *
 * $Log: AmIClosestToBall.h,v $
 * Revision 1.3  2003/10/10 10:39:08  aip3
 * Commenting predicates and specifying development plans.
 * All of our debugging prints should now be turned off.
 *
 * Revision 1.2  2003/09/22 18:09:19  aip3
 * Corrected the ifndef.
 *
 * Revision 1.1  2003/09/22 18:00:30  aip3
 * Adding predicate AmIClosestToBall.
 * It tests if the agent is closest to the ball.
 * The test can be performed against all other visible players or just opponents.
 *
 */

/**
 * @file AmIClosestToBall.h
 * Include file for the predicate class AmIClosestToBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: aip3 $
 * $Date: 2003/10/10 10:39:08 $
 */


#ifndef AMICLOSESTTOBALL_H
#define AMICLOSESTTOBALL_H

#include "Predicate.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"

RS_BEGIN_NAMESPACE

/**
 * @class AmIClosestToBall AmIClosestToBall.h
 * True if the agent believes it is closest to the ball.
 * The option to include/exclude teammates exists.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: aip3 $
 * $Date: 2003/10/10 10:39:08 $
 */
class AmIClosestToBall : public Predicate {
public:
  /** Create a predicate with default behavior. */
  AmIClosestToBall(bool includeTeammates=true);

  /** Creates a predicate that is an exact copy of @a obj. */
  AmIClosestToBall(const AmIClosestToBall& obj);

  /** Destroys the predicate. */  
  virtual ~AmIClosestToBall();


  /** Assignment operator. Assigns @a obj to this predicate. */
  AmIClosestToBall& operator= (const AmIClosestToBall& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for.

    Not used.
*/
  virtual void update(const UInt32 update_number);
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
  bool includeTeammates;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AmIClosestToBall.icc"
#endif


#endif // !PREDICATESTUB_H
