/* -*- Mode: C++ -*-
 * $Id: AmIFirstToBall.cc,v 1.5 2003/11/07 15:02:28 aip3 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AmIFirstToBall.cc
 * Unitname: Framework/predicate
 * $Revision: 1.5 $
 * Created by: aip3 2000-09-18
 * Last modified by $Author: aip3 $ $Date: 2003/11/07 15:02:28 $
 *
 *
 * HISTORY
 *
 * $Log: AmIFirstToBall.cc,v $
 * Revision 1.5  2003/11/07 15:02:28  aip3
 * Now calculates time to ball according to a new formula (stated in H-file).
 * Takes a second optional argument to specify the assumed speed of the other players.
 *
 * Revision 1.4  2003/10/10 10:39:08  aip3
 * Commenting predicates and specifying development plans.
 * All of our debugging prints should now be turned off.
 *
 * Revision 1.3  2003/10/01 22:59:45  aip3
 * Nu utgr predikatet frn att alla andra spelarna kommer att fortstta i riktningskomposanten mot bollen med nuvarande hastighet i den riktiningen. S ska det vara. Dremot rknar predikatet med att ens egen hastighet r konstant som den r NOW, vilket ska ndras.
 *
 * Revision 1.2  2003/09/22 18:21:44  aip3
 * Now this works just like AmIClosestToBall.
 * This will of course be changed later.
 *
 * Revision 1.1  2003/09/18 13:18:04  aip3
 * Added the AmIFirstToBall predicate which tests if the agent is able to reach the ball before anyone else can.
 *
 */

/**
 * @file AmIFirstToBall.cc
 * Implementation file for the predicate class AmIFirstToBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.5 $
 * $Author: aip3 $
 * $Date: 2003/11/07 15:02:28 $
 */


#include "AmIFirstToBall.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


AmIFirstToBall::AmIFirstToBall(bool includeTeammates, float assumedSpeed)
  : Predicate(), agentView(NULL), ballView(NULL), playerView(NULL), includeTeammates(includeTeammates),assumedSpeed(assumedSpeed)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}


AmIFirstToBall::AmIFirstToBall(const AmIFirstToBall& obj)
  : Predicate(obj), agentView(NULL), ballView(NULL), playerView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  includeTeammates = obj.includeTeammates;
  assumedSpeed = obj.assumedSpeed;
}


AmIFirstToBall::~AmIFirstToBall()
{
  delete ballView;
  delete agentView;
  delete playerView;
}


AmIFirstToBall& AmIFirstToBall::operator= (const AmIFirstToBall& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }
  ballView = obj.ballView;
  agentView = obj.agentView;
  playerView = obj.playerView;
  includeTeammates = obj.includeTeammates;
  assumedSpeed = obj.assumedSpeed;

  return *this;
}


FuzzyBool AmIFirstToBall::evaluate()
{
  PointUE myPos, ballPos;
  VectorUE ballSpeed;
  float staminaEffect = 1; //Should be lower if tired.
  float myTimeToBall;
  myPos = agentView->getPosition();
  ballPos = ballView->getPosition();
  ballSpeed = ballView->getSpeedVector();
  if (myPos.isUnknown()){
    return FuzzyBool(0.5);
  }
  if (ballPos.isUnknown()){
    return FuzzyBool(0.5);
  }
  if (ballSpeed.isUnknown()){  //Assume ball is still if its speed is unknown
    ballSpeed = VectorUE(0,0);
  }
  std::vector<PlayerObject> others = playerView->getOpponents();
  if (includeTeammates){
    for (std::vector<PlayerObject>::const_iterator i = playerView->getTeammates().begin();
         i != playerView->getTeammates().end(); i++)
      others.push_back(*i);
    for (std::vector<PlayerObject>::const_iterator i = playerView->getUnknownPlayers().begin();
         i != playerView->getUnknownPlayers().end(); i++)
      others.push_back(*i);
  }

  if (others.empty()) {
    //    std::cerr << "first: I'm alone" <<std::endl;
    return FuzzyBool(1.0);
  }
  
  Vector ballToPlayer(ballPos.getPoint(),myPos.getPoint());
  float relativeBallSpeed = ballToPlayer.dotProduct(ballSpeed.getVector());
  /*
    std::cerr << "%Min position: " << myPos.getPoint() << std::endl;
    std::cerr << "%Min hastigetsvektor: " << mySpeed.getVector() << std::endl;
    std::cerr << "%Bollens position: " << ballPos.getPoint() << std::endl;
    std::cerr << "%Min vektor till bollen: " << toBall << std::endl;
    std::cerr << "%Min hastighet till bollen: " << speedToBall << std::endl;
  */
  myTimeToBall = ballToPlayer.getLength() / 
    ((assumedSpeed + relativeBallSpeed)*staminaEffect);
  if(myTimeToBall<0)
    return FuzzyBool(0.0); //Hinner inte ikapp bollen
  
  std::cerr << "first: MyTime: " << myTimeToBall << std::endl;
  // --
  
  for (std::vector<PlayerObject>::const_iterator o = others.begin();
       o != others.end(); o++){
    
    PointUE oPos = o->getPosition();

    //    std::cerr << "%Brjar kolla motstndare" << std::endl;

    if (oPos.isKnown()){
      Vector ballToOpponent(ballPos.getPoint(),oPos.getPoint());
      float relativeBallSpeed = ballToOpponent.dotProduct(ballSpeed.getVector());
      float hisTimeToBall = ballToOpponent.getLength() / 
        (assumedSpeed + relativeBallSpeed);
      //      std::cerr << "first: HisTime: " << hisTimeToBall << std::endl;
      if(hisTimeToBall > 0 && hisTimeToBall < myTimeToBall)
        return FuzzyBool(0.0);
    }
  }
  //  std::cerr << "%Undersker mig sjlv" << std::endl;
  
  return FuzzyBool(1.0); //Did't find anyone closer.
}


void AmIFirstToBall::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "AmIFirstToBall.icc"
#endif
