/* -*- Mode: C++ -*-
 * $Id: AmIFirstToBall.h,v 1.5 2003/11/07 15:02:28 aip3 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AmIFirstToBall.h
 * Unitname: Framework/predicate
 * $Revision: 1.5 $
 * Created by: aip3 2003-09-18
 * Last modified by $Author: aip3 $ $Date: 2003/11/07 15:02:28 $
 *
 *
 * HISTORY
 *
 * $Log: AmIFirstToBall.h,v $
 * Revision 1.5  2003/11/07 15:02:28  aip3
 * Now calculates time to ball according to a new formula (stated in H-file).
 * Takes a second optional argument to specify the assumed speed of the other players.
 *
 * Revision 1.4  2003/10/10 10:39:08  aip3
 * Commenting predicates and specifying development plans.
 * All of our debugging prints should now be turned off.
 *
 * Revision 1.3  2003/10/01 22:59:46  aip3
 * Nu utgr predikatet frn att alla andra spelarna kommer att fortstta i riktningskomposanten mot bollen med nuvarande hastighet i den riktiningen. S ska det vara. Dremot rknar predikatet med att ens egen hastighet r konstant som den r NOW, vilket ska ndras.
 *
 * Revision 1.2  2003/09/22 18:21:44  aip3
 * Now this works just like AmIClosestToBall.
 * This will of course be changed later.
 *
 * Revision 1.1  2003/09/18 13:18:04  aip3
 * Added the AmIFirstToBall predicate which tests if the agent is able to reach the ball before anyone else can.
 *
 */

/**
 * @file AmIFirstToBall.h
 * Include file for the predicate class AmIFirstToBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.5 $
 * $Author: aip3 $
 * $Date: 2003/11/07 15:02:28 $
 */


#ifndef AMIFIRSTTOBALL_H
#define AMIFIRSTTOBALL_H

#include "Predicate.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"

RS_BEGIN_NAMESPACE

/**
 * @class AmIFirstToBall AmIFirstToBall.h
 * Tests if the agent will be the first player to reach the ball.
 * Including opponents only or all visible players in the test is optional.
 * For now, it assumes that all players will keep their speed
 * towards the ball.
 * (The projection of the speed vector on the line towards the ball)
 * It doesn't yet take the ball speed into account.
 *
 * We will implement another model that doesn't rely that much on the
 * magnitude of speed vector later.
 *
 * It will calculate the time towards the ball with the function
 * t = (distance to ball)/
 *     (K1 + Project(SpeedVector(Ball) on Vector(Ball to Player)) * S)
 * where K1 is a speed that all players are assumed to have
 * and S is a function that is 1 for all players but the Agent, for which
 * it is dependent on the current stamina level and distance to the ball.
 *     
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.5 $
 * $Author: aip3 $
 * $Date: 2003/11/07 15:02:28 $
 */
class AmIFirstToBall : public Predicate {
public:
  /** Create a predicate with default behavior. */
  AmIFirstToBall(bool includeTeammates=true, float assumeSpeed=5);

  /** Creates a predicate that is an exact copy of @a obj. */
  AmIFirstToBall(const AmIFirstToBall& obj);

  /** Destroys the predicate. */  
  virtual ~AmIFirstToBall();


  /** Assignment operator. Assigns @a obj to this predicate. */
  AmIFirstToBall& operator= (const AmIFirstToBall& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for.
    
    Not used.
*/
  virtual void update(const UInt32 update_number);
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
  bool includeTeammates;
  float assumedSpeed;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AmIFirstToBall.icc"
#endif


#endif // !PREDICATESTUB_H




