/* -*- Mode: C++ -*-
 * $Id: IsCatchAllowed.cc,v 1.4 2003/10/10 10:39:09 aip3 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsCatchAllowed.cc
 * Unitname: Framework/predicate
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: aip3 $ $Date: 2003/10/10 10:39:09 $
 *
 *
 * HISTORY
 *
 * $Log: IsCatchAllowed.cc,v $
 * Revision 1.4  2003/10/10 10:39:09  aip3
 * Commenting predicates and specifying development plans.
 * All of our debugging prints should now be turned off.
 *
 * Revision 1.3  2003/09/22 18:21:07  aip3
 * Removed redundant includes.
 *
 * Revision 1.2  2003/09/22 15:03:44  aip3
 * Lagt till koll att spelaren inte r utanfr planen.
 *
 * Revision 1.1  2003/09/22 10:19:34  aip3
 * Added predicate that tests if catching the ball is allowed for the goalie.
 *
 * Revision 1.3  2002/09/13 14:48:56  s02davro
 * FALSE -> FuzzyBool(0.0), TRUE -> FuzzyBool(1.0)
 *
 * Revision 1.2  2000/09/17 14:31:38  frehe
 * Made minor updates to the IsCatchAllowed.
 *
 * Revision 1.1  2000/09/13 00:06:00  frehe
 * Added a predicate teamplate called IsCatchAllowed.
 *
 */

/**
 * @file IsCatchAllowed.cc
 * Implementation file for the predicate class IsCatchAllowed.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.4 $
 * $Author: aip3 $
 * $Date: 2003/10/10 10:39:09 $
 */


#include "IsCatchAllowed.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


IsCatchAllowed::IsCatchAllowed()
  : Predicate(), ballView(NULL), agentView(NULL),
    gameView(NULL), facts(WorldFacts::instance())
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
}


IsCatchAllowed::IsCatchAllowed(const IsCatchAllowed& obj)
  : Predicate(obj)
{
}


IsCatchAllowed::~IsCatchAllowed()
{
  delete ballView;
  delete agentView;
  delete gameView;
  delete facts;
}


IsCatchAllowed& IsCatchAllowed::operator= (const IsCatchAllowed& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }

  return *this;
}


FuzzyBool IsCatchAllowed::evaluate()
{
  if(!agentView->isGoalie() || !gameView->isKickAllowed() || agentView->isAgentOutsideField())
    return FuzzyBool(0.0);
  PointUE agentPos = agentView->getPosition();
  if (agentPos.isUnknown()) return FuzzyBool(0.5);
  float x = agentPos.getPoint().getX();
  //  std::cerr << " x:" << x << "/" << facts->getOurGoal().getPosX()+facts->SP_penalty_area_length << "\n";
  float y = agentPos.getPoint().getY();
  if ((x > facts->getOurGoal().getPosX()+facts->SP_penalty_area_length) ||
      (abs(y) > facts->getOurGoal().getPosY()+facts->SP_penalty_area_width/2))
    return FuzzyBool(0.0);
  return FuzzyBool(1.0);
}


void IsCatchAllowed::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "IsCatchAllowed.icc"
#endif
