/* -*- Mode: C++ -*-
 * $Id: IsCatchAllowed.h,v 1.2 2003/10/10 10:39:09 aip3 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsCatchAllowed.h
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: aip3 $ $Date: 2003/10/10 10:39:09 $
 *
 *
 * HISTORY
 *
 * $Log: IsCatchAllowed.h,v $
 * Revision 1.2  2003/10/10 10:39:09  aip3
 * Commenting predicates and specifying development plans.
 * All of our debugging prints should now be turned off.
 *
 * Revision 1.1  2003/09/22 10:19:35  aip3
 * Added predicate that tests if catching the ball is allowed for the goalie.
 *
 * Revision 1.3  2000/09/17 18:03:18  frehe
 * Fixed an error in the IsCatchAllowed.
 *
 * Revision 1.2  2000/09/17 14:31:38  frehe
 * Made minor updates to the IsCatchAllowed.
 *
 * Revision 1.1  2000/09/13 00:06:00  frehe
 * Added a predicate teamplate called IsCatchAllowed.
 *
 */

/**
 * @file IsCatchAllowed.h
 * Include file for the predicate class IsCatchAllowed.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: aip3 $
 * $Date: 2003/10/10 10:39:09 $
 */


#ifndef ISCATCHALLOWED_H
#define ISCATCHALLOWED_H


#include "Predicate.h"
#include "BallView.h"
#include "GameView.h"
#include "AgentView.h"
#include "WorldFacts.h"

RS_BEGIN_NAMESPACE


/**
 * @class IsCatchAllowed IsCatchAllowed.h
 * True if the agent is allowed to catch the ball with his hands.
 * This requires that the agent is the goalie and that he is inside
 * the penalty area. Even the playmode is investigated.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: aip3 $
 * $Date: 2003/10/10 10:39:09 $
 */
class IsCatchAllowed : public Predicate {
public:
  /** Create a predicate with default behavior. */
  IsCatchAllowed();

  /** Creates a predicate that is an exact copy of @a obj. */
  IsCatchAllowed(const IsCatchAllowed& obj);

  /** Destroys the predicate. */  
  virtual ~IsCatchAllowed();


  /** Assignment operator. Assigns @a obj to this predicate. */
  IsCatchAllowed& operator= (const IsCatchAllowed& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for.

    Not used
*/
  virtual void update(const UInt32 update_number);

  BallView* ballView;
  AgentView* agentView;
  GameView* gameView;
  const WorldFacts* facts;

};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "IsCatchAllowed.icc"
#endif


#endif // !PREDICATESTUB_H
