/* -*- Mode: C++ -*-
 * $Id: PassKick.cc,v 1.2 2004/01/15 18:44:01 aip12 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassKick.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: aip12 2003-12-12
 * Last modified by $Author: aip12 $ $Date: 2004/01/15 18:44:01 $
 *
 *
 * HISTORY
 *
 * $Log: PassKick.cc,v $
 * Revision 1.2  2004/01/15 18:44:01  aip12
 * fixed the PassKick predicate
 *
 * Revision 1.1  2004/01/15 13:48:14  aip12
 * Added the PassKick predicate
 *
 * Revision 1.1  2002/12/12 00:06:00  aip12
 * Added a predicate teamplate called PassKick.
 *
 */

/**
 * @file PassKick.cc
 * Implementation file for the predicate class PassKick.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: aip12 $
 * $Date: 2004/01/15 18:44:01 $
 */


#include "PassKick.h"
#include "view_constants.h"

#define SIG(x) 1.0/(1.0 + exp(-x))

RS_BEGIN_NAMESPACE


PassKick::PassKick()
  : Predicate(), agentView(NULL), ballView(NULL), playerView(NULL), max(0)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  
  std::ifstream state("state.net");
  char buf[256];
  state.getline(buf,100);
  max = atoi(buf);
  v1= new double*[(max+1)/2+1];
  for(int i=0; i<(max+1)/2+1; i++) {
    v1[i]= new double[max+1];
  }
  v2= new double[(max+1)/2+1];
  w1= new double*[(max+1)/2+1];
  for(int i=0; i<(max+1)/2+1; i++) {
    w1[i]= new double[max+1];
  }
  w2= new double[(max+1)/2+1];
  for(int i=0; i<(max+1)/2+1; i++){
    for(int j=0; j<max+1; j++){
      //state.getline(buf,100);
      //sscanf(buf, "%f %f", &v1[i][j], &w1[i][j]);
      state >> v1[i][j];
      state >> w1[i][j];
    }
    //sscanf(buf, "%f %f", &v2[i], &w2[i]);
    state >> v2[i];
    state >> w2[i];
  }
  state.close();
}


PassKick::PassKick(const PassKick& obj)
  : Predicate(obj), agentView(NULL), ballView(NULL), playerView(NULL), max(0)
{
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  
}

PassKick::~PassKick()
{
  delete ballView;
  delete agentView;
  delete playerView;
}


PassKick& PassKick::operator= (const PassKick& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }
  ballView = obj.ballView;
  agentView = obj.agentView;
  playerView = obj.playerView;
  max=obj.max;
  
  v1=obj.v1;
  v2=obj.v2;
  w1=obj.w1;
  w2=obj.w2;
  
  return *this;
}


FuzzyBool PassKick::evaluate()
{
  //just to let it work for now
  double i[3];
  PlayerObject po = playerView->getOpponentClosestToAgent();
  i[0]=po.getPosX().getValue();
  i[1]=po.getPosY().getValue();
  i[2]=1.0f;
  
  double p[(max+1)/2+1];
  //initializition
  for(int j=0; j<(max+1)/2+1;p[j++]=0.0);
  double pp = 0.0;
  for(int j=0; j<(max+1)/2+1; j++) {
    for(int k=0; k<max+1; k++) {
      p[j]+=w1[j][k]*i[k];
    }
    p[j]=SIG(p[j]);
    pp+=p[j]*w2[j];
  }
  pp=SIG(pp);
  //srand(time(NULL));
  double v = (double)((rand())%2001-1000)/(double)2000;
  //std::cout << v << " " << pp << "\n";
  int a=((pp + v)>=0.5)?1:0;
  
  // Euristic approach: 
  //   if the predicted reinforcement is too small maybe it's time to change the action
  
  for(int j=0; j<(max+1)/2+1; p[j++]=0.0);
  for(int j=0; j<(max+1)/2+1; j++) {
    for(int k=0; k<max+1; k++) {
      p[j]+=v1[j][k]*i[k];
    }
    p[j]=SIG(p[j]);
    pp+=p[j]*v2[j];
  }
  pp=SIG(pp);
  if(pp<0.3)
    pp=(a==0)?1-pp:pp;
  return FuzzyBool(pp);
}


void PassKick::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PassKick.icc"
#endif
