/* -*- Mode: C++ -*-
 * $Id: PassKick.h,v 1.2 2004/01/15 18:44:01 aip12 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassKick.h
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Aip 12 2003-12-12
 * Last modified by $Author: aip12 $ $Date: 2004/01/15 18:44:01 $
 *
 *
 * HISTORY
 *
 * $Log: PassKick.h,v $
 * Revision 1.2  2004/01/15 18:44:01  aip12
 * fixed the PassKick predicate
 *
 * Revision 1.1  2004/01/15 13:48:15  aip12
 * Added the PassKick predicate
 *
 * Revision 1.1  2003/12/12 18:03:18  aip12
 * Created PassKick.
 *
 */

/**
 * @file PassKick.h
 * Include file for the predicate class PassKick.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: aip12 $
 * $Date: 2004/01/15 18:44:01 $
 */


#ifndef PASSKICK_H
#define PASSKICK_H


#include "Predicate.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"
#include <fstream>
#include <iostream>


RS_BEGIN_NAMESPACE


/**
 * @class PassKick PassKick.h
 * This predicate tells you when to shoot or to pass.
 * 
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: aip12 $
 * $Date: 2004/01/15 18:44:01 $
 */
class PassKick : public Predicate {
public:
  /** Create a predicate with default behavior. */
  PassKick();

  /** Creates a predicate that is an exact copy of @a obj. */
  PassKick(const PassKick& obj);

  /** Destroys the predicate. */  
  virtual ~PassKick();


  /** Assignment operator. Assigns @a obj to this predicate. */
  PassKick& operator= (const PassKick& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
  
  double **v1, *v2;
  double **w1, *w2;
  int max;
  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PassKick.icc"
#endif


#endif // !PASSKICK_H
