/* -*- Mode: C++ -*-
 * $Id: AccelerateBallToVelocity.h,v 1.3 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AccelerateBallToVelocity.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: AccelerateBallToVelocity.h,v $
 * Revision 1.3  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.2  2004/08/13 15:39:27  petan
 * Moved implementation to generateStep
 *
 * Revision 1.1  2003/09/03 14:08:43  aip19
 * Updated and documented primitive skills
 *
 *
 */


/**
 * @file AccelerateBallToVelocity.h
 * Include file for the skill AccelerateBallToVelocity.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */

#ifndef ACCELERATEBALLTOVELOCITY_H
#define ACCELERATEBALLTOVELOCITY_H

#include "SkillWithVector.h"
#include "AgentView.h"
#include "BallView.h"
#include "KickBallView.h"

RS_BEGIN_NAMESPACE


/**
 * @class AccelerateBallToVelocity AccelerateBallToVelocity.h
 * This skill accelerates the ball to the specified vector
 * If the agent cannot accelerate the ball to the specified velocity,
 * it tries to accelerate it as much as possible in the correct direction.
 *
 * @pre The agents body direction is known, the balls speed vector is known and
 *      the ball is kickable.
 * @post The ball is accelerated to the specified vector.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */
class AccelerateBallToVelocity : public SkillWithVector {
public:
  /** Default constructor. Here you can initialize the skill. */
  AccelerateBallToVelocity(const Vector& vec);

  /** Destructor should clean up after the skill. */
  virtual ~AccelerateBallToVelocity();


  /** Reset any internal state in the skill. */
  virtual void reset();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

  
private:
  /** Hide copy-constructor. */
  AccelerateBallToVelocity(const AccelerateBallToVelocity&);
  /** Hide assignment operator. */
  AccelerateBallToVelocity& operator= (const AccelerateBallToVelocity&);

  AgentView* mAgentView;
  BallView* mBallView;
  KickBallView* mKickBallView;
};


RS_END_NAMESPACE

#endif // !ACCELERATEBALLTOVELOCITY_H
