/* -*- Mode: C++ -*-
 * $Id: AlignNeckToBody.cc,v 1.6 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AlignNeckToBody.cc
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: AlignNeckToBody.cc,v $
 * Revision 1.6  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.5  2004/08/13 15:39:27  petan
 * Moved implementation to generateStep
 *
 * Revision 1.4  2003/09/03 14:08:43  aip19
 * Updated and documented primitive skills
 *
 * Revision 1.3  2003/09/02 17:59:33  frehe
 * Updated to the new command pointer semantics. RoboSoc now takes control of the pointer when sent to the actuator interface.
 *
 * Revision 1.2  2003/09/02 12:52:42  aip19
 * New primitive skills checked in, not tested.
 *
 * Revision 1.1  2003/08/29 11:01:05  petan
 * Added first command AlignNeckToBody
 *
 *
 */

/**
 * @file AlignNeckToBody.cc
 * Implementation file for the skill class AlignNeckToBody.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */


#include "AlignNeckToBody.h"
#include "view_constants.h"
#include "TurnNeckCommand.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

AlignNeckToBody::AlignNeckToBody()
  : Skill(),
    mAgentView(NULL)
{
  getView(mAgentView, AGENT_VIEW_ID);
}


// --- DESTRUCTOR ---

AlignNeckToBody::~AlignNeckToBody()
{
  viewManager->releaseView(AGENT_VIEW_ID);
}


// --- METHODS ---

void AlignNeckToBody::reset()
{
}

Plan AlignNeckToBody::generatePlan()
{
  return Plan(generateStep());
}

AgentStep AlignNeckToBody::generateStep()
{
  // Align neck to body
  TurnNeckCommand *command = new TurnNeckCommand((mAgentView->getBodyDirection() - 
		                 mAgentView->getNeckDirection()).getDeg());
  AgentStep step;
  step.setNeckCommand(command);
  return step;
}


FuzzyBool AlignNeckToBody::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}

FuzzyBool AlignNeckToBody::applicable()
{
  return mAgentView->getBodyDirection().isKnown() &&
    mAgentView->getNeckDirection().isKnown();
}

FuzzyBool AlignNeckToBody::succeed()
{
  // Has not succeeded yet
  return FuzzyBool(0.0);
}

FuzzyBool AlignNeckToBody::failed()
{
  // Has not failed yet
  return FuzzyBool(0.0);
}



RS_END_NAMESPACE

