/* -*- Mode: C++ -*-
 * $Id: AlignNeckToBody.h,v 1.4 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AlignNeckToBody.h
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: AlignNeckToBody.h,v $
 * Revision 1.4  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.3  2004/08/13 15:39:27  petan
 * Moved implementation to generateStep
 *
 * Revision 1.2  2003/09/03 14:08:44  aip19
 * Updated and documented primitive skills
 *
 * Revision 1.1  2003/08/29 11:01:05  petan
 * Added first command AlignNeckToBody
 *
 *
 */



/**
 * @file AlignNeckToBody.h
 * Include file for the skill AlignNeckToBody.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */


#ifndef ALIGNNECKTOBODY_H
#define ALIGNNECKTOBODY_H


#include "Skill.h"
#include "AgentView.h"

RS_BEGIN_NAMESPACE


/**
 * @class AlignNeckToBody AlignNeckToBody.h
 * Aligns the agents head with the agents body
 * Turns the agents neck so that the view is aligned with the agents
 * body.
 *
 * @pre The neck direction and the body direction is known.
 * @post The head is aligned to the body
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */
class AlignNeckToBody : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  AlignNeckToBody();

  /** Destructor should clean up after the skill. */
  virtual ~AlignNeckToBody();

  
  /** Reset any internal state in the skill. */
  virtual void reset();

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

  
private:
  /** Hide copy-constructor. */
  AlignNeckToBody(const AlignNeckToBody&);
  /** Hide assignment operator. */
  AlignNeckToBody& operator= (const AlignNeckToBody&);

  AgentView* mAgentView;
};


RS_END_NAMESPACE

#endif // !ALIGNNECKTOBODY_H
