/* -*- Mode: C++ -*-
 * $Id: CatchBall.cc,v 1.11 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CatchBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.11 $
 * Created by: Bjrn, Ola och Jenny
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 * $Log: CatchBall.cc,v $
 * Revision 1.11  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.10  2004/09/01 13:10:14  petan
 * Fixed catch ban cycles and ball prediction
 *
 *
 */

/**
 * @file CatchBall.cc
 * This is a definition file for the skill CatchBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.11 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */

#include "CatchBall.h"
#include "CatchCommand.h"
#include "TurnCommand.h"
#include "WorldFacts.h"
#include "AgentView.h"
#include "BallView.h"
#include "GameView.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

CatchBall::CatchBall()
  : facts(WorldFacts::instance()),
    ballView(NULL),
    agentView(NULL),
    gameView(NULL),
    mLastCatch(-facts->SP_catch_ban_cycle) {
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
}

// --- DESTRUCTOR ---

CatchBall::~CatchBall() {
    viewManager->releaseView(BALL_VIEW_ID);
    viewManager->releaseView(AGENT_VIEW_ID);
    viewManager->releaseView(GAME_VIEW_ID);
}

// --- METHODS ---

void CatchBall::reset()
{
}

Plan CatchBall::generatePlan() {
  return Plan(generateStep());
}

AgentStep CatchBall::generateStep()
{
  mLastCatch = gameView->getCurrentTime().getT();
  CatchCommand *cmd = new CatchCommand(ballView->getRelativeDirection(NEXT).getDeg());  
  AgentStep step;
  step.setBodyCommand(cmd);  
  return step;
}


FuzzyBool CatchBall::persistent() {
  return FuzzyBool(0.0);
}


FuzzyBool CatchBall::applicable() {
  return (agentView->isGoalie() &&
	  ballView->isBallCatchable() && 
	  gameView->getNextTime().getT()-mLastCatch >= facts->SP_catch_ban_cycle &&
	  ballView->getRelativeDirection(NEXT).isKnown());
}


RS_END_NAMESPACE
