/* -*- Mode: C++ -*-
 * $Id: CatchBall.h,v 1.13 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CatchBall.h
 * Unitname: Framework/skill
 * $Revision: 1.13 $
 * Created by: Bjrn, Ola och Jenny 2000-09-07
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 * $Log: CatchBall.h,v $
 * Revision 1.13  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.12  2004/09/01 13:10:14  petan
 * Fixed catch ban cycles and ball prediction
 *
 *
 */

/**
 * @file CatchBall.h
 * This is the definitionfile of the skill CatchBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.13 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $ */


#ifndef CATCHBALL_H
#define CATCHBALL_H


#include "machine_dependent_compiler_directives.h"
#include "Skill.h"

RS_BEGIN_NAMESPACE

class BallView;
class AgentView;
class GameView;
class WorldFacts;

/**
 * @class CatchBall CatchBall.h
 * The skill will try to catch the ball.
 * If the ball is catchable, this skill will produce a catchcommand in
 * the appropriate angle.
 *
 * @pre The preconditions for the skill are:
 * The agent is the goalie, the agent's position, the agent's bodydirection,
 * and the ball's position are known and the ball is catchable.
 * @post The ball has been catched.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.13 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */
class CatchBall : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  CatchBall();

  /** Destructor should clean up after the skill. */
  virtual ~CatchBall();

  
  /** Reset any internal state in the skill. */
  virtual void reset();

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();
  
  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
private:
  /** Description of this attribute. */
  //  TheType attributeName;
  
  /** Hide copy-constructor. */
  CatchBall(const CatchBall&);
  /** Hide assignment operator. */
  CatchBall& operator= (const CatchBall&);

protected:
  const WorldFacts* facts;
  BallView* ballView;
  AgentView* agentView;
  GameView* gameView;

  int mLastCatch;
};


RS_END_NAMESPACE

#endif // !CATCHBALL_H
