/* -*- Mode: C++ -*-
 * $Id: DashToPoint.cc,v 1.6 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashToPoint.cc
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: DashToPoint.cc,v $
 * Revision 1.6  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.5  2004/08/13 15:39:28  petan
 * Moved implementation to generateStep
 *
 * Revision 1.4  2004/08/13 11:59:58  petan
 * Added setMaxPower functionality
 *
 * Revision 1.3  2003/09/26 14:19:39  aip3
 * Made stuff work. Major restructuring of DefendGoalLine, MoveToLineAndTurn, and MoveToPositionOnLine: now using our own code for moving and turning. TODO: Remove obsolete inheritance of MoveToPosition by MoveToLineAndTurn; Fix bug in turning code in ToLine and decide weather to move backwards facing the ball. It's Friday, let's drink beer!
 *
 * Revision 1.2  2003/09/18 13:15:23  aip3
 * Added the PositionBetweenPoints skill which positions the agent between to points supplied to the constructor or to the setPoints method.
 *
 * Revision 1.1  2003/09/03 14:08:44  aip19
 * Updated and documented primitive skills
 *
 *
 */

/**
 * @file DashToPoint.cc
 * Implementation file for the skill class DashToPoint.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */

#include "view_constants.h"
#include "DashToPoint.h"
#include "DashCommand.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

DashToPoint::DashToPoint(const Point& pos)
  : SkillWithPosition(pos),
    mAgentView(NULL)
{
  getView(mAgentView, AGENT_VIEW_ID);
  mPower = WorldModelInterface::instance()->getWorldFacts()->SP_maxpower;
}


// --- DESTRUCTOR ---

DashToPoint::~DashToPoint()
{
  viewManager->releaseView(AGENT_VIEW_ID);
}


// --- METHODS ---

void DashToPoint::reset()
{
}

void DashToPoint::setMaxPower(Float power) {
  const WorldFacts *facts = WorldModelInterface::instance()->getWorldFacts();
  mPower = minMax(facts->SP_minpower, power, facts->SP_maxpower);
}

Plan DashToPoint::generatePlan()
{
  return Plan(generateStep());
}

AgentStep DashToPoint::generateStep()
{
  FloatUE dashPower = 0;

  if (mAgentView->getPosition().isKnown()) {
    Point relativePoint = position - mAgentView->getPosition().getPoint();
    dashPower = mAgentView->getPowerForDash(relativePoint);
    dashPower = min(dashPower.getValue(), mPower);
  }

  DashCommand *command = new DashCommand(dashPower.getValue());
  AgentStep step;
  step.setBodyCommand(command);
  return step;
}


FuzzyBool DashToPoint::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool DashToPoint::applicable()
{
  // Need to know the position of the agent to be applicable
  return mAgentView->getPosition().isKnown();
}

FuzzyBool DashToPoint::succeed()
{
  // We don't know if this succeeded
  return FuzzyBool(0.0);
}

FuzzyBool DashToPoint::failed()
{
  // Never fails.
  return FuzzyBool(0.0);
}



RS_END_NAMESPACE

