/* -*- Mode: C++ -*-
 * $Id: DashToPoint.h,v 1.5 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashToPoint.h
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: DashToPoint.h,v $
 * Revision 1.5  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.4  2004/08/13 15:39:28  petan
 * Moved implementation to generateStep
 *
 * Revision 1.3  2004/08/13 11:59:54  petan
 * Added setMaxPower functionality
 *
 * Revision 1.2  2003/09/17 18:16:48  aip5
 * Fixed virtual problem
 *
 * Revision 1.1  2003/09/03 14:08:44  aip19
 * Updated and documented primitive skills
 *
 *
 */

/**
 * @file DashToPoint.h
 * Include file for the skill DashToPoint.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */


#ifndef DASHTOPOINT_H
#define DASHTOPOINT_H

#include "SkillWithPosition.h"
#include "TooFewArgumentsException.h"
#include "AgentView.h"

RS_BEGIN_NAMESPACE

/**
 * @class DashToPoint DashToPoint.h
 * This skill enables an agent to dash to a given point
 * The agent can only dash straight ahead, so if the agent
 * isn't pointed towards the point, it will try to get as
 * close as possible
 *
 * @pre The agents position is known
 * @post The agent have gotten as close as possible to the given position
 * @par Unit:
 *      Framework
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */
class DashToPoint : public SkillWithPosition {
public:
  /** Default constructor, it is ok since the class is abstract
    all subclasses should make sure to get default values for the
    arguments. */
  DashToPoint(const Point& pos);

  /** Destructor cleaning up after the skill. */
  virtual ~DashToPoint();

  
  /** Reset any internal state in the skill. */
  virtual void reset();

  /** Sets the max power the dash can use, default is SP_maxpower */
  void setMaxPower(Float power);

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. Abstract since
    no concrete instances of this class should be created. */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. Abstract since
    no concrete instances of this class should be created. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. Abstract since no concrete instances
   of this class should be created. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();
  
  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

private:
  /** Hide copy-constructor. */
  DashToPoint(const DashToPoint&);
  /** Hide assignment operator. */
  DashToPoint& operator= (const DashToPoint&);

  AgentView *mAgentView;
  CoordPoint<Float> mPoint;
  Float mPower;
};


RS_END_NAMESPACE


#endif // !SKILLWITHARGUMENTS_H
