/* -*- Mode: C++ -*-
 * $Id: FreezeBall.cc,v 1.3 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FreezeBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: FreezeBall.cc,v $
 * Revision 1.3  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.2  2004/08/13 15:39:28  petan
 * Moved implementation to generateStep
 *
 * Revision 1.1  2003/09/03 14:08:45  aip19
 * Updated and documented primitive skills
 *
 *
 */

/**
 * @file FreezeBall.cc
 * Implementation file for the skill class FreezeBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */


#include "FreezeBall.h"
#include "view_constants.h"
#include "KickCommand.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

FreezeBall::FreezeBall()
  : Skill(),
    mAgentView(NULL),
    mKickBallView(NULL),
    mBallView(NULL)
{
  getView(mAgentView, AGENT_VIEW_ID);
  getView(mKickBallView, KICKBALL_VIEW_ID);
  getView(mBallView, BALL_VIEW_ID);
}


// --- DESTRUCTOR ---

FreezeBall::~FreezeBall()
{
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(KICKBALL_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
}


// --- METHODS ---

void FreezeBall::reset()
{
}

Plan FreezeBall::generatePlan()
{
  return Plan(generateStep());
}

AgentStep FreezeBall::generateStep()
{
  Float ballSpeed = mBallView->getSpeedVector().getLength().getValue();
  FloatUE power = mKickBallView->getKickPowerForSpeed(ballSpeed);

  AngleDeg angle = mBallView->getSpeedVector().getAngle().getValue();
  angle += 180;
  angle -= mAgentView->getBodyDirection().getDeg();

  KickCommand *command = new KickCommand(power.getValue(), angle);
  AgentStep step;
  step.setBodyCommand(command);
  return step;
}

FuzzyBool FreezeBall::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}

FuzzyBool FreezeBall::applicable()
{
  return mAgentView->getBodyDirection().isKnown() &&
    mBallView->getSpeedVector().isKnown() &&
    mBallView->isBallKickable();
}

FuzzyBool FreezeBall::succeed()
{
  // Has not succeeded yet
  return FuzzyBool(0.0);
}

FuzzyBool FreezeBall::failed()
{
  // Has not failed yet
  return FuzzyBool(0.0);
}



RS_END_NAMESPACE

