/* -*- Mode: C++ -*-
 * $Id: SearchBall.cc,v 1.5 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SearchBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 * $Log: SearchBall.cc,v $
 * Revision 1.5  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.4  2004/08/13 15:39:28  petan
 * Moved implementation to generateStep
 *
 * Revision 1.3  2003/09/03 14:08:46  aip19
 * Updated and documented primitive skills
 *
 * Revision 1.2  2003/09/02 17:59:38  frehe
 * Updated to the new command pointer semantics. RoboSoc now takes control of the pointer when sent to the actuator interface.
 *
 * Revision 1.1  2003/09/02 12:52:43  aip19
 * New primitive skills checked in, not tested.
 *
 */

/**
 * @file SearchBall.cc
 * Implementation file for the skill class SearchBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */


#include "SearchBall.h"
#include "AgentView.h"
#include "BallView.h"
#include "GameView.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "TurnNeckCommand.h"

RS_BEGIN_NAMESPACE

 
SearchBall::SearchBall()
  : Skill(),
    ballView(NULL),
    gameView(NULL),
    agentView(NULL) {
  getView(ballView, BALL_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
} 

SearchBall::~SearchBall() {
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}

void
SearchBall::reset()
{
}

Plan
SearchBall::generatePlan() {
  return Plan(generateStep());
}

AgentStep
SearchBall::generateStep()
{
  TurnCommand* cmd;

  if (gameView->getNextSeeTime() == gameView->getNextTime()) {
    
    // Turn an appropriate angle from last face direction.
    Float turnAngle;
    switch (agentView->getViewWidth()) {
    case VW_NARROW: 
      turnAngle = 45.0;
      break;
    case VW_NORMAL:
      turnAngle = 90.0;
      break;
    case VW_WIDE:
      turnAngle = 180.0;
      break;
    default:
      turnAngle = 90.0;
      break;
    }
    // Take into consideration where ball was last seen
    int sign = 1;
    AngleDegUE direction = ballView->getRelativeDirection();
    if ( direction.isKnown() && direction.getDeg() < 0) {
      sign = -1;
    }

    cmd = new TurnCommand(agentView->getAngleForTurn(AngleDeg(sign*turnAngle)));
  }
  else {
    
    // No visual information next cycle, wait.
    cmd = NULL;/*new TurnCommand((agentView->getNeckDirection() - 
	    agentView->getBodyDirection()).getDeg());*/
  }

  AgentStep step;
  step.setBodyCommand(cmd);
  return step;
}


FuzzyBool SearchBall::persistent() {
  return FuzzyBool(0.0);
}

FuzzyBool SearchBall::applicable() {
  return ballView->getPosition().isUnknown();
}


RS_END_NAMESPACE
