/* -*- Mode: C++ -*-
 * $Id: SearchBall.h,v 1.4 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SearchBall.h
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Peter Andersson 2003-09-02
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 * $Log: SearchBall.h,v $
 * Revision 1.4  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.3  2004/08/13 15:39:28  petan
 * Moved implementation to generateStep
 *
 * Revision 1.2  2003/09/03 14:08:46  aip19
 * Updated and documented primitive skills
 *
 * Revision 1.1  2003/09/02 12:52:43  aip19
 * New primitive skills checked in, not tested.
 *
 */

/**
 * @file SearchBall.h
 * Include file for the skill class SearchBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */


#ifndef SEARCHBALL_H
#define SEARCHBALL_H

#include "Skill.h"
#include "AgentView.h"
#include "BallView.h"
#include "GameView.h"

RS_BEGIN_NAMESPACE

/**
 * @class SearchBall SearchBall.h
 * This skill enables an agent to search for the ball.
 *
 * @pre We can expect visual information in
 * The next cycle and we don't know where the ball is.
 * @post We find the ball. Or keep on turning.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */
class SearchBall : public Skill {
public:
  SearchBall();
  virtual ~SearchBall();

  virtual Plan generatePlan();

  
  /** Reset any internal state in the skill. */
  virtual void reset();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
protected:
  BallView* ballView;
  GameView* gameView;
  AgentView* agentView;
};


RS_END_NAMESPACE


#endif // !SEARCHBALL_H
