/* -*- Mode: C++ -*-
 * $Id: TurnBackToPoint.h,v 1.5 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBackToPoint.h
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: TurnBackToPoint.h,v $
 * Revision 1.5  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.4  2004/08/18 12:03:06  petan
 * Removed pure virtual functions in header
 *
 * Revision 1.3  2004/08/13 15:39:28  petan
 * Moved implementation to generateStep
 *
 * Revision 1.2  2003/09/03 14:08:46  aip19
 * Updated and documented primitive skills
 *
 * Revision 1.1  2003/09/02 12:52:44  aip19
 * New primitive skills checked in, not tested.
 *
 *
 */

/**
 * @file TurnBackToPoint.h
 * Include file for the skill TurnBackToPoint.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */

#ifndef TURNBACKTOPOINT_H
#define TURNBACKTOPOINT_H

#include "SkillWithPosition.h"
#include "TooFewArgumentsException.h"
#include "AgentView.h"

RS_BEGIN_NAMESPACE


/**
 * @class TurnBackToPoint TurnBackToPoint.h
 * This skill enables an agent to turn his back towards a given position.
 *
 * @pre The agents position and body direction must be known.
 * @post The agents back is turned towards a certain position.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */
class TurnBackToPoint : public SkillWithPosition {
public:
  /** Default constructor, it is ok since the class is abstract
    all subclasses should make sure to get default values for the
    arguments. */
  TurnBackToPoint(const Point& pos);

  /** Destructor cleaning up after the skill. */
  virtual ~TurnBackToPoint();

  
  /** Reset any internal state in the skill. */
  virtual void reset();

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. Abstract since
    no concrete instances of this class should be created. */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. Abstract since
    no concrete instances of this class should be created. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. Abstract since no concrete instances
   of this class should be created. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();
  
  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

private:
  /** Hide copy-constructor. */
  TurnBackToPoint(const TurnBackToPoint&);
  /** Hide assignment operator. */
  TurnBackToPoint& operator= (const TurnBackToPoint&);

  AgentView *mAgentView;
};


RS_END_NAMESPACE


#endif // !SKILLWITHARGUMENTS_H
