/* -*- Mode: C++ -*-
 * $Id: TurnBodyToAbsoluteDirection.cc,v 1.2 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBodyToAbsoluteDirection.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-26
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: TurnBodyToAbsoluteDirection.cc,v $
 * Revision 1.2  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.1  2004/08/26 17:11:34  frehe
 * Added TurnNeckToAbsoluteDirection and TurnBodyToAbsoluteDirection.
 *
 */

/**
 * @file TurnBodyToAbsoluteDirection.cc
 * Implementation file for the skill class TurnBodyToAbsoluteDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */


#include "view_constants.h"
#include "TurnBodyToAbsoluteDirection.h"
#include "AgentView.h"
#include "TurnCommand.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

TurnBodyToAbsoluteDirection::TurnBodyToAbsoluteDirection(const AngleDeg& dir)
  : SkillWithDirection(dir),
    mAgentView(NULL)
{
  getView(mAgentView, AGENT_VIEW_ID);
}


// --- DESTRUCTOR ---

TurnBodyToAbsoluteDirection::~TurnBodyToAbsoluteDirection()
{
  viewManager->releaseView(AGENT_VIEW_ID);
}


// --- METHODS ---

void
TurnBodyToAbsoluteDirection::reset()
{
}

Plan
TurnBodyToAbsoluteDirection::generatePlan()
{
  return Plan(generateStep());
}


AgentStep
TurnBodyToAbsoluteDirection::generateStep()
{
  AngleDeg turnAngle = 0;
  if ( mAgentView->getBodyDirection().isKnown() ) {
    AngleDeg body_dir = mAgentView->getBodyDirection().getDeg();
    AngleDeg angle = direction - body_dir;
    turnAngle = mAgentView->getAngleForTurn(angle);

    std::cout << "bodydir = " << body_dir
	      << "; turndir = " << turnAngle << std::endl;
  }

  AgentStep step;
  if ( turnAngle != 0 ) {
    TurnCommand *command = new TurnCommand(turnAngle);
    step.setBodyCommand(command);
  }
  return step;
}


FuzzyBool
TurnBodyToAbsoluteDirection::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool
TurnBodyToAbsoluteDirection::applicable()
{
  return mAgentView->getPosition().isKnown() &&
    mAgentView->getBodyDirection().isKnown();
}


FuzzyBool
TurnBodyToAbsoluteDirection::succeed()
{
  // Always succedes.
  return FuzzyBool(0.0);
}


FuzzyBool
TurnBodyToAbsoluteDirection::failed()
{
  // Never fails.
  return FuzzyBool(0.0);
}


RS_END_NAMESPACE
