/* -*- Mode: C++ -*-
 * $Id: TurnBodyToAbsoluteDirection.h,v 1.2 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBodyToAbsoluteDirection.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-26
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: TurnBodyToAbsoluteDirection.h,v $
 * Revision 1.2  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.1  2004/08/26 17:11:34  frehe
 * Added TurnNeckToAbsoluteDirection and TurnBodyToAbsoluteDirection.
 *
 */

/**
 * @file TurnBodyToAbsoluteDirection.h
 * Include file for the skill TurnBodyToAbsoluteDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */


#ifndef TURN_BODY_TO_ABSOLUTE_DIRECTION_H
#define TURN_BODY_TO_ABSOLUTE_DIRECTION_H


#include "SkillWithDirection.h"


RS_BEGIN_NAMESPACE


class AgentView;


/**
 * @class TurnBodyToAbsoluteDirection TurnBodyToAbsoluteDirection.h
 * Turns the agents body so that it faces the defined absolute direction.
 * This can be used in preparation for moving.
 *
 * @pre The agents position and body direction should be known.
 * @post The agents body is turned towards the defined absolute direction.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */
class TurnBodyToAbsoluteDirection : public SkillWithDirection {
public:
  /** Default constructor, it is ok since the class is abstract
      all subclasses should make sure to get default values for the
      arguments. */
  TurnBodyToAbsoluteDirection(const AngleDeg& dir);

  /** Destructor cleaning up after the skill. */
  virtual ~TurnBodyToAbsoluteDirection();


  /** Reset any internal state in the skill. */
  virtual void reset();

  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent. Abstract since
      no concrete instances of this class should be created. */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. Abstract since
      no concrete instances of this class should be created. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
      the description of the skill. Abstract since no concrete instances
      of this class should be created. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

private:
  /// An agent view
  AgentView *mAgentView;
  
  /** Hide copy-constructor. */
  TurnBodyToAbsoluteDirection(const TurnBodyToAbsoluteDirection&);
  /** Hide assignment operator. */
  TurnBodyToAbsoluteDirection& operator= (const TurnBodyToAbsoluteDirection&);
};


RS_END_NAMESPACE


#endif // !TURN_BODY_TO_ABSOLUTE_DIRECTION_H
