/* -*- Mode: C++ -*-
 * $Id: TurnBodyToPoint.cc,v 1.9 2005/09/08 08:58:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBodyToPoint.cc
 * Unitname: Framework/skill
 * $Revision: 1.9 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:35 $
 *
 *
 * HISTORY
 *
 * $Log: TurnBodyToPoint.cc,v $
 * Revision 1.9  2005/09/08 08:58:35  frehe
 * Implemented the reset method.
 *
 * Revision 1.8  2004/08/13 15:39:29  petan
 * Moved implementation to generateStep
 *
 * Revision 1.7  2003/11/15 15:08:34  aip15
 * A dubble declaration of turnAngle in generatePlan caused the skill to never turn.
 *
 * Revision 1.6  2003/10/27 21:22:23  frehe
 * Added more unknown checks.
 *
 * Revision 1.5  2003/10/13 23:26:01  frehe
 * Fixed bug with direction.
 *
 * Revision 1.4  2003/09/03 14:12:35  aip19
 * Updated prediction
 *
 * Revision 1.2  2003/09/02 17:59:38  frehe
 * Updated to the new command pointer semantics. RoboSoc now takes control of the pointer when sent to the actuator interface.
 *
 * Revision 1.1  2003/09/02 12:52:44  aip19
 * New primitive skills checked in, not tested.
 *
 *
 */

/**
 * @file TurnBodyToPoint.cc
 * Implementation file for the skill class TurnBodyToPoint.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:35 $
 */

#include "view_constants.h"
#include "TurnBodyToPoint.h"
#include "TurnCommand.h"
#include "NewCycleStrategy.h"
#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

TurnBodyToPoint::TurnBodyToPoint(const Point& pos)
  : SkillWithPosition(pos),
    mAgentView(NULL)
{
  getView(mAgentView, AGENT_VIEW_ID);
}


// --- DESTRUCTOR ---

TurnBodyToPoint::~TurnBodyToPoint()
{
  viewManager->releaseView(AGENT_VIEW_ID);
}


// --- METHODS ---

void
TurnBodyToPoint::reset()
{
}

Plan
TurnBodyToPoint::generatePlan()
{
  return Plan(generateStep());
}

AgentStep TurnBodyToPoint::generateStep()
{
  // Predict next agent position
  const NewCycleStrategy* new_cycle_strategy
    = WorldModelInterface::instance()->getNewCycleStrategy();
  WorldModel predicted_wm(WorldModelInterface::instance()->getWorld(NOW));
  new_cycle_strategy->updateBeforeNewCycle(predicted_wm);

  // Now predicted_wm contains the predicted world
  AgentObject& agent = predicted_wm.getAgent();
  PointUE predictedPosition = agent.getPosition();

  // If the predicted position is unknown, get a good estimation
  if (predictedPosition.isUnknown()) {
    if  (mAgentView->getSpeedVector().isKnown())
      predictedPosition = mAgentView->getPosition() + 
	mAgentView->getSpeedVector();
    else
      predictedPosition = mAgentView->getPosition();
  }

  AngleDeg turnAngle = 0;
  if ( predictedPosition.isKnown() ) {
    Vector globalAngle(predictedPosition.getPoint(), position);

    if ( mAgentView->getBodyDirection().isKnown() ) {

      AngleDeg angle = globalAngle.getAngle();
      angle -= mAgentView->getBodyDirection().getDeg();
  
      turnAngle = mAgentView->getAngleForTurn(angle);
    }
  }
  
  TurnCommand *command = new TurnCommand(turnAngle);
  AgentStep step;
  step.setBodyCommand(command);
  return step;
}

FuzzyBool TurnBodyToPoint::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool TurnBodyToPoint::applicable()
{
  return mAgentView->getPosition().isKnown() &&
    mAgentView->getBodyDirection().isKnown();
}

FuzzyBool TurnBodyToPoint::succeed()
{
  // Always succedes.
  return FuzzyBool(0.0);
}

FuzzyBool TurnBodyToPoint::failed()
{
  // Never fails.
  return FuzzyBool(0.0);
}



RS_END_NAMESPACE

