/* -*- Mode: C++ -*-
 * $Id: TurnBodyToPoint.h,v 1.5 2005/09/08 08:58:36 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBodyToPoint.h
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Peter Andersson 2003-08-29
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:36 $
 *
 *
 * HISTORY
 *
 * $Log: TurnBodyToPoint.h,v $
 * Revision 1.5  2005/09/08 08:58:36  frehe
 * Implemented the reset method.
 *
 * Revision 1.4  2004/08/13 15:39:29  petan
 * Moved implementation to generateStep
 *
 * Revision 1.3  2003/09/03 14:08:46  aip19
 * Updated and documented primitive skills
 *
 * Revision 1.2  2003/09/02 17:59:38  frehe
 * Updated to the new command pointer semantics. RoboSoc now takes control of the pointer when sent to the actuator interface.
 *
 * Revision 1.1  2003/09/02 12:52:45  aip19
 * New primitive skills checked in, not tested.
 *
 *
 */

/**
 * @file TurnBodyToPoint.h
 * Include file for the skill TurnBodyToPoint.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:36 $
 */


#ifndef TURNBODYTOPOINT_H
#define TURNBODYTOPOINT_H

#include "SkillWithPosition.h"
#include "AgentView.h"

RS_BEGIN_NAMESPACE


/**
 * @class TurnBodyToPoint TurnBodyToPoint.h
 * Turns the agents body so that it faces the defined position.
 * This can be used in preparation for moving.
 *
 * @pre The agents position and body direction should be known.
 * @post The agents body is turned towards the defined position.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:36 $
 */
class TurnBodyToPoint : public SkillWithPosition {
public:
  /** Default constructor, it is ok since the class is abstract
    all subclasses should make sure to get default values for the
    arguments. */
  TurnBodyToPoint(const Point& pos);

  /** Destructor cleaning up after the skill. */
  virtual ~TurnBodyToPoint();


  /** Reset any internal state in the skill. */
  virtual void reset();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. Abstract since
    no concrete instances of this class should be created. */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();
  
  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. Abstract since
    no concrete instances of this class should be created. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. Abstract since no concrete instances
   of this class should be created. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();
  
  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

private:
  /** Hide copy-constructor. */
  TurnBodyToPoint(const TurnBodyToPoint&);
  /** Hide assignment operator. */
  TurnBodyToPoint& operator= (const TurnBodyToPoint&);

  AgentView *mAgentView;
};


RS_END_NAMESPACE


#endif // !SKILLWITHARGUMENTS_H
