/* -*- Mode: C++ -*-
 * $Id: TurnNeckToAbsoluteDirection.cc,v 1.2 2005/09/08 08:58:36 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintzfrehe <@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnNeckToAbsoluteDirection.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-26
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:36 $
 *
 *
 * HISTORY
 *
 * $Log: TurnNeckToAbsoluteDirection.cc,v $
 * Revision 1.2  2005/09/08 08:58:36  frehe
 * Implemented the reset method.
 *
 * Revision 1.1  2004/08/26 17:11:34  frehe
 * Added TurnNeckToAbsoluteDirection and TurnBodyToAbsoluteDirection.
 *
 */

/**
 * @file TurnNeckToAbsoluteDirection.cc
 * Implementation file for the skill class TurnNeckToAbsoluteDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:36 $
 */


#include "TurnNeckToAbsoluteDirection.h"
#include "view_constants.h"
#include "AgentView.h"
#include "CommandStrategy.h"
#include "NewCycleStrategy.h"
#include "WorldModelInterface.h"
#include "TurnNeckCommand.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

TurnNeckToAbsoluteDirection::TurnNeckToAbsoluteDirection(const AngleDeg& dir,
							 ServerCommand* cmd)
  : SkillWithDirectionAndCommand(dir, cmd),
    mAgentView(0)
{
  getView(mAgentView, AGENT_VIEW_ID);
}


// --- DESTRUCTOR ---

TurnNeckToAbsoluteDirection::~TurnNeckToAbsoluteDirection()
{
}


// --- METHODS ---

void
TurnNeckToAbsoluteDirection::reset()
{
}

Plan
TurnNeckToAbsoluteDirection::generatePlan()
{
  return Plan(generateStep());
}


AgentStep
TurnNeckToAbsoluteDirection::generateStep()
{
  const CommandStrategy* cmd_strategy
    = WorldModelInterface::instance()->getCommandStrategy();
  const NewCycleStrategy* new_cycle_strategy
    = WorldModelInterface::instance()->getNewCycleStrategy();

  WorldModel predicted_wm(WorldModelInterface::instance()->getWorld(NOW));

  // Predict the state of the world after executing the command
  if ( command != 0 ) {
    cmd_strategy->updateAfterCommand(predicted_wm, command);
  }
  new_cycle_strategy->updateBeforeNewCycle(predicted_wm);

  // Now predicted_wm contains the predicted world
  AgentObject& agent = predicted_wm.getAgent();
  AngleDegUE neck_dir = agent.getNeckDirection();

  AgentStep step;
  if ( neck_dir.isKnown() ) {
    AngleDeg turn_dir = direction - neck_dir.getDeg();
    AngleDegUE cur_neck_dir = mAgentView->getNeckDirection();
    std::cout << Time::currentTime << ": ";
    if ( cur_neck_dir.isKnown() ) {
      std::cout << "cur neckdir = " << cur_neck_dir.getDeg() << "; ";
    }
    std::cout << "pred neckdir = " << neck_dir.getDeg()
	      << "; turndir = " << turn_dir;

    turn_dir = minMax(WorldFacts::instance()->SP_minneckang-neck_dir.getDeg(),
		      turn_dir.getDeg(),
		      WorldFacts::instance()->SP_maxneckang-neck_dir.getDeg());

    std::cout << "; norm turndir = " << turn_dir << std::endl;

    if ( turn_dir != 0 ) {
      TurnNeckCommand* cmd = new TurnNeckCommand(turn_dir);
      step.setNeckCommand(cmd);
    }
  }
  return step;
}


FuzzyBool
TurnNeckToAbsoluteDirection::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool
TurnNeckToAbsoluteDirection::applicable()
{
  // Always applicable.
  return FuzzyBool(1.0);
}


FuzzyBool
TurnNeckToAbsoluteDirection::succeed()
{
  // The skill has not succeeded yet.
  return FuzzyBool(0.0);
}


FuzzyBool
TurnNeckToAbsoluteDirection::failed()
{
  // The skill has not failed yet.
  return FuzzyBool(0.0);
}


RS_END_NAMESPACE
