/* -*- Mode: C++ -*-
 * $Id: TurnNeckToAbsoluteDirection.h,v 1.2 2005/09/08 08:58:36 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintzfrehe <@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnNeckToAbsoluteDirection.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-26
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:36 $
 *
 *
 * HISTORY
 *
 * $Log: TurnNeckToAbsoluteDirection.h,v $
 * Revision 1.2  2005/09/08 08:58:36  frehe
 * Implemented the reset method.
 *
 * Revision 1.1  2004/08/26 17:11:34  frehe
 * Added TurnNeckToAbsoluteDirection and TurnBodyToAbsoluteDirection.
 *
 */

/**
 * @file TurnNeckToAbsoluteDirection.h
 * Include file for the skill class TurnNeckToAbsoluteDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:36 $
 */


#ifndef TURN_NECK_TO_ABSOLUTE_DIRECTION_H
#define TURN_NECK_TO_ABSOLUTE_DIRECTION_H


#include "SkillWithDirectionAndCommand.h"


RS_BEGIN_NAMESPACE


class AgentView;


/**
 * @class TurnNeckToAbsoluteDirection TurnNeckToAbsoluteDirection.h
 *
 * @pre Always applicable
 * @post The neck of the agent is turned as close as possible to
 * the desired absolute direction.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:36 $
 */
class TurnNeckToAbsoluteDirection : public SkillWithDirectionAndCommand {
 public:
  /** Default constructor. Here you can initialize the skill. */
  TurnNeckToAbsoluteDirection(const AngleDeg& dir, ServerCommand* cmd);

  /** Destructor should clean up after the skill. */
  virtual ~TurnNeckToAbsoluteDirection();


  /** Reset any internal state in the skill. */
  virtual void reset();

  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent. You can assume
      that the preconditions of the skill are true (this can be tested
      by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
      the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();


 private:
  /// An agent view
  AgentView *mAgentView;

  /** Hide copy-constructor. */
  TurnNeckToAbsoluteDirection(const TurnNeckToAbsoluteDirection&);
  /** Hide assignment operator. */
  TurnNeckToAbsoluteDirection& operator= (const TurnNeckToAbsoluteDirection&);
};


RS_END_NAMESPACE


#endif // !TURN_NECK_TO_ABSOLUTE_DIRECTION_H
