/* -*- Mode: C++ -*-
 * $Id: TurnNeckToBall.h,v 1.5 2005/09/29 08:24:59 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintzfrehe <@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnNeckToBall.h
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2003-09-02
 * Last modified by $Author: petan $ $Date: 2005/09/29 08:24:59 $
 *
 *
 * HISTORY
 *
 * $Log: TurnNeckToBall.h,v $
 * Revision 1.5  2005/09/29 08:24:59  petan
 * Updated so that agent will align neck to body if ball is outside viewable area
 *
 * Revision 1.4  2005/09/08 08:58:36  frehe
 * Implemented the reset method.
 *
 * Revision 1.3  2004/08/18 12:03:46  petan
 * Moved initialization of AgentStep
 *
 * Revision 1.2  2004/08/13 15:39:29  petan
 * Moved implementation to generateStep
 *
 * Revision 1.1  2003/09/02 17:59:16  frehe
 * Added TurnNeckToPoint and TurnNeckToBall.
 *
 */

/**
 * @file TurnNeckToBall.h
 * Include file for the skill class TurnNeckToBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: petan $
 * $Date: 2005/09/29 08:24:59 $
 */


#ifndef TURNNECKTOBALL_H
#define TURNNECKTOBALL_H


#include "SkillWithCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class TurnNeckToBall TurnNeckToBall.h
 *
 * @pre Always applicable
 * @post The neck of the agent is turned as close
 *       as possible to the ball. If the ball is unknown or
 *       outside the viewing area, the agent will align the neck
 *       to the body.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: petan $
 * $Date: 2005/09/29 08:24:59 $
 */
class TurnNeckToBall : public SkillWithCommand {
public:
  /** Default constructor. Here you can initialize the skill. */
  TurnNeckToBall(ServerCommand* cmd);

  /** Destructor should clean up after the skill. */
  virtual ~TurnNeckToBall();

  
  /** Reset any internal state in the skill. */
  virtual void reset();

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

  
private:
  /** Hide copy-constructor. */
  TurnNeckToBall(const TurnNeckToBall&);
  /** Hide assignment operator. */
  TurnNeckToBall& operator= (const TurnNeckToBall&);
};


RS_END_NAMESPACE


#endif // !TURNNECKTOPOINT_H
