/* -*- Mode: C++ -*-
 * $Id: TurnNeckToPoint.h,v 1.3 2005/09/08 08:58:36 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintzfrehe <@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnNeckToPoint.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2003-09-02
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:36 $
 *
 *
 * HISTORY
 *
 * $Log: TurnNeckToPoint.h,v $
 * Revision 1.3  2005/09/08 08:58:36  frehe
 * Implemented the reset method.
 *
 * Revision 1.2  2004/08/13 15:39:29  petan
 * Moved implementation to generateStep
 *
 * Revision 1.1  2003/09/02 17:59:17  frehe
 * Added TurnNeckToPoint and TurnNeckToBall.
 *
 */

/**
 * @file TurnNeckToPoint.h
 * Include file for the skill class TurnNeckToPoint.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:36 $
 */


#ifndef TURNNECKTOPOINT_H
#define TURNNECKTOPOINT_H


#include "SkillWithPositionAndCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class TurnNeckToPoint TurnNeckToPoint.h
 *
 * @pre Always applicable
 * @post The neck of the agent is turned as close as possible to
 * the desired point.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:36 $
 */
class TurnNeckToPoint : public SkillWithPositionAndCommand {
public:
  /** Default constructor. Here you can initialize the skill. */
  TurnNeckToPoint(const Point& pos, ServerCommand* cmd);

  /** Destructor should clean up after the skill. */
  virtual ~TurnNeckToPoint();


  /** Reset any internal state in the skill. */
  virtual void reset();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

  
private:
  /** Hide copy-constructor. */
  TurnNeckToPoint(const TurnNeckToPoint&);
  /** Hide assignment operator. */
  TurnNeckToPoint& operator= (const TurnNeckToPoint&);
};


RS_END_NAMESPACE


#endif // !TURNNECKTOPOINT_H
