/* -*- Mode: C++ -*-
 * $Id: AgentView.h,v 1.14 2005/09/19 15:31:40 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentView.h
 * Unitname: Framework/view
 * $Revision: 1.14 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: petan $ $Date: 2005/09/19 15:31:40 $
 *
 *
 * HISTORY
 *
 * $Log: AgentView.h,v $
 * Revision 1.14  2005/09/19 15:31:40  petan
 * Updated documentation for global/relative angles
 *
 * Revision 1.13  2004/08/13 11:47:07  petan
 * Changed a if construct to use minMax instead
 *
 * Revision 1.12  2003/09/18 08:27:51  aip19
 * Fixed getPowerForDash NaN problem
 *
 * Revision 1.11  2003/09/03 14:02:45  aip19
 * Updated AgentView with getPowerForDash
 *
 * Revision 1.10  2003/09/02 12:52:45  aip19
 * New primitive skills checked in, not tested.
 *
 * Revision 1.9  2003/07/21 08:59:53  s02davro
 * Added support for new communication
 *
 * Revision 1.8  2002/09/03 10:57:29  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.7  2002/08/31 15:22:45  s02davro
 * Small changes in views
 *
 * Revision 1.6  2002/08/02 21:53:02  s02davro
 * Now works by extracting information from the AgentObject in the WorldModel, parameterView depenency removed.
 *
 * Revision 1.5  2001/10/14 21:21:44  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.4  2001/01/11 22:00:17  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.3  2001/01/10 15:12:57  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.2  2000/10/18 23:05:53  frehe
 * Added lastCoachMessage and lastPlayerMessage.
 *
 * Revision 1.1  2000/09/21 13:46:39  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.2  2000/09/13 00:08:05  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also 
 * changed those views that used a static pointer to itself to do process 
 * the data in the sensor interface to use the new forEach methods 
 * implemented in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/04 15:27:19  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AgentView.h
 * Include file for the class AgentView.
 *
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.14 $
 * $Author: petan $
 * $Date: 2005/09/19 15:31:40 $
 */


#ifndef AGENTVIEW_H
#define AGENTVIEW_H

#include "View.h"
#include "HistoryDefault.h"
#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE

/**
 * @class AgentView AgentView.h
 * This view is used to keep track of the agent,
 * its position, speed and other properties.
 *
 * @todo Is the view needed?
 * @todo More information.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.14 $
 * $Author: petan $
 * $Date: 2005/09/19 15:31:40 $
 */

class AgentView : public View {
public:
  /** Create an Agent View with ViewId @a id */
  AgentView(const ViewId id);

  /** Destroys the agent view. */
  virtual ~AgentView();

  /** Get a constant object representing the agent @a hist cycles ago. */
  const AgentObject& getAgentObject(const Int hist = NOW) const;

  /** Returns @c true iff the agent is outside the field. */
  bool isAgentOutsideField() const;

  /** Get the position of the agent @a hist cycles ago. */
  PointUE getPosition(const Int hist = NOW) const;

  /** Get the (global) body direction of the agent @a hist cycles ago. */
  AngleDegUE getBodyDirection(const Int hist = NOW) const;

  /** Get the (global) neck direction of the agent @a hist cycles ago. */
  AngleDegUE getNeckDirection(const Int hist = NOW) const;
  
  /** Get the effort of the agent @a hist cycles ago. */
  FloatUE getEffort(const Int hist = NOW) const;

  /** Get the recovery of the agent @a hist cycles ago. */
  FloatUE getRecover(const Int hist) const;

  /** Get the stamina of the agent @a hist cycles ago. */
  FloatUE getStamina(const Int hist = NOW) const;

  /** Get the amount of speed of the agent @a hist cycles ago. */
  FloatUE getSpeed(const Int hist = NOW) const;

  /** Get the speed vector of the agent @a hist cycles ago. */
  VectorUE getSpeedVector(const Int hist = NOW) const;
  
  /** Get the view width of the agent @a hist cycles ago. */
  ViewWidth getViewWidth(const Int hist = NOW) const;

  /** Get the view quality of the agent @a hist cycles ago. */
  ViewQuality getViewQuality(const Int hist = NOW) const;

  /** Get the view angle of the agent @a hist cycles ago. */
  AngleDegUnknown getViewAngle(const Int hist = NOW) const;
  
  /** Returns @c true iff the agent is the goal keeper. */
  bool isGoalie() const;
  
  /** Returns the uniform number of the agent. */
  UIntUnknown getUniformNumber() const;

  /** Calculates the actual degrees needed to make a turn */
  AngleDeg getAngleForTurn(AngleDeg angle) const;

  // Documentation in .cc file
  Float getPowerForDash(Point point) const;

  /** Returns the number of dashes made upto @a hist cycles ago. */
  UInt getDashes(const Int hist = NOW) const;

  /** Returns the number of kicks made upto @a hist cycles ago. */
  UInt getKicks(const Int hist = NOW) const;

  /** Returns the number of says made upto @a hist cycles ago. */
  UInt getSays(const Int hist = NOW) const;

  /** Returns the number of turns made upto @a hist cycles ago. */
  UInt getTurns(const Int hist = NOW) const;

  /** Returns the number of turn necks made upto @a hist cycles ago. */
  UInt getTurnNecks(const Int hist = NOW) const;

  /** Returns the last message sent by the coach. */
  const std::string& getCoachMessage(const Int hist = NOW) const;

  /** Returns the last message sent by the agent itself. */
  const std::string& getOwnMessage(const Int hist = NOW) const;
  
  /** Returns the last message sent by a teammate. */
  const std::string& getTeammateMessage(const Int hist = NOW) const;

  /** Returns the last message sent by a opponent. */
  const std::string& getOpponentMessage(const Int hist = NOW) const;

  AngleDegUE getTeammateMessageDirection(const Int hist = NOW) const;

  AngleDegUE getOpponentMessageDirection(const Int hist = NOW) const;

  UInt getTeammateSpeaker(const Int hist = NOW) const;

  const std::vector<UInt>& getTeammateSpeakers(const Int hist = NOW) const;

  UInt getNumberOfOpponentSpeakers(const Int hist = NOW) const;

protected:
  //@{
  /** @name Update fuctions. */
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  //@}
private:
  /** Hidden copy-constructor. */
  AgentView(const AgentView&);

  /** Hidden assignment operator. */
  AgentView& operator= (const AgentView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AgentView.icc"
#endif


#endif // !AGENTVIEW_H


