/* -*- Mode: C++ -*-
 * $Id: BallView.cc,v 1.16 2003/06/02 07:18:10 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallView.cc
 * Unitname: Framework/view
 * $Revision: 1.16 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2003/06/02 07:18:10 $
 *
 *
 * HISTORY
 *
 * $Log: BallView.cc,v $
 * Revision 1.16  2003/06/02 07:18:10  frehe
 * Changed iostream.h to iostream.
 *
 * Revision 1.15  2002/09/26 15:28:41  s02davro
 * Moved isBallKickable to cc and corrected it
 *
 * Revision 1.14  2002/08/31 15:22:45  s02davro
 * Small changes in views
 *
 * Revision 1.13  2002/08/02 21:53:13  s02davro
 * Now works by extracting information from the BallObject, uses AgentView as before.
 *
 * Revision 1.12  2001/09/17 22:20:21  frehe
 * Added some functionality to use the realtive speed of the ball from rccparser, but it is not used at the moment.
 *
 * Revision 1.11  2001/09/17 21:27:38  andbo928
 * Removed FutureBallView references.
 *
 * Revision 1.10  2001/09/01 23:49:51  frehe
 * Updated the file for the new implementation of getValue and getAngle.
 *
 * Revision 1.9  2001/08/23 00:32:04  frehe
 * Removed dependencies.
 *
 * Revision 1.8  2001/08/22 21:04:02  frehe
 * Removed an inline statement.
 *
 * Revision 1.7  2001/01/11 22:00:20  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.6  2001/01/10 15:12:59  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.5  2000/10/23 12:06:24  frehe
 * Fixed the ball speed estimation.
 *
 * Revision 1.4  2000/10/11 18:39:11  frehe
 * Changed the ball speed estimation.
 *
 * Revision 1.3  2000/10/08 12:11:04  frehe
 * Fixed a bug in BallView.
 *
 * Revision 1.2  2000/10/04 12:03:15  frehe
 * Made some minor changes.
 *
 * Revision 1.1  2000/09/21 13:46:46  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.3  2000/09/13 00:08:06  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.2  2000/09/05 21:15:57  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/04 15:27:20  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file BallView.cc
 * Implementation file for the class BallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.16 $
 * $Author: frehe $
 * $Date: 2003/06/02 07:18:10 $
 */

#include <iostream>
#include "view_constants.h"
#include "ViewManager.h"
#include "AgentView.h"
#include "BallView.h"


RS_BEGIN_NAMESPACE

BallView::BallView(const ViewId id) 
  : View(id),
    agentView(NULL),
    facts(worldModelInterface->getWorldFacts()) {
  getView(agentView, AGENT_VIEW_ID);
}

BallView::~BallView() {
  viewManager->releaseView(AGENT_VIEW_ID);
}

bool BallView::isBallCatchable(const Int hist) const {
  if (getRelativeVector(hist).isKnown())
    return (getRelativeVector(hist).getLength() <
	    facts->SP_catchable_area_l);
  return false;
}

bool BallView::isBallKickable(const Int hist) const {
  if (getRelativeVector(hist).isKnown())
    return (getRelativeDistance(hist) < facts->SP_kickable_area);
  return false;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "BallView.icc"
#endif
