/* -*- Mode: C++ -*-
 * $Id: BallView.h,v 1.8 2004/08/13 11:47:35 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallView.h
 * Unitname: Framework/view
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: petan $ $Date: 2004/08/13 11:47:35 $
 *
 *
 * HISTORY
 *
 * $Log: BallView.h,v $
 * Revision 1.8  2004/08/13 11:47:35  petan
 * Changed absolute references to relative and made the documentation more clear
 *
 * Revision 1.7  2002/08/31 15:22:45  s02davro
 * Small changes in views
 *
 * Revision 1.6  2002/08/02 21:53:13  s02davro
 * Now works by extracting information from the BallObject, uses AgentView as before.
 *
 * Revision 1.5  2001/09/17 21:27:38  andbo928
 * Removed FutureBallView references.
 *
 * Revision 1.4  2001/08/23 00:32:04  frehe
 * Removed dependencies.
 *
 * Revision 1.3  2001/01/11 22:00:17  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.2  2001/01/10 15:12:57  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.1  2000/09/21 13:46:40  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.2  2000/09/13 00:08:06  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/04 15:27:20  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file BallView.h
 * Include file for the class BallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.8 $
 * $Author: petan $
 * $Date: 2004/08/13 11:47:35 $
 */


#ifndef BALLVIEW_H
#define BALLVIEW_H

#include "View.h"
#include "AgentView.h"

RS_BEGIN_NAMESPACE

/**
 * @class BallView BallView.h
 * This view is used to keep track of the ball.
 *
 * @todo Is the view needed?
 * @todo Information from kickview could be included.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.8 $
 * $Author: petan $
 * $Date: 2004/08/13 11:47:35 $
 */

class BallView : public View {
public:
  /** Create a ball view with ViewId @a id */
  BallView(const ViewId id);

  /** Destroys the ball view. */
  virtual ~BallView();

  /** Get a constant object representing the ball @a hist cycles ago. */
  const BallObject& getBallObject(const Int hist = NOW) const;

  /** Get the time of the last observation of the ball. */
  TimeUnknown getLastObservationTime(const Int hist = NOW) const;

  /** Get the distance to the ball @a hist cycles ago. */
  FloatUE getRelativeDistance(const Int hist = NOW) const;

  /** Get the relative direction to the ball @a hist cycles ago in
   *  relation to the agents body direction.
   */  
  AngleDegUE getRelativeDirection(const Int hist = NOW) const;

  /** Get the relative vector to the ball @a hist cycles ago in relation
   *  to the agents body direction.
   */
  VectorUE getRelativeVector(const Int hist = NOW) const;

  /** Get the position of the ball @a hist cycles ago. */
  PointUE getPosition(const Int hist = NOW) const;

  /** Get the absolute speed vector of the ball @a hist cycles ago. 
   *  I.e. absolute amount of speed and speed direction. */
  VectorUE getSpeedVector(const Int hist = NOW) const;

  /** Get the relative speed vector of the ball @a hist cycles ago.
   *  I.e. absolute amount of speed and relative speed direction. */
  VectorUE getRelativeSpeedVector(const Int hist = NOW) const;
  
  /** Returns @c true iff the ball is kickable.
   *  Obsolete: it should be made into a predicate! */
  bool isBallKickable(const Int hist = NOW) const;

  /** Returns @c true iff the ball is catchable.
   *  Obsolete: it should be made into a predicate! */
  bool isBallCatchable(const Int hist = NOW) const;
  
protected:
  /** A pointer to the agent view. */
  AgentView* agentView;

  /** The known facts of the world. (old ParamenterView) */
  const WorldFacts* facts;

  //@{
  /** @name Update functions. */
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  //@}
private:

  /** Hidden copy-constructor. */
  BallView(const BallView&);

  /** Hidden assignment operator. */
  BallView& operator= (const BallView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "BallView.icc"
#endif


#endif // !BALLVIEW_H
