/* -*- Mode: C++ -*-
 * $Id: CoachInformationStrategy.h,v 1.1 2002/08/31 16:15:53 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoachInformationStrategy.h
 * Unitname: Framework/strategy
 * $Revision: 1.1 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 16:15:53 $
 *
 *
 * HISTORY
 *
 * $Log: CoachInformationStrategy.h,v $
 * Revision 1.1  2002/08/31 16:15:53  s02davro
 * Added a strategy for CoachInformation
 *
 * Revision 1.1  2002/08/02 21:54:16  s02davro
 * Abstract class to make updates of the worldModel based on the coachmessage.
 *
 */

/**
 * @file CoachInformationStrategy.h
 * Include file for the class CoachInformationStrategy.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/08/31 16:15:53 $
 */


#ifndef COACHINFORMATIONSTRATEGY_H
#define COACHINFORMATIONSTRATEGY_H

#include "Strategy.h"

RS_BEGIN_NAMESPACE

/**
 * @class CoachInformationStrategy CoachInformationStrategy.h
 * Updates the world model when a coach message is recived.
 * This class makes updates of the worldModel based on the
 * coachmessage recived adaptive. Inherit and supply your own
 * implementation of the updateAfterCoachInformation function.
 *
 *
 * @par Unit:
 *      Framework/strategy
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2002/08/31 16:15:53 $
 */

class CoachInformationStrategy : public Strategy {
public:
  /** This function is called when we have heard something new from the coach.
   *  Overload it to make changes in the worldModel based on this new 
   *  information. */
  virtual void updateAfterCoachInformation() = 0;
};

RS_END_NAMESPACE

#endif // !COACHINFORMATIONSTRATEGY_H
