/* -*- Mode: C++ -*-
 * $Id: CommandStrategy.h,v 1.4 2003/10/14 18:03:06 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CommandStrategy.h
 * Unitname: Framework/strategy
 * $Revision: 1.4 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: s02davro $ $Date: 2003/10/14 18:03:06 $
 *
 *
 * HISTORY
 *
 * $Log: CommandStrategy.h,v $
 * Revision 1.4  2003/10/14 18:03:06  s02davro
 * Fixed command bug
 *
 * Revision 1.3  2003/09/02 17:55:57  frehe
 * Added a world model and a command to the strategy.
 *
 * Revision 1.2  2002/08/31 15:24:37  s02davro
 * Changed the interface in the strategies
 *
 * Revision 1.1  2002/08/02 21:54:28  s02davro
 * Abstract class to make updates of the worldModel based on the command executed.
 *
 */

/**
 * @file CommandStrategy.h
 * Include file for the class CommandStrategy.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: s02davro $
 * $Date: 2003/10/14 18:03:06 $
 */


#ifndef COMMANDSTRATEGY_H
#define COMMANDSTRATEGY_H

#include "Strategy.h"

RS_BEGIN_NAMESPACE

/**
 * @class CommandStrategy CommandStrategy.h
 * Updates the world model when the agent has executed a command.
 * This class makes updates of the worldModel based on the
 * command executed adaptive. Inherit and supply your own
 * implementation of the updateAfterCommand function.
 *
 * @par Unit:
 *      Framework/strategy
 * $Revision: 1.4 $
 * $Author: s02davro $
 * $Date: 2003/10/14 18:03:06 $
 */

class CommandStrategy : public Strategy {
public:
  /** This function is called when we have executed a command.
   *  Overload it to make changes in the worldModel based on this new 
   *  information. */
  virtual void updateAfterCommand(WorldModel& world, ServerCommand* cmd) const = 0;
};

RS_END_NAMESPACE

#endif // !COMMANDSTRATEGY_H
