/* -*- Mode: C++ -*-
 * $Id: FrameworkAgent.cc,v 1.13 2003/07/21 09:01:07 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FrameworkAgent.cc
 * Unitname: Framework
 * $Revision: 1.13 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:01:07 $
 *
 *
 * HISTORY
 *
 * $Log: FrameworkAgent.cc,v $
 * Revision 1.13  2003/07/21 09:01:07  s02davro
 * Now using dummy clangParser to avoid thread (signal) problems
 *
 * Revision 1.12  2003/06/02 07:18:10  frehe
 * Changed iostream.h to iostream.
 *
 * Revision 1.11  2002/12/07 17:27:07  s02davro
 * Added support for trainer
 *
 * Revision 1.10  2002/09/04 13:15:38  frehe
 * Added a test that the controller got started properly.
 *
 * Revision 1.9  2002/08/31 15:27:02  s02davro
 * New modified startup, agent, frameworksensorinterface updated
 *
 * Revision 1.8  2002/08/02 21:55:13  s02davro
 * Uses the soccerserver coachmessageparser. ParameterView -> WorldFacts.
 *
 * Revision 1.7  2001/11/15 16:49:11  frehe
 * Added support for a coach.
 *
 * Revision 1.6  2001/08/17 07:15:17  frehe
 * Made changes to compile RoboSoc with RCCParser-0.10.
 *
 * Revision 1.5  2000/10/24 15:15:33  frehe
 * Fixed startWithCoach method.
 *
 * Revision 1.4  2000/10/11 18:39:31  frehe
 * Rearrange the startWithCoach method.
 *
 * Revision 1.3  2000/09/05 21:15:56  frehe
 * Made minor changes.
 *
 * Revision 1.2  2000/09/04 15:25:40  frehe
 * Fixed some minor bugs.
 *
 * Revision 1.1  2000/09/03 22:08:46  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FrameworkAgent.cc
 * Implementation file for the class FramworkAgent.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.13 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:01:07 $
 */


#include "FrameworkAgent.h"

#include "ServerInterface.h"
#include "FrameworkSensorInterface.h"
#include "Sensors.h"
#include "Actuators.h"
#include "Decision.h"
#include "ControlInterface.h"
#include "Controller.h"

#include "ViewManager.h"
#include "view_constants.h"

#include "rs_log.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

FrameworkAgent::FrameworkAgent(Decision* _decision)
  : viewManager(ViewManager::instance()),
    facts(WorldFacts::instance()),
    serverInterface(new ServerInterface
		    (facts->SP_host, 
		     (facts->SP_coach || facts->CP_trainer)?
		     (facts->SP_coach?facts->SP_olcoach_port:facts->SP_coach_port):
		     facts->SP_port,
		     UInt32(facts->SP_buffer_size))),
    sensorInterface(new FrameworkSensorInterface()),
    clangParser(),
    sensors(new Sensors(sensorInterface, clangParser)),
    actuatorInterface(new ActuatorInterface()),
    actuators(new Actuators(actuatorInterface)),
    decision(_decision),
    controlInterface(new ControlInterface(decision)),
    controller(new Controller(serverInterface, 
			      sensors,
			      actuators, 
			      controlInterface,
			      facts->SP_simulator_step*facts->SP_slow_down_factor,
			      facts->SP_recv_step*facts->SP_slow_down_factor,
			      facts->SP_sense_body_step*facts->SP_slow_down_factor,
			      facts->SP_send_step*facts->SP_slow_down_factor,
			      facts->CP_cycles_since_io_before_shutdown,
			      facts->CP_min_ticks_before_sending_command,
			      facts->CP_ticks_before_command_warning,
			      facts->CP_max_ticks_before_sending_command,
			      facts->SP_coach,
			      facts->CP_trainer)) {
}

FrameworkAgent::~FrameworkAgent() {
  delete controller;
  delete controlInterface;
  delete actuators;
  delete actuatorInterface;
  delete sensors;
  delete sensorInterface;
  delete serverInterface;
}

bool FrameworkAgent::start() {
  RS_LOG( LA_FRAMEWORK_AGENT, "Initializing the controller..." );
  if ( !controller->init() ) {
    RS_FATAL("The controller could not be initialized!");
    return false;
  }
  
  RS_LOG( LA_FRAMEWORK_AGENT, "Initializing the decision maker..." );
  if ( !decision->init(actuatorInterface) ) {
    RS_FATAL("The decision maker could not be initialized!");
    return false;
  }
  
  RS_LOG( LA_FRAMEWORK_AGENT, "Starting the controller!" );
  if ( !controller->start() ) {
    RS_FATAL("The controller could not be started!");
    return false;
  }

  RS_LOG( LA_FRAMEWORK_AGENT, "Running the controller!" );
  controller->run();

  return true;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "FrameworkAgent.icc"
#endif
