/* -*- Mode: C++ -*-
 * $Id: FrameworkAgent.h,v 1.5 2003/07/21 09:01:07 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FrameworkAgent.h
 * Unitname: Framework
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:01:07 $
 *
 *
 * HISTORY
 *
 * $Log: FrameworkAgent.h,v $
 * Revision 1.5  2003/07/21 09:01:07  s02davro
 * Now using dummy clangParser to avoid thread (signal) problems
 *
 * Revision 1.4  2002/08/31 15:27:02  s02davro
 * New modified startup, agent, frameworksensorinterface updated
 *
 * Revision 1.3  2002/08/02 21:55:14  s02davro
 * Uses the soccerserver coachmessageparser. ParameterView -> WorldFacts.
 *
 * Revision 1.2  2001/11/15 16:49:11  frehe
 * Added support for a coach.
 *
 * Revision 1.1  2000/09/03 22:08:43  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FrameworkAgent.h
 * Include file for the class FramworkAgent.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:01:07 $
 */


#ifndef FRAMEWORKAGENT_H
#define FRAMEWORKAGENT_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string>

#include "basic_system_types.h"
#include "WorldFacts.h"
#include <rccparser/rccparser.h>

//  #include <rcssserver/clangparser.h>
//  #include <rcssserver/clangmsgbuilder.h>

// Dummy Clangparser
class CLangParser
    : public rcss::Parser
{
    bool
    doParse( std::istream& )
    { return true; }
};

RS_BEGIN_NAMESPACE

class ViewManager;

class WorldModelInterface;
class ServerInterface;
class FrameworkSensorInterface;
class Sensors;
class ActuatorInterface;
class Actuators;
class Decision;
class ControlInterface;
class Controller;

/**
 * @class FrameworkAgent FrameworkAgent.h
 * This is a class that can be used to create the basic system and
 * framework for an agent so the user do not have to create and
 * connect all the units herself. Only supply a decision object
 * and command line arguments with the necessary parameters to start
 * the agent. Now uses soccerserver coachlanguage parser and sends
 * this to the sensors. 
 *
 * @todo startAsCoach need to be fixed.
 * @todo separete coach and players FrameworkAgent?
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:01:07 $
 */
class FrameworkAgent {
public:
  /** Default construcor 
   *  Supply a decision object to make the agents decisions. */
  FrameworkAgent(Decision* decision);

  /** Destructor for the FrameworkAgent. */
  virtual ~FrameworkAgent();

  /** Start the agent. */
  virtual bool start();
  
protected:
  /** ViewManager isn't needed here I think
   *  The worldModelInterface should take care of the updates. */
  ViewManager* viewManager;

  /** The facts of the world, we need some parameters for initialization. */
  const WorldFacts *facts;

  /** Pointer to the server interface. */
  ServerInterface* serverInterface;

  /** Pointer to the server interface. */
  FrameworkSensorInterface* sensorInterface;

//    /** Coach message builder from the soccerserver. */
//    rcss::clang::MsgBuilder coachMessageBuilder;  

//    /** Coach message parser from soccerserver to parse coach data. */
//    rcss::clang::Parser coachMessageParser;
  CLangParser clangParser;

  /** Pointer to the agents sensors. */
  Sensors* sensors;

  /** Pointer to the acutator interface. */
  ActuatorInterface* actuatorInterface;

  /** Pointer to the agents acutators. */
  Actuators* actuators;

  /** Pointer to the agents decision handler. */
  Decision* decision;

  /** Pointer to the controll interface. */
  ControlInterface* controlInterface;

  /** Pointer to the agents controller. */
  Controller* controller;
};


RS_END_NAMESPACE


#endif // !FRAMEWORKAGENT_H
