/* -*- Mode: C++ -*-
 * $Id: GameView.cc,v 1.6 2003/07/21 08:58:59 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GameView.cc
 * Unitname: Framework
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 08:58:59 $
 *
 *
 * HISTORY
 *
 * $Log: GameView.cc,v $
 * Revision 1.6  2003/07/21 08:58:59  s02davro
 * Fixed playmode bug
 *
 * Revision 1.5  2002/09/04 14:43:16  s02davro
 * Added some allowed to GameView
 *
 * Revision 1.4  2002/08/02 21:55:28  s02davro
 * Now works by extracting information from the worldModelInterface and worldFacts.
 *
 * Revision 1.3  2001/01/11 22:00:20  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.2  2001/01/10 15:12:59  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.1  2000/09/21 13:46:46  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.2  2000/09/13 00:08:08  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/04 15:27:23  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file GameView.cc
 * Implementation file for the class GameView.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2003/07/21 08:58:59 $
 */


#include "GameView.h"


RS_BEGIN_NAMESPACE


GameView::GameView(const ViewId id)
  : View(id) {
}

GameView::~GameView() {
}

bool GameView::isTeleportationAllowed() {
  PlayMode pm = getPlayMode();
  if (pm == PM_BEFORE_KICK_OFF ||
      pm == PM_OUR_GOAL ||
      pm == PM_THEIR_GOAL) 
    return true;
  return false;
}
  
bool GameView::isDashAllowed() {
  PlayMode pm = getPlayMode();
  if (pm == PM_BEFORE_KICK_OFF ||
      pm == PM_HALF_TIME ||
      pm == PM_TIME_OVER ||
      pm == PM_PAUSE ||
      pm == PM_FIRST_HALF_OVER) 
    return false;
  return true;
}

bool GameView::isKickAllowed() {
  PlayMode pm = getPlayMode();
  if (pm == PM_BEFORE_KICK_OFF ||
      pm == PM_OUR_GOAL ||
      pm == PM_THEIR_GOAL ||
      pm == PM_OUR_GOALIE_GOT_BALL ||
      pm == PM_THEIR_GOALIE_GOT_BALL ||
      pm == PM_OUR_FOUL ||
      pm == PM_HALF_TIME ||
      pm == PM_TIME_OVER ||
      pm == PM_PAUSE ||
      pm == PM_FIRST_HALF_OVER ||
      pm == PM_THEIR_KICK_OFF ||
      pm == PM_THEIR_KICK_IN ||
      pm == PM_THEIR_FREE_KICK ||
      pm == PM_THEIR_CORNER_KICK ||
      pm == PM_THEIR_GOAL_KICK || 
      pm == PM_THEIR_OFFSIDE_KICK)
    return false;
  return true;
}

bool GameView::isGoalieTeleportationAllowed() {
  PlayMode pm = getPlayMode();
  if (isTeleportationAllowed() ||
      (pm == PM_OUR_GOALIE_GOT_BALL))
    return true;
  return false;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "GameView.icc"
#endif
