/* -*- Mode: C++ -*-
 * $Id: GameView.h,v 1.7 2002/09/04 14:43:16 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GameView.h
 * Unitname: Framework/view
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/09/04 14:43:16 $
 *
 *
 * HISTORY
 *
 * $Log: GameView.h,v $
 * Revision 1.7  2002/09/04 14:43:16  s02davro
 * Added some allowed to GameView
 *
 * Revision 1.6  2002/09/03 10:57:30  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.5  2002/08/31 15:27:26  s02davro
 * Updated the view
 *
 * Revision 1.4  2002/08/02 21:55:28  s02davro
 * Now works by extracting information from the worldModelInterface and worldFacts.
 *
 * Revision 1.3  2001/01/11 22:00:18  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.2  2001/01/10 15:12:58  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.1  2000/09/21 13:46:42  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.2  2000/09/13 00:08:08  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/04 15:27:23  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file GameView.h
 * Include file for the class GameView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.7 $
 * $Author: s02davro $
 * $Date: 2002/09/04 14:43:16 $
 */


#ifndef GAMEVIEW_H
#define GAMEVIEW_H

#include "View.h"
#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE

/**
 * @class GameView GameView.h
 * This view is used to keep track of the current game.
 * It includes things like the current game time,
 * the current play mode and so on.
 *
 * @todo More information?
 * @todo Is the view needed?
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.7 $
 * $Author: s02davro $
 * $Date: 2002/09/04 14:43:16 $
 */

class GameView : public View {
public:
  /** Construct a game view with ViewId @a id */
  GameView(const ViewId id);

  /** Destroys the game view. */
  virtual ~GameView();

  /** Get the current time according to the agent. */
  const Time& getCurrentTime() const;
  
  /** Get the next cycles time. */
  const Time getNextTime() const;

  /** Get the time for the last visual (see) information. */
  const TimeUnknown& getLastSeeTime() const;

  /** Get the time for the next visual (see) information. */
  const TimeUnknown& getNextSeeTime() const;

  /** Get the last play mode for the cycles that was @a hist cycles ago. */
  PlayMode getPlayMode(const Int hist = NOW) const;

  /** Get the playmodes for the specified cycle. */
  const std::vector<PlayMode>& getPlayModes(const Int hist = NOW) const;

  /** Get the side of the field that the agent is playing on. */
  Side getOurSide() const;

  /** Get the side of the field that the opponents are playing on. */
  Side getTheirSide() const;

  /** Get the score of the agent's team. */
  UInt16 getOurScore() const;

  /** Get the score for the opponents. */
  UInt16 getTheirScore() const;

  /** Get the teamname of the agent's team. */
  const std::string& getOurTeamName() const;

  /** Get the teamname of the opponents. */
  const std::string& getTheirTeamName() const;
  
  /** Returns true if teleportation is allowed in the current playmode. */
  bool isTeleportationAllowed();
  
  /** Returns true if dashing is allowed in the current playmode. */
  bool isDashAllowed();
  
  /** Returns true if kicking is allowed in the current playmode. */
  bool isKickAllowed();
  
  /** Returns true if goalieteleportation is allowed in the current playmode. */
  bool isGoalieTeleportationAllowed();
  
protected:
  //@{
  /** @name Update functions. */
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  //@}
private:
  /** Hidden copy-constructor. */
  GameView(const GameView&);

  /** Hidden assignment opeartor. */
  GameView& operator= (const GameView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "GameView.icc"
#endif


#endif // !GAMEVIEW_H
