/* -*- Mode: C++ -*-
 * $Id: HearStrategy.h,v 1.2 2002/08/31 15:24:37 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: HearStrategy.h
 * Unitname: Framework/strategy
 * $Revision: 1.2 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 15:24:37 $
 *
 *
 * HISTORY
 *
 * $Log: HearStrategy.h,v $
 * Revision 1.2  2002/08/31 15:24:37  s02davro
 * Changed the interface in the strategies
 *
 * Revision 1.1  2002/08/02 21:55:40  s02davro
 * Abstract class to make updates of the worldModel based on heard messages.
 *
 */

/**
 * @file HearStrategy.h
 * Include file for the class HearStrategy.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:24:37 $
 */


#ifndef HEARSTRATEGY_H
#define HEARSTRATEGY_H

#include "Strategy.h"

RS_BEGIN_NAMESPACE

/**
 * @class HearStrategy HearStrategy.h
 * Updates the world model when new information is heard.
 * This class makes updates of the worldModel based on the
 * new heard messages adaptive. Inherit and supply your own
 * implementation of the updateAfterHear function.
 *
 *
 * @par Unit:
 *      Framework/strategy
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:24:37 $
 */

class HearStrategy : public Strategy {
public:
  /** This function is called when we have heard a new message.
   *  Overload it to make changes in the worldModel based on this
   *  new information. */
  virtual void updateAfterHear() = 0;
};

RS_END_NAMESPACE

#endif // !HEARSTRATEGY_H
