/* -*- Mode: C++ -*-
 * $Id: InitStrategy.h,v 1.2 2002/08/31 15:24:37 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InitStrategy.h
 * Unitname: Framework/strategy
 * $Revision: 1.2 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 15:24:37 $
 *
 *
 * HISTORY
 *
 * $Log: InitStrategy.h,v $
 * Revision 1.2  2002/08/31 15:24:37  s02davro
 * Changed the interface in the strategies
 *
 * Revision 1.1  2002/08/02 21:56:00  s02davro
 * Abstract class to make updates of the worldModel based on initmessages.
 *
 */

/**
 * @file InitStrategy.h
 * Include file for the class InitStrategy.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:24:37 $
 */


#ifndef INITSTRATEGY_H
#define INITSTRATEGY_H

#include "Strategy.h"

RS_BEGIN_NAMESPACE

/**
 * @class InitStrategy InitStrategy.h
 * Updates the world model when a init message is recived.
 * This class makes updates of the worldModel based on the
 * initmessage recived adaptive. Inherit and supply your own
 * implementation of the updateAfterInit function.
 *
 * @par Unit:
 *      Framework/strategy
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:24:37 $
 */

class InitStrategy : public Strategy {
public:
  /** This function is called when we have heard a initmessage.
   *  Overload it to make changes in the worldModel based on this new 
   *  information. */
  virtual void updateAfterInit() = 0;
};

RS_END_NAMESPACE

#endif // !INITSTRATEGY_H
