/* -*- Mode: C++ -*-
 * $Id: NewCycleStrategy.h,v 1.4 2003/09/02 17:56:12 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: NewCycleStrategy.h
 * Unitname: Framework/strategy
 * $Revision: 1.4 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: frehe $ $Date: 2003/09/02 17:56:12 $
 *
 *
 * HISTORY
 *
 * $Log: NewCycleStrategy.h,v $
 * Revision 1.4  2003/09/02 17:56:12  frehe
 * Made the strategy const.
 *
 * Revision 1.3  2002/09/03 18:04:24  s02davro
 * Speed and NewCycle fix
 *
 * Revision 1.2  2002/08/31 15:24:37  s02davro
 * Changed the interface in the strategies
 *
 * Revision 1.1  2002/08/02 21:56:13  s02davro
 * Abstract class to make updates of the worldModel when a new cycle is issued.
 *
 */

/**
 * @file NewCycleStrategy.h
 * Include file for the class NewCycleStrategy.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2003/09/02 17:56:12 $
 */


#ifndef NEWCYCLESTRATEGY_H
#define NEWCYCLESTRATEGY_H

#include "Strategy.h"

RS_BEGIN_NAMESPACE

/**
 * @class NewCycleStrategy NewCycleStrategy.h
 * Updates the world model when a new cycle is issued.
 * This class makes updates of the worldModel based on the
 * new cycles adaptive. Inherit and supply your own
 * implementation of the updateAfterCoachMessage function.
 * This may be used to simulate futher worldModels in the future.
 *
 * @par Unit:
 *      Framework/strategy
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2003/09/02 17:56:12 $
 */

class NewCycleStrategy : public Strategy {
public:
  /** This function is called when a new cycle is issued.
   *  Overload it to make changes in the worldModel based on this new 
   *  information. */
  virtual void updateBeforeNewCycle(WorldModel& world) const = 0;
};

RS_END_NAMESPACE

#endif // !NEWCYCLESTRATEGY_H
