/* -*- Mode: C++ -*-
 * $Id: PlayerView.cc,v 1.6 2002/09/10 13:38:04 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayerView.cc
 * Unitname: Framework/view
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/09/10 13:38:04 $
 *
 *
 * HISTORY
 *
 * $Log: PlayerView.cc,v $
 * Revision 1.6  2002/09/10 13:38:04  s02davro
 * Added ClosestToBall and ClosestToAgent methods
 *
 * Revision 1.5  2002/08/02 21:56:39  s02davro
 * Now works by extracting information from the worldModel.
 *
 * Revision 1.4  2001/01/11 22:00:20  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.3  2001/01/10 15:12:59  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.2  2000/09/25 09:48:57  frehe
 * Fixed a bug in the CoachView.
 *
 * Revision 1.1  2000/09/21 13:46:48  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.2  2000/09/13 00:08:09  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/04 15:27:25  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file PlayerView.cc
 * Implementation file for the class PlayerView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2002/09/10 13:38:04 $
 */


#include "PlayerView.h"

RS_BEGIN_NAMESPACE

PlayerView::PlayerView(const ViewId id)
  : View(id) {
}

PlayerView::~PlayerView() {
}

PlayerObject PlayerView::getPlayerClosestToAgent(const Int hist) {
  PointUE agentPos = 
    worldModelInterface->getWorld(hist).getAgent().getPosition();
  PlayerObject teammate = getTeammateClosestToAgent(hist);
  PlayerObject opponent = getOpponentClosestToAgent(hist);
  
  if (VectorUE(agentPos, teammate.getPosition()) < 
      VectorUE(agentPos, opponent.getPosition())) 
    return teammate;
  return opponent;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PlayerView.icc"
#endif
