/* -*- Mode: C++ -*-
 * $Id: PlayerView.h,v 1.7 2003/09/28 20:00:25 aip10 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayerView.h
 * Unitname: Framework/view
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: aip10 $ $Date: 2003/09/28 20:00:25 $
 *
 *
 * HISTORY
 *
 * $Log: PlayerView.h,v $
 * Revision 1.7  2003/09/28 20:00:25  aip10
 * PassView.cc
 *
 * Revision 1.6  2002/09/10 13:38:04  s02davro
 * Added ClosestToBall and ClosestToAgent methods
 *
 * Revision 1.5  2002/08/31 15:28:47  s02davro
 * Modification of the view
 *
 * Revision 1.4  2002/08/02 21:56:39  s02davro
 * Now works by extracting information from the worldModel.
 *
 * Revision 1.3  2001/01/11 22:00:18  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.2  2001/01/10 15:12:58  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.1  2000/09/21 13:46:43  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.2  2000/09/13 00:08:10  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/04 15:27:26  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file PlayerView.h
 * Include file for the class PlayerView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.7 $
 * $Author: aip10 $
 * $Date: 2003/09/28 20:00:25 $
 */


#ifndef PLAYERVIEW_H
#define PLAYERVIEW_H

#include "View.h"
#include "HistoryDefault.h"
#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE

/**
 * @class PlayerView PlayerView.h
 * This view is used to keep track of the players,
 * their position, speed and other properties.
 *
 * @todo Speed up getClosest* by buffering the result in the view
 * @todo More information in the view.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.7 $
 * $Author: aip10 $
 * $Date: 2003/09/28 20:00:25 $
 */

class PlayerView : public View {
public:

  /** Create a player view with ViewId @a id */
  PlayerView(const ViewId id);

  /** Destroys the player view. */
  ~PlayerView();
 
 
  /** Get a pointer to the closest player @a hist cycles ago. */
  PlayerObject getPlayerClosestToAgent(const Int hist = NOW);

  /** Get a pointer to the closest opponent @a hist cycles ago. */
  PlayerObject getOpponentClosestToAgent(const Int hist = NOW);

  /** Get a pointer to the closest to the ball opponent @a hist cycles ago. */
  PlayerObject getOpponentClosestToBall(const Int hist = NOW);

  /** Get a pointer to the closest teammate @a hist cycles ago. */
  PlayerObject getTeammateClosestToAgent(const Int hist = NOW);

  /** Get a pointer to the closest to the ball teamamte @a hist cycles ago. */
  PlayerObject getTeammateClosestToBall(const Int hist = NOW);
    
  /** Get a vector with the opponents. */
  const std::vector<PlayerObject>& getOpponents(const Int hist = NOW) const;

  /** Get a vector with the teammates. */
  const std::vector<PlayerObject>& getTeammates(const Int hist = NOW) const;

  /** Get a vector with the unknown players. */
  const std::vector<PlayerObject>& getUnknownPlayers(const Int hist = NOW) const;
 
protected:
  //@{
  /** @name Update functions. */
  virtual void myUpdateAfterSee();
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  //@}
private:

  /** Hidden copy-constructor. */
  PlayerView(const PlayerView&);

  /** Hidden assignment opeartor. */
  PlayerView& operator= (const PlayerView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PlayerView.icc"
#endif


#endif // !PLAYERVIEW_H
