/* -*- Mode: C++ -*-
 * $Id: Skill.h,v 1.8 2005/09/08 08:58:25 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Skill.h
 * Unitname: Framework
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2005/09/08 08:58:25 $
 *
 *
 * HISTORY
 *
 * $Log: Skill.h,v $
 * Revision 1.8  2005/09/08 08:58:25  frehe
 * Added a reset method.
 *
 * Revision 1.7  2004/08/13 15:39:06  petan
 * Added generateStep
 *
 * Revision 1.6  2002/08/02 21:57:26  s02davro
 * Added some documentation and todos.
 *
 * Revision 1.5  2001/09/01 23:50:13  frehe
 * Added failed and succeeded.
 *
 * Revision 1.4  2001/08/23 00:32:04  frehe
 * Removed dependencies.
 *
 * Revision 1.3  2000/09/13 03:09:52  frehe
 * Added more documentation to the class.
 *
 * Revision 1.2  2000/09/05 11:09:14  frehe
 * Made the generatePlan method return a Plan instead of a Command*.
 *
 * Revision 1.1  2000/09/03 22:08:44  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Skill.h
 * Include file for the class Skill.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:25 $
 */

#ifndef SKILL_H
#define SKILL_H

#include "Plan.h"
#include "AgentStep.h"
#include "FuzzyBool.h"

RS_BEGIN_NAMESPACE

class ViewManager;

/**
 * @class Skill Skill.h
 * This class is the abstract base class for all skills in the RoboSoc system.
 *
 * @todo Use WorldModelInterface insted of ViewManager.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2005/09/08 08:58:25 $
 */
class Skill {
public:
  /** Constructs a skill with default behavior. */
  Skill();

  /** Destroys the skill. */
  virtual ~Skill();

  /** Reset any internal state in the skill. */
  virtual void reset() = 0;
  
  /** Generate a plan for how to accomplish the goal
    of the skill. */
  virtual Plan generatePlan() = 0;

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep() = 0;
  
  /** Return @c true (close to 1) if the skill would like
    to be called again. */
  virtual FuzzyBool persistent() = 0;

  /** Return @c true (close to 1) of the skill is applicable
    at the moment, i.e. its preconditions are satisfied. */
  virtual FuzzyBool applicable() = 0;  

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() = 0;

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() = 0;

protected:
  /** A pointer to the view manager. */
  ViewManager* const viewManager;
  
private:
  /** Hidden copy-constructor. */
  Skill(const Skill&);

  /** Hidden assignment operator. */
  Skill& operator= (const Skill&);
};


RS_END_NAMESPACE


#endif
