/* -*- Mode: C++ -*-
 * $Id: SkillWithDirectionAndCommand.h,v 1.1 2004/08/26 17:09:40 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SkillWithDirectionAndCommand.h
 * Unitname: Framework
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-08-26
 * Last modified by $Author: frehe $ $Date: 2004/08/26 17:09:40 $
 *
 *
 * HISTORY
 *
 * $Log: SkillWithDirectionAndCommand.h,v $
 * Revision 1.1  2004/08/26 17:09:40  frehe
 * Added SkillWithDirectionAndCommand.
 *
 */

/**
 * @file SkillWithDirectionAndCommand.h
 * Include file for the class SkillWithDirectionAndCommand.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2004/08/26 17:09:40 $
 */


#ifndef SKILL_WITH_DIRECTION_AND_COMMAND_H
#define SKILL_WITH_DIRECTION_AND_COMMAND_H


#include "SkillWithDirection.h"


RS_BEGIN_NAMESPACE


/**
 * @class SkillWithDirectionAndCommand SkillWithDirectionAndCommand.h
 * This abstract class represent skill that uses
 * a direction and a command as arguments.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2004/08/26 17:09:40 $
 */
class SkillWithDirectionAndCommand : public SkillWithDirection {
public:
  /** Constructor taking the default argument for the direction
      and the command used as arguments to the skill. */
  SkillWithDirectionAndCommand(const AngleDeg& dir,
			       ServerCommand* cmd);

  /** Destructor cleaning up after the skill. */
  virtual ~SkillWithDirectionAndCommand();


  /** Used to set the skills's arguments in a generic way.
      The first value in the vector will be interpreted as the
      direction. The command can not be set through this method. */
  virtual void setArguments(const std::vector<Float>& arguments)
    throw(TooFewArgumentsException);


  /** Used to set the skill's command argument to @a cmd. */
  virtual void setCommand(ServerCommand* cmd);

  /** Return the skill's command argument. */
  virtual ServerCommand* getCommand() const;


  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent. Abstract since
      no concrete instances of this class should be created. */
  virtual Plan generatePlan() = 0;

  /** This method is used to first set the argument of the skill
      and then generate a plan for how the goal of the skill should
      be obtained by the agent. It uses the methods setDirection and
      generatePlan(), so it should not be overriden unless it should
      made unavailable. */
  virtual Plan generatePlan(const AngleDeg& dir, ServerCommand* cmd);

  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. Abstract since
      no concrete instances of this class should be created. */
  virtual FuzzyBool persistent() = 0;

  /** This method should implement the precondition specified in
      the description of the skill. Abstract since no concrete instances
      of this class should be created. */
  virtual FuzzyBool applicable() = 0;


protected:
  /** The command argument. */
  ServerCommand* command;


private:
  /** Hide copy-constructor. */
  SkillWithDirectionAndCommand(const SkillWithDirectionAndCommand&);
  /** Hide assignment operator. */
  SkillWithDirectionAndCommand& operator= (const SkillWithDirectionAndCommand&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SkillWithDirectionAndCommand.icc"
#endif


#endif // !SKILL_WITH_DIRECTION_AND_COMMAND_H
