/* -*- Mode: C++ -*-
 * $Id: Strategy.h,v 1.2 2002/08/31 15:24:37 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Strategy.h
 * Unitname: Framework/strategy
 * $Revision: 1.2 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 15:24:37 $
 *
 *
 * HISTORY
 *
 * $Log: Strategy.h,v $
 * Revision 1.2  2002/08/31 15:24:37  s02davro
 * Changed the interface in the strategies
 *
 * Revision 1.1  2002/08/02 21:57:39  s02davro
 * Abstract base class for all worldModel updating strategies.
 *
 */

/**
 * @file Strategy.h
 * Include file for the class Strategy.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:24:37 $
 */

#ifndef STRATEGY_H
#define STRATEGY_H

#include "WorldModel.h"
#include "WorldObservation.h"
#include "WorldFacts.h"

RS_BEGIN_NAMESPACE

class WorldModelInterface;

/**
 * @class Strategy Strategy.h
 * This class is the abstract base class for all strategies.
 * Use the worldModelInterface pointer to get the agents worldModel
 * the observations. Modify it according to the strategy.
 * 
 * @todo More attribues?
 *
 * @par Unit:
 *      Framework/strategy
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/31 15:24:37 $
 */

class Strategy {
public:
  /** Construct a strategy
   *  Gets a pointer to the worldmodelinterfaces since all 
   *  strategies will need this. */
  Strategy();

  /** Destroys the strategy. */
  virtual ~Strategy();
protected:

  /** Pointer to the WorldModelInterface. 
   *  The strategies will use this to update the agents worldModel. */
  WorldModelInterface* wmi;

  /** The WorldFacts needed by some strategies to make some calculations. */
  const WorldFacts* facts;

  /** A worldObservation object 
   *  Used for easy acess to the agents last observation. */
  const WorldObservation* observation;

  /** A pointer to the worldModel the strategy is modifying. */
  WorldModel* world;
};

RS_END_NAMESPACE

#endif // !STRATEGY_H
