/* -*- Mode: C++ -*-
 * $Id: TrainerView.h,v 1.1 2004/08/27 14:58:45 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TrainerView.h
 * Unitname: Framework/view
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-08-26
 * Last modified by $Author: frehe $ $Date: 2004/08/27 14:58:45 $
 *
 *
 * HISTORY
 *
 * $Log: TrainerView.h,v $
 * Revision 1.1  2004/08/27 14:58:45  frehe
 * First version of the trainer view.
 *
 */

/**
 * @file TrainerView.h
 * Include file for the class TrainerView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2004/08/27 14:58:45 $
 */


#ifndef TRAINER_VIEW_H
#define TRAINER_VIEW_H


#include "View.h"
#include "WorldModelInterface.h"


RS_BEGIN_NAMESPACE


/**
 * @class TrainerView TrainerView.h
 * This view is used by a trainer to get information about the game.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2004/08/27 14:58:45 $
 */

class TrainerView : public View {
public:
  /** Construct a trainer view with ViewId @a id */
  TrainerView(const ViewId id);

  /** Destroys the trainer view. */
  virtual ~TrainerView();


  /** Get the score of the left team. */
  UInt16 getLeftScore() const;

  /** Get the score of the right team. */
  UInt16 getRightScore() const;

  /** Get the teamname of the left team. */
  const std::string& getLeftTeamName() const;

  /** Get the teamname of the right team. */
  const std::string& getRightTeamName() const;


protected:
  ///@{
  /// @name Update functions.
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  ///@}


private:
  /** Hidden copy-constructor. */
  TrainerView(const TrainerView&);

  /** Hidden assignment opeartor. */
  TrainerView& operator= (const TrainerView&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "TrainerView.icc"
#endif


#endif // !TRAINER_VIEW_H
