/* -*- Mode: C++ -*-
 * $Id: WorldModel.cc,v 1.11 2003/07/21 09:01:55 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WorldModel.cc
 * Unitname: BasicSystem
 * $Revision: 1.11 $
 * Created by: David Rosen 2002-06-18
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:01:55 $
 *
 *
 * HISTORY
 *
 * $Log: WorldModel.cc,v $
 * Revision 1.11  2003/07/21 09:01:55  s02davro
 * Time and log fix
 *
 * Revision 1.10  2002/12/07 17:27:08  s02davro
 * Added support for trainer
 *
 * Revision 1.9  2002/09/10 13:38:04  s02davro
 * Added ClosestToBall and ClosestToAgent methods
 *
 * Revision 1.8  2002/09/03 10:57:30  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.7  2002/08/31 15:31:17  s02davro
 * Updated the world model
 *
 * Revision 1.6  2002/08/02 21:58:45  s02davro
 * Created to store information about the agents world model.
 *

 */

/**
 * @file WorldModel.cc
 * Implementattion file for the class WorldModel.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.11 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:01:55 $
 */

#include "WorldModel.h"

RS_BEGIN_NAMESPACE

WorldModel::WorldModel() 
  : playModes(),
    opponents(),
    teammates(),
    unknowns(),
    gameTime(Time::currentTime),
    lastSeeTime(Time::currentTime),
    nextSeeTime(Time::currentTime + 1),
    nextNextSeeTime(Time::currentTime + 3) {    
}

WorldModel::WorldModel(const WorldModel& wm) {  
//    RS_LOG(LA_WORLDMODEL, "WorldModel copyconstructor.  ");
//    RS_LOG(LA_WORLDMODEL, "gameTime:        " << wm.getGameTime());
//    RS_LOG(LA_WORLDMODEL, "lastSeeTime:     " << wm.getLastSeeTime());
//    RS_LOG(LA_WORLDMODEL, "nextSeeTime:     " << wm.getNextSeeTime());
//    RS_LOG(LA_WORLDMODEL, "nextNextSeeTime: " << wm.getNextNextSeeTime());
//    RS_LOG(LA_WORLDMODEL, "ballobsTime:     " << wm.getBall().getObservationTime());
//    RS_LOG(LA_WORLDMODEL, "ballSobsTime:    " << wm.getBall().getSpeedObservationTime());
//    RS_LOG(LA_WORLDMODEL, "agentobsTime:     " << wm.getAgent().getObservationTime());
//    RS_LOG(LA_WORLDMODEL, "agentSobsTime:    " << wm.getAgent().getSpeedObservationTime());
  playModes.clear();
  opponents.clear();
  teammates.clear();
  unknowns.clear();
  closestOpponent = NULL;
  closestOpponentToBall = NULL;
  closestTeammate = NULL;
  closestTeammateToBall = NULL;
  for (std::vector<PlayMode>::const_iterator i = wm.getPlayModes().begin();
       i != wm.getPlayModes().end(); ++i)
    playModes.push_back(*i);
  for (std::vector<PlayerObject>::const_iterator i = wm.getOpponents().begin();
       i != wm.getOpponents().end(); ++i) {
    opponents.push_back(*i);
      if (&(*i) == wm.closestOpponent)
	closestOpponent = &opponents.back();
      if (&(*i) == wm.closestOpponentToBall)
	closestOpponentToBall = &opponents.back();
  }	  
  for (std::vector<PlayerObject>::const_iterator i = wm.getTeammates().begin();
       i != wm.getTeammates().end(); ++i) {
    teammates.push_back(*i);
      if (&(*i) == wm.closestTeammate)
	closestTeammate = &teammates.back();
      if (&(*i) == wm.closestTeammateToBall)
	closestTeammateToBall = &teammates.back();
  }	    
  for (std::vector<PlayerObject>::const_iterator i = wm.getUnknowns().begin();
       i != wm.getUnknowns().end(); ++i)
    unknowns.push_back(*i);
  ball = wm.getBall();
  agent = wm.getAgent();
  gameTime = wm.getGameTime();
  lastSeeTime = wm.getLastSeeTime();
  nextSeeTime = wm.getNextSeeTime();
  nextNextSeeTime = wm.getNextNextSeeTime();  
}

WorldModel::~WorldModel() {
}

WorldModel& WorldModel::operator= (const WorldModel& wm) {
//    RS_LOG(LA_WORLDMODEL, "WorldModel copyconstructor.  ");
//    RS_LOG(LA_WORLDMODEL, "gameTime:        " << wm.getGameTime());
//    RS_LOG(LA_WORLDMODEL, "lastSeeTime:     " << wm.getLastSeeTime());
//    RS_LOG(LA_WORLDMODEL, "nextSeeTime:     " << wm.getNextSeeTime());
//    RS_LOG(LA_WORLDMODEL, "nextNextSeeTime: " << wm.getNextNextSeeTime());
//    RS_LOG(LA_WORLDMODEL, "ballobsTime:     " << wm.getBall().getObservationTime());
//    RS_LOG(LA_WORLDMODEL, "ballSobsTime:    " << wm.getBall().getSpeedObservationTime());
//    RS_LOG(LA_WORLDMODEL, "agentobsTime:     " << wm.getAgent().getObservationTime());
//    RS_LOG(LA_WORLDMODEL, "agentSobsTime:    " << wm.getAgent().getSpeedObservationTime());
  playModes.clear();
  opponents.clear();
  teammates.clear();
  unknowns.clear();
  closestOpponent = NULL;
  closestOpponentToBall = NULL;
  closestTeammate = NULL;
  closestTeammateToBall = NULL;
  if (this != &wm) {
    for (std::vector<PlayMode>::const_iterator i = wm.getPlayModes().begin();
	 i != wm.getPlayModes().end(); ++i)
      playModes.push_back(*i);
    for (std::vector<PlayerObject>::const_iterator i = wm.getOpponents().begin();
	 i != wm.getOpponents().end(); ++i) {
      opponents.push_back(*i);
      if (&(*i) == wm.closestOpponent)
	closestOpponent = &opponents.back();
      if (&(*i) == wm.closestOpponentToBall)
	closestOpponentToBall = &opponents.back();
    }	
    for (std::vector<PlayerObject>::const_iterator i = wm.getTeammates().begin();
	 i != wm.getTeammates().end(); ++i) {
      teammates.push_back(*i);
      if (&(*i) == wm.closestTeammate)
	closestTeammate = &teammates.back();
      if (&(*i) == wm.closestTeammateToBall)
	closestTeammateToBall = &teammates.back();
    }	    
    for (std::vector<PlayerObject>::const_iterator i = wm.getUnknowns().begin();
	 i != wm.getUnknowns().end(); ++i)
      unknowns.push_back(*i);
    ball = wm.getBall();
    agent = wm.getAgent();
    gameTime = wm.getGameTime();
    lastSeeTime = wm.getLastSeeTime();
    nextSeeTime = wm.getNextSeeTime();
    nextNextSeeTime = wm.getNextNextSeeTime();      
  }
  return *this;
}

void WorldModel::addOpponent(const PlayerObject& opponent) {
  opponents.push_back(opponent);
}

std::vector<PlayerObject>& WorldModel::getOpponents() {
  return opponents;
}

const std::vector<PlayerObject>& WorldModel::getOpponents() const {
  return opponents;
}

const PlayerObject& WorldModel::getOpponentClosestToAgent() const {
  if (closestOpponent != NULL)
    return *closestOpponent;
  return emptyPlayer;
}

const PlayerObject& WorldModel::getOpponentClosestToBall() const {
  if (closestOpponentToBall != NULL)
    return *closestOpponentToBall;
  return emptyPlayer;
}

void WorldModel::setOpponentClosestToAgent(PlayerObject* player) {
  closestOpponent = player;
}

void WorldModel::setOpponentClosestToBall(PlayerObject* player) {
  closestOpponentToBall = player;
}

void WorldModel::clearOpponents() {
  opponents.clear();
}

void WorldModel::addTeammate(const PlayerObject& teammate) {
  teammates.push_back(teammate);
}

std::vector<PlayerObject>& WorldModel::getTeammates() {
  return teammates;
}

const std::vector<PlayerObject>& WorldModel::getTeammates() const {
  return teammates;
}

const PlayerObject& WorldModel::getTeammateClosestToAgent() const {
  if (closestTeammate != NULL)
    return *closestTeammate;
  return emptyPlayer;
}

const PlayerObject& WorldModel::getTeammateClosestToBall() const {
  if (closestTeammateToBall != NULL)
    return *closestTeammateToBall;
  return emptyPlayer;
}

void WorldModel::setTeammateClosestToAgent(PlayerObject* player) {
  closestTeammate = player;
}

void WorldModel::setTeammateClosestToBall(PlayerObject* player) {
  closestTeammateToBall = player;
}
 
void WorldModel::clearTeammates() {
  teammates.clear();
}

void WorldModel::addUnknown(const PlayerObject& unknown) {
  unknowns.push_back(unknown);
}

std::vector<PlayerObject>& WorldModel::getUnknowns() {
  return unknowns;
}

const std::vector<PlayerObject>& WorldModel::getUnknowns() const {
  return unknowns;
}

void WorldModel::clearUnknowns() {
  unknowns.clear();
}

void WorldModel::setBall(const BallObject& c) {
  ball = c;
}

BallObject& WorldModel::getBall() {
  return ball;
}

const BallObject& WorldModel::getBall() const {
  return ball;
}

void WorldModel::setAgent(const AgentObject& c) {
  agent = c;
}

AgentObject& WorldModel::getAgent() {
  return agent;
}

const AgentObject& WorldModel::getAgent() const {
  return agent;
}

void WorldModel::addPlayMode(const PlayMode& playMode) {
  playModes.push_back(playMode);
}

const std::vector<PlayMode>& WorldModel::getPlayModes() const {
  return playModes;
}

std::vector<PlayMode>& WorldModel::getPlayModes() {
  return playModes;
}

void WorldModel::clearPlayModes() {
  const PlayMode pm(getLastPlayMode());
  playModes.clear();
  playModes.push_back(pm);
}

const PlayMode WorldModel::getLastPlayMode() const {
  if (playModes.empty())
    return PM_UNKNOWN;
  return playModes.back();
}

const Time& WorldModel::getGameTime() const {
  return gameTime;
}

void WorldModel::setGameTime(Time time) {
  gameTime = time;
}

const TimeUnknown& WorldModel::getLastSeeTime() const {
  return lastSeeTime;
}

void WorldModel::setLastSeeTime(Time time) {
  lastSeeTime = time;
}
const TimeUnknown& WorldModel::getNextSeeTime() const {
  return nextSeeTime;
}

void WorldModel::setNextSeeTime(const TimeUnknown& time) {
  nextSeeTime = time;
}

const TimeUnknown& WorldModel::getNextNextSeeTime() const {
  return nextNextSeeTime;
}

void WorldModel::setNextNextSeeTime(const TimeUnknown& time) {
  nextNextSeeTime = time;
}

void WorldModel::addCommand(const ServerCommand* cmd) {
  commands.push_back(cmd);
}

const std::list<const ServerCommand*>& WorldModel::getCommands() const {
  return commands;
}

RS_END_NAMESPACE
