/* -*- Mode: C++ -*-
 * $Id: BasicCoachInformationStrategy.cc,v 1.6 2003/11/04 17:17:54 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicCoachInformationStrategy.cc
 * Unitname: BasicSystem
 * $Revision: 1.6 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: frehe $ $Date: 2003/11/04 17:17:54 $
 *
 *
 * HISTORY
 *
 * $Log: BasicCoachInformationStrategy.cc,v $
 * Revision 1.6  2003/11/04 17:17:54  frehe
 * Added support for the goalie information in the player object.
 *
 * Revision 1.5  2003/07/21 09:03:30  s02davro
 * Added support for pointing and tackling
 *
 * Revision 1.4  2002/12/07 18:02:25  s02davro
 * Player bug in CoachInfo
 *
 * Revision 1.3  2002/09/21 19:13:39  s02davro
 * Fix of ballspeed bug.
 *
 * Revision 1.2  2002/09/03 10:57:39  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.1  2002/08/31 16:16:17  s02davro
 * Added a strategy for CoachInformation
 *
 * Revision 1.1  2002/08/02 22:00:51  s02davro
 * Created the Basic classes, needs improvements.
 *
 */

/**
 * @file BasicCoachInformationStrategy.cc
 * Implementation file for the class BasicCoachInformationStrategy.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2003/11/04 17:17:54 $
 */

#include "BasicCoachInformationStrategy.h"
#include "WorldModelInterface.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

BasicCoachInformationStrategy::BasicCoachInformationStrategy() {
}

BasicCoachInformationStrategy::~BasicCoachInformationStrategy() { 
}

void BasicCoachInformationStrategy::updateAfterCoachInformation() {
  RS_LOG ( LA_STRATEGY, "BasicCoachInformationStrategy - begin");
  world = &wmi->getWorld();
  world->setGameTime(wmi->getCurrentTime());
  observation = &wmi->getObservation();  
  BallObject ball = world->getBall();
  TimeUnknown t = wmi->getCurrentTime();
  ball.setPosition(Point(observation->cball.x, observation->cball.y));
  ball.setObservationTime(t);
  ball.setSpeedVector(Vector(Point(observation->cball.dx, observation->cball.dy)));
  ball.setSpeedObservationTime(t);
  world->setBall(ball);
  world->clearTeammates();
  world->clearOpponents();
  world->clearUnknowns();
  for (std::vector<CoachPlayerObject>::const_iterator i = 
	 observation->cplayers.begin();
       i != observation->cplayers.end(); ++i) {
    PlayerObject p(Point(i->x, i->y), t, 
		   Vector(Point(i->dx, i->dy)), t, 
		   i->bodyDirection, i->neckDirection, 
		   i->pointDirection, i->tackling, i->goalie,
		   i->team, S_UNKNOWN, t, i->unum, t);
    if (i->team == T_OUR_TEAM) {
      p.setSide(facts->getOurSide());
      world->addTeammate(p);
    }
    else if (i->team == T_THEIR_TEAM) {
      p.setSide(facts->getTheirSide());
      world->addOpponent(p);
    }
    else {
      p.setSide(S_UNKNOWN);
      world->addUnknown(p);
    }
  }
  handleClosestPlayers();
  RS_LOG ( LA_STRATEGY, "BasicCoachInformationStrategy - end");
}

void BasicCoachInformationStrategy::handleClosestPlayers() {  
  // Set closest player to ball.
  PlayerObject *closestToBall = NULL;
  PointUE ballPos = world->getBall().getPosition();
  for (std::vector<PlayerObject>::iterator i = world->getTeammates().begin();
       i != world->getTeammates().end(); ++i) {
    if ((closestToBall == NULL) ||
	(closestToBall->getPosition().isUnknown()) ||
	(ballPos.getDistanceTo(i->getPosition()) <
	 ballPos.getDistanceTo(closestToBall->getPosition())))
      closestToBall = &(*i);
  }
  world->setTeammateClosestToBall(closestToBall);
  closestToBall = NULL;
  for (std::vector<PlayerObject>::iterator i = world->getOpponents().begin();
       i != world->getOpponents().end(); ++i) {
    if ((closestToBall == NULL) ||
	(closestToBall->getPosition().isUnknown()) ||
	(ballPos.getDistanceTo(i->getPosition()) <
	 ballPos.getDistanceTo(closestToBall->getPosition())))
      closestToBall = &(*i);
  }
  world->setOpponentClosestToBall(closestToBall);  
}

RS_END_NAMESPACE

