/* -*- Mode: C++ -*-
 * $Id: BasicHearStrategy.cc,v 1.5 2003/07/21 09:05:02 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicHearStrategy.cc
 * Unitname: BasicSystem
 * $Revision: 1.5 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:05:02 $
 *
 *
 * HISTORY
 *
 * $Log: BasicHearStrategy.cc,v $
 * Revision 1.5  2003/07/21 09:05:02  s02davro
 * New communication
 *
 * Revision 1.4  2002/09/21 19:13:24  s02davro
 * Fix of playmode bug.
 *
 * Revision 1.3  2002/09/04 13:50:05  s02davro
 * *** empty log message ***
 *
 * Revision 1.2  2002/08/31 15:35:24  s02davro
 * Updated the strategies and the interface
 *
 * Revision 1.1  2002/08/02 22:00:51  s02davro
 * Created the Basic classes, needs improvements.
 *
 */

/**
 * @file BasicHearStrategy.cc
 * Implementation file for the class BasicHearStrategy.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:05:02 $
 */

#include "BasicHearStrategy.h"
#include "WorldModelInterface.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

BasicHearStrategy::BasicHearStrategy() {
}

BasicHearStrategy::~BasicHearStrategy() { 
}

void BasicHearStrategy::updateAfterHear() {
  RS_LOG ( LA_STRATEGY, "BasicHearStrategy - start");
  world = &wmi->getWorld(); 
  observation = &wmi->getObservation();

  AgentObject agent = world->getAgent();  
  agent.setOwnMessage(observation->ownMessage);
  agent.setTeammateMessage(observation->teammateMessage);
  agent.setTeammateMessageDirection(observation->teammateMessageDirection);
  agent.setOpponentMessage(observation->opponentMessage);
  agent.setOpponentMessageDirection(observation->opponentMessageDirection);
  agent.setNumberOfOpponentSpeakers(observation->numberOfOpponentSpeakers);
  agent.setCoachMessage(observation->coachMessage);
  for (std::vector<UInt>::const_iterator i = observation->teammateSpeakers.begin();
       i != observation->teammateSpeakers.end(); ++i) 
    agent.addTeammateSpeaker(*i);
  world->setAgent(agent);
  world->clearPlayModes();
  for (std::vector<PlayMode>::const_iterator i = observation->playModes.begin();
       i != observation->playModes.end(); ++i) {
    world->addPlayMode(*i);
  }
  // TODO Handle messages for coach/trainer, information exist in obsWorld.chear.

  RS_LOG ( LA_STRATEGY, "BasicHearStrategy - end");
}

RS_END_NAMESPACE
