/* -*- Mode: C++ -*-
 * $Id: BasicNewCycleStrategy.cc,v 1.9 2003/12/02 16:17:01 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicNewCycleStrategy.cc
 * Unitname: BasicSystem
 * $Revision: 1.9 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: frehe $ $Date: 2003/12/02 16:17:01 $
 *
 *
 * HISTORY
 *
 * $Log: BasicNewCycleStrategy.cc,v $
 * Revision 1.9  2003/12/02 16:17:01  frehe
 * Added a check to see if the ball hasn't been seen for more than CP_cycles_before_ball_unknown cycles since the ball was seen and then set its position to unknown.
 *
 * Revision 1.8  2003/09/02 18:00:17  frehe
 * Updated the strategies to the new interface.
 *
 * Revision 1.7  2003/08/20 12:32:08  s02davro
 * Handle Arm and Tackle
 *
 * Revision 1.6  2003/07/21 09:05:31  s02davro
 * Fix to handle new communication, arm and point
 *
 * Revision 1.5  2002/09/04 07:55:40  s02davro
 * Speedfix
 *
 * Revision 1.4  2002/09/03 18:04:29  s02davro
 * Speed and NewCycle fix
 *
 * Revision 1.3  2002/09/03 10:57:39  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.2  2002/08/31 15:35:24  s02davro
 * Updated the strategies and the interface
 *
 * Revision 1.1  2002/08/02 22:00:52  s02davro
 * Created the Basic classes, needs improvements.
 *
 */

/**
 * @file BasicNewCycleStrategy.cc
 * Implementation file for the class BasicNewCycleStrategy.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2003/12/02 16:17:01 $
 */

#include "BasicNewCycleStrategy.h"
#include "WorldModelInterface.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

BasicNewCycleStrategy::BasicNewCycleStrategy() {
}

BasicNewCycleStrategy::~BasicNewCycleStrategy() { 
}

void BasicNewCycleStrategy::updateBeforeNewCycle(WorldModel &def_world) const {
  RS_LOG ( LA_STRATEGY, "BasicNewCycleStrategy - begin");  
  AgentObject agent = def_world.getAgent();
  BallObject ball = def_world.getBall();
  VectorUE moveSpeed;
  RS_LOG ( LA_STRATEGY, "Updateing game time from: " << def_world.getGameTime());  
  def_world.setGameTime(def_world.getGameTime() + 1);
  RS_LOG ( LA_STRATEGY, "                    to  : " << def_world.getGameTime());
  
  // Agent
  if (agent.getTurn().isKnown()) {
    agent.setBodyDirection(agent.getBodyDirection() + agent.getTurn());  
    agent.setNeckDirection(agent.getNeckDirection() + agent.getTurn());  
  }
  if (agent.getTurnNeck().isKnown()) {
    agent.setNeckDirection(agent.getNeckDirection() + agent.getTurnNeck());
  }
  if (agent.getSpeedVector().isKnown()) {
    moveSpeed = agent.getSpeedVector();
    if (agent.getAcceleration().isKnown()) {
      moveSpeed += agent.getAcceleration();
    }
    agent.setPosition(agent.getPosition() + moveSpeed);
    agent.setSpeedVector(moveSpeed * facts->SP_player_decay);
  }

  // Handle arm.
  if (agent.getArmExpiresIn() > 0) {    
    agent.setArmExpiresIn(agent.getArmExpiresIn() - 1);
    if (agent.getArmExpiresIn() == 0) 
      agent.setPointTarget(PointUnknown());
  }
  if (agent.getArmMoveableIn() > 0) 
    agent.setArmMoveableIn(agent.getArmMoveableIn() - 1);

  // Handle tackle.
  if (agent.getTackleExpiresIn() > 0) 
    agent.setTackleExpiresIn(agent.getTackleExpiresIn() - 1);

  // If the agent made a move command.
  if (agent.getMovePosition().isKnown()) {
    agent.setPosition(agent.getMovePosition());
    agent.setSpeedVector(VectorUE(0,0));
  }  
  
  // Ball
  if ( ball.getObservationTime().isKnown() &&
       def_world.getGameTime().getT() - ball.getObservationTime().getT() > facts->CP_cycles_before_unknown_ball ) {
    RS_LOG(LA_STRATEGY, "BallPosition set to unknown since I haven't seen it for "
	   << def_world.getGameTime().getT() << " - "
	   << ball.getObservationTime().getT() << " = "
	   << def_world.getGameTime().getT() - ball.getObservationTime().getS()
	   << " < " << facts->CP_cycles_before_unknown_ball << " cycles.");
    ball.setPosition(PointUE());
    ball.setSpeedVector(VectorUE());	  
    def_world.setBall(ball);
  }
  if (ball.getSpeedVector().isKnown()) {
    moveSpeed = ball.getSpeedVector();
    if (ball.getAcceleration().isKnown()) {
      moveSpeed += ball.getAcceleration(); 
    }   
    ball.setPosition(ball.getPosition() + moveSpeed);
    ball.setSpeedVector(moveSpeed * facts->SP_ball_decay);
  }
      
  // Players
  for (std::vector<PlayerObject>::iterator i = def_world.getTeammates().begin();
       i != def_world.getTeammates().end(); ++i) {
    if (i->getSpeedVector().isKnown()) {
      moveSpeed = i->getSpeedVector();
      if (i->getAcceleration().isKnown()) {
	moveSpeed += i->getAcceleration();
      }
      i->setPosition(i->getPosition() + moveSpeed);
      i->setSpeedVector(moveSpeed * facts->SP_player_decay);
    }
  }

  for (std::vector<PlayerObject>::iterator i = def_world.getOpponents().begin();
       i != def_world.getOpponents().end(); ++i) {
    if (i->getSpeedVector().isKnown()) {
      moveSpeed = i->getSpeedVector();
      if (i->getAcceleration().isKnown()) {
	moveSpeed += i->getAcceleration();
      }
      i->setPosition(i->getPosition() + moveSpeed);
      i->setSpeedVector(moveSpeed * facts->SP_player_decay);
    }
  }

  for (std::vector<PlayerObject>::iterator i = def_world.getUnknowns().begin();
       i != def_world.getUnknowns().end(); ++i) {
    if (i->getSpeedVector().isKnown()) {
      moveSpeed = i->getSpeedVector();
      if (i->getAcceleration().isKnown()) {
	moveSpeed += i->getAcceleration();
      }
      i->setPosition(i->getPosition() + moveSpeed);
      i->setSpeedVector(moveSpeed * facts->SP_player_decay);
    }
  }
	 
  
  // Clear Agent from movement.
  
  agent.setTurn(AngleDegUE());    
  agent.setTurnNeck(AngleDegUE());
  agent.setMovePosition(PointUnknown());
  agent.setAcceleration(VectorUE());

  // Clear communication.
  agent.clearCommunication();

  ball.setAcceleration(VectorUE());
  
  def_world.setAgent(agent);
  def_world.setBall(ball);
  RS_LOG ( LA_STRATEGY, "BasicNewCycleStrategy - end");
}

RS_END_NAMESPACE

