/* -*- Mode: C++ -*-
 * $Id: BasicSeeStrategy.cc,v 1.22 2003/12/07 17:05:38 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicSeeStrategy.cc
 * Unitname: BasicSystem
 * $Revision: 1.22 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: frehe $ $Date: 2003/12/07 17:05:38 $
 *
 *
 * HISTORY
 *
 * $Log: BasicSeeStrategy.cc,v $
 * Revision 1.22  2003/12/07 17:05:38  frehe
 * Added a missing test for unknown.
 *
 * Revision 1.21  2003/12/02 16:17:02  frehe
 * Added a check to see if the ball hasn't been seen for more than CP_cycles_before_ball_unknown cycles since the ball was seen and then set its position to unknown.
 *
 * Revision 1.20  2003/12/02 13:58:56  frehe
 * Added more debug information.
 *
 * Revision 1.19  2003/11/05 12:30:09  aip15
 * Applied the previous fix to the players speed vector as well.
 *
 * Revision 1.18  2003/11/04 17:19:27  frehe
 * Fixed an error in the speed vector calculation, suggested by aip15.
 *
 * Revision 1.17  2003/11/04 17:17:55  frehe
 * Added support for the goalie information in the player object.
 *
 * Revision 1.16  2003/10/06 09:58:53  aip19
 * Fixed silly printouts that crashed agent
 *
 * Revision 1.15  2003/10/02 15:38:12  s02davro
 * Removed improvment call that caused positioning problems
 *
 * Revision 1.14  2002/10/09 21:56:36  aip19
 * Bug fix for NARROW view width
 *
 * Revision 1.13  2002/10/09 13:29:46  aip19
 * Bugfix in TurnNeckCommand
 *
 * Revision 1.12  2002/10/08 17:00:05  aip6
 * More careful about setting the ball to unknown
 *
 * Revision 1.11  2002/09/30 19:08:34  s02davro
 * Speed based on positions is prefered over dir/distchange
 *
 * Revision 1.10  2002/09/26 15:25:23  s02davro
 * Corrected some bugs and added new speed calculations
 *
 * Revision 1.9  2002/09/23 10:30:43  s02davro
 * Removed some logmessages
 *
 * Revision 1.8  2002/09/21 20:31:22  s02davro
 * Fixed a bug in the speedcalculation, the ballspeed was somethimes set to NaN
 *
 * Revision 1.7  2002/09/21 19:12:46  s02davro
 * Now rotates the agent if we play on the right side of the field. Ballposition and speed is updated.
 *
 * Revision 1.6  2002/09/10 13:39:48  s02davro
 * Handles closest players
 *
 * Revision 1.5  2002/09/04 07:55:40  s02davro
 * Speedfix
 *
 * Revision 1.4  2002/09/03 18:04:29  s02davro
 * Speed and NewCycle fix
 *
 * Revision 1.3  2002/09/03 10:57:39  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.2  2002/08/31 15:35:25  s02davro
 * Updated the strategies and the interface
 *
 * Revision 1.1  2002/08/02 22:00:52  s02davro
 * Created the Basic classes, needs improvements.
 *
 */

/**
 * @file BasicSeeStrategy.cc
 * Implementation file for the class BasicSeeStrategy.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.22 $
 * $Author: frehe $
 * $Date: 2003/12/07 17:05:38 $
 */


#include "BasicSeeStrategy.h"
#include "WorldModelInterface.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

BasicSeeStrategy::BasicSeeStrategy() {
}

BasicSeeStrategy::~BasicSeeStrategy() { 
}

void BasicSeeStrategy::updateAfterSee() {
  RS_LOG ( LA_STRATEGY, "BasicSeeStrategy - start");
  world = &wmi->getWorld();
  observation = &wmi->getObservation();
  handleAgent();
  handleBall();
  handlePlayers();
  handleClosestPlayers();
  handleTime();
  RS_LOG ( LA_STRATEGY, "BasicSeeStrategy - end");
}

void BasicSeeStrategy::handleAgent() {  
  agent = world->getAgent();
  RS_LOG ( LA_STRATEGY, "  neckdir(1): " << agent.getNeckDirection());
  AngleDegUE neckdir = computeAgentNeckDirection();
  RS_LOG ( LA_STRATEGY, "  neckdir(2): " << neckdir);
//    neckdir.improve(agent.getNeckDirection());
  RS_LOG ( LA_STRATEGY, "  neckdir(3): " << neckdir);
  agent.setNeckDirection(neckdir);
  AngleDegUE bodydir = neckdir - observation->agent.relNeckDirection;
  RS_LOG ( LA_STRATEGY, "  bodydir(1): " << neckdir);
//    bodydir.improve(agent.getBodyDirection());
  RS_LOG ( LA_STRATEGY, "  bodydir(2): " << neckdir);
  agent.setBodyDirection(bodydir);
  PointUE agentPos = computeAgentPosition();  
  if (agentPos.isKnown()) {
    agent.setPosition(agentPos);
    agent.setObservationTime(wmi->getCurrentTime());
  }
  if (facts->getOurSide() == S_RIGHT) {
    // Rotate the agent.
    agent.setNeckDirection(agent.getNeckDirection()+180.0);
    agent.setBodyDirection(agent.getBodyDirection()+180.0);
    agent.setPosition(-agent.getPosition());
  }
  world->setAgent(agent);
}
 
AngleDegUE BasicSeeStrategy::computeAgentNeckDirection() const {

  // This is taken from ATHumboldt'97
  if ( observation->lines.size() == 0 ) {
    RS_LOG ( LA_STRATEGY, "Didn't see any lines! Face direction is unknown!");
    return AngleDegUE();
  } else {
    ServerLineObject closestLine = observation->lines[0];
    for (std::vector<ServerLineObject>::const_iterator i = observation->lines.begin();
	 i != observation->lines.end(); ++i)
      if (i->distance < closestLine.distance)
	closestLine = *i;
    AngleDegUE angle = closestLine.direction;
    switch (closestLine.lineId) {
    case SL_TOP:
      if (angle > AngleDeg(0) )
	return -angle;
      else 
	return 180.0 - angle;
      break;
    case SL_BOTTOM:
      if (angle > 0 ) 
	return 180.0 - angle;
      else 
	return -angle;
      break;
    case SL_LEFT:
      if (angle > 0)
	return AngleDeg(270) - angle;
      else if ( angle < 0 )
	return AngleDeg(90) - angle;
      else {
	RS_LOG ( LA_STRATEGY, "Couldn't compute face dir. Special case 1.");
	return AngleDegUE();
      }
      break;
    case SL_RIGHT:
      if ( angle > 0 )
	return AngleDeg(90) - angle;
      else if ( angle < 0 )
	return AngleDeg(270) - angle;
      else {
	RS_LOG( LA_STRATEGY, "Couldn't compute face dir. Special case 2.");
	return AngleDegUE();
      }
      break;
    default:
      RS_LOG ( LA_STRATEGY, "Didn't see any lines! Face direction is unknown!");
      return AngleDegUE();
    }
  }
  return AngleDegUE();
}


PointUE BasicSeeStrategy::computeAgentPosition() const {
  if ( observation->markers.size() == 0 ) {
    RS_LOG( LA_STRATEGY, "Didn't see any markers! Agent position is unknown!");
    return PointUE();
  } else {    
    ServerMarkerObject closestMarker = observation->markers[0];
    for (std::vector<ServerMarkerObject>::const_iterator i = 
	   observation->markers.begin(); i != observation->markers.end(); ++i)
      if (i->distance < closestMarker.distance)
	closestMarker = *i;
    PointUE markerPos = 
      facts->getMarker(closestMarker.markerId).getPosition();
    VectorUE rel_pos(closestMarker.direction + agent.getNeckDirection(),
		     closestMarker.distance);

    /*    if ((agent.getPosition().isKnown()) && 
	(agent.getPosition().getDistanceTo(markerPos - rel_pos) > 3.0)) {
      std::cout << "Agent moved (" 
		<< agent.getPosition().getDistanceTo(markerPos - rel_pos).getValue()
		<< ")" << std::endl;
      std::cout << "Agent oldpos (" << agent.getPosition() << ")" << std::endl;
      std::cout << "Agent newpos (" << (markerPos - rel_pos) << ")" << std::endl;      
      std::cout << "Agent neckdir (" << agent.getNeckDirection().getAngle() << ")" << std::endl;
      std::cout << "Agent bodydir (" << agent.getBodyDirection().getAngle() << ")" << std::endl;
      std::cout << "Positioning using marker: " << closestMarker.markerId 
		<< " direction: " << closestMarker.direction << " dist: "
		<< closestMarker.distance << std::endl;
      std::cout << "MarkerPos: (" << markerPos.getPoint().getX() 
		<< ", " << markerPos.getPoint().getY() << ")" << std::endl;
      if (rel_pos.isKnown())
        std::cout << "RelPos: (" << rel_pos.getVector().getLength() 
		  << ", " << rel_pos.getVector().getAngle() << ")" << std::endl;
		  }*/
    return markerPos - rel_pos;
  }
}

void BasicSeeStrategy::handlePlayers () {
  world->clearOpponents();
  world->clearTeammates();
  world->clearUnknowns();
  for (std::vector<ServerPlayerObject>::const_iterator i = 
	 observation->players.begin();
       i != observation->players.end(); ++i) {
    PlayerObject p(agent.getPosition() +
		   VectorUE(i->direction + agent.getNeckDirection(),
			    i->distance),
		   wmi->getCurrentTime());
    p.setBodyDirection(i->relBodyDirection + agent.getNeckDirection());
    p.setNeckDirection(i->relNeckDirection + agent.getNeckDirection());    
    p.setUniformNumber(i->uniformNumber);
    if (i->uniformNumber.isKnown()) 
      p.setUniformNumberObservationTime(wmi->getCurrentTime());    
    p.setTeam(i->team);
    p.setGoalie(i->goalie);
    if (i->distanceChange.isKnown() && i->directionChange.isKnown()) {
      // Setting the player's speed
      p.setSpeedObservationTime(wmi->getCurrentTime());
      PointUE agentPos(world->getAgent().getPosition());
      PointUE playerPos(p.getPosition());
      VectorUE agentToPlayer(agentPos, playerPos);
      PointUE relPos = playerPos - agentPos;
      PointUE unit = relPos / agentToPlayer.getLength();
      FloatUE tmp = i->directionChange.getRad() * agentToPlayer.getLength();
      VectorUE absVel(PointUE(i->distanceChange * unit.getX() - tmp*unit.getY(),
			      i->distanceChange * unit.getY() + tmp*unit.getX()));    
      // The vector is already relative...
      //absVel.rotate(world->getAgent().getNeckDirection());
      // We need to compensate for agent decay.?
      absVel += world->getAgent().getSpeedVector();
      p.setSpeedVector(absVel);
    }
    switch (i->team) {
    case T_OUR_TEAM:
      p.setTeamObservationTime(wmi->getCurrentTime());
      p.setSide(facts->getOurSide());
      world->addTeammate(p);
      break;
    case T_THEIR_TEAM:
      p.setTeamObservationTime(wmi->getCurrentTime());
      p.setSide(facts->getTheirSide());
      world->addOpponent(p);
      break;
    case T_UNKNOWN:
    default:
      p.setSide(S_UNKNOWN);
      world->addUnknown(p);
      break;
    }
  }
}  

void BasicSeeStrategy::handleClosestPlayers() {  
  // Set closest player to ball and agent.
  PlayerObject *closestToAgent = NULL;
  PlayerObject *closestToBall = NULL;
  PointUE agentPos = world->getAgent().getPosition();
  PointUE ballPos = world->getBall().getPosition();
  for (std::vector<PlayerObject>::iterator i = world->getTeammates().begin();
       i != world->getTeammates().end(); ++i) {
    if ((closestToAgent == NULL) ||
	(closestToAgent->getPosition().isUnknown()) ||
	(agentPos.getDistanceTo(i->getPosition()) <
	 agentPos.getDistanceTo(closestToAgent->getPosition())))
      closestToAgent = &(*i);
    if ((closestToBall == NULL) ||
	(closestToBall->getPosition().isUnknown()) ||
	(ballPos.getDistanceTo(i->getPosition()) <
	 ballPos.getDistanceTo(closestToBall->getPosition())))
      closestToBall = &(*i);
  }
  world->setTeammateClosestToAgent(closestToAgent);
  world->setTeammateClosestToBall(closestToBall);
  closestToAgent = NULL;
  closestToBall = NULL;
  for (std::vector<PlayerObject>::iterator i = world->getOpponents().begin();
       i != world->getOpponents().end(); ++i) {
    if ((closestToAgent == NULL) ||
	(closestToAgent->getPosition().isUnknown()) ||
	(agentPos.getDistanceTo(i->getPosition()) <
	 agentPos.getDistanceTo(closestToAgent->getPosition())))
      closestToAgent = &(*i);
    if ((closestToBall == NULL) ||
	(closestToBall->getPosition().isUnknown()) ||
	(ballPos.getDistanceTo(i->getPosition()) <
	 ballPos.getDistanceTo(closestToBall->getPosition())))
      closestToBall = &(*i);
  }
  world->setOpponentClosestToAgent(closestToAgent);
  world->setOpponentClosestToBall(closestToBall);  
}

void BasicSeeStrategy::handleBall() {
  RS_LOG(LA_STRATEGY, "handleBall");
  
  BallObject currentBall = world->getBall();
  BallObject ball = world->getBall();
  // The agent has seen the ball in the last cycle
  if (observation->ball.distance.isKnown()) {
    RS_LOG(LA_STRATEGY, "saw ball!");
    ball.setPosition(agent.getPosition() + 
		     VectorUE(observation->ball.direction + 
			      agent.getNeckDirection(),
			      observation->ball.distance));
    VectorUE speed1(calculateSpeed1(observation->ball.distanceChange,
				    observation->ball.directionChange,
				    ball.getPosition()));
    VectorUE speed2(calculateSpeed2(currentBall, ball));    
    if (speed1.isKnown() || speed2.isKnown()) {
      if (speed2.isKnown()) {
	RS_LOG(LA_STRATEGY, "BallSpeed (position) : " << Point(speed2.getVector()));
	ball.setSpeedVector(speed2);
      }
      if (speed1.isKnown()) {
	RS_LOG(LA_STRATEGY, "BallSpeed (dirchange): " << Point(speed1.getVector()));
	ball.setSpeedVector(speed1);
      }
      ball.setObservationTime(wmi->getCurrentTime());    
    }
  } else {
    if ( ball.getPosition().isKnown() ) {
      if ( agent.getPosition().isKnown() ) {
	// The agent didn't see the ball and the position of the ball is
	// known. If this wasn't expected we should set ball position is
	// unknown.
	AngleDegUE angleToBall = 
	  agent.getPosition().getDirectionTo(ball.getPosition());
	AngleDeg viewAngle = 0;
	switch (agent.getViewWidth()) {
	case VW_NARROW:
	  viewAngle = 22.5;
	  break;
	case VW_NORMAL:
	  viewAngle = 45;
	  break;
	case VW_WIDE:
	  viewAngle = 90;
	  break;
	default:
	  viewAngle = 45;
	}

	if ( ball.getObservationTime().isKnown() ) {
	  RS_LOG(LA_STRATEGY, "didn't see known ball and agent: angleToBall = "
		 << angleToBall << ", viewAngle = " << viewAngle
		 << ", neckAngle = " << agent.getNeckDirection()
		 << ", distToBall = "
		 << agent.getPosition().getDistanceTo(ball.getPosition())
		 << ", ballObsTimeT = " << ball.getObservationTime().getT()
		 << ", gameTimeT = " << world->getGameTime().getT());
	} else {
	  RS_LOG(LA_STRATEGY, "didn't see known ball and agent: angleToBall = "
		 << angleToBall << ", viewAngle = " << viewAngle
		 << ", neckAngle = " << agent.getNeckDirection()
		 << ", distToBall = "
		 << agent.getPosition().getDistanceTo(ball.getPosition()));
	}
    
	// Check the error values here.
	if (agent.getPosition().getDistanceTo(ball.getPosition()) < (facts->SP_feel_distance - 0.5)) {
	  RS_LOG(LA_STRATEGY, "BallPosition set to unknown (feel) belived distance: "
		 << agent.getPosition().getDistanceTo(ball.getPosition()));
	  if ( ball.getObservationTime().isKnown() )
	    RS_LOG(LA_STRATEGY, "ObservationTimespan: "
		   << world->getGameTime().getT() - ball.getObservationTime().getT()
		   << ", " << world->getGameTime().getS() - ball.getObservationTime().getS());
	  ball.setPosition(PointUE());
	  ball.setSpeedVector(VectorUE());
	} else if (abs(angleToBall - agent.getNeckDirection()) <= (viewAngle - 5)) {
	  // Check the error values here.
	  RS_LOG(LA_STRATEGY, "BallPosition set to unknown (angle) belived angle: "
		 << angleToBall - agent.getNeckDirection());
	  if (ball.getObservationTime().isKnown())
	    RS_LOG(LA_STRATEGY, "ObservationTimespan: "
		   << world->getGameTime().getT() - ball.getObservationTime().getT()
		   << ", " << world->getGameTime().getS() - ball.getObservationTime().getS());
	  ball.setPosition(PointUE());
	  ball.setSpeedVector(VectorUE());
	} else if ( ball.getObservationTime().isKnown() &&
		    world->getGameTime().getT() - ball.getObservationTime().getT() > facts->CP_cycles_before_unknown_ball ) {
	  RS_LOG(LA_STRATEGY, "BallPosition set to unknown since I haven't seen it for "
		 << world->getGameTime().getT() << " - "
		 << ball.getObservationTime().getT() << " = "
		 << world->getGameTime().getT() - ball.getObservationTime().getT()
		 << " < " << facts->CP_cycles_before_unknown_ball
		 << " cycles.");
	  ball.setPosition(PointUE());
	  ball.setSpeedVector(VectorUE());	  
	}
      } else {
	// Ball is known but agent is unknown
	RS_LOG(LA_STRATEGY, "didn't see known ball and unknown agent");
      }
    } else {
      // Ball is already unknown
      RS_LOG(LA_STRATEGY, "didn't see unknown ball");
    }
  }
  world->setBall(ball);
}


void BasicSeeStrategy::handleTime() {
  ViewWidth vw = world->getAgent().getViewWidth();
  ViewQuality vq = world->getAgent().getViewQuality();
  
  // Imported from WorldStateView.
  if (vw == VW_NARROW || (vw == VW_NORMAL && vq == VQ_LOW)) {
    // see info every cycle
    world->setNextSeeTime(world->getGameTime() + 1);
  } else if((vw == VW_NORMAL && vq == VQ_HIGH) ||
	    (vw == VW_WIDE && vq == VQ_LOW)) {
    // see info 2 of 3 cycles
    if(world->getLastSeeTime() == world->getGameTime() - 1) {
      world->setNextSeeTime(world->getGameTime() + 2);
      world->setNextNextSeeTime(world->getNextSeeTime() + 1);
    } else {
      world->setNextSeeTime(world->getGameTime() + 1);
      world->setNextNextSeeTime(world->getNextSeeTime() + 2);
    }
  } else if(vw == VW_WIDE && vq == VQ_HIGH) {
    // see info every 3rd cycle
    world->setNextSeeTime(world->getGameTime() + 3);
    world->setNextNextSeeTime(world->getNextSeeTime() + 3);
  } else {
    // huh? if something is unknown... should not happen
    world->setNextSeeTime(TimeUnknown());
    world->setNextNextSeeTime(TimeUnknown());
  }
  world->setLastSeeTime(world->getGameTime());
}

VectorUE BasicSeeStrategy::calculateSpeed2(const BallObject& cBall,
					   const BallObject& ball) {
  VectorUE absVel;
  if (cBall.getPosition().isKnown() && 
      cBall.getObservationTime().isKnown()) { 
    UInt currentTime = wmi->getCurrentTime().getT();
    UInt lastSeen = cBall.getObservationTime().getT();
    UInt seenCyclesAgo = currentTime - lastSeen;
    if (seenCyclesAgo <= 4 && seenCyclesAgo > 0 &&
	wmi->getWorld(-seenCyclesAgo).getBall().getPosition().isKnown()) {	
      // Set speed based on last positionestimation.
      // Check if the ball might have been moved.
      absVel = VectorUE(wmi->getWorld(-seenCyclesAgo).getBall().getPosition(),
			ball.getPosition());
      absVel /= seenCyclesAgo;
      if (absVel.getLength() > facts->SP_ball_speed_max) {
	// The ball might have been moved, set speed to unknown.
	absVel = VectorUE();
      }
    }
  }
  return absVel;
}

VectorUE BasicSeeStrategy::calculateSpeed1(FloatUE distanceChange, 
					   AngleDegUE directionChange,
					   PointUE ballPos) {
  VectorUE absVel;
  if (distanceChange.isKnown() && directionChange.isKnown()) {
    PointUE agentPos(world->getAgent().getPosition());
    VectorUE agentToBall(agentPos, ballPos);
    PointUE relPos = ballPos - agentPos;      
    PointUE unit = relPos / agentToBall.getLength();
    FloatUE tmp = directionChange.getRad() * agentToBall.getLength();
    absVel = VectorUE(PointUE(distanceChange * unit.getX() - tmp*unit.getY(),
			      distanceChange * unit.getY() + tmp*unit.getX()));
    // The vector is already relative...
    //absVel.rotate(world->getAgent().getNeckDirection());
    // We need to compensate for agent decay.?
    absVel += world->getAgent().getSpeedVector();
  }
  return absVel;
}



RS_END_NAMESPACE




