/* -*- Mode: C++ -*-
 * $Id: BasicSeeStrategy.h,v 1.5 2002/09/26 15:25:23 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicSeeStrategy.h
 * Unitname: Framework/strategy
 * $Revision: 1.5 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: s02davro $ $Date: 2002/09/26 15:25:23 $
 *
 *
 * HISTORY
 *
 * $Log: BasicSeeStrategy.h,v $
 * Revision 1.5  2002/09/26 15:25:23  s02davro
 * Corrected some bugs and added new speed calculations
 *
 * Revision 1.4  2002/09/10 13:39:48  s02davro
 * Handles closest players
 *
 * Revision 1.3  2002/09/03 10:57:39  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.2  2002/08/31 15:35:25  s02davro
 * Updated the strategies and the interface
 *
 * Revision 1.1  2002/08/02 22:00:52  s02davro
 * Created the Basic classes, needs improvements.
 *
 */

/**
 * @file BasicSeeStrategy.h
 * Include file for the class BasicSeeStrategy.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/09/26 15:25:23 $
 */


#ifndef BASICSEESTRATEGY_H
#define BASICSEESTRATEGY_H

#include "basic_system_types.h"
#include "SeeStrategy.h"

RS_BEGIN_NAMESPACE

/**
 * @class BasicSeeStrategy BasicSeeStrategy.h
 *
 * @par Unit:
 *      Framework/strategy
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/09/26 15:25:23 $
 */

class BasicSeeStrategy : public SeeStrategy {
public:
  BasicSeeStrategy();
  virtual ~BasicSeeStrategy();
  virtual void  updateAfterSee();
protected:
  virtual AngleDegUE computeAgentNeckDirection() const;
  virtual PointUE computeAgentPosition() const;
  virtual void handleAgent();
  virtual void handlePlayers();
  virtual void handleClosestPlayers();
  virtual void handleBall();
  virtual void handleTime();
  AgentObject agent;
  virtual VectorUE calculateSpeed1(FloatUE distanceChange,
				   AngleDegUE directionChange,
				   PointUE ballPos);
  virtual VectorUE calculateSpeed2(const BallObject& cball,
				   const BallObject& ball);
};

RS_END_NAMESPACE

#endif // !BASICSEESTRATEGY_H
