/* -*- Mode: C++ -*-
 * $Id: BasicSenseStrategy.cc,v 1.6 2004/09/05 19:38:25 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicSenseStrategy.cc
 * Unitname: BasicSystem
 * $Revision: 1.6 $
 * Created by: David Rosn 2002-07-01
 * Last modified by $Author: frehe $ $Date: 2004/09/05 19:38:25 $
 *
 *
 * HISTORY
 *
 * $Log: BasicSenseStrategy.cc,v $
 * Revision 1.6  2004/09/05 19:38:25  frehe
 * Improved support for focus.
 *
 * Revision 1.5  2003/08/20 12:32:08  s02davro
 * Handle Arm and Tackle
 *
 * Revision 1.4  2002/09/03 18:04:30  s02davro
 * Speed and NewCycle fix
 *
 * Revision 1.3  2002/09/03 10:57:39  s02davro
 * LastSeeTime, NextSeeTime, NextNextSeeTime
 *
 * Revision 1.2  2002/08/31 15:35:25  s02davro
 * Updated the strategies and the interface
 *
 * Revision 1.1  2002/08/02 22:00:53  s02davro
 * Created the Basic classes, needs improvements.
 *
 */

/**
 * @file BasicSenseStrategy.cc
 * Implementation file for the class BasicSenseStrategy.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2004/09/05 19:38:25 $
 */

#include "BasicSenseStrategy.h"
#include "WorldModelInterface.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

BasicSenseStrategy::BasicSenseStrategy() {
}

BasicSenseStrategy::~BasicSenseStrategy() { 
}

void BasicSenseStrategy::updateAfterSense() {
  RS_LOG ( LA_STRATEGY, "BasicSenseStrategy - start");
  world = &wmi->getWorld();
  world->setGameTime(wmi->getCurrentTime());
  observation = &wmi->getObservation();
  AgentObject agent = world->getAgent();
  agent.setViewQuality(observation->agent.viewQuality);
  agent.setViewWidth(observation->agent.viewWidth);
  agent.setStamina(observation->agent.stamina);
  agent.setEffort(observation->agent.effort);
  agent.setNeckDirection(agent.getBodyDirection() + 
			 observation->agent.relNeckDirection);
  agent.setDashes(observation->agent.dashes);
  agent.setKicks(observation->agent.kicks);
  agent.setSays(observation->agent.says);
  agent.setTurns(observation->agent.turns);
  agent.setTurnNecks(observation->agent.turnNecks);
  agent.setAttentionTos(observation->agent.focusState.focusCount);
  agent.setTackles(observation->agent.tackleState.tackleCount);
  agent.setPointTos(observation->agent.armState.count);
  
  agent.setSpeedVector(VectorUE(observation->agent.speedDirection + 
				agent.getBodyDirection(),
				observation->agent.speedAmount));
  agent.setArmMoveableIn(observation->agent.armState.moveableIn);
  agent.setArmExpiresIn(observation->agent.armState.expiresIn);
  agent.setPointTarget(observation->agent.armState.target);
  if ( observation->agent.focusState.focusSide == facts->getOurSide() ) {
    agent.setFocusedTeam(T_OUR_TEAM);
  } else if ( observation->agent.focusState.focusSide == facts->getTheirSide() ) {
    agent.setFocusedTeam(T_THEIR_TEAM);
  } else {
    agent.setFocusedTeam(T_UNKNOWN);
  }
  agent.setFocusedUniformNumber(observation->agent.focusState.focusUniformNr);
  agent.setTackleExpiresIn(observation->agent.tackleState.tackleExpiresIn);
  
  world->setAgent(agent);
  RS_LOG ( LA_STRATEGY, "BasicSenseStrategy - end");
}

RS_END_NAMESPACE

