/* -*- Mode: C++ -*-
 * $Id: FieldStrategy.cc,v 1.4 2004/01/16 12:49:16 aip10 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FieldStrategy.cc
 * Unitname: BasicSystem
 * $Revision: 1.4 $
 * Created by: aip10 2003/11/10
 * Last modified by $Author: aip10 $ $Date: 2004/01/16 12:49:16 $
 *
 *
 * HISTORY
 *
 * $Log: FieldStrategy.cc,v $
 * Revision 1.4  2004/01/16 12:49:16  aip10
 * changed descriptions
 *
 * Revision 1.3  2004/01/12 12:03:41  aip10
 * New strategy commited
 *

 */

/**
 * @file FieldStrategy.cc
 * Implementation file for the class FieldStrategy
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.4 $
 * $Author: aip10 $
 * $Date: 2004/01/16 12:49:16 $
 */

#include "FieldStrategy.h"
#include <math.h>
#include <iostream>

RS_BEGIN_NAMESPACE

const int fieldSizeX = 105;
const int fieldSizeY = 68;


//Constructor
FieldStrategy::FieldStrategy(const string& name_, float xSize_, float ySize_)
  : name(name_),
    xSize(xSize_),
    ySize(ySize_)
{
	numZonesX =(int) ceil(fieldSizeX/xSize);
	numZonesY =(int) ceil(fieldSizeY/ySize);
	numZones = numZonesX * numZonesY;
	relations = new float*[numZones];
	for (int i=0; i<numZones; i++)
	  relations[i] = new float[numZones+1];
	for(int i=0; i< numZones; i++){
		for(int j=0; j<=numZones; j++){
			setRelation(i,j,0.5);
		}
	}
}// end constructor

//Destructor
FieldStrategy::~FieldStrategy(){
}//end destructor

//getAverageRelation
float FieldStrategy::getAverageRelation(int zone){
	
	return relations[zone][numZones];

}

//getBestZone
float FieldStrategy::getBestZone(Point us, Point *bestLeft, Point *bestRight){

        RS_LOG (LA_DECISION,"Entered getbesZone");
	int zone = getZone(us);
	 RS_LOG (LA_DECISION,"Got zone");
	float best=0;
	int bestZone=0;

	for(int i=0; i<numZones; i++){
	  
		if(relations[zone][i]>=best){
			best=relations[zone][i];

			 RS_LOG (LA_DECISION,"GotRelation");
			bestZone=i;
		}
	}
	getPoints(bestZone, &bestLeft,&bestRight);
	return best;

}// end getBestZone

// getName
string FieldStrategy::getName(){
	return name;
}

// getNumZones
int FieldStrategy::getNumZones(){
	return numZones;
}

// getXSize
float FieldStrategy::getXSize(){
	return xSize;
}// end getXSize


// getYSize
float FieldStrategy::getYSize(){
	return ySize;
}// end getYSize


// getRelation
float FieldStrategy::getRelation(Point us, Point them){

	int zoneUs, zoneThem;

	zoneUs = getZone(us);
	zoneThem = getZone(them);
 
	return relations[zoneUs][zoneThem];
}// end getRelation


// getRelation
float FieldStrategy::getRelation(int from, int to){
	return relations[from][to];
}


// setRelation
void FieldStrategy::setRelation(int zone, int target, float value){
	float old = relations[zone][target];
	relations[zone][target]=value;
	float total = relations[zone][numZones] * numZones;
	total = total-old+relations[zone][target];
	relations[zone][numZones] = total/numZones;
	
}


//////////////////////////////////PRIVATE//////////////////////////////////////////
/*********************************************************************************/

// getPoints
void FieldStrategy::getPoints(int zone, Point **left, Point **right){
	int y = zone%numZonesY;
	int x = zone/numZonesY;
	(*left)->setX(x*xSize-fieldSizeX/2);
	(*left)->setY((y+1)*ySize-fieldSizeY/2);
	(*right)->setX((x+1)*xSize-fieldSizeX/2);
	(*right)->setY(y*ySize-fieldSizeY/2);
	//what if outside of field???

}// end getPoints

// getZone
int FieldStrategy::getZone(Point us){
	double x = us.getX()+(fieldSizeX/2);
	double y = us.getY()+(fieldSizeY/2);
	return (int)(ceil(x/xSize-1)*numZonesY+ceil(y/ySize-1));
}// end getZone


RS_END_NAMESPACE
