/* -*- Mode: C++ -*-
 * $Id: SkillStub.cc,v 1.7 2004/08/30 07:28:50 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SkillStub.cc
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: petan $ $Date: 2004/08/30 07:28:50 $
 *
 *
 * HISTORY
 *
 * $Log: SkillStub.cc,v $
 * Revision 1.7  2004/08/30 07:28:50  petan
 * Fixed AgentStep.
 *
 * Revision 1.6  2003/08/29 12:31:15  petan
 * Changed comments to clarify usage of succeed and fail
 *
 * Revision 1.5  2002/09/13 14:48:56  s02davro
 * FALSE -> FuzzyBool(0.0), TRUE -> FuzzyBool(1.0)
 *
 * Revision 1.4  2001/09/06 13:13:39  andbo928
 * Added class specifier to a lot of methods. Added succeed and failed methods to cc file.
 *
 * Revision 1.3  2000/09/16 18:39:39  frehe
 * Fixed the log messages.
 *
 * Revision 1.2  2000/09/16 18:36:48  frehe
 * Made minor fixes the skill stubs.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file SkillStub.cc
 * Implementation file for the skill class SkillStub.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: petan $
 * $Date: 2004/08/30 07:28:50 $
 */


#include "SkillStub.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

SkillStub::SkillStub()
  // This is an initializer list
  // which means that all the objects inside the
  // class can be initialized after the colon.
  // You initialize the variable myVar with
  // myVar(constructor arguments).
  // You should also initialize the base classes, but
  // only the classes that you directly inherit from.
  // If you do not initialize a variable its default
  // constructor will be called.
  : Skill()
{
}


// --- DESTRUCTOR ---

SkillStub::~SkillStub()
{
}


// --- METHODS ---

Plan SkillStub::generatePlan()
{
  // Return an empty plan.
  return Plan();
}

AgentStep SkillStub::generateStep()
{
  // Return an empty step-
  return AgentStep();
}

FuzzyBool SkillStub::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool SkillStub::applicable()
{
  // Always applicable.
  return FuzzyBool(1.0);
}

FuzzyBool SkillStub::succeed()
{
  // The skill has not succeeded yet.
  return FuzzyBool(0.0);
}

FuzzyBool SkillStub::failed()
{
  // The skill has not failed yet.
  return FuzzyBool(0.0);
}



RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "SkillStub.icc"
#endif
// To here if you do not have any inline definitions!
