/* -*- Mode: C++ -*-
 * $Id: SkillStub.h,v 1.7 2004/08/30 07:28:50 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SkillStub.h
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: petan $ $Date: 2004/08/30 07:28:50 $
 *
 *
 * HISTORY
 *
 * $Log: SkillStub.h,v $
 * Revision 1.7  2004/08/30 07:28:50  petan
 * Fixed AgentStep.
 *
 * Revision 1.6  2003/09/02 18:00:41  frehe
 * Changed a erroneous name.
 *
 * Revision 1.5  2001/09/06 13:13:39  andbo928
 * Added class specifier to a lot of methods. Added succeed and failed methods to cc file.
 *
 * Revision 1.4  2001/09/01 14:45:12  frehe
 * Updated the skill stub to the new abstract Skill class.
 *
 * Revision 1.3  2000/09/16 18:39:39  frehe
 * Fixed the log messages.
 *
 * Revision 1.2  2000/09/16 18:36:48  frehe
 * Made minor fixes the skill stubs.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file SkillStub.h
 * Include file for the skill class SkillStub.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: petan $
 * $Date: 2004/08/30 07:28:50 $
 */


#ifndef SKILLSTUB_H
#define SKILLSTUB_H


#include "Skill.h"


RS_BEGIN_NAMESPACE


/**
 * @class SkillStub SkillStub.h
 * The first sentence is a brief description of the skill class SkillStub.
 * The rest is a longer description of the skill class SkillStub.
 * Use it to describe the skill!
 *
 * @pre The preconditions for the skill are ...
 * @post The intended postconditions for the skill are...
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: petan $
 * $Date: 2004/08/30 07:28:50 $
 */
class SkillStub : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  SkillStub();

  /** Destructor should clean up after the skill. */
  virtual ~SkillStub();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** Generate a step for how to accomplish the goal
      of the skill. */
  virtual AgentStep generateStep();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

  
private:
  /** Hide copy-constructor. */
  SkillStub(const SkillStub&);
  /** Hide assignment operator. */
  SkillStub& operator= (const SkillStub&);
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SkillStub.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !SKILLSTUB_H
