/* -*- Mode: C++ -*-
 * $Id: TeamDecision.cc,v 1.8 2005/09/22 09:51:32 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TeamDecision.cc
 * Unitname: Framework/template/Team
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 2000-09-16
 * Last modified by $Author: frehe $ $Date: 2005/09/22 09:51:32 $
 */

/**
 * @file TeamDecision.cc
 * Implementation of the class TeamDecision.
 *
 * @par Unit:
 *      Framework/template/Team
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2005/09/22 09:51:32 $
 */

#include "TeamDecision.h"
#include "GameView.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "ActuatorInterface.h"
#include "ViewManager.h"
#include "view_constants.h"
#include "InitCommand.h"
#include "rs_assert.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE


TeamDecision::TeamDecision(ActuatorInterface* const actuator_interface)
  : Decision(actuator_interface),
    viewManager(ViewManager::instance()),
    worldModelInterface(WorldModelInterface::instance()),
    facts(worldModelInterface->getWorldFacts()),
    gameView(NULL),
    ballView(NULL),
    agentView(NULL),
    playerView(NULL)
{
  getView(gameView, GAME_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}


TeamDecision::~TeamDecision()
{
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}


ServerCommand* TeamDecision::initAgent()
{
  InitCommand* init_cmd = new InitCommand(facts->CP_team_name, 
					  facts->SP_version, 
					  facts->CP_goalie);
  return init_cmd;
}


bool TeamDecision::start()
{
  RS_REQUIRE( initialized == true );
  return true;
}


bool TeamDecision::decide()
{
  return false;
}


void TeamDecision::onAuralCoachSensorData()
{
}


void TeamDecision::onAuralRefereeSensorData()
{
}


void TeamDecision::onAuralPlayerSensorData()
{
}


void TeamDecision::onAuralSelfSensorData()
{
}


void TeamDecision::onInit()
{
  // Position the players with some distance at the beginning of game.
  Point movePoint = Point(-10,-10);
  if (agentView->getUniformNumber().isKnown()) {
    UInt nr = agentView->getUniformNumber().getValue();
    switch (nr) {      
    case 1:
      movePoint = Point(-10, 0);
      break;
    case 2:
      movePoint = Point(-5, -15);
      break;
    case 3:
      movePoint = Point(-5, 15);
      break;
    case 4:
      movePoint = Point(-25, 10);
      break;
    case 5:
      movePoint = Point(-25, -10);
      break;
    case 6:
      movePoint = Point(-25, 20);
      break;
    case 7:
      movePoint = Point(-25, -20);
      break;
    case 8:
      movePoint = Point(-45, 10);
      break;
    case 9:
      movePoint = Point(-25, -10);
      break;
    case 10:
      movePoint = Point(-25, 20);
      break;
    case 11:
      movePoint = Point(-25, -20);
      break;
    default:
      movePoint = Point(-10,-10);
    }
  }
  MoveCommand* move_cmd = new MoveCommand(movePoint);
  actuatorInterface->addToPlan(move_cmd);
}


void TeamDecision::onCommandWarning()
{
  decide();
}


RS_END_NAMESPACE
