/* -*- Mode: C++ -*-
 * $Id: TeamDecision.h,v 1.5 2005/09/22 09:51:32 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TeamDecision.h
 * Unitname: Framework/template/Team
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-09-16
 * Last modified by $Author: frehe $ $Date: 2005/09/22 09:51:32 $
 */

/**
 * @file TeamDecision.h
 * Include file for the class TeamDecision.
 *
 * @par Unit:
 *      Framework/template/Team
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/22 09:51:32 $
 */


#ifndef TEAMDECISION_H
#define TEAMDECISION_H


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE

class ViewManager;
class GameView;
class BallView;
class AgentView;
class PlayerView;


/**
 * @class TeamDecision TeamDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2005/09/22 09:51:32 $
 */
class TeamDecision : public Decision {
public:
  TeamDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~TeamDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  virtual bool decide();

  // Event handlers
  //virtual void onActuatorSensorData();
  virtual void onAuralCoachSensorData();
  virtual void onAuralRefereeSensorData();
  virtual void onAuralPlayerSensorData();
  virtual void onAuralSelfSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  //virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  //virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  //virtual void onNewCycle(const bool forced);
  //virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  ViewManager* const viewManager;
  const WorldModelInterface* const worldModelInterface;
  const WorldFacts *facts;
  GameView* gameView;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


#endif // !TEAMDECISION_H
