/* -*- Mode: C++ -*-
 * $Id: TestTeamDecision.h,v 1.7 2003/09/08 07:47:23 aip19 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TestTeamDecision.h
 * Unitname: Framework/test/TestTeam
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-16
 * Last modified by $Author: aip19 $ $Date: 2003/09/08 07:47:23 $
 *
 *
 * HISTORY
 *
 * $Log: TestTeamDecision.h,v $
 * Revision 1.7  2003/09/08 07:47:23  aip19
 * Updated templates to use SearchBall and made the HOWTOS a bit clearer
 *
 * Revision 1.6  2002/09/13 15:10:19  s02davro
 * *** empty log message ***
 *
 * Revision 1.5  2002/09/13 14:48:57  s02davro
 * FALSE -> FuzzyBool(0.0), TRUE -> FuzzyBool(1.0)
 *
 * Revision 1.4  2002/09/13 14:10:39  s02davro
 * Updated the TestTeam to use new version of RoboSoc
 *
 * Revision 1.3  2001/09/01 15:06:52  frehe
 * Updated the file to work with the current version of RoboSoc.
 *
 * Revision 1.2  2000/09/16 18:39:42  frehe
 * Fixed the log messages.
 *
 * Revision 1.1  2000/09/16 18:37:09  frehe
 * Add a team that can be used as the basis for test teams.
 *
 */

/**
 * @file TeamDecision.h
 * Include file for the class TeamDecision.
 *
 * @par Unit:
 *      Framework/test/TestTeam
 * $Revision: 1.7 $
 * $Author: aip19 $
 * $Date: 2003/09/08 07:47:23 $
 */


#ifndef TEAMDECISION_H
#define TEAMDECISION_H


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "WorldModelInterface.h"
#include "SearchBall.h"
#include "GotoBall.h"

RS_BEGIN_NAMESPACE

class ViewManager;
class GameView;
class BallView;
class AgentView;
class PlayerView;


/**
 * @class TestTeamDecision TestTeamDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.7 $
 * $Author: aip19 $
 * $Date: 2003/09/08 07:47:23 $
 */
class TestTeamDecision : public Decision {
public:
  TestTeamDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~TestTeamDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  virtual bool decide();

  // Event handlers
  //virtual void onActuatorSensorData();
  virtual void onAuralCoachSensorData();
  virtual void onAuralRefereeSensorData();
  virtual void onAuralPlayerSensorData();
  virtual void onAuralSelfSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  //virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  //virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  //virtual void onNewCycle(const bool forced);
  //virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  ViewManager* const viewManager;
  const WorldModelInterface* const worldModelInterface;
  const WorldFacts *facts;
  GameView* gameView;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;

  SearchBall searchBall;  
  GotoBall gotoBall;
};


RS_END_NAMESPACE


#endif // !TEAMDECISION_H
