/* -*- Mode: C++ -*-
 * $Id: ViewStub.h,v 1.3 2002/09/13 22:44:32 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewStub.h
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: frehe $ $Date: 2002/09/13 22:44:32 $
 *
 *
 * HISTORY
 *
 * $Log: ViewStub.h,v $
 * Revision 1.3  2002/09/13 22:44:32  frehe
 * Removed history from the view stub.
 *
 * Revision 1.2  2000/09/17 14:17:32  frehe
 * Made minor updates to the ViewStub.
 *
 * Revision 1.1  2000/09/13 00:06:21  frehe
 * Added a view template called ViewStub.
 *
 */

/**
 * @file ViewStub.h
 * Include file for the view class ViewStub.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2002/09/13 22:44:32 $
 */


#ifndef VIEWSTUB_H
#define VIEWSTUB_H


#include "View.h"


RS_BEGIN_NAMESPACE


/**
 * @class ViewStub ViewStub.h
 * A stub class for views that can be used as a template
 * for creating new views. It uses history.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2002/09/13 22:44:32 $
 */
class ViewStub : public ViewW {
public:
  /** Construct a view with the ViewId @a id. */
  ViewStub(const ViewId id);

  /** Destroys the view. */
  ~ViewStub();


  /** Initialize the view.
   Do not have to implemented, can be removed! */
  virtual bool init();

  
protected:
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  
private:
  // Hide copy-constructor and assignment.
  ViewStub(const ViewStub&);
  ViewStub& operator= (const ViewStub&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ViewStub.icc"
#endif


#endif // !VIEWSTUB_H
