/* -*- Mode: C++ -*-
 * $Id: AttackerDefenderGoalieScoreScenario.h,v 1.1 2005/08/13 13:36:57 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttackerDefenderGoalieScoreScenario.h
 * Unitname: Framework/trainer
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-09-03
 * Last modified by $Author: frehe $ $Date: 2005/08/13 13:36:57 $
 *
 *
 * HISTORY
 *
 * $Log: AttackerDefenderGoalieScoreScenario.h,v $
 * Revision 1.1  2005/08/13 13:36:57  frehe
 * Readded the files since the trainer seems to have been deleted from the CVS.
 *
 * Revision 1.2  2004/09/07 17:28:16  frehe
 * Changed the design to allow reading of test cases from file.
 *
 * Revision 1.1  2004/09/03 09:44:15  frehe
 * Added new score scenario with both goalie and defenders.
 *
 */

/**
 * @file AttackerDefenderGoalieScoreScenario.h
 * Include file for the class AttackerDefenderGoalieScoreScenario.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:36:57 $
 */


#ifndef ATTACKER_DEFENDER_GOALIE_SCORE_SCENARIO_H
#define ATTACKER_DEFENDER_GOALIE_SCORE_SCENARIO_H


#include "Scenario.h"
#include <vector>


RS_BEGIN_NAMESPACE


class ViewManager;
class WorldModelInterface;
class WorldFacts;
class PlayerView;
class TrainerView;
class GameView;


/**
 * @class AttackerDefenderGoalieScoreScenario AttackerDefenderGoalieScoreScenario.h
 * A concrete scenario which sets up an attack  at a random position in
 * the penalty area, the ball at a random position in the kickable area
 * of the player and expect it to score against a goalie within a limited
 * number of cycles.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:36:57 $
 */
class AttackerDefenderGoalieScoreScenario : public Scenario {
public:
  /**
   * Create a scenario using @random_seed to initialize the random number
   * generator. @a gmode determines the goalie mode, see the goalie for
   * documentation. The vector @a modes determines the number of defenders
   * to place on the field and their defend mode.
   * @a cycles is the maximum number of cycles allowed to score.
   */
  AttackerDefenderGoalieScoreScenario(int random_seed, int gmode,
				      const std::vector<int>& modes,
				      int cycles = 25);

  /** Destroys the scenario. */
  virtual ~AttackerDefenderGoalieScoreScenario();


  /** Return a plan to setup the scenario. */
  virtual Plan setup();


  /** Return true if the scenario is running. */
  virtual bool running();


  /** Should return true if the scenario is running and succeeded. */
  virtual bool success();

  /** Should return true if the scenario is running and failed. */
  virtual bool failure();


private:
  ViewManager* mViewManager;
  const WorldModelInterface* mWorldModelInterface;
  const WorldFacts* mFacts;
  PlayerView* mPlayerView;
  TrainerView* mTrainerView;
  GameView* mGameView;

  int mGoalieMode;
  std::vector<int> mDefenderModes;

  bool mRunning;
  int mStartScore;
  int mStartTime;
  int mCycles;
};


RS_END_NAMESPACE


#endif // !ATTACKER_DEFENDER_GOALIE_SCORE_SCENARIO_H
