/* -*- Mode: C++ -*-
 * $Id: AttackerDefendersDribbleScenario.h,v 1.2 2005/08/18 20:45:56 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttackerDefendersDribbleScenario.h
 * Unitname: Framework/trainer
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-09-07
 * Last modified by $Author: frehe $ $Date: 2005/08/18 20:45:56 $
 *
 *
 * HISTORY
 *
 * $Log: AttackerDefendersDribbleScenario.h,v $
 * Revision 1.2  2005/08/18 20:45:56  frehe
 * Added a max y dist parameter. Now 10m, maybe it should be lower.
 *
 * Revision 1.1  2005/08/13 13:36:58  frehe
 * Readded the files since the trainer seems to have been deleted from the CVS.
 *
 * Revision 1.1  2004/09/07 21:25:36  frehe
 * Added a scenario for testing dribble.
 *
 */

/**
 * @file AttackerDefendersDribbleScenario.h
 * Include file for the class AttackerDefendersDribbleScenario.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/08/18 20:45:56 $
 */


#ifndef ATTACKER_DEFENDERS_DRIBBLE_SCENARIO_H
#define ATTACKER_DEFENDERS_DRIBBLE_SCENARIO_H


#include "Scenario.h"
#include "PlayerObject.h"
#include <vector>


RS_BEGIN_NAMESPACE


class ViewManager;
class WorldModelInterface;
class WorldFacts;
class PlayerView;
class TrainerView;
class GameView;
class BallView;


/**
 * @class AttackerDefendersDribbleScenario AttackerDefendersDribbleScenario.h
 * A concrete scenario which sets up an attacker  at a random position
 * along a line in the middle of the field and one or more defenders in
 * the area in front of the attacker. The goal is for the attacker to do
 * dribble the defenders and pass another line with the ball under control.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/08/18 20:45:56 $
 */
class AttackerDefendersDribbleScenario : public Scenario {
public:
  /**
   * Create a scenario using @random_seed to initialize the random number
   * generator. @a gmode determines the  mode, see the  for
   * documentation. The vector @a modes determines the number of defenderss
   * to place on the field and their defend mode.
   * @a cycles is the maximum number of cycles allowed to dribble.
   */
  AttackerDefendersDribbleScenario(int random_seed,
				   const std::vector<int>& defenders_modes,
				   int dribble_dist, int min_defender_dist,
				   int max_defender_dist,
				   int max_y_dist,
				   int cycles = 50);

  /** Destroys the scenario. */
  virtual ~AttackerDefendersDribbleScenario();


  /** Return a plan to setup the scenario. */
  virtual Plan setup();


  /** Return true if the scenario is running. */
  virtual bool running();


  /** Should return true if the scenario is running and succeeded. */
  virtual bool success();

  /** Should return true if the scenario is running and failed. */
  virtual bool failure();


protected:
  /// Return the current playerr object
  const PlayerObject& find_player() const;
  

private:
  ViewManager* mViewManager;
  const WorldModelInterface* mWorldModelInterface;
  const WorldFacts* mFacts;
  PlayerView* mPlayerView;
  TrainerView* mTrainerView;
  GameView* mGameView;
  BallView* mBallView;

  std::vector<int> mDefendersModes;
  int m_dribble_dist;
  int m_min_defender_dist;
  int m_max_defender_dist;
  int m_max_y_dist;
  Float m_start_x;
  Float m_start_y;
  int m_player_unum;
  
  bool mRunning;
  int mStartDribble;
  int mStartTime;
  int mCycles;
};


RS_END_NAMESPACE


#endif // !ATTACKER_DEFENDERS_DRIBBLE_SCENARIO_H
