/* -*- Mode: C++ -*-
 * $Id: AttackerGoalieScoreScenario.h,v 1.1 2005/08/13 13:36:58 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttackerGoalieScoreScenario.h
 * Unitname: Framework/trainer
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-09-01
 * Last modified by $Author: frehe $ $Date: 2005/08/13 13:36:58 $
 *
 *
 * HISTORY
 *
 * $Log: AttackerGoalieScoreScenario.h,v $
 * Revision 1.1  2005/08/13 13:36:58  frehe
 * Readded the files since the trainer seems to have been deleted from the CVS.
 *
 * Revision 1.3  2004/09/07 17:28:16  frehe
 * Changed the design to allow reading of test cases from file.
 *
 * Revision 1.2  2004/09/01 22:01:16  frehe
 * Fixed indentation.
 *
 * Revision 1.1  2004/09/01 09:03:47  frehe
 * Added new scenario with one attacker and one goalie.
 *
 */

/**
 * @file AttackerGoalieScoreScenario.h
 * Include file for the class AttackerGoalieScoreScenario.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:36:58 $
 */


#ifndef ATTACKER_GOALIE_SCORE_SCENARIO_H
#define ATTACKER_GOALIE_SCORE_SCENARIO_H


#include "Scenario.h"


RS_BEGIN_NAMESPACE


class ViewManager;
class WorldModelInterface;
class WorldFacts;
class PlayerView;
class TrainerView;
class GameView;


/**
 * @class AttackerGoalieScoreScenario AttackerGoalieScoreScenario.h
 * A concrete scenario which sets up an attack  at a random position in
 * the penalty area, the ball at a random position in the kickable area
 * of the player and expect it to score against a goalie within a limited
 * number of cycles.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:36:58 $
 */
class AttackerGoalieScoreScenario : public Scenario {
public:
  /**
   * Create a scenario using @random_seed to initialize the random number
   * generator. @a mode determines the goalie mode, see the goalie for
   * documentation. @a cycles is the maximum number of cycles allowed to
   * score.
   */
  AttackerGoalieScoreScenario(int random_seed, int mode = 1, int cycles = 25);

  /** Destroys the scenario. */
  virtual ~AttackerGoalieScoreScenario();


  /** Return a plan to setup the scenario. */
  virtual Plan setup();


  /** Return true if the scenario is running. */
  virtual bool running();


  /** Should return true if the scenario is running and succeeded. */
  virtual bool success();

  /** Should return true if the scenario is running and failed. */
  virtual bool failure();


private:
  ViewManager* mViewManager;
  const WorldModelInterface* mWorldModelInterface;
  const WorldFacts* mFacts;
  PlayerView* mPlayerView;
  TrainerView* mTrainerView;
  GameView* mGameView;

  int mMode;

  bool mRunning;
  int mStartScore;
  int mStartTime;
  int mCycles;
};


RS_END_NAMESPACE


#endif // !ATTACKER_GOALIE_SCORE_SCENARIO_H
