/* -*- Mode: C++ -*-
 * $Id: AttackersDefendersPassScenario.h,v 1.2 2005/08/18 21:06:50 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttackersDefendersPassScenario.h
 * Unitname: Framework/trainer
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-09-07
 * Last modified by $Author: frehe $ $Date: 2005/08/18 21:06:50 $
 *
 *
 * HISTORY
 *
 * $Log: AttackersDefendersPassScenario.h,v $
 * Revision 1.2  2005/08/18 21:06:50  frehe
 * Added a max y dist parameter. Now 5m, maybe it should be dependent on the actual position of the receiver not the possible positions.
 *
 * Revision 1.1  2005/08/13 13:36:58  frehe
 * Readded the files since the trainer seems to have been deleted from the CVS.
 *
 * Revision 1.1  2004/09/07 23:05:37  frehe
 * Added a scenario for testing passing.
 *
 */

/**
 * @file AttackersDefendersPassScenario.h
 * Include file for the class AttackersDefendersPassScenario.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/08/18 21:06:50 $
 */


#ifndef ATTACKERS_DEFENDERS_PASS_SCENARIO_H
#define ATTACKERS_DEFENDERS_PASS_SCENARIO_H


#include "Scenario.h"
#include <vector>


RS_BEGIN_NAMESPACE


class ViewManager;
class WorldModelInterface;
class WorldFacts;
class PlayerView;
class TrainerView;
class GameView;
class BallView;


/**
 * @class AttackersDefendersPassScenario AttackersDefendersPassScenario.h
 * A concrete scenario which sets up an attackers at a random position
 * along a line in the middle of the field, one or more defenders in the
 * area in front of the attacker and the rest of the attackers in 
 * random positions in an area behind the defenders. The goal is for the
 * attacker to pass one of the other attackers (on the other side of a line)
 * so it has the ball under control.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/08/18 21:06:50 $
 */
class AttackersDefendersPassScenario : public Scenario {
public:
  /**
   * Create a scenario using @random_seed to initialize the random number
   * generator. @a gmode determines the  mode, see the  for
   * documentation. The vector @a modes determines the number of defenderss
   * to place on the field and their defend mode.
   * @a cycles is the maximum number of cycles allowed to pass.
   */
  AttackersDefendersPassScenario(int random_seed,
				 int attackers,
				 const std::vector<int>& defenders_modes,
				 int min_pass_dist, int max_pass_dist,
				 int min_y_dist, int max_y_dist,
				 int min_defender_dist,
				 int max_defender_dist,
				 int max_y_diff,
				 int cycles = 50);

  /** Destroys the scenario. */
  virtual ~AttackersDefendersPassScenario();


  /** Return a plan to setup the scenario. */
  virtual Plan setup();


  /** Return true if the scenario is running. */
  virtual bool running();


  /** Should return true if the scenario is running and succeeded. */
  virtual bool success();

  /** Should return true if the scenario is running and failed. */
  virtual bool failure();
  

private:
  ViewManager* mViewManager;
  const WorldModelInterface* mWorldModelInterface;
  const WorldFacts* mFacts;
  PlayerView* mPlayerView;
  TrainerView* mTrainerView;
  GameView* mGameView;
  BallView* mBallView;

  std::vector<int> mDefendersModes;
  int m_attackers;
  int m_min_pass_dist;
  int m_max_pass_dist;
  int m_min_y_dist;
  int m_max_y_dist;
  int m_min_defender_dist;
  int m_max_defender_dist;
  int m_max_y_diff;
  Float m_start_x;
  Float m_start_y;
  int m_player_unum;
  
  bool mRunning;
  int mStartPass;
  int mStartTime;
  int mCycles;
};


RS_END_NAMESPACE


#endif // !ATTACKERS_DEFENDERS_PASS_SCENARIO_H
