/* -*- Mode: C++ -*-
 * $Id: OnlyAttackerScoreScenario.h,v 1.1 2005/08/13 13:36:58 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: OnlyAttackerScoreScenario.h
 * Unitname: Framework/trainer
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-08-31
 * Last modified by $Author: frehe $ $Date: 2005/08/13 13:36:58 $
 *
 *
 * HISTORY
 *
 * $Log: OnlyAttackerScoreScenario.h,v $
 * Revision 1.1  2005/08/13 13:36:58  frehe
 * Readded the files since the trainer seems to have been deleted from the CVS.
 *
 * Revision 1.3  2004/09/07 17:28:17  frehe
 * Changed the design to allow reading of test cases from file.
 *
 * Revision 1.2  2004/09/01 09:03:21  frehe
 * Parameterized the number of cycles before failure and check the play mode in failure.
 *
 * Revision 1.1  2004/08/31 17:12:10  frehe
 * Added a scenario with only an attacker.
 *
 */

/**
 * @file OnlyAttackerScoreScenario.h
 * Include file for the class OnlyAttackerScoreScenario.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:36:58 $
 */


#ifndef ONLY_ATTACKER_SCORE_SCENARIO_H
#define ONLY_ATTACKER_SCORE_SCENARIO_H


#include "Scenario.h"


RS_BEGIN_NAMESPACE


class ViewManager;
class WorldModelInterface;
class WorldFacts;
class PlayerView;
class TrainerView;
class GameView;


/**
 * @class OnlyAttackerScoreScenario OnlyAttackerScoreScenario.h
 * A concrete scenario which sets up an attacker at a random position in
 * the penalty area, the ball at a random position in the kickable area
 * of the player and expect it to score within a limited number of cycles.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:36:58 $
 */
class OnlyAttackerScoreScenario : public Scenario {
public:
  /**
   * Create a scenario using@random_seed to initialize the random number
   * generator. @a cycles is the maximum number of cycles allowed to score.
   */
  OnlyAttackerScoreScenario(int random_seed, int cycles = 25);

  /** Destroys the scenario. */
  virtual ~OnlyAttackerScoreScenario();


  /** Return a plan to setup the scenario. */
  virtual Plan setup();


  /** Return true if the scenario is running. */
  virtual bool running();


  /** Should return true if the scenario is running and succeeded. */
  virtual bool success();

  /** Should return true if the scenario is running and failed. */
  virtual bool failure();


private:
  ViewManager* mViewManager;
  const WorldModelInterface* mWorldModelInterface;
  const WorldFacts* mFacts;
  PlayerView* mPlayerView;
  TrainerView* mTrainerView;
  GameView* mGameView;

  bool mRunning;
  int mStartScore;
  int mStartTime;
  int mCycles;
};


RS_END_NAMESPACE


#endif // !ONLY_ATTACKER_SCORE_SCENARIO_H
