/* -*- Mode: C++ -*-
 * $Id: Scenario.h,v 1.2 2005/08/18 20:22:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Scenario.h
 * Unitname: Framework/trainer
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-30
 * Last modified by $Author: frehe $ $Date: 2005/08/18 20:22:35 $
 *
 *
 * HISTORY
 *
 * $Log: Scenario.h,v $
 * Revision 1.2  2005/08/18 20:22:35  frehe
 * Added a min level to remove certain test cases.
 *
 * Revision 1.1  2005/08/13 13:36:58  frehe
 * Readded the files since the trainer seems to have been deleted from the CVS.
 *
 * Revision 1.2  2004/09/07 17:28:17  frehe
 * Changed the design to allow reading of test cases from file.
 *
 * Revision 1.1  2004/08/31 12:21:43  frehe
 * First version with external scenarios.
 *
 */

/**
 * @file Scenario.h
 * Include file for the class Scenario.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/08/18 20:22:35 $
 */


#ifndef SCENARIO_H
#define SCENARIO_H


#include "machine_dependent_compiler_directives.h"
#include "Plan.h"


RS_BEGIN_NAMESPACE


/**
 * @class Scenario Scenario.h
 * The abstract base class for all scenarios for the RoboSoc trainer.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/08/18 20:22:35 $
 */
class Scenario {
public:
  /** Create a default scenario. */
  Scenario(int level);

  /** Destroys the scenario. */
  virtual ~Scenario();


  /** Return a plan to setup the scenario. */
  virtual Plan setup() = 0;


  /** Return true if the scenario is running. */
  virtual bool running() = 0;


  /** Should return true if the scenario is running and succeeded. */
  virtual bool success() = 0;

  /** Should return true if the scenario is running and failed. */
  virtual bool failure() = 0;


  /** Return the level of the scenario. */
  int level() const;


private:
  int level_;
};


RS_END_NAMESPACE


#endif // !SCENARIO_H
