/* -*- Mode: C++ -*-
 * $Id: ThreeOnThreeScenario.h,v 1.1 2005/08/13 13:36:58 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ThreeOnThreeScenario.h
 * Unitname: Framework/trainer
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-09-29
 * Last modified by $Author: frehe $ $Date: 2005/08/13 13:36:58 $
 *
 *
 * HISTORY
 *
 * $Log: ThreeOnThreeScenario.h,v $
 * Revision 1.1  2005/08/13 13:36:58  frehe
 * Readded the files since the trainer seems to have been deleted from the CVS.
 *
 * Revision 1.1  2004/09/29 20:24:22  frehe
 * Added support for three on three scenario.
 *
 */

/**
 * @file ThreeOnThreeScenario.h
 * Include file for the class ThreeOnThreeScenario.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:36:58 $
 */


#ifndef THREE_ON_THREE_SCENARIO_H
#define THREE_ON_THREE_SCENARIO_H


#include "Scenario.h"
#include <vector>


RS_BEGIN_NAMESPACE


class ViewManager;
class WorldModelInterface;
class WorldFacts;
class PlayerView;
class TrainerView;
class GameView;


/**
 * @class ThreeOnThreeScenario ThreeOnThreeScenario.h
 * A concrete scenario which sets up three attackers outside the penalty
 * area and two defenderd and a goalie inside the penalty area. The ball
 * is placed with one of the attackers.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:36:58 $
 */
class ThreeOnThreeScenario : public Scenario {
public:
  /**
   * Create a scenario using @random_seed to initialize the random number
   * generator. @attackers_pos is where to place the attackers and
   * @a defenders_pos is where to place the defenders. The ball is made
   * kickable for one of the attackers.
   */
  ThreeOnThreeScenario(int random_seed,
		       const std::vector<Point>& attackers_pos,
		       const std::vector<Point>& defenders_pos,
		       int cycles = 300);

  /** Destroys the scenario. */
  virtual ~ThreeOnThreeScenario();


  /** Return a plan to setup the scenario. */
  virtual Plan setup();


  /** Return true if the scenario is running. */
  virtual bool running();


  /** Should return true if the scenario is running and succeeded. */
  virtual bool success();

  /** Should return true if the scenario is running and failed. */
  virtual bool failure();


private:
  ViewManager* mViewManager;
  const WorldModelInterface* mWorldModelInterface;
  const WorldFacts* mFacts;
  PlayerView* mPlayerView;
  TrainerView* mTrainerView;
  GameView* mGameView;

  std::vector<Point> mAttackersPos;
  std::vector<Point> mDefendersPos;

  bool mRunning;
  int mStartScore;
  int mStartTime;
  int mCycles;
};


RS_END_NAMESPACE


#endif // !THREE_ON_THREE_SCENARIO_H
