/* -*- Mode: C++ -*-
 * $Id: TrainerDecision.h,v 1.2 2005/08/18 20:22:35 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TrainerDecision.h
 * Unitname: Framework/trainer
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-23
 * Last modified by $Author: frehe $ $Date: 2005/08/18 20:22:35 $
 *
 *
 * HISTORY
 *
 * $Log: TrainerDecision.h,v $
 * Revision 1.2  2005/08/18 20:22:35  frehe
 * Added a min level to remove certain test cases.
 *
 * Revision 1.1  2005/08/13 13:36:58  frehe
 * Readded the files since the trainer seems to have been deleted from the CVS.
 *
 * Revision 1.5  2004/09/07 17:28:17  frehe
 * Changed the design to allow reading of test cases from file.
 *
 * Revision 1.4  2004/08/31 17:12:26  frehe
 * Updated to the new test case scenario wrapper.
 *
 * Revision 1.3  2004/08/31 12:21:43  frehe
 * First version with external scenarios.
 *
 * Revision 1.2  2004/08/30 14:52:02  frehe
 * Simple trainer with one fixed scenario, score from random position in penalty area.
 *
 * Revision 1.1  2004/08/23 20:46:45  frehe
 * First version of trainer.
 *
 */

/**
 * @file TrainerDecision.h
 * Include file for the class TrainerDecision.
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/08/18 20:22:35 $
 */


#ifndef TRAINER_DECISION_H
#define TRAINER_DECISION_H


#include "Decision.h"
#include "WorldModelInterface.h"


RS_BEGIN_NAMESPACE


class ViewManager;
class GameView;
class BallView;
class AgentView;
class PlayerView;
class TrainerView;
class TestCase;


/**
 * @class TrainerDecision TrainerDecision.h
 * The decision maker for the trainer.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Framework/trainer
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2005/08/18 20:22:35 $
 */
class TrainerDecision : public Decision {
public:
  TrainerDecision(TestCase* test_case, int min_level,
		  ActuatorInterface* const actuator_interface = NULL);
  virtual ~TrainerDecision();

  virtual ServerCommand* initAgent();
  virtual bool init(ActuatorInterface* const actuator_interface);
  
  virtual bool start();
  virtual bool decide();

  // Event handlers
  //virtual void onActuatorSensorData();
  virtual void onAuralSensorData();
  virtual void onAuralCoachSensorData();
  virtual void onAuralRefereeSensorData();
  virtual void onAuralPlayerSensorData();
  virtual void onAuralSelfSensorData();
  //virtual void onSubstitution();
  virtual void onInit();
  virtual void onPhysicalSensorData();
  //virtual void onSensorError();
  virtual void onVisualCoachSensorData();
  virtual void onVisualSensorData();

  virtual void onBeforeSensorData();
  virtual void onAfterSensorData();
  virtual void onBeforeTick();
  virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();
  //virtual void onDelayedActions();
  //virtual void onMissingActions();


protected:
  ViewManager* const viewManager;
  const WorldModelInterface* const worldModelInterface;
  const WorldFacts *facts;
  GameView* gameView;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
  TrainerView* trainerView;

  int min_level_;
  TestCase* currentTestCase;
  
  struct timeval timeOfLastCycle;
  bool got_visual_data;
};


RS_END_NAMESPACE


#endif // !TRAINER_DECISION_H
