header "pre_include_hpp" {
    // gets inserted before antlr generated includes in the header file
}
header "post_include_hpp" {
    // gets inserted after antlr generated includes in the header file
    // outside any generated namespace specifications
    
#include "TestCase.h"
#include "Scenario.h"
}

header "pre_include_cpp" {
    // gets inserted before the antlr generated includes in the cpp file
#include "OnlyAttackerScoreScenario.h"
#include "AttackerGoalieScoreScenario.h"
#include "AttackerDefenderGoalieScoreScenario.h"
#include "AttackerDefendersDribbleScenario.h"
#include "AttackersDefendersPassScenario.h"
#include "ThreeOnThreeScenario.h"
    
#include <iostream>
}

header "post_include_cpp" {
    // gets inserted after the antlr generated includes in the cpp file
}

header {
    // gets inserted after generated namespace specifications in the header
    // file. But outside the generated class.
}

options {
    language="Cpp";
    namespace="RS";        // encapsulate code in this namespace
    namespaceStd="std";
    namespaceAntlr="antlr";
    genHashLines = true;    // generated #line's or turn it off.
}

{
    // global stuff in the cpp file
}
class TrainerParser extends Parser;
options {
    k = 2;
	buildAST = false;
    //defaultErrorHandler=false; // do not generate exception handlers
    exportVocab=Trainer;
}
/*
tokens {
}
*/
{
    // additional methods and members
    public:
    void random_seed(int random_seed) { random_seed_ = random_seed; };

    private:
    int random_seed_;
}


test_case returns [TestCase* test_case]
{
    std::string name;
    int iterations;
    int verbose;
    test_case = 0;
    Scenario* sce = 0;
}
    : "TestCase"
        LPAREN
        name=string COMMA iterations=long_ COMMA verbose=long_
        RPAREN
        {
            test_case = new TestCase(name, iterations, verbose);
        }
        LCURLY
        (
            sce=scenario
            {
                test_case->add_scenario(sce);
            }
        )+
        RCURLY
    ;

scenario returns [Scenario* s]
    : s=only_attacker_score
    | s=attacker_goalie_score
    | s=attacker_defenders_goalie_score
    | s=attacker_defenders_dribble
    | s=attackers_defenders_pass
    | s=three_on_three
    ;

only_attacker_score returns [Scenario* s]
{
    int cycles;
}
    : "OnlyAttackerScore"
        LPAREN cycles=long_ RPAREN
        {
            s = new OnlyAttackerScoreScenario(random_seed_, cycles);
        }
    ;

attacker_goalie_score returns [Scenario* s]
{
    int goalie_mode;
    int cycles;
}
    : "AttackerGoalieScore"
        LPAREN goalie_mode=long_ COMMA cycles=long_ RPAREN
        {
            s = new AttackerGoalieScoreScenario(random_seed_,
                                                goalie_mode, cycles);
        }
    ;

attacker_defenders_goalie_score returns [Scenario* s]
{
    int goalie_mode;
    std::vector<int> defenders_modes;
    int cycles;
}
    : "AttackerDefendersGoalieScore"
        LPAREN
        goalie_mode=long_ COMMA
        LANGLE defenders_modes=long_seq RANGLE COMMA
        cycles=long_
        RPAREN
        {
            s = new AttackerDefenderGoalieScoreScenario(random_seed_,
                                                        goalie_mode,
                                                        defenders_modes,
                                                        cycles);
        }
    ;

attacker_defenders_dribble returns [Scenario* s]
{
    std::vector<int> defenders_modes;
    int dribble_dist;
    int min_defender_dist;
    int max_defender_dist;
    int max_y_dist;
    int cycles;
}
    : "AttackerDefendersDribble"
        LPAREN
        LANGLE defenders_modes=long_seq RANGLE COMMA
        dribble_dist=long_ COMMA
        min_defender_dist=long_ COMMA
        max_defender_dist=long_ COMMA
        max_y_dist=long_ COMMA
        cycles=long_
        RPAREN
        {
            s = new AttackerDefendersDribbleScenario(random_seed_,
                                                     defenders_modes,
                                                     dribble_dist,
                                                     min_defender_dist,
                                                     max_defender_dist,
                                                     max_y_dist,
                                                     cycles);
        }
    ;

attackers_defenders_pass returns [Scenario* s]
{
    int attackers;
    std::vector<int> defenders_modes;
    int min_pass_dist;
    int max_pass_dist;
    int min_y_dist;
    int max_y_dist;
    int min_defender_dist;
    int max_defender_dist;
    int max_y_diff;
    int cycles;
}
    : "AttackersDefendersPass"
        LPAREN
        attackers=long_ COMMA
        LANGLE defenders_modes=long_seq RANGLE COMMA
        min_pass_dist=long_ COMMA
        max_pass_dist=long_ COMMA
        min_y_dist=long_ COMMA
        max_y_dist=long_ COMMA
        min_defender_dist=long_ COMMA
        max_defender_dist=long_ COMMA
        max_y_diff=long_ COMMA
        cycles=long_
        RPAREN
        {
            s = new AttackersDefendersPassScenario(random_seed_,
                                                   attackers,
                                                   defenders_modes,
                                                   min_pass_dist,
                                                   max_pass_dist,
                                                   min_y_dist,
                                                   max_y_dist,
                                                   min_defender_dist,
                                                   max_defender_dist,
                                                   max_y_diff,
                                                   cycles);
        }
    ;

three_on_three returns [Scenario* s]
{
    Point a1,a2,a3;
    Point d1,d2,d3;
    int cycles;
}
    : "ThreeOnThree"
        LPAREN
        a1=point COMMA a2=point COMMA a3=point COMMA
        d1=point COMMA d2=point COMMA d3=point COMMA
        cycles=long_
        RPAREN
        {
            std::vector<Point> attackers;
            attackers.push_back(a1);
            attackers.push_back(a2);
            attackers.push_back(a3);
            std::vector<Point> defenders;
            defenders.push_back(d1);
            defenders.push_back(d2);
            defenders.push_back(d3);
            s = new ThreeOnThreeScenario(random_seed_, attackers, defenders, cycles);
        }
    ;

pos_long_ returns [long l]
{
    l = -1;
}
    : t:NUM_INT { l = atol(t->getText().c_str()); }
    ;

long_ returns [long l]
{
    l = -1;
    bool neg = false;
}
    :   ( MINUS { neg=true; } )?
        l = pos_long_ { if ( neg ) { l *= -1; } }
    ;

pos_float_ returns [double d]
{
    d = -1;
}
    : f:NUM_FLOAT { d = atof(f->getText().c_str()); }
    ;

float_ returns [double d]
{
    d = -1;
    bool neg = false;
}
    :   ( MINUS { neg=true; } )?
        d = pos_float_ { if ( neg ) { d *= -1; } }
    ;

double_ returns [double d]
{
    d = -1;
    bool neg = false;
}
    :   ( MINUS { neg=true; } )?
        ( 
            d = pos_float_
        | 
            d = pos_long_
        )
        {
            if ( neg ) { d *= -1; }
        }
    ;

bool_ returns [bool b]
{
    b = false;
}
    : "true" { b = true; }
    | "false" { b = false; }
    ;

string returns [std::string s]
{
    s = "";
}
    : t:STRING_LITERAL
        {
            s = std::string(t->getText(), 1, t->getText().size()-2);
        }
    ;

long_seq returns [std::vector<int> ls]
{
    int l;
}
    : l = long_ { ls.push_back(l); }
        (COMMA^ l = long_
            {
                ls.push_back(l);
            }
        )*
    ;

point returns [Point p]
{
    double x, y;
}
    : LPAREN x=double_ y=double_ RPAREN { p = Point(x,y); }
    ;

ident returns [std::string n]
{
    n = "";
}
    :   t1:IDENT { n = t1->getText(); }
        //( DOT t2:IDENT { n += t2->getText(); } )*
    ;

ident_with_namespaces returns [std::string nn]
{
    std::string n;
}
    :   n = ident
        {
            nn = n;
        }
        ( COLON COLON n=ident
            {
                nn.append("::");
                nn.append(n);
            }
        )*
    ;


{
    // global stuff in the cpp file
}
class TrainerLexer extends Lexer;
options {
    k = 2;
    exportVocab=Trainer;
	testLiterals=false;    // don't automatically test for literals
}
{
   // additional methods and members
}


// OPERATORS
//QUESTION		:	'?'		;
LPAREN			:	'('		;
RPAREN			:	')'		;
LBRACK			:	'['		;
RBRACK			:	']'		;
LCURLY			:	'{'		;
RCURLY			:	'}'		;
LANGLE          :   '<'     ;
RANGLE          :   '>'     ;
COLON			:	':'		;
COMMA			:	','		;
//DOT		        :	'.'		;
ASSIGN			:	'='		;
AT              :   '@'     ;
/*
EQUAL			:	"=="	;
BNOT			:	'~'		;
LNOT			:	'!'		;
NOT_EQUAL		:	"!="	;
DIV				:	'/'		;
DIV_ASSIGN		:	"/="	;
PLUS			:	'+'		;
PLUS_ASSIGN		:	"+="	;
INC				:	"++"	;
*/
MINUS			:	'-'		;
SLASH			:	'/'		;
/*
MINUS_ASSIGN	:	"-="	;
DEC				:	"--"	;
STAR			:	'*'		;
STAR_ASSIGN		:	"*="	;
MOD				:	'%'		;
MOD_ASSIGN		:	"%="	;
SR				:	">>"	;
SR_ASSIGN		:	">>="	;
BSR				:	">>>"	;
BSR_ASSIGN		:	">>>="	;
GE				:	">="	;
GT				:	">"		;
SL				:	"<<"	;
SL_ASSIGN		:	"<<="	;
LE				:	"<="	;
LT_				:	'<'		;
BXOR			:	'^'		;
BXOR_ASSIGN		:	"^="	;
BOR				:	'|'		;
BOR_ASSIGN		:	"|="	;
LOR				:	"||"	;
BAND			:	'&'		;
BAND_ASSIGN		:	"&="	;
LAND			:	"&&"	;
*/
SEMI			:	';'		;


/*
protected
DIGIT:  '0'..'9'
	;

protected
UINT:	( DIGIT )+
	;

LONG:   ( '+' | '-' )? UINT
    ;

protected
REAL:   ( LONG )? '.' UINT
    ;

protected
EXP :   ( REAL | LONG ) ( 'e' | 'E') LONG
    ;

DOUBLE: ( LONG | REAL | EXP )
    ;

protected
CHAR: ( 'a'..'z' | 'A'..'Z' )
    ;

STRING: ( CHAR | '_' ) ( CHAR | DIGIT | '_' )*
    ;
*/

/*
STRING: '"'! ( CHAR | '_' ) ( CHAR | DIGIT | '_' )* '"'!
    ;
*/


// Whitespace -- ignored
WS	:	(	' '
		|	'\t'
		|	'\f'
			// handle newlines
		|	(	options {generateAmbigWarnings=false;}
			:	"\r\n"  // Evil DOS
			|	'\r'    // Macintosh
			|	'\n'    // Unix (the right way)
			)
			{ newline(); }
		)+
		{ _ttype = antlr::Token::SKIP; }
	;

// Single-line comments
SL_COMMENT
	:	"//"
		(~('\n'|'\r'))* ('\n'|'\r'('\n')?)
		{
            //std::cout << "Found comment: '" << text.substr(_begin, text.length()-_begin) << "'" << std::endl;	
            _ttype = antlr::Token::SKIP;
            newline();
        }
	;

// multiple-line comments
ML_COMMENT
	:	"/*"
		(	/*	'\r' '\n' can be matched in one alternative or by matching
				'\r' in one iteration and '\n' in another.  I am trying to
				handle any flavor of newline that comes in, but the language
				that allows both "\r\n" and "\r" and "\n" to all be valid
				newline is ambiguous.  Consequently, the resulting grammar
				must be ambiguous.  I'm shutting this warning off.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:
			{ LA(2)!='/' }? '*'
		|	'\r' '\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('*'|'\n'|'\r')
		)*
		"*/"
		{ _ttype = antlr::Token::SKIP; newline(); }
	;


// character literals
CHAR_LITERAL
	:	'\'' ( ESC | ~('\''|'\n'|'\r'|'\\') ) '\''
	;

// string literals
STRING_LITERAL
	:	'"' (ESC|~('"'|'\\'|'\n'|'\r'))* '"'
	;


// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
// There are various ambiguities hushed in this rule.  The optional
// '0'...'9' digit matches should be matched here rather than letting
// them go back to STRING_LITERAL to be matched.  ANTLR does the
// right thing by matching immediately; hence, it's ok to shut off
// the FOLLOW ambig warnings.
protected
ESC
	:	'\\'
		(	'n'
		|	'r'
		|	't'
		|	'b'
		|	'f'
		|	'"'
		|	'\''
		|	'\\'
		|	('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
		|	'0'..'3'
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	'0'..'7'
				(
					options {
						warnWhenFollowAmbig = false;
					}
				:	'0'..'7'
				)?
			)?
		|	'4'..'7'
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	'0'..'7'
			)?
		)
	;


// hexadecimal digit (again, note it's protected!)
protected
HEX_DIGIT
	:	('0'..'9'|'A'..'F'|'a'..'f')
	;


// a dummy rule to force vocabulary to be all characters (except special
//   ones that ANTLR uses internally (0 to 2)
protected
VOCAB
	:	'\3'..'\377'
	;


// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
IDENT
	options {testLiterals=true;}
	:	('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$'|'.')*
	;


// a numeric literal
NUM_INT
{
    bool isDecimal=false;
}
	:	'.' //{_ttype = DOT;}
        (('0'..'9')+ (EXPONENT)? (FLOAT_SUFFIX)? { _ttype = NUM_FLOAT; })?
	|	(	'0' {isDecimal = true;} // special case for just '0'
			(	('x'|'X')
				(											// hex
					// the 'e'|'E' and float suffix stuff look
					// like hex digits, hence the (...)+ doesn't
					// know when to stop: ambig.  ANTLR resolves
					// it correctly by matching immediately.  It
					// is therefor ok to hush warning.
					options {
						warnWhenFollowAmbig=false;
					}
				:	HEX_DIGIT
				)+
			|	('0'..'7')+									// octal
			)?
		|	('1'..'9') ('0'..'9')*  {isDecimal=true;}		// non-zero decimal
		)
		(	('l'|'L')

            // only check to see if it's a float if looks like decimal so far
		|	{isDecimal}?
			(	'.' ('0'..'9')* (EXPONENT)? (FLOAT_SUFFIX)?
			|	EXPONENT (FLOAT_SUFFIX)?
			|	FLOAT_SUFFIX
			)
			{ _ttype = NUM_FLOAT; }
		)?
	;


// a couple protected methods to assist in matching floating point numbers
protected
EXPONENT
	:	('e'|'E') ('+'|'-')? ('0'..'9')+
	;


protected
FLOAT_SUFFIX
	:	'f'|'F'|'d'|'D'
	;
