/* -*- Mode: C++ -*-
 * $Id: OffsideTrapView.cc,v 1.10 2003/10/12 21:50:38 aip4 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: OffsideTrapView.cc
 * Unitname: Framework/view
 * $Revision: 1.10 $
 * Created by: Anders and Erik 2002
 * Last modified by $Author: aip4 $ $Date: 2003/10/12 21:50:38 $
 *
 *
 * HISTORY
 *
 * $Log: OffsideTrapView.cc,v $
 * Revision 1.10  2003/10/12 21:50:38  aip4
 * Commenting
 *
 * Revision 1.9  2003/10/11 19:05:17  aip4
 * commenting
 *
 * Revision 1.8  2003/10/11 17:37:06  aip4
 * Major update
 *
 * Revision 1.7  2003/10/11 14:44:27  aip4
 * unsuitable update
 *
 * Revision 1.6  2003/10/11 13:51:37  aip4
 * unsuitable stup
 *
 * Revision 1.5  2003/10/11 13:45:48  aip4
 * More than one to put pressure
 *
 * Revision 1.4  2003/10/02 23:14:14  aip4
 * changed maxDistance
 *
 * Revision 1.3  2003/10/02 23:07:00  aip4
 * changed linepos
 *
 * Revision 1.2  2003/10/02 21:02:29  aip4
 * fixed canPressureOpponent
 *
 * Revision 1.1  2003/09/29 01:17:31  aip4
 * Update after deletion
 *
 * Revision 1.1  2003/09/28 22:25:12  aip4
 * Uppdaterat fel funktionsnamn
 *
 * Revision 1.0  2002/10/22 14:42:16  aip4
 * Offside view created, only basic testing performed
 *
 */

/**
 * @file OffsideTrapView.cc
 * Implementation file for the view class OffsideTrapView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.10 $
 * $Author: aip4 $
 * $Date: 2003/10/12 21:50:38 $
 */


#include "OffsideTrapView.h"
#include <vector>

RS_BEGIN_NAMESPACE

OffsideTrapView::OffsideTrapView(const ViewId id, Float maxTrapDist)
  : View(id),
    facts(worldModelInterface->getWorldFacts()),
    maxTrapDistance(maxTrapDist) {
   getView(playerView, PLAYER_VIEW_ID);
   getView(ballView, BALL_VIEW_ID);
   getView(agentView, AGENT_VIEW_ID);
}

OffsideTrapView::~OffsideTrapView() {
}

bool OffsideTrapView::canPressureOpponent() {
  //So far we have found that in all situations where
  //an offside trap i suitable there is always some players
  //that can put pressure on the ball keeper. Therefore
  //we always return true. This can easily be changed if necessary.
  //For example a requirement of at least four players behind the opponent
  //could be set.
  return true;
}

void OffsideTrapView::getPlayersToSetPress(std::vector<PlayerObject>& pressurePlayers) {
  //Start with 100 meters then get better values (hopefully!)
  FloatUE minDistUpperSide = 100;
  FloatUE minDistLowerSide = 100;
  
  PointUE ballPos = ballView->getPosition();
  if(ballPos.isKnown()){
    // Get teammates 
    std::vector<PlayerObject> friends = playerView->getTeammates();

    // Include current agent in iteration 
    friends.push_back(agentView->getAgentObject());
    
    // Initiate pressurePlayers with something 
    pressurePlayers.push_back(playerView->getTeammateClosestToBall());
    pressurePlayers.push_back(playerView->getTeammateClosestToBall());
    
    // Loop through all players in our team 
    for (std::vector<PlayerObject>::iterator i = friends.begin(); i != friends.end(); i++) {
      if(i->getPosition().isKnown()) {
	// If player is closest to ball than and in second 
	// quadrant (ball in origo). Then it is the first player
	//  in the vector 

	if ((ballPos.getY() < i->getPosition().getY()) &&
	    (ballPos.getDistanceTo(i->getPosition()) < minDistUpperSide) &&
	    (ballPos.getX() > i->getPosition().getX())) {
	  minDistUpperSide = ballPos.getDistanceTo(i->getPosition());
	  pressurePlayers[0] = *i;
	}
	// If player is closest to ball than and in third 
	// quadrant (ball in origo). Then it is the second player
	//  in the vector 
	else if ((ballPos.getDistanceTo(i->getPosition()) < minDistLowerSide) &&
		 (ballPos.getX() > i->getPosition().getX())) {
	  minDistLowerSide = ballPos.getDistanceTo(i->getPosition());
	  pressurePlayers[1] = *i;	  
	}
      }
    }
  }
}

// This algorithm is very simple:
//    Whoever is closest to the ball, has the ball
bool OffsideTrapView::opponentHasBall() {
  // Some local variables to simplify writing 
  PointUE closestOpponentPos = 
    playerView->getOpponentClosestToBall().getPosition();
  PointUE closestTeammatePos = 
    playerView->getTeammateClosestToBall().getPosition();
  PointUE ballPos = ballView->getPosition();

  if (closestOpponentPos.isKnown() &&
      closestTeammatePos.isKnown() &&
      ballPos.isKnown()) {
    FloatUE oppDist = closestOpponentPos.getDistanceTo(ballPos);
    FloatUE ourDist = closestTeammatePos.getDistanceTo(ballPos);
    // If current player is closer than closest teammate
    if (ballView->getRelativeDistance().isKnown())
      if (ballView->getRelativeDistance().getValue() < ourDist)
	ourDist = ballView->getRelativeDistance().getValue();
    return (oppDist < ourDist);
  }
  else {
    return false;
  }
}

FloatUE OffsideTrapView::getOpponentPosClosestToGoal() {
  //We start from the center-line
  FloatUE last = 0;

  // Loop through all opponents and return the position of 
  // the one that is closes to our goal.
  std::vector<PlayerObject> opps = playerView->getOpponents();
  for (std::vector<PlayerObject>::iterator i = opps.begin(); i != opps.end(); i++) {
    if(i->getPosX().isKnown()) {
      if(i->getPosX() < last) {
	last = i->getPosX();
      }
    }
  }
  return last;
}

FloatUE OffsideTrapView::getTeammatePosNextClosestToGoal() {
  // We start from the center line
  FloatUE last = 0;
  FloatUE nextLast = 0;

  // Get all teammates 
  std::vector<PlayerObject> friends = playerView->getTeammates();

  // Include current agent in iteration  
  friends.push_back(agentView->getAgentObject());

  // Loop through our team and return the one that is
  // next closest to our goal 
  for (std::vector<PlayerObject>::iterator i = friends.begin(); i != friends.end(); i++) {
    if(i->getPosX().isKnown()) {
      if(i->getPosX() < last) {
	nextLast=last;
	last = i->getPosX();
      }
      else if(i->getPosX() < nextLast) {
	nextLast = i->getPosX();
      }
    }
  }
  return nextLast;
}

bool OffsideTrapView::suitable() {

  // Some local variables to simplify writing
  FloatUE defenderPos = getTeammatePosNextClosestToGoal();
  FloatUE lastOpponentPos = getOpponentPosClosestToGoal();
  FloatUE trapDistance = lastOpponentPos - defenderPos;
  FloatUE ballPos = ballView->getPosition().getX();

  return (// distance from last defender to line not to big
	  (trapDistance.getValue() < maxTrapDistance) && 
	  // opponent not already in offside 
	  (trapDistance > 0) &&               
	  // opponent must be on our side of the field 
	  (lastOpponentPos < 0) &&            
	  // the ball must be behind of the opponent 
	  (ballPos > lastOpponentPos) &&      
	  // opponent must have the ball 
	  opponentHasBall());
}

bool OffsideTrapView::unsuitable(int xcoord) {
  FloatUE lastBackPos = getTeammatePosNextClosestToGoal();
  
  // Has the ball passed the given x-coordinate?
  if(ballView->getPosition().isKnown()) {
    if ( ballView->getPosition().getX() < xcoord) {
      std::cout << "Ball" << std::endl;
      return true;
    }
  }
  
  // Is there a defender with a too long distance to run?
  if(lastBackPos.isKnown()) {
    if((xcoord - lastBackPos.getValue()) > maxTrapDistance) {
      std::cout << "Distance" << std::endl;
      return true;
    }
  }
  
  return false;
}

inline void OffsideTrapView::myUpdateAfterCoachMessage() {}

inline void OffsideTrapView::myNewCycle() {}

inline void OffsideTrapView::myUpdateAfterInit() {}

inline void OffsideTrapView::myUpdateAfterSee() {}

inline void OffsideTrapView::myUpdateAfterHear() {}

inline void OffsideTrapView::myUpdateAfterSense() {}

inline void OffsideTrapView::myUpdateAfterCommand() {}



RS_END_NAMESPACE



