/* -*- Mode: C++ -*-
 * $Id: OffsideView.cc,v 1.3 2003/09/22 17:30:21 aip4 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: OffsideView.cc
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Anders and Erik 2002
 * Last modified by $Author: aip4 $ $Date: 2003/09/22 17:30:21 $
 *
 *
 * HISTORY
 *
 * $Log: OffsideView.cc,v $
 * Revision 1.3  2003/09/22 17:30:21  aip4
 * Fixed a couple of bugs
 *
 * Revision 1.1  2002/10/22 14:42:16  aip7
 * Offside view created, only basic testing performed
 */

/**
 * @file OffsideView.cc
 * Implementation file for the view class OffsideView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: aip4 $
 * $Date: 2003/09/22 17:30:21 $
 */


#include "OffsideView.h"
#include <vector>

RS_BEGIN_NAMESPACE

/**
 * Constructor. Use getView instead.
 * @see getView
 */
OffsideView::OffsideView(const ViewId id)
  : View(id),
    facts(worldModelInterface->getWorldFacts()) {
   getView(playerView, PLAYER_VIEW_ID);
   getView(ballView, BALL_VIEW_ID);
   getView(agentView, AGENT_VIEW_ID);
}

OffsideView::~OffsideView() {
}

/** Finds the opponents offsideline. */
FloatUE OffsideView::getOpponent() {

   //Set start value to be midline (can't be offside on your own half).
   FloatUE current = FloatUE(0);

   //Find the second "last" opponent player.
   FloatUE last = FloatUE(0);
   FloatUE nextLast = FloatUE(0);
   std::vector<PlayerObject> opps = playerView->getOpponents();
   for (std::vector<PlayerObject>::iterator i = opps.begin(); i != opps.end(); i++) {
      if(i->getPosX().isKnown()) {
	 if(i->getPosX() > last) {
	    nextLast = last;
	    last = i->getPosX();
	 }
	 else if(i->getPosX() > nextLast) {
	    nextLast = i->getPosX();
	 }
      }
   }
   
   if(nextLast > current) {
      current = nextLast;
   }

   //Can't be offside if behind the ball.
   if(ballView->getPosition().isKnown() &&
      ballView->getPosition().getX() > current) {
      current = ballView->getPosition().getX();
   }

   return current;
}

/** Checks if the agent is offside. */
bool OffsideView::amIOffside() {
   return (agentView->getPosition().isKnown() &&
	   getOpponent() <= agentView->getPosition().getX() + FloatUE(0.1));
}

/** Finds our offsideline. */
FloatUE OffsideView::getOur() {

   //Set start value to be midline.
   FloatUE current = FloatUE(0);

   //Find the second "last" teammate.
   FloatUE last = FloatUE(0);
   FloatUE nextLast = FloatUE(0);
   std::vector<PlayerObject> friends = playerView->getTeammates();
   for (std::vector<PlayerObject>::iterator i = friends.begin(); i != friends.end(); i++) {
      if(i->getPosX().isKnown()) {
	 if(i->getPosX() < last) {
	    nextLast = last;
	    last = i->getPosX();
	 }
	 else if(i->getPosX() < nextLast) {
	    nextLast = i->getPosX();
	 }
      }
   }
   
   //Check that I'm not second last (or last).
   if(agentView->getPosition().isKnown()) {
      if(agentView->getPosition().getX() < last) {
	nextLast = last;
	last = agentView->getPosition().getX();
      }
      else if (agentView->getPosition().getX() < nextLast) {
	 nextLast = agentView->getPosition().getX();
      }
   }
   
   //Update if second last player is behind midline (duh?)
   if(nextLast < current) {
      current = nextLast;
   }

   //Can't be offside if behind the ball.
   if(ballView->getPosition().isKnown() &&
      ballView->getPosition().getX() < current) {
      current = ballView->getPosition().getX();
   }

   return current;
}

/** Checks if the agent is inhibits the offside. */
bool OffsideView::amILast() {
   return (agentView->getPosition().isKnown() &&
	   getOur() >= agentView->getPosition().getX() - FloatUE(0.1));
}


// Below is stuff that just had to be here (?)
const BallObject& OffsideView::getBall() const {
  return worldModelInterface->getWorld().getBall();
}

const AgentObject& OffsideView::getAgent() const {
  return worldModelInterface->getWorld().getAgent();
}

inline void OffsideView::myUpdateAfterCoachMessage() {}

inline void OffsideView::myNewCycle() {}

inline void OffsideView::myUpdateAfterInit() {}

inline void OffsideView::myUpdateAfterSee() {}

inline void OffsideView::myUpdateAfterHear() {}

inline void OffsideView::myUpdateAfterSense() {}

inline void OffsideView::myUpdateAfterCommand() {}



RS_END_NAMESPACE



