/* -*- Mode: C++ -*-
 * $Id: OffsideView.h,v 1.1 2002/10/22 14:42:16 aip7 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: OffsideView.h
 * Unitname: Framework/view
 * $Revision: 1.1 $
 * Created by: Anders and Erik 2002
 * Last modified by $Author: aip7 $ $Date: 2002/10/22 14:42:16 $
 *
 *
 * HISTORY
 *
 * $Log: OffsideView.h,v $
 * Revision 1.1  2002/10/22 14:42:16  aip7
 * Offside view created, only basic testing performed
 *
 */

/**
 * @file OffsideView.h
 * Include file for the view class OffsideView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: aip7 $
 * $Date: 2002/10/22 14:42:16 $
 */


#ifndef OFFSIDEVIEW_H
#define OFFSIDEVIEW_H


#include "View.h"
#include "AgentView.h"
#include "BallView.h"
#include "PlayerView.h"
#include "view_constants.h"
#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE

class BallView;
class AgentView;

/**
 * @class OffsideView OffsideView.h
 * This view vill try to find offside lines.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: aip7 $
 * $Date: 2002/10/22 14:42:16 $
 */
class OffsideView : public View {
public:
  /** Construct a view with the ViewId @a id. */
  OffsideView(const ViewId id);

  /** Destroys the view. */
  ~OffsideView();

  /** Finds the opponents offsideline. */
  FloatUE getOpponent();
  
  /** Checks if the agent is offside. */
  bool amIOffside();

  /** Finds our offsideline. */
  FloatUE getOur();

  /** Checks if the agent is inhibits the offside. */
  bool amILast();
	 
protected:  
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  const AgentObject& getAgent() const;
  const BallObject& getBall() const;
  const WorldFacts* facts;

private:
  // Hide copy-constructor and assignment.
  OffsideView(const OffsideView&);
  OffsideView& operator= (const OffsideView&);  

  PlayerView *playerView;
  BallView *ballView;
  AgentView *agentView;

};

RS_END_NAMESPACE


#endif // !OFFSIDEVIEW_H
