/* -*- Mode: C++ -*-
 * $Id: PassView.cc,v 1.13 2004/01/16 12:41:58 aip10 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * -*- Mode: C++ -*-
 * $Id: PassView.cc,v 1.13 2004/01/16 12:41:58 aip10 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassView.cc
 * Unitname: Framework/view
 * $Revision: 1.13 $
 * Created by: aip10
 * Last modified by $Author: aip10 $ $Date: 2004/01/16 12:41:58 $
 *
 *
 * HISTORY
 *
 * $Log: PassView.cc,v $
 * Revision 1.13  2004/01/16 12:41:58  aip10
 * changed descriptions
 *
 * Revision 1.12  2003/12/02 13:34:59  frehe
 * Removed member opponent and used a const reference directly when needed.
 *
 * Revision 1.11  2003/12/01 22:18:57  aip10
 * modified
 *
 * Revision 1.10  2003/12/01 20:55:02  frehe
 * Added missing newline at end of file.
 *
 * Revision 1.9  2003/12/01 19:56:22  aip10
 * MoveWithBallToPosition removed and added to skill GoToPassingPosition
 *
 * Revision 1.8  2003/11/07 15:27:43  aip10
 * printf statements removed
 *
 * Revision 1.7  2003/10/12 17:04:54  aip10
 * New evaluations added
 *
 * Revision 1.6  2003/10/12 14:05:23  aip10
 * Modified pointers
 *
 * Revision 1.5  2003/10/11 10:41:00  aip10
 * Move To Better position implemented
 *
 * Revision 1.4  2003/10/07 09:13:10  aip19
 * Fixed linkage errors and compile errors
 *
 * Revision 1.3  2003/10/06 12:12:18  aip10
 * Added secondary evaluation
 *
 * Revision 1.2  2003/09/28 20:39:34  frehe
 * Made minor changes to make it compile without warnings.
 *
 * Revision 1.1  2003/09/28 20:00:24  aip10
 * PassView.cc
 *
 * Revision 1.1  2000/09/04 15:27:20  ginba729
 * Created the file
 *
 */

/** To DO*******************************
 * > primaryEvaluation -> need to work out how to sort the vector
 * > secondaryEvaluation -> check syntax for iterator pointer reference
 * > playerEvaluation -> just need to use correct function calls
 * > offside
 *****************************************************************
 */


/**
 * @file PassView.cc
 * Implementation file for the class PassView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.13 $
 * $Author: aip10 $
 * $Date: 2004/01/16 12:41:58 $
 */

#include <iostream>
#include "view_constants.h"
#include "ViewManager.h"
#include "WorldFacts.h"
#include "PassView.h"

RS_BEGIN_NAMESPACE

PassView::PassView(const ViewId id) 
  : View(id),
    playerView(NULL),
    agentView(NULL),
	ballView(NULL),
    facts(WorldFacts::instance())
	{
  //worldModelInterface->getWorldFacts()
  getView(agentView, AGENT_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
}



PassView::~PassView() {
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
}

//............................isAvailablePlayer()..............................//
bool PassView::isAvailablePlayer(){
 
  PlayerVector teammates = playerView->getTeammates();
  PlayerVector::iterator team = teammates.begin();
 
  while (team != teammates.end()){
    //PlayerObject player = *team;
    //if (playerEvaluation(&player)>threshold) 
    return true;
    team++;
  }
  //delete &teammates;
  return false;
}
 
//...............................passPartner()................................//
float PassView::passPartner(PlayerObject **bestPartner, FieldStrategy *strategy){
	
  PlayerObject *chosenPlayer = 0;
  // call primaryEvaluation()
  //opponents = playerView->getOpponents();
  SortedVector sortedPlayers = primaryEvaluation();
  //call secondaryEvaluation() with return from primaryEvaluation()
  float eval = secondaryEvaluation(sortedPlayers, &chosenPlayer);
	
  // make bestPartner equal to chosen player from secondaryEvaluation()
  *bestPartner = chosenPlayer;
  return eval;

}



////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////PROTECTED///////////////////////////////////////////

//.............................primaryEvaluation()............................//
SortedVector PassView::primaryEvaluation(){	
	
  // PlayerVector to store sorted PlayerObjects
  //PlayerVector * players = new PlayerVector();
  
  PlayerVector teammates = playerView->getTeammates();
  SortedVector evaluatedPlayers;
  PlayerVector::iterator team = teammates.begin();
  EvaluatedPlayer player;
  SortedVector::iterator sort;
  while (team != teammates.end()){
    float eval = playerEvaluation(&(*team));
    
    //if(eval >threshold){
      sort = evaluatedPlayers.begin();
      if (sort!=evaluatedPlayers.end())
      	while(sort!=evaluatedPlayers.end() && sort->value>eval){
		sort++;
      	}
      player.player = &(*team);
      player.value = eval;
      evaluatedPlayers.insert(sort,player);

    //}
    team++;
  }
  //delete &teammates;
  return evaluatedPlayers; 
}

//..............................secondaryEvaluation()........................//
float PassView::secondaryEvaluation(SortedVector candidates, PlayerObject **bestPlayer){
	
  float best = -1;
  float eval;
  SortedVector::iterator candidate = candidates.begin();
	
  if (agentView->getPosition().isKnown())	
  while (candidate != candidates.end()){
    /*insert eval function here*/
	eval =markedEval(((EvaluatedPlayer)*candidate).player,true,agentView->getPosition().getPoint())
	  +partnerBodyDirection(((EvaluatedPlayer)*candidate).player)
	  +fieldGain(((EvaluatedPlayer)*candidate).player,agentView->getPosition().getPoint())
	  +partnerViewDivergence(((EvaluatedPlayer)*candidate).player)
	  +strategyModel(((EvaluatedPlayer)*candidate).player);
	
    if(eval>best){
      *bestPlayer = ((EvaluatedPlayer)*candidate).player;
      best = eval;
    }
    candidate++;
  }
  return best;
}

//...............................playerEvaluation().......................//
const float PassView::playerEvaluation(PlayerObject *player){
  /*	if(offside(player)) return 0;
	else{*/
   if (agentView->getPosition().isKnown()){
    float markeval = markedEval(player,false,agentView->getPosition().getPoint());
    if (markeval!=0)
      return markeval+fieldGain(player, agentView->getPosition().getPoint());
    
   }
  return 0.0;
  /*}*/
}


//...........................fieldGain().......................................//

float PassView::fieldGain(PlayerObject *player, Point position){
	

      //MarkerObject goalMarker = facts
      Vector goalVector = position.getVectorTo(facts->getTheirGoal().getPosition());    
      if (player->getPosition().isKnown())
      {
      Vector playerVector = position.getVectorTo(player->getPosition().getPoint());
	
      Vector projectedVector = playerVector.projectOn(goalVector);
      if ((projectedVector.getAngle().getDeg()<-90)||(projectedVector.getAngle().getDeg()>90))
	return 0;
	
      // return the decimal percentage of the projection of playerVector on goalVector
      return projectedVector.getLength()/goalVector.getLength();
      }
      else return  0;
}

	
//.........................markedEval()....................................//

float PassView::markedEval(PlayerObject *player, bool Thorough, Point position){
  
  PlayerVector::const_iterator op;
  float mul = 1;
  Point startPoint, endPoint;
  Float distance;
  
  const PlayerVector& opponents = playerView->getOpponents();
  
  AngleRad angle;
  
  if (!Thorough)
  {
		if (player->getPosition().isKnown())
		{
		  startPoint = player->getPosition().getPoint();
		  op = opponents.begin();

		  while (op != opponents.end())
		  {
			  if((*op).getPosition().isKnown())
			  {
	
				  endPoint = (*op).getPosition().getPoint();
				  distance = startPoint.getDistanceTo(endPoint);
         		    
				  if (distance <= Area*facts->SP_kickable_area){

				  if (distance <= facts->SP_kickable_area*2){ //if the oppponent is in kickable area
					return 0;
					}
				  else mul = mul * (distance/facts->SP_kickable_area);
				  }
				  
			  }
		  op++;
		  }  
		  //delete &opponents;
		  return mul; 
		}
		else {
		return 0; //solve this problem
		}	
  }
  else {
	     //thorough evaluation
	 if (player->getPosition().isKnown())
		{
		  startPoint = player->getPosition().getPoint();
		  op = opponents.begin();

		  while (op != opponents.end())
		  {
			  if(op->getPosition().isKnown())
			  {
				  endPoint = op->getPosition().getPoint();
				  distance = startPoint.getDistanceTo(endPoint);
				  
				  angle = player->getPosition().getPoint().getDirectionTo(position);
				  angle = angle - player->getPosition().getPoint().getDirectionTo(endPoint);
				           
				  if (distance<= Area*facts->SP_kickable_area){

				  if (distance <= facts->SP_kickable_area*2){
					return 0; //may require modification,'cause not always the enemy reach the ball
					}

				  else mul = mul * (distance/facts->SP_kickable_area) * sin(abs(angle.getRad()) /2);
				  }
				  
			  }
		  op++;
		  }
		  return mul; 
		}
		else {
		return 0; 
		}
  }
}


//***********************partnerBodyDirection()**************************//
float PassView::partnerBodyDirection(PlayerObject *player) {	

	if(ballView->getSpeedVector().getAngle().isKnown()&&
		player->getBodyDirection().isKnown()){
		
		AngleDegUE difference = ballView->getSpeedVector().getAngle() - 
			player->getBodyDirection();
		
		if(difference==0) return 1;

		return 1-(difference.getDeg()/181); //WHAT TO DO WITH THE ANGLEE??
	}
	else return 0.01;
} 

//**************************partnerViewDivergence()*****************************//

float PassView::partnerViewDivergence(PlayerObject *player){

	if(agentView->getPosition().isKnown()&& player->getNeckDirection().isKnown()){
	
	  // get my position for evaluation
      Point myPosition = agentView->getPosition().getPoint(); 
      //MarkerObject goalMarker = facts
      Vector partnerVector = myPosition.getVectorTo(player->getPosition().getPoint()); 
		
	  VectorUE sightVector(player->getNeckDirection(),partnerVector.getLength());
	  AngleDegUE difference = partnerVector.getAngle() - sightVector.getAngle();
		
	  if(difference==0)return 1;
	  return 1-(difference.getDeg()/181);
	}
	else return 0.01;
}

//**********************************strategyModel()********************************//
float PassView::strategyModel(PlayerObject *player){
	
	if(strategy==NULL){
        return 0;
	}else{
		return 0;
	}
}

//**********************************moveToBetterPosition()********************************//
Point PassView::moveToBetterPosition(PlayerObject *player){

	if(agentView->getPosition().isKnown()){
	  
		//get the direction of the viewManager
		AngleDeg watchingAngle = agentView->getBodyDirection().getDeg() + agentView->getNeckDirection().getDeg();
		AngleDeg upperAngle = watchingAngle - agentView->getViewAngle().getDeg()/2;
		AngleDeg bottomAngle = watchingAngle + agentView->getViewAngle().getDeg()/2;
        AngleDeg movingAngle = upperAngle;
		
		Point bestPoint (0,0);
		Point point;
		float evaluation;
		float MaxEvaluation = 0;
		float x=0,y=0;

        while (movingAngle < bottomAngle){

			//get the point in arc

			Vector direction (movingAngle,moveDistance);
			Vector xProjection (0,moveDistance);
			Vector yProjection (90,moveDistance);

			xProjection = direction.projectOn(xProjection);
			yProjection = direction.projectOn(yProjection);
            
			x = xProjection.getLength();
			y = yProjection.getLength();
			

			if ((movingAngle <= AngleDeg(-90)) || (movingAngle >= AngleDeg(90)))
				x = -x;
			if ((movingAngle <= AngleDeg(0)) && (movingAngle >= AngleDeg(-180)))
				y = -y;
			x = agentView->getPosition().getPoint().getX() + x;
			y = agentView->getPosition().getPoint().getY() + y;
			
			point.setX(x);
			point.setY(y);

			//evaluate the point 
			evaluation = markedEval(player,true,point);
			if (evaluation > MaxEvaluation){
				bestPoint = point;
				MaxEvaluation = evaluation;
			}

		movingAngle = AngleDeg(moveAngle + movingAngle.getDeg());
		
		}
	//move to the best point
	if (MaxEvaluation == 0)
	    bestPoint = point;
     return bestPoint;     
     }	
  return Point(0,0);	
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PassView.icc"
#endif
