/* -*- Mode: C++ -*-
 * $Id: AgentObject.cc,v 1.10 2004/09/05 20:37:06 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentObject.cc
 * Unitname: Library
 * $Revision: 1.10 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2004/09/05 20:37:06 $
 *
 *
 * HISTORY
 *
 * $Log: AgentObject.cc,v $
 * Revision 1.10  2004/09/05 20:37:06  frehe
 * Improved support for focus.
 *
 * Revision 1.9  2003/11/04 17:17:32  frehe
 * Moved the goalie information from AgentObject to PlayerObject.
 *
 * Revision 1.8  2003/08/20 12:20:41  s02davro
 * Handle tackles, pointTos attentionTos
 *
 * Revision 1.7  2003/07/21 09:07:56  s02davro
 * Messages, arm, tackling
 *
 * Revision 1.6  2002/09/21 19:06:49  s02davro
 * Copycontructor and assigment didn't handle the messagedirection
 *
 * Revision 1.5  2002/08/31 14:46:01  s02davro
 * Small changes in the game objects.
 *
 * Revision 1.4  2002/08/02 21:28:51  s02davro
 * Changed some types, UCO->UE, Added some information about what is being done for the moment, See the attributes powerUsed, movePosition, turn and neckTurn for more information. Updated the documentation.
 *
 * Revision 1.3  2001/10/14 21:21:45  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.2  2000/10/18 23:05:41  frehe
 * Added lastCoachMessage and lastPlayerMessage.
 *
 * Revision 1.1  2000/09/01 03:24:33  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AgentObject.cc
 * Implementation of the class AgentObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.10 $
 * $Author: frehe $
 * $Date: 2004/09/05 20:37:06 $
 */

#include "AgentObject.h"

RS_BEGIN_NAMESPACE

AgentObject::AgentObject(const PointUE& def_point,
			 const TimeUnknown& def_time,
			 const VectorUE& def_speed,
			 const TimeUnknown& def_speed_time,
			 const AngleDegUE& def_body_dir,
			 const AngleDegUE& def_neck_dir,
			 const AngleDegUE& def_point_dir,
			 const bool def_tackling,
			 const bool def_goalie,
			 const Team def_team,
			 const Side def_side,
			 const TimeUnknown& def_team_time,
			 const UIntUnknown def_u_number,
			 const TimeUnknown& def_unum_time,
			 const ViewWidth def_view_width,
			 const ViewQuality def_view_quality,
			 const FloatUE def_stamina,
			 const FloatUE def_effort,
			 const FloatUE def_recover,
			 const VectorUE& def_acceleration,
			 const Float& def_power,
			 const PointUnknown& def_move,
			 const AngleDegUE& def_turn,
			 const AngleDegUE& def_neckTurn)        
  : PlayerObject(def_point, def_time, def_speed, def_speed_time, 
		 def_body_dir, def_neck_dir, def_point_dir, def_tackling,
		 def_goalie, def_team, def_side, 
		 def_team_time, def_u_number, def_unum_time, 
		 def_acceleration),
    stamina(def_stamina),
    effort(def_effort),
    recover(def_recover),
    dashes(0),
    kicks(0),
    says(0),
    turns(0),
    turnNecks(0),
    attentionTos(0),    
    tackles(0),
    pointTos(0),
    viewWidth(def_view_width),
    viewQuality(def_view_quality),
    cyclesSinceCatch(0),
    cyclesSinceSay(0),
    coachMessage(),
    ownMessage(),
    teammateMessage(),
    opponentMessage(),
    teammateMessageDirection(),
    opponentMessageDirection(),
    teammateSpeakers(),
    numberOfOpponentSpeakers(0),
    powerUsed(def_power),
    movePosition(def_move),
    turn(def_turn), 
    neckTurn(def_neckTurn),
    focusedTeam(T_UNKNOWN),
    focusedUniformNumber(),
    pointTarget(),
    armMoveableIn(0),
    armExpiresIn(0),
    tackleExpiresIn(0) {
}


AgentObject::AgentObject(const AgentObject& obj)
  : PlayerObject(obj),
    stamina (obj.stamina),
    effort (obj.effort),
    recover(obj.recover),
    dashes (obj.dashes),
    kicks (obj.kicks),
    says (obj.says),
    turns (obj.turns),
    turnNecks (obj.turnNecks),
    attentionTos (obj.attentionTos),
    tackles (obj.tackles),
    pointTos (obj.pointTos),
    viewWidth (obj.viewWidth),
    viewQuality (obj.viewQuality),
    cyclesSinceCatch (obj.cyclesSinceCatch),
    cyclesSinceSay (obj.cyclesSinceSay),
    coachMessage (obj.coachMessage),
    ownMessage (obj.ownMessage),
    teammateMessage (obj.teammateMessage),
    opponentMessage (obj.opponentMessage),
    opponentMessageDirection (obj.opponentMessageDirection),
    teammateMessageDirection (obj.teammateMessageDirection), 
    numberOfOpponentSpeakers (obj.numberOfOpponentSpeakers),
    powerUsed(obj.powerUsed),
    movePosition(obj.movePosition),
    turn(obj.turn),
    neckTurn(obj.neckTurn),
    focusedTeam(obj.focusedTeam),
    focusedUniformNumber(obj.focusedUniformNumber),
    pointTarget(obj.pointTarget),
    armMoveableIn(obj.armMoveableIn),
    armExpiresIn(obj.armExpiresIn),
    tackleExpiresIn(obj.tackleExpiresIn) {
  for (std::vector<UInt>::const_iterator i = obj.teammateSpeakers.begin();
       i != obj.teammateSpeakers.end(); ++i) 
    teammateSpeakers.push_back(*i);  
}

AgentObject& AgentObject::operator= (const AgentObject& obj) {
  if (this != &obj) {
    PlayerObject::operator=(obj);
    stamina                = obj.stamina;
    effort                 = obj.effort;
    recover                = obj.recover;
    dashes                 = obj.dashes;
    kicks                  = obj.kicks;
    turns                  = obj.turns;
    says                   = obj.says;
    turnNecks              = obj.turnNecks;
    attentionTos           = obj.attentionTos;
    tackles                = obj.tackles;
    pointTos               = obj.pointTos;
    viewWidth              = obj.viewWidth;
    viewQuality            = obj.viewQuality;
    cyclesSinceCatch       = obj.cyclesSinceCatch;
    cyclesSinceSay         = obj.cyclesSinceSay;
    coachMessage           = obj.coachMessage;
    ownMessage             = obj.ownMessage;
    teammateMessage        = obj.teammateMessage;
    teammateMessageDirection = obj.teammateMessageDirection;    
    opponentMessage        = obj.opponentMessage;
    opponentMessageDirection = obj.opponentMessageDirection;
    numberOfOpponentSpeakers = obj.numberOfOpponentSpeakers;
    powerUsed              = obj.powerUsed;
    movePosition           = obj.movePosition;
    turn                   = obj.turn;
    neckTurn               = obj.neckTurn;
    focusedTeam            = obj.focusedTeam;
    focusedUniformNumber   = obj.focusedUniformNumber;
    pointTarget            = obj.pointTarget;
    armMoveableIn          = obj.armMoveableIn;
    armExpiresIn           = obj.armExpiresIn;
    tackleExpiresIn        = obj.tackleExpiresIn;
    teammateSpeakers.clear();
    for (std::vector<UInt>::const_iterator i = obj.teammateSpeakers.begin();
	 i != obj.teammateSpeakers.end(); ++i) 
      teammateSpeakers.push_back(*i);  
  }
  return *this;  
}

void AgentObject::clearCommunication() {
  ownMessage = "";
  teammateMessage = "";
  opponentMessage = "";
  coachMessage = "";
  teammateMessageDirection = AngleDegUE();
  opponentMessageDirection = AngleDegUE();
  teammateSpeakers.clear();
  numberOfOpponentSpeakers = 0;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "AgentObject.icc"
#endif
