/* -*- Mode: C++ -*-
 * $Id: AgentObject.h,v 1.9 2004/09/05 20:37:06 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentObject.h
 * Unitname: Library
 * $Revision: 1.9 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2004/09/05 20:37:06 $
 *
 *
 * HISTORY
 *
 * $Log: AgentObject.h,v $
 * Revision 1.9  2004/09/05 20:37:06  frehe
 * Improved support for focus.
 *
 * Revision 1.8  2003/11/04 17:17:32  frehe
 * Moved the goalie information from AgentObject to PlayerObject.
 *
 * Revision 1.7  2003/08/20 12:20:42  s02davro
 * Handle tackles, pointTos attentionTos
 *
 * Revision 1.6  2003/07/21 09:07:56  s02davro
 * Messages, arm, tackling
 *
 * Revision 1.5  2002/08/31 14:46:01  s02davro
 * Small changes in the game objects.
 *
 * Revision 1.4  2002/08/02 21:28:51  s02davro
 * Changed some types, UCO->UE, Added some information about what is being done for the moment, See the attributes powerUsed, movePosition, turn and neckTurn for more information. Updated the documentation.
 *
 * Revision 1.3  2001/10/14 21:21:45  frehe
 * Added better support for messages from self and from other players.
 *
 * Revision 1.2  2000/10/18 23:05:36  frehe
 * Added lastCoachMessage and lastPlayerMessage.
 *
 * Revision 1.1  2000/09/01 03:24:30  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AgentObject.h
 * Include file for the class AgentObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2004/09/05 20:37:06 $
 */

#ifndef AGENTOBJECT_H
#define AGENTOBJECT_H

#include "PlayerObject.h"

RS_BEGIN_NAMESPACE

/**
 * @class AgentObject AgentObject.h
 * Class used to represent the agent.
 *
 *
 * @todo Make a toString and operator<<.  
 * @todo Add functionality to make heterogenouse player information easy to use.
 * @todo Handle the sent Commands in a good way (remove power,move,turn,neckturn).
 *       
 *
 * @par Unit:
 *      Library
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2004/09/05 20:37:06 $
 */

class AgentObject : public PlayerObject {
public:
  /** Construct the AgentObject. Default with all values unknown. */
  AgentObject(const PointUE& def_point = PointUE(),
	      const TimeUnknown& def_time = TimeUnknown(),
	      const VectorUE& def_speed = VectorUE(),
	      const TimeUnknown& def_speed_time = TimeUnknown(),
	      const AngleDegUE& def_body_dir = AngleDegUE(),
	      const AngleDegUE& def_neck_dir = AngleDegUE(),
	      const AngleDegUE& def_point_dir = AngleDegUE(),
	      const bool def_tackling = false,
	      const bool def_goalie = false,
	      const Team def_team = T_UNKNOWN,
	      const Side def_side = S_UNKNOWN,
	      const TimeUnknown& def_team_time = TimeUnknown(),
	      const UIntUnknown def_u_number = UIntUnknown(),
	      const TimeUnknown& def_unum_time = TimeUnknown(),
	      const ViewWidth def_view_width = VW_UNKNOWN,
	      const ViewQuality def_view_quality = VQ_UNKNOWN,	      
	      const FloatUE def_stamina = FloatUE(),
	      const FloatUE def_effort = FloatUE(),
	      const FloatUE def_recover = FloatUE(),
	      const VectorUE& def_acceleration = VectorUE(),
	      const Float& def_power = Float(),
	      const PointUnknown& def_move = PointUnknown(),
	      const AngleDegUE& def_turn = AngleDegUE(),
	      const AngleDegUE& def_neckTurn = AngleDegUE());
  
  /** Copy constructor for the agent. */
  AgentObject(const AgentObject& obj);
  
  /** Assignment operator. */
  AgentObject& operator= (const AgentObject& obj);
  
  /** Set all agent attributes to unknown. */
  void reset();

  /** Set the agents stamina. */
  void setStamina(const FloatUE new_stamina);

  /** Returns the agents stamina. */
  FloatUE getStamina() const;

  /** Set the agents effort */
  void setEffort(const FloatUE new_effort);
  
  /** Returns the agents effort. */
  FloatUE getEffort() const;

  /** Set the agents recovery value. */
  void setRecover(const FloatUE new_recover);

  /** Returns the agent recovery value. */
  FloatUE getRecover() const;

  /** Set the number of dashes executed by the agent. */
  void setDashes(const UInt new_dashes);
  
  /** Returns the number of dashes executed by the agent. */
  UInt getDashes() const;

  /** Set the number of kicks executed by the agent. */
  void setKicks(const UInt new_kicks);

  /** Returns the number of kicks executed by the agent. */
  UInt getKicks() const;

  /** Set the number of says executed by the agent. */
  void setSays(const UInt new_says);

  /** Returns the number of says executed by the agent. */
  UInt getSays() const;

  /** Set the number of turns executed by the agent. */
  void setTurns(const UInt new_turns);

  /** Returns the number of turns executed by the agent. */
  UInt getTurns() const;

  /** Set the number of neckturns executed by the agent. */
  void setTurnNecks(const UInt new_turn_necks);

  /** Returns the number of neckturns executed by the agent. */
  UInt getTurnNecks() const;

  /** Set the number of attentiontos executed by the agent. */
  void setAttentionTos(const UInt new_attention_tos);

  /** Returns the number of attentiontos executed by the agent. */
  UInt getAttentionTos() const;

  /** Set the number of tackles executed by the agent. */
  void setTackles(const UInt new_tackles);

  /** Returns the number of tackles executed by the agent. */
  UInt getTackles() const;

  /** Set the number of pointtos executed by the agent. */
  void setPointTos(const UInt new_point_tos);

  /** Returns the number of pointtos executed by the agent. */
  UInt getPointTos() const;
  
  /** Set the number of cycles since the agent (goalie) made a catch. */
  void setCyclesSinceCatch(const UInt new_cycs_since_catch);
  
  /** Returns the cycles since last catch. */
  UInt getCyclesSinceCatch() const;
  
  /** Set the number of cycles since the agent last spoke. */
  void setCyclesSinceSay(const UInt new_cycs_since_say);

  /** Return the number of cycles since the agent last spoke. */
  UInt getCyclesSinceSay() const;
  
  /** Sets the agents ViewWidth. */
  void setViewWidth(const ViewWidth new_view_width);

  /** Gets the agents ViewWidth. */
  ViewWidth getViewWidth() const;
  
  /** Set the agents ViewQUality. */
  void setViewQuality(const ViewQuality new_view_quality);

  /** Gets the agents ViewQuality. */
  ViewQuality getViewQuality() const;

  /** Set the last message heard from coach. */
  void setCoachMessage(const std::string& msg);

  /** Returns the last message heard from coach. */
  const std::string& getCoachMessage() const;
  
  /** Set the last message heard from ourself. */
  void setOwnMessage(const std::string& msg);

  /** Returns the last message heard from ourself. */
  const std::string& getOwnMessage() const;
  
  /** Set the last message heard from another player. */
  void setTeammateMessage(const std::string& msg);

  /** Returns the last message heard from another player. */
  const std::string& getTeammateMessage() const;
  
  /** Set the direction of the last teammate message. */
  void setTeammateMessageDirection(const AngleDegUE& dir);

  /** Returns the direction of the last playermessage. */
  AngleDegUE getTeammateMessageDirection() const;

  /** Get the number of the heard teammate. */
  UInt getTeammateSpeaker() const;
  
  /** Get the numbers of all the teammates that tried to speak this cykle. */
  const std::vector<UInt>& getTeammateSpeakers() const;

  /** Add a teammate spekaer. */
  void addTeammateSpeaker(UInt nr);

  /** Set the last message heard from an opponent. */
  void setOpponentMessage(const std::string& msg);

  /** Returns the last message heard from the opponents. */
  const std::string& getOpponentMessage() const;

  /** Set the direction of the last opponent message. */
  void setOpponentMessageDirection(const AngleDegUE& dir);

  /** Returns the direction of the last opponent message. */
  AngleDegUE getOpponentMessageDirection() const;

  UInt getNumberOfOpponentSpeakers() const;
  
  void setNumberOfOpponentSpeakers(UInt nr);  

  void clearCommunication();

  /** Set the power currently used (for dashing) by the agent. */
  void setPowerUsed(const Float& power);

  /** Returns the power used at the moment. */
  Float getPowerUsed() const;

  /** Set the position the agent is about to move to. */
  void setMovePosition(const PointUnknown& def_move);

  /** Returns the position the agent is about to move to. */
  PointUnknown getMovePosition() const;

  /** Set the turn the agent is about to execute. */
  void setTurn(const AngleDegUE& power);
  
  /** Returns the turn the agent is about to execute. */
  AngleDegUE getTurn() const;
  
  /** Set the neckturn the agent is about to execute. */
  void setTurnNeck(const AngleDegUE& power);
  
  /** Returns the neckturn the agent is about to execute. */
  AngleDegUE getTurnNeck() const;

  /** Get the team currently focused on (if any). */
  Team getFocusedTeam() const;

  /** Set the team currently focused on (if any). */
  void  setFocusedTeam(const Team& new_focused_team);

  /** Get the player currently focused on (if any). */
  UIntUnknown getFocusedUniformNumber() const;

  /** Set the player currently focused on (if any). */
  void  setFocusedUniformNumber(const UIntUnknown& new_focused_nr);

  /** Set the point currently pointed at (if any). */
  PointUnknown getPointTarget() const;

  /** Get the point currently pointed at (if any). */
  void setPointTarget(const PointUnknown& new_point);

  /** Set the number of cycles until the arm will be mobile. */
  UInt getArmMoveableIn() const;

  /** Get the number of cycles until the arm will be mobile. */
  void setArmMoveableIn(const UInt new_moveable_in);
  
  /** The number of cycles until the current point expires. */
  UInt getArmExpiresIn() const;

  /** The number of cycles until the current point expires. */
  void setArmExpiresIn(const UInt new_expires_in);

  /** The number of cycles until the tackle expires. */
  UInt getTackleExpiresIn() const;

  /** The number of cycles until the tackle expires. */
  void setTackleExpiresIn(const UInt new_expires_in);
  
protected:
  /** Stamina for the agent. */
  FloatUE stamina;

  /** Effort for the agent. */
  FloatUE effort;

  /** Recovery for the agent. */
  FloatUE recover;

  /** Number of executed dashes. */
  UInt dashes;

  /** Number of executed kicks. */
  UInt kicks;

  /** Number of executed says. */
  UInt says;

  /** Number of executed turns. */
  UInt turns;

  /** Number of executed neckturns. */
  UInt turnNecks;

  /** Number of executed attentiontos. */
  UInt attentionTos;

  /** Number of executed tackles. */
  UInt tackles;

  /** Number of executed pointtos. */
  UInt pointTos;

  /** The agens viewWidth.
   *  The valid values are VW_WIDE (90deg), VW_NORMAL (45deg)
   *  VW_NARROW (22.5deg). */
  ViewWidth viewWidth;

  /** The agents viewQuality.
   *  The valid values are VQ_HIGH (good), VQ_LOW (not recomended). */      
  ViewQuality viewQuality;
  
  /** The number of cycles since the agent made a catch. */
  UInt cyclesSinceCatch;
  
  /** The number of cycles since the agent spoke. */
  UInt cyclesSinceSay;

  /** Message heard from ourself. */
  std::string ownMessage;

  /** Message heard from a teammate. */
  std::string teammateMessage;

  /** Message heard from an opponent. */
  std::string opponentMessage;

  /** Message heard from the coach. */
  std::string coachMessage;
  
  /** Direction of the message from the teammate. */
  AngleDegUE teammateMessageDirection;

  /** Direction of the message from the opponent. */
  AngleDegUE opponentMessageDirection;

  /** Teammate speakers. 
   *  List of temmates that is speaking this cykle. First in list is
   *  the number of the teammate whos message you heard. */
  std::vector<UInt> teammateSpeakers;
  
  /** Number of opponent speakers. */
  Int numberOfOpponentSpeakers;
  
  /** The power currently used by the agent. */
  Float powerUsed;
  
  /** The position the agent is about to move to. */
  PointUnknown movePosition;
  
  /** The turn the agent is about to execute. */
  AngleDegUE turn;
  
  /** The neckturn the agent is about to execute. */
  AngleDegUE neckTurn;

  /** The team currently focused on (if any). */
  Team focusedTeam;

  /** The player currently focused on (if any). */
  UIntUnknown focusedUniformNumber;

  /** The point currently pointed at (if any). */
  PointUnknown pointTarget;

  /** The number of cycles until the arm will be mobile 
   *  0 means that the arm is currently mobile. */
  UInt armMoveableIn;
  
  /** The number of cycles until the current point expires. 
   *  0 means that we're not pointing for the moment. */
  UInt armExpiresIn;

  /** The number of cycles until the tackle expires. 
   *  0 means that we're not in tackle mode and is therefore mobile. */
  UInt tackleExpiresIn;      
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AgentObject.icc"
#endif


#endif // !AGENTOBJECT_H
