/* -*- Mode: C++ -*-
 * $Id: AngleDegreesUnknown.h,v 1.9 2005/08/13 13:31:17 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AngleDegreesUnknown.h
 * Unitname: Library
 * $Revision: 1.9 $
 * Created by: Fredrik Heintz 2000-08-31
 * Last modified by $Author: frehe $ $Date: 2005/08/13 13:31:17 $
 *
 *
 * HISTORY
 *
 * $Log: AngleDegreesUnknown.h,v $
 * Revision 1.9  2005/08/13 13:31:17  frehe
 * Fixed several errors to make it compile with gcc-3.4.3
 *
 * Revision 1.8  2002/09/03 10:56:30  s02davro
 * Small modifications
 *
 * Revision 1.7  2002/08/31 14:50:27  s02davro
 * Modifications, corrections of angle classes
 *
 * Revision 1.6  2002/08/02 21:32:07  s02davro
 * Updated the documentation. Rewrote some things to be able to handle the WithError concept.
 *
 * Revision 1.5  2001/08/29 21:08:27  frehe
 * Replaces class T with typename T where T can be a built in type.
 *
 * Revision 1.4  2000/09/18 15:20:04  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.3  2000/09/04 15:21:48  frehe
 * Changed the name of the trigonometric functions to the same as in the standard math library (sin, cos, tan, asin, acos, atan, and atan2).
 *
 * Revision 1.2  2000/09/01 03:23:51  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:36  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AngleDegreesUnknown.h
 * Include file for the template AngleDegreesUnknown.
 *
1 * @par Unit:
 *      Library
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:31:17 $
 */


#ifndef ANGLEDEGREESUNKNOWN_H
#define ANGLEDEGREESUNKNOWN_H


#include "AngleDegrees.h"
#include "WithUnknown.h"
#include "UnknownValueException.h"
#include "rs_throw.h"

#define AUNKNOWN_HEAD std::string("#AUnknown")

RS_BEGIN_NAMESPACE


/**
 * @class AngleDegreesUnknown AngleDegreesUnknown.h
 * Template class for representing angles in degrees that can be unknown.
 * The template argument only defines what type the angles should be 
 * represented by.
 * The only demands on the template argument T is that the trigonometric
 * and arithmetic functions are defined on it.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2005/08/13 13:31:17 $
 */

template<typename T> 
class AngleDegreesUnknown : public WithUnknown<AngleDegrees<T> > {
public:
  /** Default constructor, the value is set to unknown. */
  AngleDegreesUnknown();

  /** Construct a AngleDegreesUnknown with value set to ang. */
  AngleDegreesUnknown(const T& ang);

  /** Construct a AngleDegreesUnknown using a WithUnknown<T>. */
  AngleDegreesUnknown(const WithUnknown<T>& ang);

  /** Construct object using the value in a AngleDegrees<T>. */
  AngleDegreesUnknown(const AngleDegrees<T>& ang);

  /** Construct object using the value in a AngleRadians<T>. */
  AngleDegreesUnknown(const AngleRadians<T>& ang);

  /** Construct object using a (known or unknown) AngleDegrees<T>. */
  AngleDegreesUnknown(const WithUnknown<AngleDegrees<T> >& ang);  

  /** Copyconstructor. */
  AngleDegreesUnknown(const AngleDegreesUnknown<T>& ang);

  /** Assignment operator. */
  AngleDegreesUnknown<T>& operator= (const AngleDegreesUnknown<T> obj);    
  
  /** @name Arithmetic operators. */
  //@{
  AngleDegreesUnknown<T>& operator+= (const AngleDegreesUnknown<T>& op);
  AngleDegreesUnknown<T>& operator-= (const AngleDegreesUnknown<T>& op);
  AngleDegreesUnknown<T>& operator*= (const AngleDegreesUnknown<T>& op);
  AngleDegreesUnknown<T>& operator/= (const AngleDegreesUnknown<T>& op);
  AngleDegreesUnknown<T> operator+ (const AngleDegreesUnknown<T>& op) const;
  AngleDegreesUnknown<T> operator- (const AngleDegreesUnknown<T>& op) const;
  AngleDegreesUnknown<T> operator* (const AngleDegreesUnknown<T>& op) const;
  AngleDegreesUnknown<T> operator/ (const AngleDegreesUnknown<T>& op) const;
  AngleDegreesUnknown<T> operator- () const;
  //@}

  /** @name Increment and decrement operators. */
  //@{
  AngleDegreesUnknown<T>& operator++ ();
  AngleDegreesUnknown<T>& operator-- ();
  //@}

  /** @name Comparison operators.
   *  @see WithUnknown for more information */
  //@{
  bool operator== (const AngleDegreesUnknown<T>& op) const;
  bool operator!= (const AngleDegreesUnknown<T>& op) const;
  bool operator< (const AngleDegreesUnknown<T>& op) const;
  bool operator<= (const AngleDegreesUnknown<T>& op) const;
  bool operator> (const AngleDegreesUnknown<T>& op) const;
  bool operator>= (const AngleDegreesUnknown<T>& op) const;
  bool operator!  () const;
  //@}
  
  /** Make the angle unknown. */
  AngleDegreesUnknown<T>& makeUnknown();

  /** Check if the angle is known. */
  bool isKnown() const;

  /** Check if the angle is unknown. */
  bool isUnknown() const;

  /** Get the angle. 
   *  @throws UnknownValueException if the angle is unknown. */
  AngleDegrees<T> getAngle() const throw(UnknownValueException);

  /** Get the angle in radians.
   *  @throws UnknownValueException if the angle is unknown. */
  T getRad() const throw(UnknownValueException);

  /** Get the angle in degrees.
   *  @throws UnknownValueException if the angle is unknown. */
  T getDeg() const throw(UnknownValueException);
};

template<typename T>
AngleDegreesUnknown<T>::AngleDegreesUnknown()
  : WithUnknown<AngleDegrees<T> >() {
}

template<typename T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const T& ang)
  : WithUnknown<AngleDegrees<T> >(ang) {
}

template<typename T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const WithUnknown<T>& ang)
  : WithUnknown<AngleDegrees<T> >() {
  if (ang.isKnown()) {
    WithUnknown<AngleDegrees<T> >::operator=(ang.getValue());
  }
}

template<typename T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const AngleDegrees<T>& ang)
  : WithUnknown<AngleDegrees<T> >(ang) {
}

template<typename T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const WithUnknown<AngleDegrees<T> >& ang) 
  : WithUnknown<AngleDegrees<T> >(ang) {
}

template<typename T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const AngleDegreesUnknown<T>& ang)
  : WithUnknown<AngleDegrees<T> >(ang) {
}

template<typename T>
AngleDegreesUnknown<T>& 
AngleDegreesUnknown<T>::operator= (const AngleDegreesUnknown<T> obj) {
  if (&obj != this) {
    if (obj.isKnown()) {
      WithUnknown<AngleDegrees<T> >::operator=(obj.getValue());
    } else {
      makeUnknown();
    }
  }
  return *this;
}

template<typename T>
inline AngleDegreesUnknown<T>& 
AngleDegreesUnknown<T>::operator+= (const AngleDegreesUnknown<T>& op) {
  WithUnknown<AngleDegrees<T> >::operator+= (op);
  return *this;
}

template<typename T>
inline AngleDegreesUnknown<T>& 
AngleDegreesUnknown<T>::operator-= (const AngleDegreesUnknown<T>& op) {
  WithUnknown<AngleDegrees<T> >::operator-= (op);
  return *this;
}

template<typename T>
inline AngleDegreesUnknown<T>& 
AngleDegreesUnknown<T>::operator*= (const AngleDegreesUnknown<T>& op) {
  WithUnknown<AngleDegrees<T> >::operator*= (op);
  return *this;
}

template<typename T>
inline AngleDegreesUnknown<T>& 
AngleDegreesUnknown<T>::operator/= (const AngleDegreesUnknown<T>& op) {
  WithUnknown<AngleDegrees<T> >::operator/- (op);
  return *this;
}

template<typename T>
inline AngleDegreesUnknown<T> 
AngleDegreesUnknown<T>::operator+ (const AngleDegreesUnknown<T>& op) const {
  return AngleDegreesUnknown<T>(WithUnknown<AngleDegrees<T> >::operator+ (op));
}

template<typename T>
inline AngleDegreesUnknown<T> 
AngleDegreesUnknown<T>::operator- (const AngleDegreesUnknown<T>& op) const {
  return AngleDegreesUnknown<T>(WithUnknown<AngleDegrees<T> >::operator- (op));
}

template<typename T>
inline AngleDegreesUnknown<T> 
AngleDegreesUnknown<T>::operator* (const AngleDegreesUnknown<T>& op) const {
  return AngleDegreesUnknown<T>(WithUnknown<AngleDegrees<T> >::operator* (op));
}

template<typename T>
inline AngleDegreesUnknown<T> 
AngleDegreesUnknown<T>::operator/ (const AngleDegreesUnknown<T>& op) const {
  return AngleDegreesUnknown<T>(WithUnknown<AngleDegrees<T> >::operator/ (op));
}

template<typename T>
inline bool 
AngleDegreesUnknown<T>::operator== (const AngleDegreesUnknown<T>& op) const {
  if ( &op != this)
    return WithUnknown<AngleDegrees<T> >::operator== (op);
  return true;
}

template<typename T>
inline bool 
AngleDegreesUnknown<T>::operator!= (const AngleDegreesUnknown<T>& op) const {
  return !(*this == op);
}

template<typename T>
inline bool 
AngleDegreesUnknown<T>::operator< (const AngleDegreesUnknown<T>& op) const {
  return WithUnknown<AngleDegrees<T> >::operator< (op);
}

template<typename T>
inline bool 
AngleDegreesUnknown<T>::operator<= (const AngleDegreesUnknown<T>& op) const {
  return WithUnknown<AngleDegrees<T> >::operator<= (op);
}

template<typename T>
inline bool 
AngleDegreesUnknown<T>::operator> (const AngleDegreesUnknown<T>& op) const {
  return WithUnknown<AngleDegrees<T> >::operator> (op);
}

template<typename T>
inline bool 
AngleDegreesUnknown<T>::operator>= (const AngleDegreesUnknown<T>& op) const {
  return WithUnknown<AngleDegrees<T> >::operator>= (op);
}

template<typename T>
inline bool AngleDegreesUnknown<T>::operator! () const {
  return WithUnknown<AngleDegrees<T> >::operator! ();
}

template<typename T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator++ () {
  WithUnknown<AngleDegrees<T> >::operator++ ();
  return *this;
}

template<typename T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator-- () {
  WithUnknown<AngleDegrees<T> >::operator-- ();
  return *this;
}

template<typename T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator- () const {
  return AngleDegreesUnknown<T>(WithUnknown<AngleDegrees<T> >::operator- ());
}

template<typename T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::makeUnknown() {
  WithUnknown<AngleDegrees<T> >::makeUnknown();
  return *this;
}

template<typename T>
inline bool AngleDegreesUnknown<T>::isKnown() const {
  return WithUnknown<AngleDegrees<T> >::isKnown();
}

template<typename T>
inline bool AngleDegreesUnknown<T>::isUnknown() const {
  return WithUnknown<AngleDegrees<T> >::isUnknown();
}

template<typename T>
inline AngleDegrees<T> 
AngleDegreesUnknown<T>::getAngle() const throw(UnknownValueException) {  
  return WithUnknown<AngleDegrees<T> >::getValue();
}

template<typename T>
inline T AngleDegreesUnknown<T>::getRad() const throw(UnknownValueException) {
  return WithUnknown<AngleDegrees<T> >::getValue().getRad();
}


template<typename T>
inline T AngleDegreesUnknown<T>::getDeg() const throw(UnknownValueException) {
  return WithUnknown<AngleDegrees<T> >::getValue().getDeg();
}

template<typename T>
inline AngleDegreesUnknown<T> operator- (const T& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return (-op2) + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator- (const WithUnknown<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return (-op2) + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator- (const AngleDegrees<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return (-op2) + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator- (const AngleRadians<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return (-op2) + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> 
operator- (const WithUnknown<AngleDegrees<T> >& op1,
	   const AngleDegreesUnknown<T>& op2) {
  return (-op2) + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator+ (const T& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator+ (const WithUnknown<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator+ (const AngleDegrees<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator+ (const AngleRadians<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> 
operator+ (const WithUnknown<AngleDegrees<T> >& op1,
	   const AngleDegreesUnknown<T>& op2) {
  return op2 + op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator* (const T& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator* (const WithUnknown<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator* (const AngleDegrees<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator* (const AngleRadians<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUnknown<T> 
operator* (const WithUnknown<AngleDegrees<T> >& op1,
	   const AngleDegreesUnknown<T>& op2) {
  return op2 * op1;
}

template<typename T>
inline AngleDegreesUnknown<T> operator/ (const T& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return AngleDegreesUnknown<T>(op1) / op2;
}

template<typename T>
inline AngleDegreesUnknown<T> operator/ (const WithUnknown<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return AngleDegreesUnknown<T>(op1) / op2;
}

template<typename T>
inline AngleDegreesUnknown<T> operator/ (const AngleDegrees<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return AngleDegreesUnknown<T>(op1) / op2;
}

template<typename T>
inline AngleDegreesUnknown<T> operator/ (const AngleRadians<T>& op1,
					 const AngleDegreesUnknown<T>& op2) {
  return AngleDegreesUnknown<T>(op1) / op2;
}

template<typename T>
inline AngleDegreesUnknown<T> 
operator/ (const WithUnknown<AngleDegrees<T> >& op1,
	   const AngleDegreesUnknown<T>& op2) {
  return AngleDegreesUnknown<T>(op1) / op2;
}

template<typename T>
inline bool operator==(const T& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 == op1;
}

template<typename T>
inline bool operator==(const WithUnknown<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 == op1;
}

template<typename T>
inline bool operator==(const AngleDegrees<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 == op1;
}

template<typename T>
inline bool operator==(const AngleRadians<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 == op1;
}

template<typename T>
inline bool operator==(const WithUnknown<AngleDegrees<T> >& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 == op1;
}

template<typename T>
inline bool operator!=(const T& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 != op1;
}

template<typename T>
inline bool operator!=(const WithUnknown<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 != op1;
}

template<typename T>
inline bool operator!=(const AngleDegrees<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 != op1;
}

template<typename T>
inline bool operator!=(const AngleRadians<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 != op1;
}

template<typename T>
inline bool operator!=(const WithUnknown<AngleDegrees<T> >& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 != op1;
}

template<typename T>
inline bool operator<(const T& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 > op1;
}

template<typename T>
inline bool operator<(const WithUnknown<T>& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 > op1;
}

template<typename T>
inline bool operator<(const AngleDegrees<T>& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 > op1;
}

template<typename T>
inline bool operator<(const AngleRadians<T>& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 > op1;
}

template<typename T>
inline bool operator<(const WithUnknown<AngleDegrees<T> >& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 > op1;
}

template<typename T>
inline bool operator<=(const T& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 >= op1;
}

template<typename T>
inline bool operator<=(const WithUnknown<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 >= op1;
}

template<typename T>
inline bool operator<=(const AngleDegrees<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 >= op1;
}

template<typename T>
inline bool operator<=(const AngleRadians<T>& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 >= op1;
}

template<typename T>
inline bool operator<=(const WithUnknown<AngleDegrees<T> >& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 >= op1;
}

template<typename T>
inline bool operator>(const AngleDegreesUnknown<T>& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 < op1;
}

template<typename T>
inline bool operator>(const T& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 < op1;
}

template<typename T>
inline bool operator>(const WithUnknown<T>& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 < op1;
}

template<typename T>
inline bool operator>(const AngleDegrees<T>& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 < op1;
}

template<typename T>
inline bool operator>(const AngleRadians<T>& op1,
		      const AngleDegreesUnknown<T> op2) {
  return op2 < op1;
}

template<typename T>
inline bool operator>=(const WithUnknown<AngleDegrees<T> >& op1,
		       const AngleDegreesUnknown<T> op2) {
  return op2 <= op1;
}

template<typename T>
inline std::ostream& operator <<(std::ostream& os, 
				 const AngleDegreesUnknown<T>& ang) {
#ifdef AUNKNOWN_HEAD
  if (ang.isKnown())
    return os << AUNKNOWN_HEAD << "(" << ang.getAngle() << ")";
  else 
    return os << AUNKNOWN_HEAD << "(UNKNOWN)";
#else
  if (ang.isKnown())
    return os << ang.getAngle();
  else
    return os << "(UNKNOWN)";
#endif
}

template<typename T>
inline const std::string toString(const AngleDegreesUnknown<T>& ang) {
#ifdef AUNKNOWN_HEAD
  if (ang.isKnown())    
    return AUNKNOWN_HEAD + "(" + toString(ang.getAngle()) + ")";
  else 
    return AUNKNOWN_HEAD + "(UNKNOWN)";
#else
  if (ang.isKnown())
    return toString(ang.getAngle());
  else
    return toString("(UNKNOWN)");
#endif
}

template<typename T>
inline WithUnknown<T> sin(const AngleDegreesUnknown<T>& x) {
  if ( x.isUnknown() )
    return WithUnknown<T>();
  else
    return WithUnknown<T>(sin(x.getAngle()));
}


template<typename T>
inline WithUnknown<T> cos(const AngleDegreesUnknown<T>& x) {
  if ( x.isUnknown() )
    return WithUnknown<T>();
  else
    return WithUnknown<T>(cos(x.getAngle()));
}


template<typename T>
inline WithUnknown<T> tan(const AngleDegreesUnknown<T>& x) {
  if ( x.isUnknown() )
    return WithUnknown<T>();
  else
    return WithUnknown<T>(tan(x.getAngle()));
}

#ifdef RS_TRIG_IN_DEG

template<typename T>
inline AngleDegreesUnknown<T> ASin(const WithUnknown<T>& op) {
  if ( op.isUnknown() )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(ASin(op.getValue()));
}

template<typename T>
inline AngleDegreesUnknown<T> ACos(const WithUnknown<T>& op) {
  if ( op.isUnknown() )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(ACos(op.getValue()));
}

template<typename T>
inline AngleDegreesUnknown<T> ATan(const WithUnknown<T>& op) {
  if ( op.isUnknown() )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(ATan(op.getValue()));
}

template<typename T>
inline AngleDegreesUnknown<T> ATan2(const WithUnknown<T>& op1,
				    const WithUnknown<T>& op2) {
  if ( op1.isUnknown() ||
       op2.isUnknown() )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(ATan2(op1.getValue(), op2.getValue()));
}

#endif // RS_TRIG_IN_DEG

RS_END_NAMESPACE

#endif // !ANGLEDEGREESUNKNOWN_H
