/* -*- Mode: C++ -*-
 * $Id: AttentionToCommand.cc,v 1.3 2003/08/20 12:21:47 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttentionToCommand.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2003/08/20 12:21:47 $
 *
 *
 * HISTORY
 *
 * $Log: AttentionToCommand.cc,v $
 * Revision 1.3  2003/08/20 12:21:47  s02davro
 * Corrected Command type
 *
 * Revision 1.2  2003/07/21 09:11:46  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.1  2002/10/01 16:06:28  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file AttentionToCommand.cc
 * Implementation of the class AttentionToCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */

#include "AttentionToCommand.h"
#include "string_utils.h"

RS_BEGIN_NAMESPACE

AttentionToCommand::AttentionToCommand(const Team def_team, const UInt def_unumber)
  : ImmediateCommand(CMD_AttentionTo),
    team(def_team),
    uniformNumber(def_unumber),
    off(false) {
}

AttentionToCommand::AttentionToCommand()
  : ImmediateCommand(CMD_AttentionTo),
    off(true) {
}

AttentionToCommand::~AttentionToCommand() {
}

const std::string AttentionToCommand::serverString() const {
  return "(attentionto " + (off?toString("off)"):
			    (((team == T_OUR_TEAM)?toString("our "):
			      toString("opp ")) + toString(uniformNumber) + ")")); 
}

AttentionToCommand* AttentionToCommand::clone() const {
  return new AttentionToCommand(*this);
}

Team AttentionToCommand::getTeam() const {
  return team;
}

UInt AttentionToCommand::getUniformNumber() const {
  return uniformNumber;
}

bool AttentionToCommand::isOff() const {
  return off;
}

RS_END_NAMESPACE

