/* -*- Mode: C++ -*-
 * $Id: AttentionToCommand.h,v 1.3 2003/08/20 12:21:47 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AttentionToCommand.h
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2003/08/20 12:21:47 $
 *
 *
 * HISTORY
 *
 * $Log: AttentionToCommand.h,v $
 * Revision 1.3  2003/08/20 12:21:47  s02davro
 * Corrected Command type
 *
 * Revision 1.2  2003/07/21 09:11:47  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.1  2002/10/01 16:06:27  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file AttentionToCommand.h
 * Include file for the class AttentionToCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */


#ifndef ATTENTIONTOCOMMAND_H
#define ATTENTIONTOCOMMAND_H

#include "ImmediateCommand.h"
#include "soccer_types.h"

RS_BEGIN_NAMESPACE

/**
 * @class AttentionToCommand AttentionToCommand.h
 * Class used to represent the attention to command.
 * Use this to set a players attention to anoter.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */

class AttentionToCommand : public ImmediateCommand {
public:
  /** Set attention to a player. */
  AttentionToCommand(const Team def_team, const UInt def_unumber);

  /** Turn of the attention. */
  AttentionToCommand();

  ~AttentionToCommand();

  /** Returns the server string that will execute the AttentionTo command. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  AttentionToCommand* clone() const;

  Team getTeam() const;
  UInt getUniformNumber() const;
  bool isOff() const;  
private:
  Team team;
  UInt uniformNumber;
  bool off;
};

RS_END_NAMESPACE

#endif //ATTENTIONTOCOMMAND_H
