/* -*- Mode: C++ -*-
 * $Id: ByeCommand.h,v 1.6 2003/07/21 09:11:47 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ByeCommand.h
 * Unitname: Library
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:47 $
 *
 *
 * HISTORY
 *
 * $Log: ByeCommand.h,v $
 * Revision 1.6  2003/07/21 09:11:47  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.5  2000/09/14 20:38:36  frehe
 * Added support for building shared libraries with libtool.
 *
 * Revision 1.4  2000/09/09 11:49:43  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.3  2000/09/05 21:08:48  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.2  2000/09/03 11:10:25  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/01 11:43:39  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ByeCommand.h
 * Include file for the class ByeCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:47 $
 */

#ifndef BYECOMMAND_H
#define BYECOMMAND_H

#include "ImmediateCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class ByeCommand ByeCommand.h
 * Class used to represent the server command (bye).
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:47 $
 */
class ByeCommand : public ImmediateCommand {
public:
  ByeCommand();
  ~ByeCommand();

  /** Returns the server string that will execute the ByeCommand
      at the soccer server. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  ByeCommand* clone() const;
};

RS_END_NAMESPACE

#endif // !BYECOMMAND_H
