/* -*- Mode: C++ -*-
 * $Id: ChangeModeCommand.cc,v 1.2 2003/07/22 14:50:09 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ChangeModeCommand.cc
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/22 14:50:09 $
 *
 *
 * HISTORY
 *
 * $Log: ChangeModeCommand.cc,v $
 * Revision 1.2  2003/07/22 14:50:09  s02davro
 * Removed imposible playmodes
 *
 * Revision 1.1  2003/07/21 08:29:12  s02davro
 * Added new commands (v9, trainer, coach, new command hierarchy)
 *
 * Revision 1.2  2000/09/09 11:49:52  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.1  2000/09/01 11:43:42  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ChangeModeCommand.cc
 * Implementation of the class ChangeModeCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/07/22 14:50:09 $
 */

#include "ChangeModeCommand.h"
#include "rs_log.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

ChangeModeCommand::ChangeModeCommand(PlayMode pm)
  : TrainerCommand(CMD_ChangeMode),
    playMode(pm) {}


ChangeModeCommand::~ChangeModeCommand() {}

const std::string ChangeModeCommand::serverString() const {
  std::string side;
  std::string pm;
  bool sidePM = false;
  switch (playMode) {
  case PM_OUR_OFFSIDE_KICK:
  case PM_OUR_KICK_OFF:
  case PM_OUR_KICK_IN:
  case PM_OUR_FREE_KICK:
  case PM_OUR_INDIRECT_FREE_KICK:
  case PM_OUR_CORNER_KICK:
  case PM_OUR_GOAL_KICK:
  case PM_OUR_GOAL:
  case PM_OUR_PENALTY_KICK:
  case PM_OUR_PENALTY_SETUP:
  case PM_OUR_PENALTY_READY:
  case PM_OUR_PENALTY_TAKEN:
  case PM_OUR_PENALTY_MISS:
  case PM_OUR_PENALTY_SCORE:
  case PM_OUR_FOUL_CHARGE:
  case PM_OUR_FOUL_PUSH:
  case PM_OUR_FOUL_MULTIPLEATTACKER:
  case PM_OUR_FOUL_BALLOUT:
  case PM_OUR_FOUL_BACKPASS:
  case PM_OUR_FOUL_FREEKICKFAULT:
  case PM_OUR_FOUL_CATCHFAULT:
    side = "l";
    sidePM = true;
    break;
  case PM_THEIR_OFFSIDE_KICK:
  case PM_THEIR_KICK_OFF:
  case PM_THEIR_KICK_IN:
  case PM_THEIR_FREE_KICK:
  case PM_THEIR_INDIRECT_FREE_KICK:
  case PM_THEIR_CORNER_KICK:
  case PM_THEIR_GOAL_KICK:
  case PM_THEIR_GOAL:
  case PM_THEIR_PENALTY_KICK:
  case PM_THEIR_PENALTY_SETUP:
  case PM_THEIR_PENALTY_READY:
  case PM_THEIR_PENALTY_TAKEN:
  case PM_THEIR_PENALTY_MISS:
  case PM_THEIR_PENALTY_SCORE:
  case PM_THEIR_FOUL_CHARGE:
  case PM_THEIR_FOUL_PUSH:
  case PM_THEIR_FOUL_MULTIPLEATTACKER:
  case PM_THEIR_FOUL_BALLOUT:
  case PM_THEIR_FOUL_BACKPASS:
  case PM_THEIR_FOUL_FREEKICKFAULT:
  case PM_THEIR_FOUL_CATCHFAULT:
    side = "r";
    sidePM = true;
    break;
  default:
    sidePM = false;
  }
  switch (playMode) {
  case PM_BEFORE_KICK_OFF:
    pm = "before_kick_off";
    break;
  case PM_HALF_TIME:
    pm = "first_half_over";
    break;
  case PM_TIME_OVER:
    pm = "time_over";
    break;
  case PM_PLAY_ON:
    pm = "play_on";
    break;
  case PM_DROP_BALL:
    pm = "drop_ball";
    break;    
  case PM_OUR_OFFSIDE_KICK:    
  case PM_THEIR_OFFSIDE_KICK:
    pm = "offside_";
    break;
  case PM_OUR_KICK_OFF:
  case PM_THEIR_KICK_OFF:
    pm = "kick_off_";
    break;
  case PM_OUR_KICK_IN:
  case PM_THEIR_KICK_IN:
    pm = "kick_in_";
    break;
  case PM_OUR_FREE_KICK:
  case PM_THEIR_FREE_KICK:
    pm = "free_kick_";
    break;
  case PM_OUR_INDIRECT_FREE_KICK:
  case PM_THEIR_INDIRECT_FREE_KICK:
    pm = "indirect_free_kick_";
    break;
  case PM_OUR_CORNER_KICK:
  case PM_THEIR_CORNER_KICK:
    pm = "corner_kick_";
    break;
  case PM_OUR_GOAL_KICK:
  case PM_THEIR_GOAL_KICK:
    pm = "goal_kick_";
    break;
  case PM_OUR_GOAL:
  case PM_THEIR_GOAL:
    pm = "goal_";
    break;
  case PM_OUR_PENALTY_KICK:
  case PM_THEIR_PENALTY_KICK:
    pm = "penalty_kick_";
    break;
  case PM_OUR_PENALTY_SETUP:
  case PM_THEIR_PENALTY_SETUP:
    pm = "penalty_setup_";
    break;
  case PM_OUR_PENALTY_READY:
  case PM_THEIR_PENALTY_READY:
    pm = "penalty_ready_";
    break;
  case PM_OUR_PENALTY_TAKEN:
  case PM_THEIR_PENALTY_TAKEN:
    pm = "penalty_taken_";
    break;
  case PM_OUR_PENALTY_MISS:
  case PM_THEIR_PENALTY_MISS:
    pm = "penalty_miss_";
    break;
  case PM_OUR_PENALTY_SCORE:
  case PM_THEIR_PENALTY_SCORE:
    pm = "penalty_score_";
    break;
  case PM_PAUSE:
    pm = "pause";
    break;
  case PM_HUMAN:
    pm = "human_judge";
    break;
  case PM_OUR_FOUL_CHARGE:
  case PM_THEIR_FOUL_CHARGE:
    pm = "foul_charge_";
    break;
  case PM_OUR_FOUL_PUSH:
  case PM_THEIR_FOUL_PUSH:
    pm = "foul_push_";
    break;
  case PM_OUR_FOUL_MULTIPLEATTACKER:
  case PM_THEIR_FOUL_MULTIPLEATTACKER:
    pm = "foul_multiple_attack_";
    break;
  case PM_OUR_FOUL_BALLOUT:
  case PM_THEIR_FOUL_BALLOUT:
    pm = "foul_ballout_";
    break;
  case PM_OUR_FOUL_BACKPASS:
  case PM_THEIR_FOUL_BACKPASS:
    pm = "back_pass_";
    break;
  case PM_OUR_FOUL_FREEKICKFAULT:
  case PM_THEIR_FOUL_FREEKICKFAULT:
    pm = "free_kick_fault_";
    break;
  case PM_OUR_FOUL_CATCHFAULT:
  case PM_THEIR_FOUL_CATCHFAULT:
    pm = "catch_fault_";
    break;
  default:
    RS_WARNING("Not supported playmode.");
    return "";
  }
  return "(change_mode " + pm + (sidePM?side:"") + ")";
}

ChangeModeCommand* ChangeModeCommand::clone() const {
  return new ChangeModeCommand(*this);
}

RS_END_NAMESPACE

