/* -*- Mode: C++ -*-
 * $Id: ChangeViewCommand.cc,v 1.3 2003/07/21 09:11:47 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ChangeViewCommand.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:47 $
 *
 *
 * HISTORY
 *
 * $Log: ChangeViewCommand.cc,v $
 * Revision 1.3  2003/07/21 09:11:47  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.2  2000/09/09 11:49:53  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.1  2000/09/02 14:16:27  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ChangeViewCommand.cc
 * Implementation of the class ChangeViewCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:47 $
 */


#include "ChangeViewCommand.h"
#include "string_utils.h"

RS_BEGIN_NAMESPACE

ChangeViewCommand::ChangeViewCommand(const ViewWidth view_w,
				     const ViewQuality view_q)
  : ImmediateCommand(CMD_ChangeView),
    viewWidth(view_w),
    viewQuality(view_q) {}

ChangeViewCommand::~ChangeViewCommand() {}

const std::string ChangeViewCommand::serverString() const {
  return "(change_view " + toString(viewWidth) + " " + toString(viewQuality) + ")";
}

ChangeViewCommand* ChangeViewCommand::clone() const {
  return new ChangeViewCommand(*this);
}

ViewWidth ChangeViewCommand::getViewWidth() const {
  return viewWidth;
}

ViewQuality ChangeViewCommand::getViewQuality() const {
  return viewQuality;
}

RS_END_NAMESPACE
