/* -*- Mode: C++ -*-
 * $Id: ClangVersionCommand.cc,v 1.2 2003/08/20 12:21:47 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ClangVersionCommand.cc
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: David Rosen 2003-06-30
 * Last modified by $Author: s02davro $ $Date: 2003/08/20 12:21:47 $
 *
 *
 * HISTORY
 *
 * $Log: ClangVersionCommand.cc,v $
 * Revision 1.2  2003/08/20 12:21:47  s02davro
 * Corrected Command type
 *
 * Revision 1.1  2003/07/21 08:29:13  s02davro
 * Added new commands (v9, trainer, coach, new command hierarchy)
 *
 * Revision 1.1  2001/11/15 16:48:31  frehe
 * Added a clangVersion command.
 *
 */

/**
 * @file ClangVersionCommand.cc
 * Implementation of the class ClangVersionCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */

#include "ClangVersionCommand.h"
#include "string_utils.h"

RS_BEGIN_NAMESPACE

ClangVersionCommand::ClangVersionCommand(UInt def_minver, UInt def_maxver)
  : ImmediateCommand(CMD_ClangVersion),
    minver(def_minver),
    maxver(def_maxver) {}

ClangVersionCommand::~ClangVersionCommand() {}

const std::string ClangVersionCommand::serverString() const {
  return "(clang (ver " + toString(minver) + " " + toString(maxver) + "))";
}

ClangVersionCommand* ClangVersionCommand::clone() const {
  return new ClangVersionCommand(*this);
}

UInt ClangVersionCommand::getMinVersion() const {
  return minver;
}

UInt ClangVersionCommand::getMaxVersion() const {
  return maxver;
}

RS_END_NAMESPACE
