/* -*- Mode: C++ -*-
 * $Id: ClangVersionCommand.h,v 1.2 2003/08/20 12:21:47 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ClangVersionCommand.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: David Rosen 2003-06-30
 * Last modified by $Author: s02davro $ $Date: 2003/08/20 12:21:47 $
 *
 *
 * HISTORY
 *
 * $Log: ClangVersionCommand.h,v $
 * Revision 1.2  2003/08/20 12:21:47  s02davro
 * Corrected Command type
 *
 * Revision 1.1  2003/07/21 08:29:13  s02davro
 * Added new commands (v9, trainer, coach, new command hierarchy)
 *
 * Revision 1.1  2001/11/15 16:48:31  frehe
 * Added a clangVersion command.
 *
 */

/**
 * @file ClangVersionCommand.h
 * Include file for the class ClangVersionCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */

#ifndef CLANGVERSIONCOMMAND_H
#define CLANGVERSIONCOMMAND_H

#include "ImmediateCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class ClangVersionCommand ClangVersionCommand.h
 * Class used to represent a generic clangVersion command
 * defined in RoboSoc.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */
class ClangVersionCommand : public ImmediateCommand {
public:
  ClangVersionCommand(UInt def_minver, UInt def_maxver);
  virtual ~ClangVersionCommand();
  
  /** Returns the server string that will execute the ClangVersionCommand
      at the soccer server. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  ClangVersionCommand* clone() const;
  
  UInt getMinVersion() const;
  UInt getMaxVersion() const;
private:
  /** The actual commands. */
  UInt minver;
  UInt maxver;
};

RS_END_NAMESPACE

#endif // !CLANGVERSIONCOMMAND_H
