/* -*- Mode: C++ -*-
 * $Id: CoachOrTrainerCommand.h,v 1.1 2003/07/21 08:29:13 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoachOrTrainerCommand.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-09
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 08:29:13 $
 *
 *
 * HISTORY
 *
 * $Log: CoachOrTrainerCommand.h,v $
 * Revision 1.1  2003/07/21 08:29:13  s02davro
 * Added new commands (v9, trainer, coach, new command hierarchy)
 *
 *
 */

/**
 * @file CoachOrTrainerCommand.h
 * Include file for the class CoachOrTrainerCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2003/07/21 08:29:13 $
 */

#ifndef COACHORTRAINERCOMMAND_H
#define COACHORTRAINERCOMMAND_H

#include "ImmediateCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class CoachOrTrainerCommand CoachOrTrainerCommand.h
 * Class used as the base class of all CoachOrTrainer commands
 * defined in RoboSoc. (Can be sent directly to server, and
 * more than one every cycle, only coach or trainer.)
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2003/07/21 08:29:13 $
 */
class CoachOrTrainerCommand : public ImmediateCommand {
public:
  CoachOrTrainerCommand(const CommandType);
  virtual ~CoachOrTrainerCommand();

  /** Returns the server string that will execute the ServerCommand
      at the soccer server. */
  virtual const std::string serverString() const = 0;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual CoachOrTrainerCommand* clone() const = 0;
};

RS_END_NAMESPACE

#endif // !COACHORTRAINERCOMMAND_H
