/* -*- Mode: C++ -*-
 * $Id: Command.h,v 1.8 2003/07/21 09:11:48 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Command.h
 * Unitname: Library
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:48 $
 *
 *
 * HISTORY
 *
 * $Log: Command.h,v $
 * Revision 1.8  2003/07/21 09:11:48  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.7  2001/09/17 21:54:39  frehe
 * Added a operator<< for Command*.
 *
 * Revision 1.6  2001/09/03 19:38:58  frehe
 * Removed dependencies on <iostream>.
 *
 * Revision 1.5  2000/09/09 11:49:44  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.4  2000/09/03 11:10:26  frehe
 * Made the clone method constant.
 *
 * Revision 1.3  2000/09/01 11:43:00  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 01:24:37  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Command.h
 * Include file for the class Command.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.8 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:48 $
 */

#ifndef COMMAND_H
#define COMMAND_H

#include <string>
#include <iosfwd>
#include "library_types.h"

RS_BEGIN_NAMESPACE

/** A type for representing the different commands that can be sent. */
enum CommandType {
  /** General commands. */
  CMD_Unknown, 
  CMD_Bye, 
  CMD_Say,
  /** Field players commands. */
  CMD_Init,
  CMD_Catch,
  CMD_ChangeView, 
  CMD_Dash,
  CMD_Kick, 
  CMD_Move,
  CMD_SenseBody,
  CMD_Turn, 
  CMD_TurnNeck,
  CMD_AttentionTo,
  CMD_PointTo,
  CMD_Tackle,
  CMD_ClangVersion,
  /** Trainer commands, CMD_Trainer can be used for future developments. */
  CMD_Trainer,  
  CMD_TrainerInit,
  CMD_TrainerSubstitution,
  CMD_ChangeMode,
  CMD_MoveObject,
  CMD_CheckBall,
  CMD_Start,
  CMD_Recover,
  CMD_Ear,
  /** Coach (and trainer) command, CMD_Coach can be used for new coach commands. */
  CMD_Coach, 
  CMD_CoachInit,
  CMD_CoachSubstitution,
  CMD_Look,
  CMD_Eye,
  CMD_TeamNames,
  /** Only Coach. */
  CMD_TeamGraphics, 
  /** Other commands (internal). */
  CMD_SetCyclesSinceIOBeforeShutdown,
  CMD_SetMinTicksBeforeSendingCommand,
  CMD_SetMaxTicksBeforeSendingCommand,
  CMD_SetTicksBeforeCommandWarning,
  CMD_SetTicksBetweenCycles,
  CMD_SetTicksBetweenSenses,
  CMD_SetTicksBetweenVisuals,
  CMD_Generic
};

/**
 * @class Command Command.h
 * Class used as the base class of all commands defined in
 * RoboSoc.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.8 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:48 $
 */
class Command {
public:
  /** Create a command of the type @a c_type. */
  Command(const CommandType c_type);

  /** Destroys the command. */
  virtual ~Command();
  
  /** Get the type of the command. Should be made obsolete in
      future versions of RoboSoc. */
  CommandType getType() const;

  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual Command* clone() const = 0;
  
private:
  CommandType cmdType;
};

RS_END_NAMESPACE

#endif // !COMMAND_H
