/* -*- Mode: C++ -*-
 * $Id: ImmediateCommand.h,v 1.2 2003/07/21 09:11:48 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ImmediateCommand.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-09
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:48 $
 *
 *
 * HISTORY
 *
 * $Log: ImmediateCommand.h,v $
 * Revision 1.2  2003/07/21 09:11:48  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.1  2000/09/09 11:49:45  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 */

/**
 * @file ImmediateCommand.h
 * Include file for the class ImmediateCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:48 $
 */

#ifndef IMMEDIATECOMMAND_H
#define IMMEDIATECOMMAND_H

#include "ServerCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class ImmediateCommand ImmediateCommand.h
 * Class used as the base class of all immediate commands
 * defined in RoboSoc. (Can be sent directly to server, and
 * more than one every cycle.)
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:48 $
 */
class ImmediateCommand : public ServerCommand {
public:
  ImmediateCommand(const CommandType);
  virtual ~ImmediateCommand();

  /** Returns the server string that will execute the ServerCommand
      at the soccer server. */
  virtual const std::string serverString() const = 0;

  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual ImmediateCommand* clone() const = 0;
};

RS_END_NAMESPACE

#endif // !IMMEDIATECOMMAND_H
