/* -*- Mode: C++ -*-
 * $Id: InitCommand.cc,v 1.4 2003/07/21 09:11:49 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InitCommand.cc
 * Unitname: Library
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:49 $
 *
 *
 * HISTORY
 *
 * $Log: InitCommand.cc,v $
 * Revision 1.4  2003/07/21 09:11:49  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.3  2002/12/07 17:31:01  s02davro
 * InitCommand handle trainer
 *
 * Revision 1.2  2000/09/09 11:49:53  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.1  2000/09/02 14:16:28  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file InitCommand.cc
 * Implementation of the class InitCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */


#include "InitCommand.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


InitCommand::InitCommand(const std::string& team_name,
			 const Float ver,
			 const bool goalie)
  : ImmediateCommand(CMD_Init),
    teamName(team_name),
    version(ver),
    goalKeeper(goalie) {}

InitCommand::~InitCommand() {
}

const std::string InitCommand::serverString() const {
  return "(init " + teamName + " (version " + toString(version) + ")" + 
    ((goalKeeper) ? " (goalie))" : ")");
}

InitCommand* InitCommand::clone() const {
  return new InitCommand(*this);
}


RS_END_NAMESPACE
