/* -*- Mode: C++ -*-
 * $Id: InitCommand.h,v 1.6 2003/07/21 09:11:49 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InitCommand.h
 * Unitname: Library
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:49 $
 *
 *
 * HISTORY
 *
 * $Log: InitCommand.h,v $
 * Revision 1.6  2003/07/21 09:11:49  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.5  2002/12/07 17:31:01  s02davro
 * InitCommand handle trainer
 *
 * Revision 1.4  2000/09/09 11:49:45  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.3  2000/09/05 21:08:49  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.2  2000/09/03 11:10:27  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 14:16:23  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file InitCommand.h
 * Include file for the class InitCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */

#ifndef INITCOMMAND_H
#define INITCOMMAND_H

#include "ImmediateCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class InitCommand InitCommand.h
 * Class used to represent the server command
 * (init TeamName (version V) (goalie)).
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */
class InitCommand : public ImmediateCommand {
public:
  InitCommand(const std::string& team_name,
	      const Float vers,
	      const bool goalie);
  ~InitCommand();

  /** Returns the server string that will execute the InitCommand
      at the soccer server. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual InitCommand* clone() const;  

private:
  std::string teamName;
  Float version;
  bool goalKeeper;
};

RS_END_NAMESPACE

#endif // !INITCOMMAND_H
