/* -*- Mode: C++ -*-
 * $Id: KickCommand.cc,v 1.3 2003/07/21 09:11:49 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickCommand.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:49 $
 *
 *
 * HISTORY
 *
 * $Log: KickCommand.cc,v $
 * Revision 1.3  2003/07/21 09:11:49  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.2  2000/09/09 11:49:53  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.1  2000/09/02 14:16:28  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file KickCommand.cc
 * Implementation of the class KickCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */

#include "KickCommand.h"
#include "string_utils.h"

RS_BEGIN_NAMESPACE

KickCommand::KickCommand(const Float pow, const AngleDeg& ang)
  : PeriodicCommand(CMD_Kick),
    power(pow),
    angle(ang) {
}

KickCommand::~KickCommand() {
}

const std::string KickCommand::serverString() const {
  return "(kick " + toString(power) + " " + toString(angle.getDeg()) + ")";
}


KickCommand* KickCommand::clone() const {
  return new KickCommand(*this);
}

Float KickCommand::getPower() const {
  return power;
}
 
AngleDeg KickCommand::getAngle() const {
  return angle;
}

RS_END_NAMESPACE
