/* -*- Mode: C++ -*-
 * $Id: KickCommand.h,v 1.5 2003/07/21 09:11:49 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickCommand.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:49 $
 *
 *
 * HISTORY
 *
 * $Log: KickCommand.h,v $
 * Revision 1.5  2003/07/21 09:11:49  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.4  2000/09/09 11:49:45  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.3  2000/09/05 21:08:50  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.2  2000/09/03 11:10:28  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 14:16:24  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file KickCommand.h
 * Include file for the class KickCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */

#ifndef KICKCOMMAND_H
#define KICKCOMMAND_H

#include "PeriodicCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class KickCommand KickCommand.h
 * Class used to represent the server command (kick Power Direction).
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */
class KickCommand : public PeriodicCommand {
public:
  KickCommand(const Float pow, const AngleDeg& ang);
  ~KickCommand();

  /** Returns the server string that will execute the Kick command. */
  const std::string serverString() const;

  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  KickCommand* clone() const;

  Float getPower() const;
  AngleDeg getAngle() const;
private:
  Float power;
  AngleDeg angle;
};

RS_END_NAMESPACE

#endif // !KICKCOMMAND_H
