/* -*- Mode: C++ -*-
 * $Id: LineObject.cc,v 1.3 2002/08/02 21:35:27 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: LineObject.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/08/02 21:35:27 $
 *
 *
 * HISTORY
 *
 * $Log: LineObject.cc,v $
 * Revision 1.3  2002/08/02 21:35:27  s02davro
 * Changed types, added documentation.
 *
 * Revision 1.2  2001/09/03 19:38:58  frehe
 * Removed dependencies on <iostream>.
 *
 * Revision 1.1  2000/09/01 01:24:45  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file LineObject.cc
 * Implementation of the class LineObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2002/08/02 21:35:27 $
 */


#include <ostream>
#include "LineObject.h"
#include "rs_error.h"
#include "rs_debug.h"


RS_BEGIN_NAMESPACE


LineObject::LineObject(const SideLineName def_lineName,
		       const Vector& def_vector) 
  : lineVector(def_vector),
    lineId(def_lineName) {  
}


LineObject::LineObject(const LineObject& obj)
  : lineVector(obj.lineVector),
    lineId(obj.lineId) {
}


LineObject& LineObject::operator= (const LineObject& obj) {
  if (this != &obj) {
    lineVector = obj.lineVector;
    lineId = obj.lineId;
  }
  return *this;
}


std::ostream& operator<<(std::ostream& os, const LineObject& line) {
  os << "#Line(" << line.getLineId() << ")";
  return os;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "LineObject.icc"
#endif
