/* -*- Mode: C++ -*-
 * $Id: LineObject.h,v 1.5 2002/08/31 14:46:01 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: LineObject.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/08/31 14:46:01 $
 *
 *
 * HISTORY
 *
 * $Log: LineObject.h,v $
 * Revision 1.5  2002/08/31 14:46:01  s02davro
 * Small changes in the game objects.
 *
 * Revision 1.4  2002/08/02 21:35:27  s02davro
 * Changed types, added documentation.
 *
 * Revision 1.3  2001/09/03 19:38:58  frehe
 * Removed dependencies on <iostream>.
 *
 * Revision 1.2  2000/09/01 01:24:40  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file LineObject.h
 * Include file for the class LineObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/08/31 14:46:01 $
 */


#ifndef LINEOBJECT_H
#define LINEOBJECT_H


#include <iosfwd>

#include "library_types.h"
#include "soccer_types.h"


RS_BEGIN_NAMESPACE


/**
 * @class LineObject LineObject.h
 * Class used to represent the four side lines in the soccer field.
 * All values are known and without error since this is known facts from
 * the soccerserver.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/08/31 14:46:01 $
 */
class LineObject {
public:
  /** Default constructor. */
  LineObject(const SideLineName def_lineName = SL_UNKNOWN,
	     const Vector& def_vector = Vector());

  /** Copyconstructor. */
  LineObject(const LineObject& obj);
  
  /** Assignment operator. */
  LineObject& operator= (const LineObject& obj);
  
  /** Sets the ID of the LineObject 
   *  @see SideLineName in soccer_types.h for range of values. */
  void setLineId(const SideLineName line);

  /** Get the ID of the object.
   *  @see SideLineName in soccer_types.h for range of values. */
  SideLineName getLineId() const;

  /** Set the vector describing the line. */
  void setVector(const Vector& c);

  /** Get the vector. */
  Vector getVector() const;
  
private:
  /** Representation of the vector. */
  Vector lineVector;

  /** Representation of the ID. */
  SideLineName lineId;
};


std::ostream& operator<<(std::ostream&, const LineObject&);


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "LineObject.icc"
#endif


#endif  // !LINEOBJECT_H
