/* -*- Mode: C++ -*-
 * $Id: MarkerObject.h,v 1.2 2002/08/02 21:35:55 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MarkerObject.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/08/02 21:35:55 $
 *
 *
 * HISTORY
 *
 * $Log: MarkerObject.h,v $
 * Revision 1.2  2002/08/02 21:35:55  s02davro
 * Change types, added documentation.
 *
 * Revision 1.1  2000/09/01 03:24:31  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file MarkerObject.h
 * Include file for the class MarkerObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/02 21:35:55 $
 */


#ifndef MARKEROBJECT_H
#define MARKEROBJECT_H

#include "library_types.h"
#include "soccer_types.h"

RS_BEGIN_NAMESPACE


/**
 * @class MarkerObject MarkerObject.h
 * Class used to represent a marker. The position is known and without
 * error since this is given facts in the soccerfield.
 *
 * @todo Make a toString.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2002/08/02 21:35:55 $
 */

class MarkerObject {
public:
  /** Default constructor. */
  MarkerObject(const MarkerName def_markerName = UNKNOWN_MARKER,
	       const Point& def_point = Point());

  /** Copyconstructor. */
  MarkerObject(const MarkerObject& obj);
  
  /** Assignment operator. */
  MarkerObject& operator= (const MarkerObject& obj);
  
  /** Set the ID of the marker. 
   *  See see MarkerName in soccer_types.h for range of values. */
  void setMarkerId(const MarkerName marker);

  /** Get the ID of the marker. 
   *  See see MarkerName in soccer_types.h for range of values. */
  MarkerName getMarkerId() const;

  /** Get the x coordinate for the marker. */
  Float getPosX() const;

  /** Get the y coordinate for the marker. */
  Float getPosY() const;

  /** Get the position of the marker. */
  Point getPosition() const;

private:
  /** Representation of the markers position. */
  Point position;

  /** Representation of the markers ID. */
  MarkerName markerId;
};

std::ostream& operator<<(std::ostream& os, const MarkerObject& marker);


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "MarkerObject.icc"
#endif


#endif // !MARKEROBJECT_H
