/* -*- Mode: C++ -*-
 * $Id: MoveCommand.cc,v 1.3 2003/07/21 09:11:49 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveCommand.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:49 $
 *
 *
 * HISTORY
 *
 * $Log: MoveCommand.cc,v $
 * Revision 1.3  2003/07/21 09:11:49  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.2  2000/09/09 11:49:54  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.1  2000/09/02 14:16:28  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file MoveCommand.cc
 * Implementation of the class MoveCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */


#include "MoveCommand.h"
#include "string_utils.h"

RS_BEGIN_NAMESPACE

MoveCommand::MoveCommand(const Point& dest)
  : PeriodicCommand(CMD_Move),
    destination(dest) {}

MoveCommand::~MoveCommand() {}

const std::string MoveCommand::serverString() const {
  return "(move " + toString(destination.getX()) + " "
    + toString(destination.getY()) + ")";
}

MoveCommand* MoveCommand::clone() const {
  return new MoveCommand(*this);
}

Point MoveCommand::getDestination() const {
  return destination;
}

RS_END_NAMESPACE

