/* -*- Mode: C++ -*-
 * $Id: MoveObjectCommand.cc,v 1.2 2003/07/22 14:49:43 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveObjectCommand.cc
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: David Rosen 2003-06-30
 * Last modified by $Author: s02davro $ $Date: 2003/07/22 14:49:43 $
 *
 *
 * HISTORY
 *
 * $Log: MoveObjectCommand.cc,v $
 * Revision 1.2  2003/07/22 14:49:43  s02davro
 * Bugfix (b) -> (ball) etc.
 *
 * Revision 1.1  2003/07/21 08:29:15  s02davro
 * Added new commands (v9, trainer, coach, new command hierarchy)
 *
 * Revision 1.1  2001/11/15 16:48:31  frehe
 * Added a clangVersion command.
 *
 */

/**
 * @file MoveObjectCommand.cc
 * Implementation of the class MoveObjectCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/07/22 14:49:43 $
 */

#include "MoveObjectCommand.h"
#include "string_utils.h"

RS_BEGIN_NAMESPACE

MoveObjectCommand::MoveObjectCommand(BallObject def_ball, Point def_position, Vector def_speed)
  : TrainerCommand(CMD_MoveObject),
    moveBall(true),
    ball(def_ball),
    angle(0),
    position(def_position),
    speed(def_speed) {
}

MoveObjectCommand::MoveObjectCommand(PlayerObject def_player, 
				     const std::string& def_team_name,
				     Point def_position, 
				     AngleDeg def_angle, Vector def_speed)
  : TrainerCommand(CMD_MoveObject),
    moveBall(false),
    player(def_player),
    team_name(def_team_name),
    angle(def_angle),
    position(def_position),
    speed(def_speed) {
  if (player.getUniformNumber().isUnknown() || (player.getTeam() == T_UNKNOWN)) {
    RS_WARNING("Trying to move a unknown player");
  }
}

MoveObjectCommand::~MoveObjectCommand() {
}

const std::string MoveObjectCommand::serverString() const {
  if (moveBall)
    return toString("(move (ball) ") + toString(position.getX()) + " " + toString(position.getY()) + 
      " " + toString(angle.getDeg()) + 
      " " + toString(Point(speed).getX()) + " " + toString(Point(speed).getY()) + ")";
  else
    return toString("(move (player ") + team_name + " " + 
      toString(player.getUniformNumber().getValue()) + ") " + 
      toString(position.getX()) + " " + toString(position.getY()) + " " + 
      toString(angle.getDeg()) + " " +toString(Point(speed).getX()) + " " + 
      toString(Point(speed).getY()) + ")";
}

MoveObjectCommand* MoveObjectCommand::clone() const {
  return new MoveObjectCommand(*this);
}

RS_END_NAMESPACE
