/* -*- Mode: C++ -*-
 * $Id: MoveObjectCommand.h,v 1.2 2003/07/22 14:49:43 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveObjectCommand.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: David Rosen 2003-06-30
 * Last modified by $Author: s02davro $ $Date: 2003/07/22 14:49:43 $
 *
 *
 * HISTORY
 *
 * $Log: MoveObjectCommand.h,v $
 * Revision 1.2  2003/07/22 14:49:43  s02davro
 * Bugfix (b) -> (ball) etc.
 *
 * Revision 1.1  2003/07/21 08:29:15  s02davro
 * Added new commands (v9, trainer, coach, new command hierarchy)
 *
 * Revision 1.1  2001/11/15 16:48:31  frehe
 * Added a clangVersion command.
 *
 */

/**
 * @file MoveObjectCommand.h
 * Include file for the class MoveObjectCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/07/22 14:49:43 $
 */

#ifndef MOVEOBJECTCOMMAND_H
#define MOVEOBJECTCOMMAND_H

#include "TrainerCommand.h"
#include "BallObject.h"
#include "PlayerObject.h"
#include "rs_log.h"
#include "rs_error.h"

RS_BEGIN_NAMESPACE

/**
 * @class MoveObjectCommand MoveObjectCommand.h
 * Class used to represent a command to move object in RoboSoc.
 * This command should only be used by the trainer.
 * @todo Command.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/07/22 14:49:43 $
 */

class MoveObjectCommand : public TrainerCommand {
public:
  MoveObjectCommand(BallObject ball, 
		    Point position, 
		    Vector speed = Vector());
  MoveObjectCommand(PlayerObject player,
		    const std::string& team_name,
		    Point position,
		    AngleDeg angle = AngleDeg(),
		    Vector speed = Vector());
  ~MoveObjectCommand();
  
  /** Returns the server string that will execute the MoveObjectCommand
      at the soccer server. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command. */
  MoveObjectCommand* clone() const;
private:
  /** The actual command data. */
  bool moveBall;
  BallObject ball;
  PlayerObject player;
  std::string team_name;
  Point position;
  AngleDeg angle;
  Vector speed;
};

RS_END_NAMESPACE

#endif // !MOVEOBJECTCOMMAND_H
