/* -*- Mode: C++ -*-
 * $Id: PeriodicCommand.h,v 1.2 2003/07/21 09:11:49 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PeriodicCommand.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-09
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:11:49 $
 *
 *
 * HISTORY
 *
 * $Log: PeriodicCommand.h,v $
 * Revision 1.2  2003/07/21 09:11:49  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.1  2000/09/09 11:49:46  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 */

/**
 * @file PeriodicCommand.h
 * Include file for the class PeriodicCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */

#ifndef PERIODICCOMMAND_H
#define PERIODICCOMMAND_H

#include "ServerCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class PeriodicCommand PeriodicCommand.h
 * Class used as the base class of all periodic commands
 * defined in RoboSoc. This commands may only be sent
 * ones every cycle to the server.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:11:49 $
 */
class PeriodicCommand : public ServerCommand {
public:
  PeriodicCommand(const CommandType);
  virtual ~PeriodicCommand();
  
  /** Returns the server string that will execute the ServerCommand
      at the soccer server. */
  virtual const std::string serverString() const = 0;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual PeriodicCommand* clone() const = 0;
};

RS_END_NAMESPACE

#endif // !PERIODICCOMMAND_H
