/* -*- Mode: C++ -*-
 * $Id: PlayerObject.cc,v 1.5 2003/11/04 17:17:32 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayerObject.cc
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2003/11/04 17:17:32 $
 *
 *
 * HISTORY
 *
 * $Log: PlayerObject.cc,v $
 * Revision 1.5  2003/11/04 17:17:32  frehe
 * Moved the goalie information from AgentObject to PlayerObject.
 *
 * Revision 1.4  2003/07/21 09:21:12  s02davro
 * Tackle, arm update
 *
 * Revision 1.3  2002/08/31 14:46:02  s02davro
 * Small changes in the game objects.
 *
 * Revision 1.2  2002/08/02 21:36:41  s02davro
 * Change types, removed some methods. Relative -> Global position and angles.
 *
 * Revision 1.1  2000/09/01 03:24:34  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file PlayerObject.cc
 * Implementation of the class PlayerObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2003/11/04 17:17:32 $
 */


#include "PlayerObject.h"


RS_BEGIN_NAMESPACE


PlayerObject::PlayerObject(const PointUE& def_point,
			   const TimeUnknown& def_time,
			   const VectorUE& def_speed,
			   const TimeUnknown& def_speed_time,
			   const AngleDegUE& def_body_dir,
			   const AngleDegUE& def_neck_dir,
			   const AngleDegUE& def_point_dir,
			   const bool def_tackling,
			   const bool def_goalie,
			   const Team def_team,
			   const Side def_side,
			   const TimeUnknown& def_team_time,
			   const UIntUnknown def_u_number,
			   const TimeUnknown& def_unum_time,
			   const VectorUE& def_acceleration)
  : MobileObject(def_point, def_time, def_speed, 
		 def_speed_time, def_acceleration),
    bodyDir(def_body_dir),
    neckDir(def_neck_dir),    
    pointDir(def_point_dir),
    tackling(def_tackling),
    goalie(def_goalie),
    team(def_team),
    side(def_side),
    teamObservationTime(def_team_time),
    uNumber(def_u_number),
    uNumberObservationTime(def_unum_time) {
}  

PlayerObject::PlayerObject(const PlayerObject& obj)
  : MobileObject(obj),
    bodyDir(obj.bodyDir),
    neckDir(obj.neckDir),    
    pointDir(obj.pointDir),
    tackling(obj.tackling),
    goalie(obj.goalie),
    team(obj.team),
    side(obj.side),
    teamObservationTime(obj.teamObservationTime),
    uNumber(obj.uNumber),
    uNumberObservationTime(obj.uNumberObservationTime) {
}  

PlayerObject& PlayerObject::operator= (const PlayerObject& obj) {
  if (this != &obj) {
    MobileObject::operator=(obj);
    bodyDir        = obj.bodyDir;
    neckDir        = obj.neckDir;
    pointDir       = obj.pointDir;
    tackling       = obj.tackling;
    goalie         = obj.goalie;
    team           = obj.team;
    side           = obj.side;
    teamObservationTime = obj.teamObservationTime;
    uNumber        = obj.uNumber;
    uNumberObservationTime = obj.uNumberObservationTime;
  }
  return *this;
}

void PlayerObject::setTeam(const Side our_side) {
  if ( our_side == side )
    team = T_OUR_TEAM;
  else if ( our_side == S_UNKNOWN )
    team = T_UNKNOWN;
  else
    team = T_THEIR_TEAM;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PlayerObject.icc"
#endif
