/* -*- Mode: C++ -*-
 * $Id: PlayerObject.h,v 1.5 2003/11/04 17:17:33 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayerObject.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2003/11/04 17:17:33 $
 *
 *
 * HISTORY
 *
 * $Log: PlayerObject.h,v $
 * Revision 1.5  2003/11/04 17:17:33  frehe
 * Moved the goalie information from AgentObject to PlayerObject.
 *
 * Revision 1.4  2003/07/21 09:21:12  s02davro
 * Tackle, arm update
 *
 * Revision 1.3  2002/08/31 14:46:02  s02davro
 * Small changes in the game objects.
 *
 * Revision 1.2  2002/08/02 21:36:41  s02davro
 * Change types, removed some methods. Relative -> Global position and angles.
 *
 * Revision 1.1  2000/09/01 03:24:32  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file PlayerObject.h
 * Include file for the class PlayerObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2003/11/04 17:17:33 $
 */


#ifndef PLAYEROBJECT_H
#define PLAYEROBJECT_H


#include "MobileObject.h"


RS_BEGIN_NAMESPACE


/**
 * @class PlayerObject PlayerObject.h
 * Class used to represent a player.
 *
 *
 * @todo Make a toString and operator<<
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2003/11/04 17:17:33 $
 */
class PlayerObject : public MobileObject {
public:
  /** Default constructor, the players  parameters are unknown as default. */
  PlayerObject(const PointUE& def_point = PointUE(),
	       const TimeUnknown& def_time = TimeUnknown(),
	       const VectorUE& def_speed = VectorUE(),
	       const TimeUnknown& def_speed_time = TimeUnknown(),
	       const AngleDegUE& def_body_dir = AngleDegUE(),
	       const AngleDegUE& def_neck_dir = AngleDegUE(),
	       const AngleDegUE& def_point_dir = AngleDegUE(),
	       const bool def_tackling = false,
	       const bool def_goalie = false,
	       const Team def_team = T_UNKNOWN,
	       const Side def_side = S_UNKNOWN,
	       const TimeUnknown& def_team_time = TimeUnknown(),
	       const UIntUnknown def_u_number = UIntUnknown(),
	       const TimeUnknown& def_unum_time = TimeUnknown(),
	       const VectorUE& def_acceleration = VectorUE());

  /** Copyconstructor. */
  PlayerObject(const PlayerObject& obj);

  /** Assignment operator. */
  PlayerObject& operator= (const PlayerObject& obj);

  /** Set the (global) bodydirection of the player. */
  void setBodyDirection(const AngleDegUE& ang);

  /** Get the (global) bodydirection of the player. */
  AngleDegUE getBodyDirection() const;

  /** Set the (global) neckdirection of the player. */
  void setNeckDirection(const AngleDegUE& ang);

  /** Get the (global) neckdirection of the player. */
  AngleDegUE getNeckDirection() const;

  /** Returns the relative neckdirection (to body) of the player.
   *  relneckdirection = neckdirection - bodydirection  */
  AngleDegUE getRelativeNeckDirection() const;

  /** Set the point direction of the player. */
  void setPointDirection(const AngleDegUE& ang);

  /** Get the point direction of the player. */
  AngleDegUE getPointDirection() const;

  /** Set the tackling status of the player. */
  void setTackling(const bool tackle);

  /** Get the tackling status of the player. */
  bool getTackling() const;

  /** Set the players team
   *  Values are T_OUR_TEAM, T_THEIR_TEAM and T_UNKNOWN */
  void setTeam(const Team tm);
  
  /** Set the players team based on what side it plays for. */
  void setTeam(const Side our_side);

  /** Returns the players team. 
   *  Values are T_OUR_TEAM, T_THEIR_TEAM and T_UNKNOWN */
  Team getTeam() const;
  
  /** Set to true if the player is a goalie. */
  void setGoalie(const bool goalie_);

  /** Returns true if the player is a goalie. */
  bool isGoalie() const;
  
  /** Set the players side
   *  Values are S_LEFT, S_RIGHT, S_NEUTRAL, S_UNKNOWN */
  void setSide(const Side new_side);

  /** Returns the players side
   *  Values are S_LEFT, S_RIGHT, S_NEUTRAL, S_UNKNOWN */
  Side getSide() const;
  
  /** Set the time for last observation of players team. */
  void setTeamObservationTime(const TimeUnknown& def_team_time);

  /** Get the last observation of the players team time. */
  TimeUnknown getTeamObservationTime() const;

  /** Set the players uniformnumber. */
  void setUniformNumber(const UIntUnknown u_num);

  /** Return the players uniformnumber. */
  UIntUnknown getUniformNumber() const;

  /** Set the time for last observation of players uniformnumber. */
  void setUniformNumberObservationTime(const TimeUnknown& def_unum_time);

  /** Return the time when we last observed the players uniformnumber. */
  TimeUnknown getUniformNumberObservationTime() const;

protected:
  /** Representation of the players (global) bodydirection. */
  AngleDegUE bodyDir;

  /** Representation of the players (global) neckdirection. */
  AngleDegUE neckDir;

  /** Representation of the players pointed to direction. */
  AngleDegUE pointDir;

  /** Representation of tackling status. */
  bool tackling;

  /** The team */
  Team team;

  /** Is goalie? */
  bool goalie;
  
  /** The current side the player is playing for. */
  Side side;

  /** Time of last observation of team. */
  TimeUnknown teamObservationTime;

  /** Representation of the players uniformnumber. */
  UIntUnknown uNumber;

  /** Time of last observation of uniformnumber. */
  TimeUnknown uNumberObservationTime;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PlayerObject.icc"
#endif


#endif // !PLAYEROBJECT_H
