/* -*- Mode: C++ -*-
 * $Id: PointToCommand.cc,v 1.3 2003/08/20 12:21:47 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PointToCommand.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2003/08/20 12:21:47 $
 *
 *
 * HISTORY
 *
 * $Log: PointToCommand.cc,v $
 * Revision 1.3  2003/08/20 12:21:47  s02davro
 * Corrected Command type
 *
 * Revision 1.2  2003/07/21 09:19:19  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.1  2002/10/01 16:06:27  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file PointToCommand.cc
 * Implementation of the class PointToCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */

#include "PointToCommand.h"
#include "string_utils.h"

RS_BEGIN_NAMESPACE

PointToCommand::PointToCommand(const Float& def_distance, const AngleDeg& def_direction)
  : ImmediateCommand(CMD_PointTo),
    distance(def_distance),
    direction(def_direction),
    off(false) {}

PointToCommand::PointToCommand(const Vector& def_vector)
  : ImmediateCommand(CMD_PointTo),
    distance(def_vector.getLength()),
    direction(def_vector.getAngle()),
    off(false) {}

PointToCommand::PointToCommand()
  : ImmediateCommand(CMD_PointTo),
    distance(),
    direction(),
    off(true) {}

PointToCommand::~PointToCommand() {
}

const std::string PointToCommand::serverString() const {
  return "(pointto " + (off?toString("off"):(toString(distance) + " " +
					     toString(direction.getDeg()))) + ")";
}

PointToCommand* PointToCommand::clone() const {
  return new PointToCommand(*this);
}

Float PointToCommand::getDistance() const {
  return distance;
}

AngleDeg PointToCommand::getAngle() const {
  return direction;
}

bool PointToCommand::isOff() const {
  return off;
}

RS_END_NAMESPACE
