/* -*- Mode: C++ -*-
 * $Id: PointToCommand.h,v 1.3 2003/08/20 12:21:47 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PointToCommand.h
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: David Rosn 2002-09-30
 * Last modified by $Author: s02davro $ $Date: 2003/08/20 12:21:47 $
 *
 *
 * HISTORY
 *
 * $Log: PointToCommand.h,v $
 * Revision 1.3  2003/08/20 12:21:47  s02davro
 * Corrected Command type
 *
 * Revision 1.2  2003/07/21 09:19:19  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.1  2002/10/01 16:06:27  s02davro
 * Added some new (version 8) commandsd
 *
 *
 */

/**
 * @file PointToCommand.h
 * Include file for the class PointToCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */


#ifndef POINTTOCOMMAND_H
#define POINTTOCOMMAND_H

#include "ImmediateCommand.h"
#include "library_types.h"

RS_BEGIN_NAMESPACE

/**
 * @class PointToCommand PointToCommand.h
 * Class used to represent the server command (pointto Distance Direction).
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2003/08/20 12:21:47 $
 */

class PointToCommand : public ImmediateCommand {
public:
  PointToCommand(const Float& def_distance, const AngleDeg& def_direction);
  PointToCommand(const Vector& def_vector);
  /** Use this constructor to turn the pointing off. */
  PointToCommand();
  ~PointToCommand();

  /** Returns the server string that will execute the PointTo command. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual PointToCommand* clone() const;

  Float getDistance() const;
  AngleDeg getAngle() const;
  bool isOff() const;
private:
  Float distance;
  AngleDeg direction;
  bool off;
};

RS_END_NAMESPACE

#endif // !POINTTOCOMMAND_H
