/* -*- Mode: C++ -*-
 * $Id: SayCommand.cc,v 1.5 2004/09/29 20:24:59 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SayCommand.cc
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2004/09/29 20:24:59 $
 *
 *
 * HISTORY
 *
 * $Log: SayCommand.cc,v $
 * Revision 1.5  2004/09/29 20:24:59  frehe
 * Added a copy constructor.
 *
 * Revision 1.4  2004/09/08 22:16:03  frehe
 * Updated the SayCommand with quotes around the message.
 *
 * Revision 1.3  2003/07/21 09:19:19  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.2  2000/09/09 11:49:54  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.1  2000/09/02 14:16:28  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file SayCommand.cc
 * Implementation of the class SayCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2004/09/29 20:24:59 $
 */

#include "SayCommand.h"

RS_BEGIN_NAMESPACE

SayCommand::SayCommand(const std::string& msg)
  : ImmediateCommand(CMD_Say),
    message(msg) {
}

SayCommand::SayCommand(const SayCommand& cmd)
  : ImmediateCommand(cmd),
    message(cmd.message)
{
}

SayCommand::~SayCommand(){
}

const std::string SayCommand::serverString() const {
  return "(say \"" + message + "\")";
}

SayCommand* SayCommand::clone() const {
  return new SayCommand(*this);
}

RS_END_NAMESPACE
