/* -*- Mode: C++ -*-
 * $Id: SayCommand.h,v 1.6 2004/09/29 20:24:59 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SayCommand.h
 * Unitname: Library
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2004/09/29 20:24:59 $
 *
 *
 * HISTORY
 *
 * $Log: SayCommand.h,v $
 * Revision 1.6  2004/09/29 20:24:59  frehe
 * Added a copy constructor.
 *
 * Revision 1.5  2003/07/21 09:19:19  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.4  2000/09/09 11:49:46  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.3  2000/09/05 21:08:51  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.2  2000/09/03 11:10:28  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 14:16:25  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file SayCommand.h
 * Include file for the class SayCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2004/09/29 20:24:59 $
 */

#ifndef SAYCOMMAND_H
#define SAYCOMMAND_H

#include "ImmediateCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class SayCommand SayCommand.h
 * Class used to represent the server command (say Message).
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2004/09/29 20:24:59 $
 */

class SayCommand : public ImmediateCommand {
public:
  SayCommand(const std::string& msg);
  SayCommand(const SayCommand& cmd);
  ~SayCommand();

  /** Returns the server string that will execute the AttentionTo command. */
  const std::string serverString() const;  

  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  SayCommand* clone() const;

private:
  std::string message;
};

RS_END_NAMESPACE

#endif // !SAYCOMMAND_H
