/* -*- Mode: C++ -*-
 * $Id: Sector.cc,v 1.1 2004/09/20 10:42:17 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Sector.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-09-20
 * Last modified by $Author: frehe $ $Date: 2004/09/20 10:42:17 $
 *
 *
 * HISTORY
 *
 * $Log: Sector.cc,v $
 * Revision 1.1  2004/09/20 10:42:17  frehe
 * Added sector helpers.
 *
 */

/**
 * @file Sector.cc
 * Implementation of the class Sector.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2004/09/20 10:42:17 $
 */


#include "Sector.h"
#include <algorithm>


RS_BEGIN_NAMESPACE


template<typename T>
inline bool between(const T& min_val, const T& val, const T& max_val)
{ return ( (min_val <= val) && (val <= max_val)); }


Sector::Sector(const AngleDeg& min_angle,
	       const AngleDeg& max_angle)
  : min_angle_(min_angle),
    max_angle_(max_angle)
{
  normalize();
}


Sector::Sector(const Point& from,
	       const Point& first_pos,
	       const Point& last_pos)
  : min_angle_(from.getDirectionTo(first_pos)),
    max_angle_(from.getDirectionTo(last_pos))
{
  normalize();
}


Sector::~Sector()
{
}


AngleDeg
Sector::min_angle() const
{
  return min_angle_;
}


AngleDeg
Sector::max_angle() const
{
  return max_angle_;
}


AngleDeg
Sector::center() const
{
  return ( min_angle_ + width()/2.0 );
}


AngleDeg
Sector::width() const
{
  return (max_angle_ - min_angle_);
}


bool
Sector::contains(const AngleDeg& angle) const
{
  return between(min_angle_, angle, max_angle_);
}


void
Sector::normalize()
{
  // Make sure they are sorted
  if ( min_angle_ > max_angle_ ) {
    std::swap(min_angle_, max_angle_);
  }
}


RS_END_NAMESPACE
