/* -*- Mode: C++ -*-
 * $Id: Sector.h,v 1.2 2005/09/01 07:04:51 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Sector.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-09-20
 * Last modified by $Author: petan $ $Date: 2005/09/01 07:04:51 $
 *
 *
 * HISTORY
 *
 * $Log: Sector.h,v $
 * Revision 1.2  2005/09/01 07:04:51  petan
 * Added simple doxygen comments
 *
 * Revision 1.1  2004/09/20 10:42:17  frehe
 * Added sector helpers.
 *
 */

/**
 * @file Sector.h
 * Include file for the class Sector.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: petan $
 * $Date: 2005/09/01 07:04:51 $
 */


#ifndef SECTOR_H
#define SECTOR_H


#include "library_types.h"


RS_BEGIN_NAMESPACE


/**
 * @class Sector Sector.h
 * Class used to represent a circle sector in RoboSoc.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: petan $
 * $Date: 2005/09/01 07:04:51 $
 */
class Sector {
public:

  /**
   * Creates a new sector fron two angles.
   * @param min_angle The minimum angle of the sector
   * @param max_angle The maximum angle of the sector
   */
  Sector(const AngleDeg& min_angle,
	 const AngleDeg& max_angle);

  /**
   * Creates a sector from three points, the source and two destination points.
   *
   * @param from The source point of the circle sector
   * @param first_pos One position on the cirle sector
   * @param last_pos The other position on the circle sector
   */
  Sector(const Point& from,
	 const Point& first_pos,
	 const Point& last_pos);

  /**
   * Unallocates the sector
   */
  ~Sector();

  /**
   * Gets the absolute minimum angle of the circle sector
   * @return The absolute minimum angle of the circle sector
   */
  AngleDeg min_angle() const;

  /**
   * Gets the absolute maximum angle of the circle sector
   * @return The absolute maximum angle of the circle sector
   */
  AngleDeg max_angle() const;

  /**
   * Gets the global angle that is the center of the circle sector
   * @return The global angle that is the center of the circle sector
   */
  AngleDeg center() const;

  /**
   * The width of the circle sector
   * @return The width of the circle sector
   */
  AngleDeg width() const;

  /**
   * Checks if the circle sector contains a specific absolute angle.
   * @param angle The absolute angle to check against
   * @return true if the sector contains the angle
   */
  bool contains(const AngleDeg& angle) const;


protected:
  void normalize();
  
  
private:
  AngleDeg min_angle_;
  AngleDeg max_angle_;
};


RS_END_NAMESPACE


#endif // !SECTOR_H
