/* -*- Mode: C++ -*-
 * $Id: ServerCommand.h,v 1.9 2003/07/21 09:19:19 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ServerCommand.h
 * Unitname: Library
 * $Revision: 1.9 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:19:19 $
 *
 *
 * HISTORY
 *
 * $Log: ServerCommand.h,v $
 * Revision 1.9  2003/07/21 09:19:19  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.8  2000/09/09 11:49:47  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.7  2000/09/05 21:08:51  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.6  2000/09/05 09:36:40  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.5  2000/09/03 14:23:19  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.4  2000/09/03 11:10:29  frehe
 * Made the clone method constant.
 *
 * Revision 1.3  2000/09/01 11:43:01  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 01:24:41  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 */

/**
 * @file ServerCommand.h
 * Include file for the class ServerCommand.
 * ServerCommands are commands that could be sent to the soccerserver.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.9 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:19:19 $
 */

#ifndef SERVERCOMMAND_H
#define SERVERCOMMAND_H

#include "Command.h"

RS_BEGIN_NAMESPACE

/**
 * @class ServerCommand ServerCommand.h
 * Class used as the base class of all server commands
 * defined in RoboSoc.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.9 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:19:19 $
 */
class ServerCommand : public Command {
public:
  ServerCommand(const CommandType);
  virtual ~ServerCommand();
  
  /** Returns the server string that will execute the ServerCommand
    at the soccer server. */
  virtual const std::string serverString() const = 0;

  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual ServerCommand* clone() const = 0;
};

RS_END_NAMESPACE

#endif // !SERVERCOMMAND_H
