/* -*- Mode: C++ -*-
 * $Id: Time.cc,v 1.5 2003/07/21 09:22:02 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Time.cc
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:22:02 $
 *
 *
 * HISTORY
 *
 * $Log: Time.cc,v $
 * Revision 1.5  2003/07/21 09:22:02  s02davro
 * Fixed small bug(?) in time
 *
 * Revision 1.4  2002/09/03 10:54:04  s02davro
 * Updated Time and TimeUnknown
 *
 * Revision 1.3  2002/08/02 21:36:58  s02davro
 * Added documentation, changed types.
 *
 * Revision 1.2  2001/09/03 19:38:59  frehe
 * Removed dependencies on <iostream>.
 *
 * Revision 1.1  2000/09/01 01:24:47  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Time.cc
 * Implementation of the class Time.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:22:02 $
 */


#include <string>
#include <iostream>
#include "Time.h"
#include "string_utils.h"
#include "rs_error.h"


#ifdef RS_WARNING
#  define TIME_WARNING( str ) RS_WARNING( std::string("Time: ") + str )
#else
#  define TIME_WARNING( str )
#endif

#ifdef RS_ERROR
#  define TIME_ERROR( str ) RS_ERROR( std::string("Time: ") + str )
#else
#  define TIME_ERROR( str )
#endif


RS_BEGIN_NAMESPACE

bool Time::clockStopped = false;
Time Time::currentTime = Time(0,0);
Time Time::lastStopClockTime = Time(0,0);
Time Time::secondLastStopClockTime = Time(0,0);


Time::Time(const UInt vt)
  : t(vt),
    s(0) {
}

Time::Time(const UInt vt, const UInt vs)
  : t(vt),
    s(vs) {
}

Time::Time(const Time& obj)
  : t(obj.t),
    s(obj.s) {
}

Time& Time::operator=(const Time& a) {
  if ( this != &a ) {
    t = a.t;
    s = a.s;
  }
  return *this;
}

Time Time::operator-(const Time& a) const { 
  if ((t - a.t) < 0) return Time(0,0);
  else if ((s - a.s) < 0) return Time(t - a.t, 0);
  else return Time(t - a.t, s - a.s);
}

Time Time::operator-(const UInt a) const {
  if ( s==0 && t-a > lastStopClockTime.t )
    return Time(t-a,0);

  if ( s>0 ) { 
    if ( a <= s ) 
      return Time(t,s-a); // Just take off from stopped time 
    else { // a > s 
      Time new_time = Time( t-(a-s),0 );
      // take off from stopped time, then server time 
      if ( new_time < lastStopClockTime ) 
	RS_LOG( LA_TIME, "can't be sure of this subtraction (1) " + toString(*this)
		+ " - " + toString(a));
      return new_time;
    }
  } else { // t-a <= lastStopClockTime.t 
    UInt stopped_time =  a - (t - lastStopClockTime.t);  
    if ( stopped_time > lastStopClockTime.s ) {
      if ( !lastStopClockTime.t )
	// if == 0, OK--account for players starting at different times 
	return Time(0,0);
      UInt tmp = lastStopClockTime.t - (stopped_time - lastStopClockTime.s);
      if ( tmp <= secondLastStopClockTime.t ) 
	RS_LOG( LA_TIME, "can't be sure of this subtraction (2) " 
		+ toString(*this) + " - "
		+ toString(a) + "  " + toString(lastStopClockTime));
      return Time( tmp, 0 );
    }
    return Time( lastStopClockTime.t, lastStopClockTime.s - stopped_time );
  }
}

Time Time::operator+(const Time& a) const { 
  return Time(t + a.t, s + a.s);
}

Time Time::operator +(const UInt a) const {  
  if (clockStopped)
    return Time(t,s+a);
  else
    return Time(t+a,0);
}

std::ostream& operator<<(std::ostream&  os, const Time& ct) {
#ifdef TIME_HEAD
  return os << TIME_HEAD << "(" << ct.getT() << ", " << ct.getS() << ")";
#else
  return os << "(" << ct.getT() << ", " << ct.getS() << ")";
#endif
}

const std::string toString(const Time& obj) {
#ifdef TIME_HEAD
  return TIME_HEAD + "(" + toString(obj.getT()) + ", " + 
    toString(obj.getS()) + ")";
#else
  return "(" + toString(obj.getT()) + ", " + ::toString(obj.getS()) + ")";
#endif
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "Time.icc"
#endif
