/* -*- Mode: C++ -*-
 * $Id: TimeUnknown.cc,v 1.3 2002/09/03 10:54:05 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TimeUnknown.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/09/03 10:54:05 $
 *
 *
 * HISTORY
 *
 * $Log: TimeUnknown.cc,v $
 * Revision 1.3  2002/09/03 10:54:05  s02davro
 * Updated Time and TimeUnknown
 *
 * Revision 1.2  2002/08/02 21:37:10  s02davro
 * Changed inheritance, now looks like the other Unknown classes.
 *
 * Revision 1.1  2000/09/01 01:24:47  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file TimeUnknown.cc
 * Implementation of the class TimeUnknown.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: s02davro $
 * $Date: 2002/09/03 10:54:05 $
 */


#include "TimeUnknown.h"
#include "string_utils.h"

RS_BEGIN_NAMESPACE


TimeUnknown::TimeUnknown()
  : WithUnknown<Time>() {
}

TimeUnknown::TimeUnknown(const UInt vt)
  : WithUnknown<Time>(Time(vt)) {
}

TimeUnknown::TimeUnknown(const UInt vt, const UInt vs)
  : WithUnknown<Time>(Time(vt, vs)) {
}

TimeUnknown::TimeUnknown(const Time& obj)
  : WithUnknown<Time>(obj) {
}

TimeUnknown::TimeUnknown(const WithUnknown<Time>& obj)
  : WithUnknown<Time>(obj) {
}

TimeUnknown::TimeUnknown(const TimeUnknown& obj) {
  if (obj.isKnown())
    value = obj.getValue();
  unknown = obj.isUnknown();
}

TimeUnknown& TimeUnknown::operator=(const TimeUnknown& a) {
  if ( this != &a ) {
    if (a.isKnown())
      value = a.getValue();
    unknown = a.isUnknown();
  }
  return *this;
}


std::ostream& operator<<(std::ostream&  os, const TimeUnknown& obj) {
#ifdef TIMEUNKNOWN_HEAD
  if ( obj.isKnown() )
    return os << TIMEUNKNOWN_HEAD << "(" << obj.getT() << ", " 
	      << obj.getS() << ")";
  else
    return os << TIMEUNKNOWN_HEAD << "(UNKNOWN)";
#else
  if ( obj.isKnown() )
    return os << "(" << obj.getT() << ", " << obj.getS() << ")";
  else
    return os << "(UNKNOWN)";
#endif
}

const std::string toString(const TimeUnknown& obj) {
#ifdef TIMEUNKNOWN_HEAD
  if ( obj.isKnown() )
    return TIMEUNKNOWN_HEAD + "(" + toString(obj.getT()) + ", " 
      + toString(obj.getS()) + ")";
  else
    return TIMEUNKNOWN_HEAD + "(UNKNOWN)";
#else
  if ( obj.isKnown() )
    return "(" + toString(obj.getT()) + ", " + toString(obj.getS()) + ")";
  else
    return "(UNKNOWN)";
#endif
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "TimeUnknown.icc"
#endif

