/* -*- Mode: C++ -*-
 * $Id: TimeUnknown.h,v 1.5 2002/09/03 10:54:05 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TimeUnknown.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2002/09/03 10:54:05 $
 *
 *
 * HISTORY
 *
 * $Log: TimeUnknown.h,v $
 * Revision 1.5  2002/09/03 10:54:05  s02davro
 * Updated Time and TimeUnknown
 *
 * Revision 1.4  2002/08/31 14:55:32  s02davro
 * Small modifications
 *
 * Revision 1.3  2002/08/02 21:37:10  s02davro
 * Changed inheritance, now looks like the other Unknown classes.
 *
 * Revision 1.2  2000/09/20 15:10:57  frehe
 * Added documentation.
 *
 * Revision 1.1  2000/09/01 01:24:42  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file TimeUnknown.h
 * Include file for the class TimeUnknown.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/09/03 10:54:05 $
 */


#ifndef TIMEUNKNOWN_H
#define TIMEUNKNOWN_H

#include "Time.h"
#include "WithUnknown.h"
#include "UnknownValueException.h"
#include "library_types_with_unknown.h"

#define TIMEUNKNOWN_HEAD std::string("#TimeUnknown")

RS_BEGIN_NAMESPACE

/**
 * @class TimeUnknown TimeUnknown.h
 * Class used to represent the soccer server time that can be unknown.
 * This class is based on CMUnited's implementation created by P. Stone & Co.
 *
 * @todo Check to see if not more arguments could be unknown.
 * @todo Needs testing.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2002/09/03 10:54:05 $
 */
class TimeUnknown : public WithUnknown<Time> {
public:
  /** The default constructor will make the object value unknown. */
  TimeUnknown();

  /** Constuct a object based on the gametime. */
  TimeUnknown(const UInt vt);
  
  /** Constuct a object based on the gametime and cycletime. */
  TimeUnknown(const UInt vt, const UInt vs);

  /** Construct a object based on a known timevalue. */
  TimeUnknown(const Time& obj);

  /** Construct a TimeUnknown using a WithUnknown<Time> */
  TimeUnknown(const WithUnknown<Time>& obj);

  /** Construct a object based on a (unknown) timevalue. */
  TimeUnknown(const TimeUnknown& obj);

  /** Assignment operator. */
  TimeUnknown& operator = (const TimeUnknown&);

  /** @name Arithmetic operators. */
  //@{
  TimeUnknown& operator +=(const TimeUnknown& op);
  TimeUnknown& operator -=(const TimeUnknown& op);
  TimeUnknown  operator - (const TimeUnknown& op) const;
  TimeUnknown  operator + (const TimeUnknown& op) const;

  /** @name Increment and decrement operators. */
  //@{
  TimeUnknown& operator ++();
  TimeUnknown& operator --();
  //@}

  /** @name Comparison operators. 
   *  @see WithUnknown for more information. */
  //@{
  bool operator ==(const TimeUnknown&) const;
  bool operator !=(const TimeUnknown&) const;
  bool operator < (const TimeUnknown&) const;
  bool operator <=(const TimeUnknown&) const;
  bool operator > (const TimeUnknown&) const;
  bool operator >=(const TimeUnknown&) const;
  bool operator ! () const;
  //@}

  /** Make the time unknown. */
  TimeUnknown& makeUnknown();

  /** Check if the time is known. */
  bool isKnown() const;

  /** Check if the time is unknown. */
  bool isUnknown() const;

  /** Get the time.
   *  @trows UnknownValueException if the time is unknown. */
  Time getTime() const throw(UnknownValueException);

  /** Get the current gametime. */
  UInt getS() const throw(UnknownValueException);

  /** Set the current gametime. */
  TimeUnknown& setS(const UInt);

  /** Get the current cycletime. */
  UInt getT() const throw(UnknownValueException);

  /** Set the current cycletime. */
  TimeUnknown& setT(const UInt);
};

RS_END_NAMESPACE


  
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "TimeUnknown.icc"
#endif

#endif // !TIMEUNKNOWN_H
