/* -*- Mode: C++ -*-
 * $Id: TrainerChangeModeCommand.h,v 1.2 2005/11/15 10:27:55 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TrainerChangeModeCommand.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-23
 * Last modified by $Author: petan $ $Date: 2005/11/15 10:27:55 $
 *
 *
 * HISTORY
 *
 * $Log: TrainerChangeModeCommand.h,v $
 * Revision 1.2  2005/11/15 10:27:55  petan
 * Updated documentation for trainer commands.
 *
 * Revision 1.1  2004/08/23 20:45:29  frehe
 * Added trainer commands.
 *
 */

/**
 * @file TrainerChangeModeCommand.h
 * Include file for the class TrainerChangeModeCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: petan $
 * $Date: 2005/11/15 10:27:55 $
 */


#ifndef TRAINER_CHANGE_MODE_COMMAND_H
#define TRAINER_CHANGE_MODE_COMMAND_H


#include "TrainerCommand.h"
#include "soccer_types.h"


RS_BEGIN_NAMESPACE


/**
 * @class TrainerChangeModeCommand TrainerChangeModeCommand.h
 * Class used to represent the trainer command (change_mode PLAY_MODE).
 *
 * Change the play-mode to PLAY MODE. PLAY MODE must match one of the modes defined in SoccerServer Manual Section 4.7.1.
 * Note that for most play-mode requests the soccerserver will only change the play-mode. The position of the ball
 * usually remains unchanged, but in some cases players will be moved. E.g. in free-kick and kick-in
 * playmodes they will be moved away from the ball if they stand within a certain radius.
 * When changing to `before kick off' they will be even moved to their own side.
 * Possible replies by the soccerserver:
 * <ol><li> (ok change mode) <br> The command succeeded. </li>
 * <li>(error illegal mode)<br> The specified mode was not valid.</li>
 * <li>(error illegal command form)<br> The PLAY MODE argument was omitted.</li>
 * </ol>
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: petan $
 * $Date: 2005/11/15 10:27:55 $
 */
class TrainerChangeModeCommand : public TrainerCommand {
public:
  TrainerChangeModeCommand(const PlayMode play_mode);

  ~TrainerChangeModeCommand();

  /** Returns the server string that will execute the ChangeModeCommand
      at the soccer server. */
  const std::string serverString() const;

  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  TrainerChangeModeCommand* clone() const;


private:
  PlayMode play_mode_;
};


RS_END_NAMESPACE


#endif // !TRAINER_CHANGE_MODE_COMMAND_H
