/* -*- Mode: C++ -*-
 * $Id: TrainerCommand.h,v 1.1 2003/07/21 08:29:16 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TrainerCommand.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-09
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 08:29:16 $
 *
 *
 * HISTORY
 *
 * $Log: TrainerCommand.h,v $
 * Revision 1.1  2003/07/21 08:29:16  s02davro
 * Added new commands (v9, trainer, coach, new command hierarchy)
 *
 * Revision 1.1  2000/09/09 11:49:45  frehe
 * Added a new level in the command hieararchy, namely TrainerCommand and PeriodicCommand.
 *
 */

/**
 * @file TrainerCommand.h
 * Include file for the class TrainerCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2003/07/21 08:29:16 $
 */

#ifndef TRAINERCOMMAND_H
#define TRAINERCOMMAND_H

#include "ImmediateCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class TrainerCommand TrainerCommand.h
 * Class used as the base class of all trainer commands
 * defined in RoboSoc. (Can be sent directly to server, and
 * more than one every cycle, only by trainer)
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: s02davro $
 * $Date: 2003/07/21 08:29:16 $
 */
class TrainerCommand : public ImmediateCommand {
public:
  TrainerCommand(const CommandType);
  virtual ~TrainerCommand();

  /** Returns the server string that will execute the ServerCommand
      at the soccer server. */
  virtual const std::string serverString() const = 0;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual TrainerCommand* clone() const = 0;
};

RS_END_NAMESPACE

#endif // !TRAINERCOMMAND_H
