/* -*- Mode: C++ -*-
 * $Id: TrainerEarCommand.h,v 1.1 2005/11/15 10:27:55 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TrainerEarCommand.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: David Rosen 2003-06-30
 * Last modified by $Author: petan $ $Date: 2005/11/15 10:27:55 $
 *
 *
 * HISTORY
 *
 * $Log: TrainerEarCommand.h,v $
 * Revision 1.1  2005/11/15 10:27:55  petan
 * Updated documentation for trainer commands.
 *
 * Revision 1.1  2003/07/21 08:29:14  s02davro
 * Added new commands (v9, trainer, coach, new command hierarchy)
 *
 * Revision 1.1  2001/11/15 16:48:31  frehe
 * Added a clangVersion command.
 *
 */

/**
 * @file EarCommand.h
 * Include file for the class EarCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: petan $
 * $Date: 2005/11/15 10:27:55 $
 */

#ifndef TRAINEREARCOMMAND_H
#define TRAINEREARCOMMAND_H

#include "TrainerCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class TrainerEarCommand TrainerEarCommand.h
 * Class used to represent the ear command
 * defined in RoboSoc.
 * (ear MODE) It turns on or off the sending of auditory information to the trainer.
 * MODE must be one of on and off. If (ear on) is sent, the server sends all auditory information
 * to the trainer. If (ear off) is sent, the server stops sending auditory information to the trainer.
 * Possible replies by the soccerserver: (ok ear on) (ok ear off) Both replies indicate that
 * the command succeeded. (error illegal mode) MODE did not match on or off. 
 * (error illegal command form) The MODE argument was omitted.
 *
 * For more information, see the Soccer Server Manual
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: petan $
 * $Date: 2005/11/15 10:27:55 $
 */
class TrainerEarCommand : public TrainerCommand {
public:
  TrainerEarCommand(bool def_on = false);
  ~TrainerEarCommand();
  
  /** Returns the server string that will execute the EarCommand
      at the soccer server. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  TrainerEarCommand* clone() const;
  
private:
  /** The actual commands. */
  bool on;
};

RS_END_NAMESPACE

#endif // !TRAINEREARCOMMAND_H
