/* -*- Mode: C++ -*-
 * $Id: TrainerMoveObjectCommand.cc,v 1.3 2004/08/31 12:29:07 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TrainerMoveObjectCommand.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2004-08-23
 * Last modified by $Author: frehe $ $Date: 2004/08/31 12:29:07 $
 *
 * HISTORY
 *
 * $Log: TrainerMoveObjectCommand.cc,v $
 * Revision 1.3  2004/08/31 12:29:07  frehe
 * Fixed bug with uninitialized member.
 *
 * Revision 1.2  2004/08/26 17:08:41  frehe
 * Fixed implementation of moving players.
 *
 * Revision 1.1  2004/08/23 20:45:29  frehe
 * Added trainer commands.
 *
 */

/**
 * @file TrainerMoveObjectCommand.cc
 * Implementation of the class TrainerMoveObjectCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2004/08/31 12:29:07 $
 */


#include "TrainerMoveObjectCommand.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


TrainerMoveObjectCommand::TrainerMoveObjectCommand(Float x, Float y)
  : TrainerCommand(CMD_MoveObject),
    ball_(true),
    x_(x),
    y_(y),
    vel_(false),
    dir_(false),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const Point& pos)
  : TrainerCommand(CMD_MoveObject),
    ball_(true),
    x_(pos.getX()),
    y_(pos.getY()),
    vel_(false),
    dir_(false),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(Float x, Float y,
						   Float vel_x, Float vel_y)
  : TrainerCommand(CMD_MoveObject),
    ball_(true),
    x_(x),
    y_(y),
    vel_(true),
    vel_x_(vel_x),
    vel_y_(vel_y),
    dir_(false),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const Point& pos,
						   const Point& vel)
  : TrainerCommand(CMD_MoveObject),
    ball_(true),
    x_(pos.getX()),
    y_(pos.getY()),
    vel_(true),
    vel_x_(vel.getX()),
    vel_y_(vel.getY()),
    dir_(false),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const std::string& team,
						   UInt unum,
						   const Point& pos)
  : TrainerCommand(CMD_MoveObject),
    ball_(false),
    team_(team),
    unum_(unum),
    x_(pos.getX()),
    y_(pos.getY()),
    vel_(false),
    dir_(false),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const std::string& team,
						   UInt unum,
						   Float x,
						   Float y)
  : TrainerCommand(CMD_MoveObject),
    ball_(false),
    team_(team),
    unum_(unum),
    x_(x),
    y_(y),
    vel_(false),
    dir_(false),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const std::string& team,
						   UInt unum,
						   const Point& pos,
						   Float dir)
  : TrainerCommand(CMD_MoveObject),
    ball_(false),
    team_(team),
    unum_(unum),
    x_(pos.getX()),
    y_(pos.getY()),
    vel_(false),
    dir_(true),
    bdir_(dir),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const std::string& team,
						   UInt unum,
						   Float x,
						   Float y,
						   Float dir)
  : TrainerCommand(CMD_MoveObject),
    ball_(false),
    team_(team),
    unum_(unum),
    x_(x),
    y_(y),
    vel_(false),
    dir_(true),
    bdir_(dir),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const std::string& team,
						   UInt unum,
						   const Point& pos,
						   Float dir,
						   const Point& vel)
  : TrainerCommand(CMD_MoveObject),
    ball_(false),
    team_(team),
    unum_(unum),
    x_(pos.getX()),
    y_(pos.getY()),
    vel_(true),
    vel_x_(vel.getX()),
    vel_y_(vel.getY()),
    dir_(true),
    bdir_(dir),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const std::string& team,
						   UInt unum,
						   Float x, Float y,
						   Float dir,
						   Float vel_x, Float vel_y)
  : TrainerCommand(CMD_MoveObject),
    ball_(false),
    team_(team),
    unum_(unum),
    x_(x),
    y_(y),
    vel_(true),
    vel_x_(vel_x),
    vel_y_(vel_y),
    dir_(true),
    bdir_(dir),
    cmd_(false)
{
}


TrainerMoveObjectCommand::TrainerMoveObjectCommand(const std::string& cmd)
  : TrainerCommand(CMD_MoveObject),
    cmd_(true),
    cmd_string_(cmd)
{
}


TrainerMoveObjectCommand::~TrainerMoveObjectCommand()
{
}


const std::string
TrainerMoveObjectCommand::serverString() const
{
  if ( cmd_ ) {
    return cmd_string_;
  } else {
    std::string cmd = "(move ";

    if ( ball_  ) {
      cmd += "(ball) ";
    } else {
      cmd += "(player " + team_ + " " + toString(unum_) + ") ";
    }

    cmd += toString(x_) + " " + toString(y_);
    if ( dir_ ) {
      cmd += " " + toString(bdir_);
    } else if ( vel_ ) {
      // If we have velocity we must have direction as well!
      cmd += " 0";
    }

    if ( vel_ ) {
      cmd += " " + toString(vel_x_) + " " + toString(vel_y_);
    }

    cmd += ")";
    return cmd;
  }
}


TrainerMoveObjectCommand*
TrainerMoveObjectCommand::clone() const {
  return new TrainerMoveObjectCommand(*this);
}


RS_END_NAMESPACE
