/* -*- Mode: C++ -*-
 * $Id: TrainerRecoverCommand.h,v 1.2 2005/11/15 10:27:55 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TrainerRecoverCommand.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-23
 * Last modified by $Author: petan $ $Date: 2005/11/15 10:27:55 $
 *
 *
 * HISTORY
 *
 * $Log: TrainerRecoverCommand.h,v $
 * Revision 1.2  2005/11/15 10:27:55  petan
 * Updated documentation for trainer commands.
 *
 * Revision 1.1  2004/08/23 20:45:30  frehe
 * Added trainer commands.
 *
 */

/**
 * @file TrainerRecoverCommand.h
 * Include file for the class TrainerRecoverCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: petan $
 * $Date: 2005/11/15 10:27:55 $
 */


#ifndef TRAINER_RECOVER_COMMAND_H
#define TRAINER_RECOVER_COMMAND_H


#include "TrainerCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class TrainerRecoverCommand TrainerRecoverCommand.h
 * Class used to represent the trainer command (recover).
 *
 * (recover) This command resets players' stamina, recovery, effort
 * and hear capacity to the values at the beginning of the game. 
 * Possible replies by the soccerserver: (ok recover) The command succeeded.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: petan $
 * $Date: 2005/11/15 10:27:55 $
 */
class TrainerRecoverCommand : public TrainerCommand {
public:
  TrainerRecoverCommand();
  
  ~TrainerRecoverCommand();
  
  /** Returns the server string that will execute the RecoverCommand
      at the soccer server. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  TrainerRecoverCommand* clone() const;
};


RS_END_NAMESPACE


#endif // !TRAINER_RECOVER_COMMAND_H
