/* -*- Mode: C++ -*-
 * $Id: TrainerStartCommand.h,v 1.2 2005/11/15 10:27:55 petan Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TrainerStartCommand.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2004-08-23
 * Last modified by $Author: petan $ $Date: 2005/11/15 10:27:55 $
 *
 *
 * HISTORY
 *
 * $Log: TrainerStartCommand.h,v $
 * Revision 1.2  2005/11/15 10:27:55  petan
 * Updated documentation for trainer commands.
 *
 * Revision 1.1  2004/08/23 20:45:30  frehe
 * Added trainer commands.
 *
 */

/**
 * @file TrainerStartCommand.h
 * Include file for the class TrainerStartCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: petan $
 * $Date: 2005/11/15 10:27:55 $
 */


#ifndef TRAINER_START_COMMAND_H
#define TRAINER_START_COMMAND_H


#include "TrainerCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class TrainerStartCommand TrainerStartCommand.h
 * Class used to represent the trainer command (start).
 * (start) This commands starts the server, e.g. sets the play-mode to `kick off l'.
 * This essentially simulates pressing the kick off button on the monitor.
 * If the trainer does not send an init command, then the first commands
 *  of any type received from the trainer will cause the server to start,
 *  e.g. set the play-mode to `kick off l'. Possible replies by the soccerserver:
 *  (ok start) The command succeeded.
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: petan $
 * $Date: 2005/11/15 10:27:55 $
 */
class TrainerStartCommand : public TrainerCommand {
public:
  TrainerStartCommand();
  
  ~TrainerStartCommand();
  
  /** Returns the server string that will execute the StartCommand
      at the soccer server. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  TrainerStartCommand* clone() const;
};


RS_END_NAMESPACE


#endif // !TRAINER_START_COMMAND_H
