/* -*- Mode: C++ -*-
 * $Id: TurnCommand.h,v 1.5 2003/07/21 09:20:16 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnCommand.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:20:16 $
 *
 *
 * HISTORY
 *
 * $Log: TurnCommand.h,v $
 * Revision 1.5  2003/07/21 09:20:16  s02davro
 * Changed and updated the command hierarchy
 *
 * Revision 1.4  2000/09/09 11:49:47  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.3  2000/09/05 21:08:52  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.2  2000/09/03 11:10:32  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 14:16:26  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file TurnCommand.h
 * Include file for the class TurnCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:20:16 $
 */

#ifndef TURNCOMMAND_H
#define TURNCOMMAND_H

#include "PeriodicCommand.h"

RS_BEGIN_NAMESPACE

/**
 * @class TurnCommand TurnCommand.h
 * Class used to represent the server command
 * (turn Angle)
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:20:16 $
 */
class TurnCommand : public PeriodicCommand {
public:
  TurnCommand(const AngleDeg& ang);
  ~TurnCommand();
 
  /** Returns the server string that will execute the TurnCommand
      at the soccer server. */
  const std::string serverString() const;
  
  /** Returns a new copy of the command.
      REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  TurnCommand* clone() const;

  AngleDeg getAngle() const;
private:
  AngleDeg angle;
};

RS_END_NAMESPACE

#endif // !TURNCOMMAND_H
