/* -*- Mode: C++ -*-
 * $Id: library_types.h,v 1.9 2003/07/21 09:23:15 s02davro Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: library_types.h
 * Unitname: Library
 * $Revision: 1.9 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: s02davro $ $Date: 2003/07/21 09:23:15 $
 *
 *
 * HISTORY
 *
 * $Log: library_types.h,v $
 * Revision 1.9  2003/07/21 09:23:15  s02davro
 * Removed the command types from library_types, now in Command.h
 *
 * Revision 1.8  2002/10/10 08:04:49  s02davro
 * Added CoordRectangle
 *
 * Revision 1.7  2002/10/01 16:07:14  s02davro
 * Added the new commands to makefile and library_types.h
 *
 * Revision 1.6  2002/09/04 13:54:05  s02davro
 * Removed rectangle again
 *
 * Revision 1.5  2002/09/04 13:23:26  s02davro
 * Added Circle and rectangle
 *
 * Revision 1.4  2002/08/02 21:38:04  s02davro
 * Added some things?
 *
 * Revision 1.3  2001/11/15 16:48:32  frehe
 * Added a coach command.
 *
 * Revision 1.2  2000/09/01 01:24:44  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file library_types.h
 * Include file with the types used by the RoboSoc library.
 *
 * @todo Split up the types in this file on more files.
 *       Maybe have one file for the "basic" types, one with
 *       Unknown types and one with soccer related types.
 * @todo Remove the CommandType by using polymorphism instead.
 * @todo Find better names for some of the enumerations.
 * @todo Decide how to capitalize the enumeration constants.
 * @par Done:
 *      Since they are constants they should be all capitals.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.9 $
 * $Author: s02davro $
 * $Date: 2003/07/21 09:23:15 $
 */


#ifndef LIBRARY_TYPES_H
#define LIBRARY_TYPES_H


#include "machine_dependent_types.h"
#include "AngleDegrees.h"
#include "AngleRadians.h"
#include "CoordPoint.h"
#include "CoordVector.h"
#include "CoordCircle.h"
#include "CoordRectangle.h"
#include "ObjectCollection.h"


RS_BEGIN_NAMESPACE


/** A type for representing the standard unsigned integer.
  It is used when the precision of the number only defines
  how exact the result will be. To increase/decrease the
  precision the user can redefine this type. */
typedef UInt16 UInt;

/** A type for representing the standard signed integer.
  It is used when the precision of the number only defines
  how exact the result will be. To increase/decrease the
  precision the user can redefine this type. */
typedef Int32 Int;

/** A type for representing the standard floating point
  number. It is used when the precision of the number only
  defines how exact the result will be. To increase/decrease
  the precision the user can redefine this type. */
typedef Float64 Float;


/** A type for representing the standard angles in degrees. */
typedef AngleDegrees<Float> AngleDeg;

/** A type for representing the standard angles in radians. */
typedef AngleRadians<Float> AngleRad;


/** A type for representing standard points. */
typedef CoordPoint<Float> Point;

/** A type for representing standard vectors. */
typedef CoordVector<Float> Vector;

/** A type for representing standard circles. */
typedef CoordCircle<Float> Circle;

/** A type for representing standard circles. */
typedef CoordRectangle<Float> Rectangle;

/** Type for representing a collection of Int. */
typedef ObjectCollection<Int> IntCollection;

/** Type for representing a collection of Float. */
typedef ObjectCollection<Float> FloatCollection;

/** Type for representing a collection of UInt. */
typedef ObjectCollection<UInt> UIntCollection;

/** Type for representing a collection of Vector. */
typedef ObjectCollection<Vector> VectorCollection;

/** Type for representing a collection of Point. */
typedef ObjectCollection<Point> PointCollection;

/** Type for representing a collection of AngleDeg. */
typedef ObjectCollection<AngleDeg> AngleDegCollection;

RS_END_NAMESPACE


#endif // !LIBRARY_TYPES_H
