/* -*- Mode: C++ -*-
 * $Id: quantize_utils.cc,v 1.1 2005/09/19 17:46:26 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: quantize_utils.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2005-09-19
 * Last modified by $Author: frehe $ $Date: 2005/09/19 17:46:26 $
 *
 *
 * HISTORY
 *
 * $Log: quantize_utils.cc,v $
 * Revision 1.1  2005/09/19 17:46:26  frehe
 * Added quantize helpers.
 *
 */

/**
 * @file quantize_utils.cc
 * Include file for the RoboSoc quantize utilities.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2005/09/19 17:46:26 $
 */


#ifndef QUANTIZE_UTILS_H
#define QUANTIZE_UTILS_H


#include "quantize_utils.h"
#include "library_types.h"
#include "math_utils.h"


RS_BEGIN_NAMESPACE


Float64
invQMin(const Float64 def_value, const Float64 qstep)
{
  return (rint(def_value/qstep)-0.5)*qstep;
}

  
Float64
invQMax(const Float64 def_value, const Float64 qstep)
{
  return (rint(def_value/qstep)+0.5)*qstep;
}


Float64
distanceMin(const Float64 def_value, const Float64 qstep)
{
  return exp( invQMin( log( invQMin(def_value - 1.0e-10, 0.1)), qstep));
}


Float64
distanceMax(const Float64 def_value, const Float64 qstep)
{
  return exp( invQMax( log( invQMax(def_value + 1.0e-10, 0.1)), qstep));
}
  

AngleDeg
invQMin(const AngleDeg def_value, const Float64 qstep)
{
  return AngleDeg((rint(def_value.getDeg()/qstep)-0.5)*qstep);
}


AngleDeg
invQMax(const AngleDeg def_value, const Float64 qstep)
{
  return AngleDeg((rint(def_value.getDeg()/qstep)+0.5)*qstep);
}


AngleDeg
distanceMin(const AngleDeg def_value, const Float64 qstep)
{
  return AngleDeg( exp( invQMin(log( invQMin(def_value.getDeg() - 1.0e-10, 0.1)), qstep)));
}


AngleDeg
distanceMax(const AngleDeg def_value, const Float64 qstep)
{
  return AngleDeg( exp( invQMax( log( invQMax(def_value.getDeg() + 1.0e-10, 0.1)), qstep)));
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "quantize_utils.icc"
#endif


#endif // !QUANTIZE_UTILS_H
