/* -*- Mode: C++ -*-
 * $Id: sector_helpers.cc,v 1.1 2004/09/20 10:42:17 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: sector_helpers.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-09-20
 * Last modified by $Author: frehe $ $Date: 2004/09/20 10:42:17 $
 *
 *
 * HISTORY
 *
 * $Log: sector_helpers.cc,v $
 * Revision 1.1  2004/09/20 10:42:17  frehe
 * Added sector helpers.
 *
 */

/**
 * @file sector_helpers.cc
 * Implementation file for the RoboSoc sector utilities.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2004/09/20 10:42:17 $
 */


#include "sector_helpers.h"
#include <algorithm>


RS_BEGIN_NAMESPACE


typedef std::vector<Point> Points;
typedef std::vector<AngleDeg> Angles;
typedef std::vector<Sector> Sectors;


Sector
get_largest_sector(const Point& from,
		   const std::vector<Point>& to,
		   const Sector& sector)
{
  Angles angles;
  angles.push_back(sector.max_angle());
  for ( Points::const_iterator p = to.begin(); p != to.end(); ++p ) {
    AngleDeg angle = from.getDirectionTo(*p);
    if ( sector.contains(angle) ) {
      angles.push_back(angle);
    }
  }
  std::sort(angles.begin(), angles.end());
  
  AngleDeg prev_angle = sector.min_angle();
  Sector best_sector(0, 0);
  for ( Angles::iterator a = angles.begin(); a != angles.end(); ++a ) {
    if ( (*a - prev_angle) > best_sector.width() ) {
      best_sector = Sector(prev_angle, *a);
    }
    prev_angle = *a;
  }

  return best_sector;
}


std::vector<Sector>
get_sector_wider_than(const Point& from,
		      const std::vector<Point>& to,
		      const Sector& sector,
		      const AngleDeg& min_width)
{
  Angles angles;
  angles.push_back(sector.max_angle());
  for ( Points::const_iterator p = to.begin(); p != to.end(); ++p ) {
    AngleDeg angle = from.getDirectionTo(*p);
    if ( sector.contains(angle) ) {
      angles.push_back(angle);
    }
  }
  std::sort(angles.begin(), angles.end());
  
  AngleDeg prev_angle = sector.min_angle();
  Sectors sectors;
  for ( Angles::iterator a = angles.begin(); a != angles.end(); ++a ) {
    if ( (*a - prev_angle) > min_width ) {
      sectors.push_back(Sector(prev_angle, *a));
    }
    prev_angle = *a;
  }

  return sectors;
}


RS_END_NAMESPACE
