/* -*- Mode: C++ -*-
 * $Id: sector_helpers.h,v 1.1 2004/09/20 10:42:17 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: sector_helpers.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2004-09-20
 * Last modified by $Author: frehe $ $Date: 2004/09/20 10:42:17 $
 *
 *
 * HISTORY
 *
 * $Log: sector_helpers.h,v $
 * Revision 1.1  2004/09/20 10:42:17  frehe
 * Added sector helpers.
 *
 */

/**
 * @file sector_helpers.h
 * Include file for the RoboSoc sector utilities.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2004/09/20 10:42:17 $
 */


#ifndef SECTOR_HELPERS_H
#define SECTOR_HELPERS_H


#include "Sector.h"
#include <vector>


RS_BEGIN_NAMESPACE


/**
 * Return the largest sector defined as the angles between the
 * points in @a to as seen from @a from which are within @a sector.
 */
Sector get_largest_sector(const Point& from, const std::vector<Point>& to,
			  const Sector& sector);


/**
 * Return all sectors defined as the angles between the points in @a to
 * as seen from @a from which are within @a sectorlarger than @a min_width.
 */
std::vector<Sector> get_sector_wider_than(const Point& from,
					  const std::vector<Point>& to,
					  const Sector& sector,
					  const AngleDeg& min_width);


RS_END_NAMESPACE


#endif // SECTOR_HELPERS_H
