/* -*- Mode: C++ -*-
 * $Id: string_utils.cc,v 1.5 2004/08/23 20:45:20 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: string_utils.cc
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-08-29
 * Last modified by $Author: frehe $ $Date: 2004/08/23 20:45:20 $
 *
 *
 * HISTORY
 *
 * $Log: string_utils.cc,v $
 * Revision 1.5  2004/08/23 20:45:20  frehe
 * Added a toString for play modes, not really fixed.
 *
 * Revision 1.4  2003/06/02 08:58:37  frehe
 * Updated toString to stringstream.
 *
 * Revision 1.3  2001/06/19 14:01:50  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.2  2001/02/22 14:54:08  frehe
 * Made a minor change to string_utils.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file string_utils.cc
 * Implementation of the RoboSoc string utilities, currently only toString().
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2004/08/23 20:45:20 $
 */


#include "string_utils.h"


RS_BEGIN_NAMESPACE


std::string toString(const ViewWidth view_w)
{
  switch ( view_w ) {
  case VW_NORMAL:
    return "normal";
  case VW_WIDE:
    return "wide";
  case VW_NARROW:
    return "narrow";
  default:
    return "unknown";
  }
}


std::string toString(const ViewQuality view_q)
{
  switch ( view_q ) {
  case VQ_HIGH:
    return "high";
  case VQ_LOW:
    return "low";
  default:
    return "unknown";
  }
}


std::string toString(const PlayMode play_mode)
{
  switch ( play_mode ) {
  case PM_UNKNOWN:
    return "PM_UNKNOWN";
  case PM_BEFORE_KICK_OFF:
    return "PM_BEFORE_KICK_OFF";
  case PM_HALF_TIME:
    return "PM_HALF_TIME";
  case PM_TIME_OVER:
    return "PM_TIME_OVER";
  case PM_EXTENDED_TIME:
    return "PM_EXTENDED_TIME";
  case PM_PLAY_ON:
    return "PM_PLAY_ON";
  case PM_DROP_BALL:
    return "PM_DROP_BALL";
  case PM_OUR_OFFSIDE_KICK:
    return "PM_OUR_OFFSIDE_KICK";
  case PM_THEIR_OFFSIDE_KICK:
    return "PM_THEIR_OFFSIDE_KICK";
  case PM_OUR_KICK_OFF:
    return "PM_OUR_KICK_OFF";
  case PM_THEIR_KICK_OFF:
    return "PM_THEIR_KICK_OFF";
  case PM_OUR_KICK_IN:
    return "PM_OUR_KICK_IN";
  case PM_THEIR_KICK_IN:
    return "PM_THEIR_KICK_IN";
  case PM_OUR_FREE_KICK:
    return "PM_OUR_FREE_KICK";
  case PM_THEIR_FREE_KICK:
    return "PM_THEIR_FREE_KICK";
  case PM_OUR_INDIRECT_FREE_KICK:
    return "PM_OUR_INDIRECT_FREE_KICK";
  case PM_THEIR_INDIRECT_FREE_KICK:
    return "PM_THEIR_INDIRECT_FREE_KICK";
  case PM_OUR_CORNER_KICK:
    return "PM_OUR_CORNER_KICK";
  case PM_THEIR_CORNER_KICK:
    return "PM_THEIR_CORNER_KICK";
  case PM_OUR_GOAL_KICK:
    return "PM_OUR_GOAL_KICK";
  case PM_THEIR_GOAL_KICK:
    return "PM_THEIR_GOAL_KICK";
  case PM_OUR_GOAL:
    return "PM_OUR_GOAL";
  case PM_THEIR_GOAL:
    return "PM_THEIR_GOAL";
  case PM_OUR_GOALIE_GOT_BALL:
    return "PM_OUR_GOALIE_GOT_BALL";
  case PM_THEIR_GOALIE_GOT_BALL:
    return "PM_THEIR_GOALIE_GOT_BALL";
  case PM_OUR_PENALTY_KICK:
    return "PM_OUR_PENALTY_KICK";
  case PM_THEIR_PENALTY_KICK:
    return "PM_THEIR_PENALTY_KICK";
  case PM_OUR_PENALTY_SETUP:
    return "PM_OUR_PENALTY_SETUP";
  case PM_THEIR_PENALTY_SETUP:
    return "PM_THEIR_PENALTY_SETUP";
  case PM_OUR_PENALTY_READY:
    return "PM_OUR_PENALTY_READY";
  case PM_THEIR_PENALTY_READY:
    return "PM_THEIR_PENALTY_READY";
  case PM_OUR_PENALTY_TAKEN:
    return "PM_OUR_PENALTY_TAKEN";
  case PM_THEIR_PENALTY_TAKEN:
    return "PM_THEIR_PENALTY_TAKEN";
  case PM_OUR_PENALTY_MISS:
    return "PM_OUR_PENALTY_MISS";
  case PM_THEIR_PENALTY_MISS:
    return "PM_THEIR_PENALTY_MISS";
  case PM_OUR_PENALTY_SCORE:
    return "PM_OUR_PENALTY_SCORE";
  case PM_THEIR_PENALTY_SCORE:
    return "PM_THEIR_PENALTY_SCORE";
  case PM_OUR_PENALTY_ONFIELD:
    return "PM_OUR_PENALTY_ONFIELD";
  case PM_THEIR_PENALTY_ONFIELD:
    return "PM_THEIR_PENALTY_ONFIELD";
  case PM_OUR_PENALTY_FOUL:
    return "PM_OUR_PENALTY_FOUL";
  case PM_THEIR_PENALTY_FOUL:
    return "PM_THEIR_PENALTY_FOUL";
  case PM_OUR_PENALTY_WINNER:
    return "PM_OUR_PENALTY_WINNER";
  case PM_THEIR_PENALTY_WINNER:
    return "PM_THEIR_PENALTY_WINNER";
  case PM_PENALTY_DRAW:
    return "PM_PENALTY_DRAW";
  case PM_FIRST_HALF_OVER:
    return "PM_FIRST_HALF_OVER";
  case PM_PAUSE:
    return "PM_PAUSE";
  case PM_HUMAN:
    return "PM_HUMAN";
  case PM_OUR_FOUL:
    return "PM_OUR_FOUL";
  case PM_THEIR_FOUL:
    return "PM_THEIR_FOUL";  
  case PM_OUR_FOUL_CHARGE:
    return "PM_OUR_FOUL_CHARGE";
  case PM_THEIR_FOUL_CHARGE:
    return "PM_THEIR_FOUL_CHARGE";
  case PM_OUR_FOUL_PUSH:
    return "PM_OUR_FOUL_PUSH";
  case PM_THEIR_FOUL_PUSH:
    return "PM_THEIR_FOUL_PUSH";
  case PM_OUR_FOUL_MULTIPLEATTACKER:
    return "PM_OUR_FOUL_MULTIPLEATTACKER";
  case PM_THEIR_FOUL_MULTIPLEATTACKER:
    return "PM_THEIR_FOUL_MULTIPLEATTACKER";
  case PM_OUR_FOUL_BALLOUT:
    return "PM_OUR_FOUL_BALLOUT";
  case PM_THEIR_FOUL_BALLOUT:
    return "PM_THEIR_FOUL_BALLOUT";
  case PM_OUR_FOUL_BACKPASS:
    return "PM_OUR_FOUL_BACKPASS";
  case PM_THEIR_FOUL_BACKPASS:
    return "PM_THEIR_FOUL_BACKPASS";
  case PM_OUR_FOUL_FREEKICKFAULT:
    return "PM_OUR_FOUL_FREEKICKFAULT";
  case PM_THEIR_FOUL_FREEKICKFAULT:
    return "PM_THEIR_FOUL_FREEKICKFAULT";
  case PM_OUR_FOUL_CATCHFAULT:
    return "PM_OUR_FOUL_CATCHFAULT";
  case PM_THEIR_FOUL_CATCHFAULT:
    return "PM_THEIR_FOUL_CATCHFAULT";
  case PM_TIME_UP_WITHOUT_TEAM:
    return "PM_TIME_UP_WITHOUT_TEAM";
  default:
    return "PM_UNKNOWN";
  }
}


RS_END_NAMESPACE
