/* -*- Mode: C++ -*-
 * $Id: string_utils.h,v 1.6 2004/09/08 18:32:56 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: string_utils.h
 * Unitname: Library
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-08-29
 * Last modified by $Author: frehe $ $Date: 2004/09/08 18:32:56 $
 *
 *
 * HISTORY
 *
 * $Log: string_utils.h,v $
 * Revision 1.6  2004/09/08 18:32:56  frehe
 * Added a from_string template.
 *
 * Revision 1.5  2004/08/23 20:45:20  frehe
 * Added a toString for play modes, not really fixed.
 *
 * Revision 1.4  2003/06/02 08:58:37  frehe
 * Updated toString to stringstream.
 *
 * Revision 1.3  2003/06/02 07:11:12  aip20
 * Change strstream to sstream.
 *
 * Revision 1.2  2001/08/29 21:08:29  frehe
 * Replaces class T with typename T where T can be a built in type.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file string_utils.h
 * Include file for the RoboSoc string utilities, currently only toString().
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2004/09/08 18:32:56 $
 */


#ifndef STRING_UTILS_H
#define STRING_UTILS_H


#include <string>
#include <sstream>
//#include "library_types.h"
#include "soccer_types.h"


RS_BEGIN_NAMESPACE


/** @name String Utilities
  A collection of functions to convert from objects to
  a string describing the object. */
//@{
/** Function used to convert an object to a string, mainly
  using a strstream and the operator<< defined on the object.
  If no operator<< exist then compile time errors will occur. */
std::string toString(const ViewWidth view_w);
std::string toString(const ViewQuality view_q);
std::string toString(const PlayMode play_mode);

template<typename T>
std::string toString(const T& obj)
{
  std::ostringstream str;
  str << obj;
  return str.str();
}
//@}


/// Convert from a string to a specific type
template <typename Type>
Type from_string(const std::string& s)
{
  std::stringstream str;
  Type var;
  str << s;
  str >> var;
  return var;  
}


RS_END_NAMESPACE


#endif // STRING_UTILS_H
